
!********************************************************************************
!*   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
!*   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
!*   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
!*
!*
!*   This file is part of the package SNF.
!*
!*   SNF is free software; you can redistribute it and/or modify
!*   it under the terms of the GNU General Public License as published by
!*   the Free Software Foundation; either version 2 of the License, or
!*   (at your option) any later version.
!*
!*   This program is distributed in the hope that it will be useful,
!*   but WITHOUT ANY WARRANTY; without even the implied warranty of
!*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*   GNU General Public License for more details.
!*
!*   You should have received a copy of the GNU General Public License
!*   along with this program; if not, write to the Free Software
!*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!*
!********************************************************************************
program snf
  use f90_kind
  !----------------------------------------------------------------------
  ! this program needs the files "coord" and "control". and what is more
  ! important, it needs the file "restart" with all raw data.
  ! from that point the complete hessian and dipol moment derivatives 
  ! are calculated as avaible from an c1 calculation. these results will
  ! then be transformed to a symmetry adapted basis optained by diagonalization 
  ! of projection operators. finally the frequencies, normalmodes and intensities 
  ! are collected and sorted in one global spectrum. 
  !
  ! written by Carsten Kind, Markus Reiher and Johannes Neugebauer, 
  !            Uni-Erlangen 1999-2001
  !----------------------------------------------------------------------
  use snf_spectrum,             only: type_peak, print_ir, destroy_spectrum, &
       & calc_frequencies, calc_ir_intensities,calc_raman_intensities,sort_spectrum, print_raman,&
       & print_norm_modes,get_translation_and_rotation, &
       & zeropoint_energy,assign_irrep_number,calc_mode_properties, &
       & get_relcross,sum_degen,print_rcs,get_relir
  use snf_molecule,       only: type_atom,type_molecule,print_molecule,&
    & destroy_molecule,hauptax_trans,print_isomass,isotope_equal,atom_equal,&
    & setisomass,testlin
  use snf_turbomole,      only: read_tm_coord
  use snf_char_table,     only: type_char_table,create_char_table,&
    & destroy_char_table,print_char_table,create_product_table,&
    & print_product_table
  use snf_representation, only: create_representation_for_vib,destroy_representation, &
       print_representation
  use snf_projection,     only: create_proj_op,destroy_proj_op,&
       print_proj_op
  use snf_character,      only: create_character,create_irreps,destroy_character,destroy_irreps, &
       print_character,print_irreps
  use snf_hessian,        only: create_hessian_xyz,print_hessian, &
       &                        destroy_hessian_xyz,expand_hessian,&
       &                        check_hessian,balance_hessian,symmetrize_hessian
  use snf_force,          only: hessian_mass_weighting,write_tm_force, &
       &                        calc_atom_force
  use snf_restart,        only: read_restart,write_restart,backup_restart
  use snf_error,          only: serror
  use snf_xmol,           only: print_xmol_norm
  use snf_control,        only: type_ctrl,read_control,print_ctrl,init_ctrl,destroy_control
  use snf_dipol,          only: create_dipol_dxyz,print_dipol_dxyz, &
       &                        destroy_dipol_dxyz, expand_dipol, &
       &                        create_dipol_dq, symm_dipol
  use snf_polar,          only: create_polar_dxyz,print_polar_dxyz, &
       &                        destroy_polar_dxyz, expand_polar, &
       &                        create_polar_dq , symm_polar
  use snf_irred_dist,     only: create_irred_dist,create_red_grad,&
                                create_red_dip,create_red_pol
  use snf_irred_atoms,    only: create_irred_atoms,destroy_irred_atoms,&
       &                        print_irred_atoms,check_geometry,get_point_grp,is_point_grp
  use snf_thermo,         only: get_tdvalues
  use snf_tools
  use snf_norm_modes
  use snf_gaussian  , only: print_gaussian_frequencies
  use snf_mopac     , only: print_mopac_frequencies
  use snf_cioslowski, only: create_cioslowski,print_cioslowski
  use snf_gnuplot,    only: generate_plots
  use snf_param
  use snf_symmat,     only: symel_mat
  use snf_dummy,      only: rm_dummies
  implicit none

  type(type_ctrl) :: ctrl,ctrl_restart  ! essential control flags and parameters
  type(type_atom), pointer :: &
       & molecule(:)     ! geometry atoms and atomic masses of the molecule  
  type(type_atom),allocatable :: mol(:)
  type(type_molecule)      :: molekel  ! information about the whole molecule
  real(double),pointer :: &
       & gradients(:,:,:)      ! energy gradients of the negative distortion (nvar,nvar)
  real(double),pointer :: &
       & dipols(:,:,:)         ! dipol moments of the negative distortion (nvar,3)
  real(double),pointer :: &
       & pols(:,:,:,:)           ! polarizabilities pols(1..8,nvar=3*n_atoms,&
       !&no_steps=no_distortions,no_frequencies(for dynamic polarizabilities)
  real(double),allocatable :: &
       & distortion(:)         ! generalized distortion in the cartesian basis

  real(double),pointer :: &
       & c2p(:,:),&                ! transformation matrix from cartesian 
       & hessian_xyz(:,:)      ! hessian in the cartesian basis (nvar,nvar)
  real(double),allocatable :: &
       & hessian_m(:,:), &     ! hessian in the mass weighted cartesian basis (nvar,nvar)
       & hessian_p(:,:), &     ! hessian in the symmetryadapted basis of (irrep,irrep)
       & hessian_q(:), &       ! hessian in the basis of normal modes (irrep)
       & norm_modes(:,:)       ! transformation matrix from symmetryadapted
  !                              basis to normal coordinates (irrep,irrep)
  !                              transformation matrix from cartesian basis 
  !                              to normal coordinates  (nvar,irrep)
  real(double),pointer :: &
       & dipol_dxyz(:,:), &    ! derivatives of dipol moments in cartesian basis (nvar,3)
       & dipol_dq(:,:),   &    ! derivatives of dipol moments in normal coordinates (irrep,3)
       & polar_dxyz(:,:,:), &    ! derivatives of polarizability tensor in cartesian basis (nvar,?)
       & polar_dq(:,:,:)         ! derivatives of polarizability tensor in normal coordinates (irrep,?)

  real(double),pointer :: &
       & representation(:,:,:),&! representations of symmetry operations s -->(s,:,:)
       & proj_op(:,:,:), &     ! projectors on irreps i-->(i,:,:)
       & character(:)          ! characters of the representations

  integer,pointer :: &
       & irreps(:)             ! vector with number of vectors per irrep
  type(type_char_table) :: &
       & char_table            ! character table
  type(type_peak), pointer :: &
       & spectrum(:), &       ! spectral information  with frequencies,modes and intensities 
       & global_spectrum(:),& ! spectral information  with frequencies,modes and intensities 
       & sum_spectrum(:)      ! spectral information, summed over degenerate modes
  integer :: &
       & irrep,  &             ! index of irrep
       &     irrep_dim         ! number of vectors in actual irrep, therefore 
  !                              dimension parameter
  integer,pointer          :: &
       &  istepfl(:), &      ! points to calculate(=6*nirred_atoms)
       & atom_index(:), &    ! index of irreducible atom (1:natoms)
       & symm_index(:), &    ! symmetry op. to optain irred. atom (1:natoms)
       & selatom(:), &       ! atom-numbers which special isotopes are selected for
       & irred_atom_index(:) ! index list with irreducible atoms (1:nirred_atoms)

  integer,pointer          :: &
       & dist_index(:), &    ! index of irreducible dist. 
       & dsymm_index(:), &   ! symmetry op. to optain irred. dist. 
       & irred_dist_index(:) ! index list with irreducible dist. 

  character(len=8),pointer :: &
       & nvmach(:)           ! list of all used machines 
  character(len=132) :: errbuff ! buffer for error messages

  real(double),allocatable :: eig_proj_op(:)
  real(double) :: zeropoint ! zero point energy
  real(double),pointer :: massatom(:) ! masses for special isotopes
  integer zeros

  !cmr thermochemistry
  real(double)         :: mol_mass
  real(double),pointer :: freqs(:)
  !cmr 
  character(len=7),pointer :: prod_table(:,:)


  real(double),pointer :: cioslowski(:) ! cioslowski charge decomposition
  ! less important
  integer :: i,j,k,nirred_atoms,nvar,offset,nfreq,nirred_dist
  integer :: number_projected_modes,out
  logical :: new_restart,fdum,static
  ! diagonalization
  integer :: lwork,info,ilaenv,status
  double precision,allocatable :: work(:),tmpmat(:,:)
!JN  out=6
  out=44
  open(unit=out,file='snf.out',status='unknown')
  call print_header(out)
  call last_mod(out)
  nullify(molecule,gradients,dipols,pols,         &
       &  hessian_xyz,c2p,&
       &  dipol_dxyz,dipol_dq,polar_dxyz,polar_dq,           &
       &  representation,proj_op,character,irreps,           &
       &  cioslowski,nvmach,    &
       &  irred_atom_index,symm_index,atom_index,istepfl,    &
       &  global_spectrum,spectrum,sum_spectrum)
  !----------------------------------------------------------------
  ! read coord file, write data to "molecule"
  ! determine symmetry and distortion from control file
  ! read gradients and dipol moments, and polarizabilities from restart file
  !----------------------------------------------------------------
  call init_ctrl(ctrl)
  call testfile('snf_control')
  call testfile('control')
  call testfile('restart')
  call testfile('coord')
  call read_control(ctrl,selatom,massatom)
  if(associated(ctrl%tm_ctrl)) then
     ctrl%mode=ctrl%tm_ctrl%mode
     call read_tm_coord(ctrl%tm_ctrl%coord,molecule)
  endif
  if(associated(selatom)) then
     call setisomass(molecule,selatom,massatom)
     deallocate(selatom,massatom)
  end if
  fdum=.false.
  do i=1,ubound(molecule,1)
    if(molecule(i)%element.eq.'Q') fdum=.true.
  end do
  ! calculate molecule mass and moments of inertia
  nvar=3*ubound(molecule,1)
  if(.not.is_point_grp(molecule,1.D-3,atom_equal,ctrl%symmetry)) then
     errbuff=''
     write(errbuff,'(2a)') 'Molecule does not fit into symmetry:',ctrl%symmetry
     call serror('snf',2,errbuff,out)
  endif
  if(ctrl%isosym.ne.'def') then
    if(.not.is_point_grp(molecule,1.D-3,isotope_equal,ctrl%isosym)) then
      errbuff=''
      write(errbuff,'(2a)') 'Molecule does not fit into symmetry:',ctrl%isosym
      call serror('snf',2,errbuff,out)
    endif
  endif
  ctrl%nvar=nvar

  call print_ctrl(ctrl,unit=out)
  if(any(molecule(:)%changem)) call print_isomass(molecule,unit=out)
  call print_molecule(molecule,unit=out)

  if(ctrl%dynpol) ctrl_restart%dynpol=.true.
  if(associated(ctrl%dynfreq)) then
    nfreq=ubound(ctrl%dynfreq,1)
  else
    nfreq=1
  end if
  allocate(ctrl_restart%dynfreq(nfreq))
  call read_restart(ctrl%restart,ctrl_restart,&
       &   gradients,dipols,pols,istepfl,nvmach,status,ctrl%raman,&
       & unit=out)
  if(status.eq.-9) then
    call serror('snf',2,'Restart file does not exist.',out)
  else if(status.eq. -1) then
    call serror('snf',2,'Some calculations are not finished.',out)
  else if(status.eq. 0) then
    call serror('snf',2,'Not all calculations have been done yet.',out)
  else if(status.eq. -2) then
    call serror('snf',2,'Not all data necessary for frequency analysis##&
     &available in file restart.',out)
  endif

  call destroy_control(ctrl_restart)
  if(fdum) then
    call serror('snf',-1,'Dummy atoms are neglected in all further &
     &evaluations',out)
    call rm_dummies(molecule,gradients,dipols,pols,nvmach,&
     & istepfl,2*ctrl%numderiv,ctrl%raman,nfreq)
    nvar=3*ubound(molecule,1)
    ctrl%nvar=nvar
  end if
  !----------------------------------------------------------------
  ! one distortion value for all cartesian coordinates
  ! note: generalized distortions may be necessary in order to 
  !       perform distortions along the normal coordinates
  !----------------------------------------------------------------
  allocate(distortion(nvar))
  distortion(:)=ctrl%cstep  
  !----------------------------------------------------------------
  ! determine total number of translational and rotational modes
  !----------------------------------------------------------------  
  allocate(mol(ubound(molecule,1)))
  mol=molecule
  call hauptax_trans(mol,molekel)
  deallocate(mol)
  zeros=0
  do i=1,3
     if(molekel%p_inertia(i).lt. 1.d0) zeros=zeros+1
  enddo

  number_projected_modes=6-zeros
  !----------------------------------------------------------------
  ! construct symmetry information
  !    character table
  !    representations
  !    projection operators
  !    characters and irreducible representations
  !    list of irreducible atoms with symmetry operations
  !----------------------------------------------------------------
  call create_char_table(char_table,ctrl%symmetry)
  call check_geometry(molecule,char_table,atom_equal,unit=out)
  call create_irred_atoms(molecule,char_table,atom_index,symm_index, &
       &                  irred_atom_index,nirred_atoms)
  if(ctrl%print_irred_at) &
       & call print_irred_atoms(atom_index,symm_index,&
       &         irred_atom_index,nirred_atoms,unit=out)
  !----------------------------------------------------------------
  ! create hessian matrix from gradient and perform mass weighting
  ! calculate cartesian derivative dipol moments and print them out
  !----------------------------------------------------------------
  if(ctrl%maxsym) call create_irred_dist(molecule,char_table,dist_index, &
       &  dsymm_index,irred_dist_index,nirred_dist,nirred_atoms,&
       &  irred_atom_index,2*ctrl%numderiv,distortion,&
       &  ctrl%prirdist,unit=out)
  call create_representation_for_vib(representation,molecule,char_table)
  call create_character(representation,character)
  call create_irreps(character,irreps,char_table)
  call create_proj_op(representation,char_table,proj_op)
  if(ctrl%print_symmetry) then
     call print_char_table(char_table,unit=out)
     call create_product_table(char_table,prod_table)
     call print_product_table(char_table,prod_table,unit=out)
     deallocate(prod_table)
     write(out,*)
     write(out,*)'Symmetry elements'
     write(out,*)'-----------------'
     do i=1,char_table%nsymel
        call printmat(symel_mat(char_table%symel(i)),text=char_table%symel(i),unit=out,fmt='F10.5')
     enddo
     ! call print_representation(representation,char_table,unit=out)
     ! call print_proj_op(proj_op,char_table,unit=out)
     call print_character(character,char_table,unit=out)
     call print_irreps(irreps,char_table,unit=out)
  endif
  if(ctrl%maxsym) call create_red_grad(gradients,dist_index, &
     & dsymm_index,nirred_atoms,&
     & irred_atom_index,2*ctrl%numderiv,representation)
  call create_hessian_xyz(hessian_xyz,gradients,distortion)
  call expand_hessian(hessian_xyz,representation,symm_index)
  call check_hessian(hessian_xyz,ctrl,symm_index,istepfl,new_restart,unit=out)
  call symmetrize_hessian(hessian_xyz,representation)
  if(new_restart) then
     if(fdum) then
       call serror('snf',-1,'SNF is not yet able to write new restart &
         &files##if dummies are present. The restart file has to be &
         &changed manually.',unit=out)
     else
       call backup_restart(ctrl%restart,unit=out)
       call write_restart(gradients,dipols,pols,istepfl,nvmach,ctrl,unit=out)
     end if
  endif
  if(associated(nvmach))  deallocate(nvmach)
  if(associated(istepfl)) deallocate(istepfl)
  if(ctrl%print_hessian) &
       & call printmat(hessian_xyz,text='cartesian hessian',unit=out) 
  call balance_hessian(hessian_xyz)
  !call print_hessian(hessian_xyz,'nf_hessian.res')
  !  if(ctrl%print_force) &
  !       & call calc_atom_force(molecule,hessian_xyz,unit=out)
  allocate(hessian_m(nvar,nvar))
  call hessian_mass_weighting(hessian_xyz,hessian_m,molecule)
  !  call write_tm_force(hessian_xyz)
  !----------------------------------------------------------------
  ! calculate cartesian derivative dipol moments and 
  ! polarizabilities (if requested) and print them out 
  !----------------------------------------------------------------
  if(ctrl%maxsym) call create_red_dip(dipols,dist_index, &
     & dsymm_index,nirred_atoms,&
     & irred_atom_index,2*ctrl%numderiv,char_table)
  call create_dipol_dxyz(dipol_dxyz,dipols,distortion)
  call expand_dipol(dipol_dxyz,representation,symm_index,char_table)
  call symm_dipol(dipol_dxyz,representation,char_table)
  if(ctrl%print_dip) &
       & call printmat(dipol_dxyz,text='gradient of dipole moments',unit=out)
  if (ctrl%raman.and.(.not.associated(ctrl%dynfreq))) then
     allocate(polar_dxyz(ubound(pols,1),ubound(pols,2),1))
     if(ctrl%maxsym) call create_red_pol(pols(:,:,:,1),dist_index, &
       & dsymm_index,nirred_atoms,&
       & irred_atom_index,2*ctrl%numderiv,char_table)
     call create_polar_dxyz(polar_dxyz(:,:,1),pols(:,:,:,1),distortion)
     call expand_polar(polar_dxyz(:,:,1),representation,symm_index,char_table)
     call symm_polar(polar_dxyz(:,:,1),representation,char_table)
     if(ctrl%print_pol) then
       if(ctrl%dynpol) then
         write(out,*)
         write(out,*)
         write(out,'(a,f16.5,a)') ' Dynamic polarizability derivatives for &
          &laser wavelength:        514.40000 nm'
         write(out,*)
       end if
       call printmat(polar_dxyz(:,:,1),text='gradient of polarizability tensor',&
          & unit=out)
      end if
  else if(ctrl%raman.and.ctrl%dynpol.and.associated(ctrl%dynfreq)) then
     allocate(polar_dxyz(ubound(pols,1),ubound(pols,2),ubound(ctrl%dynfreq,1)))
     do i=1,ubound(ctrl%dynfreq,1)
       if(ctrl%maxsym) call create_red_pol(pols(:,:,:,i),dist_index, &
          & dsymm_index,nirred_atoms,&
          & irred_atom_index,2*ctrl%numderiv,char_table)
       call create_polar_dxyz(polar_dxyz(:,:,i),pols(:,:,:,i),distortion)
       call expand_polar(polar_dxyz(:,:,i),representation,symm_index,char_table)
       call symm_polar(polar_dxyz(:,:,i),representation,char_table)
       if(ctrl%print_pol) then
         write(out,*)
         write(out,*)
         if(abs(ctrl%dynfreq(i)).le.1.0d-10) then
           write(out,'(a)') ' Dynamic polarizability derivatives for &
            &laser wavelength:  infinity'
         else
           write(out,'(a,f16.5)') ' Dynamic polarizability derivatives for &
            &laser wavelength: ',1.0d0/ctrl%dynfreq(i)*nm2freq
         end if
         write(out,*)
         call printmat(polar_dxyz(:,:,i),text='gradient of polarizability tensor',&
          & unit=out)
       end if 
     end do
  end if
  if(ctrl%print_cioslowski) then
     call create_cioslowski(dipol_dxyz,cioslowski)
     call print_cioslowski(out,cioslowski)
     deallocate(cioslowski)
  endif
  !----------------------------------------------------------------
  ! continue in ctrl%isosym symmetry if unusual isotope settings appear
  !----------------------------------------------------------------
  !>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
  if(any(molecule(:)%changem).or.ctrl%isosym.ne.'def') then
     if(ctrl%isosym.eq.'def') then
        call get_point_grp(molecule,1.D-3,isotope_equal,ctrl%isosym)
     endif
     write(out,*)
     write(out,'(''======================================================='')')
     write(out,'(''======================================================='')')
     write(out,'('' ALL FOLLOWING RESULTS WERE OBTAINED USING '',a3,'' SYMMETRY'')')  ctrl%isosym
     write(out,'(''======================================================='')')
     write(out,'(''======================================================='')')
     write(out,*)
     deallocate(proj_op,representation,character,irreps)
     call destroy_char_table(char_table)
     call create_char_table(char_table,ctrl%isosym)
     call create_representation_for_vib(representation,molecule,char_table)
     call create_character(representation,character)
     call create_irreps(character,irreps,char_table)
     call create_proj_op(representation,char_table,proj_op)
  endif
  !<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  !----------------------------------------------------------------

  !----------------------------------------------------------------
  ! test linearity of molecule
  !----------------------------------------------------------------
  if(testlin(molecule)) call serror('snf',-1,'Found linear molecule.',out)
  !----------------------------------------------------------------
  ! Header printed here, so that it is printed only once.
  !----------------------------------------------------------------
  write(out,*)
  write(out,'('' Identifying translational and rotation normal modes'')')
  write(out,'('' ---------------------------------------------------'')')
  !----------------------------------------------------------------
  ! solve problem for every irreducible representation separately
  !----------------------------------------------------------------
  allocate(global_spectrum(nvar-number_projected_modes))
  if(ctrl%raman) then
    do i=1,ubound(global_spectrum,1)
      allocate(global_spectrum(i)%raman(nfreq))
    end do
  end if
  offset=0
  do irrep=1,char_table%nirrep
     irrep_dim=irreps(irrep)*char_table%l(1,irrep)
     if(irrep_dim .ne. 0) then
        lwork=(ilaenv(1,'DSYTRD','l',nvar,nvar,nvar,nvar)&
             & +2)*nvar
        allocate(eig_proj_op(nvar),work(lwork))
        !----------------------------------------------------------------
        ! calculate eigenvalues and eigenvectors of the 
        ! projection operators.
        !----------------------------------------------------------------
        call dsyev('V','L',nvar,proj_op(1,1,irrep),nvar,eig_proj_op(1),work(1),lwork,info)
        if(info.ne.0) then 
          call serror('snf',2,'Could not calculate eigenvectors##&
           &of the projection operators.',out)
        end if
        deallocate(eig_proj_op,work)

        allocate(c2p(nvar,irrep_dim))
        c2p=proj_op(1:nvar,nvar-irrep_dim+1:nvar,irrep)
        !----------------------------------------------------------------
        ! do everything twice to get an assessment of the quality of 
        ! rotational and translational modes
        ! you may comment it out, if you want. nothing relies on this.
        !----------------------------------------------------------------
        lwork=(ilaenv(1,'DSYTRD','l',irrep_dim,irrep_dim,irrep_dim,irrep_dim)&
             & +2)*irrep_dim
        allocate(hessian_p(irrep_dim,irrep_dim),&
             &   hessian_q(irrep_dim),&
             &   spectrum(irrep_dim), &
             &   work(lwork), &
             &   tmpmat(nvar,irrep_dim), &
             &   norm_modes(nvar,irrep_dim))
        tmpmat=0.D0
        !----------------------------------------------------------------
        ! "block diagonalization" 
        ! [S_mu^(m)]^+ F^(m) S_mu^(m) = F_mu^(s)
        !----------------------------------------------------------------
        call dgemm('N','N',nvar,irrep_dim,nvar,1.D0,hessian_m(1,1),nvar,c2p(1,1),nvar,0.D0,tmpmat(1,1),nvar)
        hessian_p=0.D0
        call dgemm('T','N',irrep_dim,irrep_dim,nvar,1.D0,c2p(1,1),nvar,tmpmat(1,1),nvar,0.D0,hessian_p(1,1),irrep_dim)
        call balance_hessian(hessian_p)
        !--------------------------------------------------------------------
        ! calculate eigenvalues and eigenvectors of this block of the hessian
        ! [U_mu^(s)]^+ F_mu^(s) U_mu^(s) = F_mu^(q)
        !--------------------------------------------------------------------
        call dsyev('V','L',irrep_dim,hessian_p(1,1),irrep_dim,hessian_q(1),work(1),lwork,info)
        if(info.ne.0) then 
          call serror('snf',2,'Could not calculate eigenvectors##&
           &of the hessian.',out)
        end if
        !--------------------------------------------------------------------
        ! calculate "original" transformation matrix
        ! U_mu^(m) = S_mu^(m) U_mu^(s) 
        !--------------------------------------------------------------------
        call dgemm('N','N',nvar,irrep_dim,irrep_dim,1.D0,c2p(1,1),nvar, &
         &hessian_p(1,1),irrep_dim,0.D0,norm_modes(1,1),nvar)
        call calc_frequencies(hessian_q,spectrum)
        do i=1,irrep_dim
           allocate(spectrum(i)%norm_mode(nvar))
           spectrum(i)%norm_mode(:)=norm_modes(:,i)
           spectrum(i)%irrep=char_table%mulliken(irrep)
           spectrum(i)%g=char_table%g(irrep)
           spectrum(i)%number_in_irrep=i
        end do
        call get_translation_and_rotation(spectrum,molecule,-1,unit=out)

        deallocate(norm_modes)
        call destroy_spectrum(spectrum)
        deallocate(hessian_p,hessian_q,work,tmpmat)
        !----------------------------------------------------------------
        ! end of rotational and translatinal analysis
        !----------------------------------------------------------------
        ! now project out all translational and rotational modes and 
        ! do the whole work again
        !----------------------------------------------------------------
        call project_off_transrot(molecule,c2p,'all')

        irrep_dim=ubound(c2p,2)
     endif
     if(irrep_dim .ne. 0) then
        lwork=(ilaenv(1,'DSYTRD','l',nvar,nvar,nvar,nvar)&
             & +2)*nvar
        allocate(hessian_p(irrep_dim,irrep_dim),&
             &   hessian_q(irrep_dim),&
             &   spectrum(irrep_dim), &
             &   work(lwork), &
             &   tmpmat(nvar,irrep_dim), &
             &   norm_modes(nvar,irrep_dim))
        !----------------------------------------------------------------
        ! transform hessian to symmetry adapted basis
        ! and diagonalize it.
        !----------------------------------------------------------------
        call dgemm('N','N',nvar,irrep_dim,nvar,1.D0,hessian_m(1,1),nvar,c2p(1,1),nvar,0.D0,tmpmat(1,1),nvar)
        call dgemm('T','N',irrep_dim,irrep_dim,nvar,1.D0,c2p(1,1),nvar,tmpmat(1,1),nvar,0.D0,hessian_p(1,1),irrep_dim)
        call balance_hessian(hessian_p)
        !----------------------------------------------------------------
        ! calculate eigenvalues and eigenvectors of the symmetry
        ! adapted hessian
        !----------------------------------------------------------------
        call dsyev('V','L',irrep_dim,hessian_p(1,1),irrep_dim,hessian_q(1),work(1),lwork,info)
        if(info.ne.0) then 
          call serror('snf',2,'Could not calculate eigenvectors##&
           &of the symmetry adapted hessian.',out)
        end if
        !----------------------------------------------------------------
        ! create transformation matrix 'normal_modes' from 
        ! cartesian to normal coordinates
        !----------------------------------------------------------------
        call dgemm('N','N',nvar,irrep_dim,irrep_dim,1.D0,c2p(1,1),nvar,hessian_p(1,1),irrep_dim,0.D0,norm_modes(1,1),nvar)
        call calc_frequencies(hessian_q,spectrum)
        !----------------------------------------------------------------
        ! perform transformation of cartesian derivative of dipol moments
        ! calculate ir (and raman) spectra with eigenvalues and 
        ! transformed dipol derivatives
        !----------------------------------------------------------------
        call create_dipol_dq(dipol_dxyz,norm_modes,molecule,dipol_dq)
        call calc_ir_intensities(dipol_dq,spectrum)
        if (ctrl%raman.and.(.not.associated(ctrl%dynfreq))) then
          allocate(polar_dq(6,ubound(norm_modes,2),1))
          do i=1,irrep_dim
            allocate(spectrum(i)%raman(nfreq))
          end do
          call create_polar_dq(polar_dxyz(:,:,1),norm_modes,molecule,polar_dq(:,:,1))
          call calc_raman_intensities(polar_dq(:,:,1),spectrum,ctrl%temp,&
           &ctrl%theta,1) 
        else if(ctrl%raman.and.ctrl%dynpol.and.associated(ctrl%dynfreq)) then
          allocate(polar_dq(6,ubound(norm_modes,2),nfreq))
          do i=1,irrep_dim
            allocate(spectrum(i)%raman(nfreq))
          end do
          do i=1,ubound(ctrl%dynfreq,1)
            call create_polar_dq(polar_dxyz(:,:,i),norm_modes,molecule,polar_dq(:,:,i))
            call calc_raman_intensities(polar_dq(:,:,i),spectrum,ctrl%temp,&
             &ctrl%theta,i,ctrl%dynfreq(i)) 
          end do
        else if(.not.ctrl%raman) then
          !----------------------------------------------------------------
          ! This is done for compatibility with other parts of the program
          !----------------------------------------------------------------
          do i=1,irrep_dim
            allocate(spectrum(i)%raman(1))
          end do
        endif
        !----------------------------------------------------------------
        ! assign several values concerning the spectrum
        !----------------------------------------------------------------
        do i=1,irrep_dim
           allocate(spectrum(i)%norm_mode(nvar))
           spectrum(i)%norm_mode(:)=norm_modes(:,i)
           spectrum(i)%irrep=char_table%mulliken(irrep)
           spectrum(i)%g=char_table%g(irrep)
           spectrum(i)%number_in_irrep=i
           spectrum(i)%red_mass = 0.d0
           do j=1,nvar
              spectrum(i)%red_mass = spectrum(i)%red_mass + &
                   & (spectrum(i)%norm_mode(j)**2 &
                   & /(molecule((j+2)/3)%mass))
           enddo
           spectrum(i)%red_mass=1.d0/spectrum(i)%red_mass
           spectrum(i)%force = 0.d0
        enddo
        deallocate(hessian_q,hessian_p,dipol_dq,work,tmpmat,norm_modes)
        if (associated(polar_dq)) deallocate(polar_dq)

        !----------------------------------------------------------------
        ! copy irrep data to global spectrum
        !----------------------------------------------------------------
        global_spectrum(offset+1:offset+irrep_dim)=spectrum(:)
        offset=offset+irrep_dim
        deallocate(spectrum)
     endif
     if(associated(c2p)) deallocate(c2p)
  enddo ! loop over irreps
  ! In case of unexpected incorrect translational or rotational modes
  ! dummy modes are introduced
  do i=offset+1,nvar-number_projected_modes
     allocate(global_spectrum(i)%norm_mode(nvar))
     global_spectrum(i)%norm_mode = 0.D0
     global_spectrum(i)%freq = 0.D0
     global_spectrum(i)%irrep='ERR'
     global_spectrum(i)%g=0
     global_spectrum(i)%number_in_irrep = 0
     if(ctrl%raman) then 
       allocate(global_spectrum(i)%raman(nfreq))
       global_spectrum(i)%raman(:)%int=0.D0
     end if
     global_spectrum(i)%red_mass=0.D0
  enddo
  write(out,*)
  write(out,'(a)') ' The following results may differ from the previous '
  write(out,'(a)') ' values, because now the translational and rotational modes'
  write(out,'(a)') ' are projected off instead of being identified.'
  write(out,'(a)') ' Therefore the following data are more accurate.'
  !----------------------------------------------------------------
  ! clean up, and print out.
  !----------------------------------------------------------------
  call sort_spectrum(global_spectrum)
  call assign_irrep_number(global_spectrum,char_table)
  call get_relir(global_spectrum)
  call print_ir(global_spectrum,unit=out)
  static=.not.ctrl%dynpol
  if(ctrl%raman.and.(.not.associated(ctrl%dynfreq))) then
    if(ctrl%dynpol) then
      write(out,'(3(/),a)') ' Following Raman results obtained for &
       &laser wavelength:       514.50000 nm'
      write(out,'(a)') 
    end if
    call get_relcross(global_spectrum,1)
    call print_raman(global_spectrum,1,out)
    if(ctrl%print_rcs) call print_rcs(global_spectrum,1,static,&
    &ctrl%theta,out)
  else if(ctrl%raman.and.ctrl%dynpol.and.associated(ctrl%dynfreq)) then
    !----------------------------------------------------------------
    ! if dynamic polarizabilities with non-standart laser wavelengths
    ! have been selected, print raman intensities for these
    ! wavelengths.
    !----------------------------------------------------------------
    do i=1,nfreq
      if(abs(ctrl%dynfreq(i)).le.1.0d-10) then
        write(out,'(3(/),a)') ' Following Raman results obtained for &
         &laser wavelength:  infinity'
      else
        write(out,'(3(/),a,f16.5)') ' Following Raman results obtained &
         &for laser wavelength: ',1.0d0/ctrl%dynfreq(i)*nm2freq
      end if
      write(out,'(a)') 
      call get_relcross(global_spectrum,i)
      call print_raman(global_spectrum,i,out)
      if(ctrl%print_rcs) call print_rcs(global_spectrum,i,static,&
       &ctrl%theta,out)
    end do
  end if
  !-------------------------------------------------------------
  ! additional output: intensities summed over degenerate modes
  !-------------------------------------------------------------
  if(ctrl%sumdeg) then
    write(out,'((/),152(''#''))') 
    write(out,'(3(/),a)') ' Intensities summed over degenerate modes'
    write(out,'(a,2(/))') ' ----------------------------------------'
    call sum_degen(global_spectrum,sum_spectrum,ctrl%raman)
    call get_relir(sum_spectrum)
    call print_ir(sum_spectrum,unit=out)
    if(ctrl%raman.and.(.not.associated(ctrl%dynfreq))) then
      if(ctrl%dynpol) then
        write(out,'(3(/),a)') ' Following Raman results obtained for &
         &laser wavelength:       514.50000 nm'
        write(out,'(a)') 
      end if
      call get_relcross(sum_spectrum,1)
      call print_raman(sum_spectrum,1,out)
      if(ctrl%print_rcs) call print_rcs(sum_spectrum,1,static,&
       &ctrl%theta,out)
    else if(ctrl%raman.and.ctrl%dynpol.and.associated(ctrl%dynfreq)) then
      do i=1,nfreq
        if(abs(ctrl%dynfreq(i)).le. 1.0d-10) then
          write(out,'(3(/),a)') ' Following Raman results obtained for &
           &laser wavelength:  infinity'
        else
          write(out,'(3(/),a,f16.5)') ' Following Raman results obtained &
           &for laser wavelength: ',1.0d0/ctrl%dynfreq(i)*nm2freq
        end if
        write(out,'(a)') 
        call get_relcross(sum_spectrum,i)
        call print_raman(sum_spectrum,i,out)
        if(ctrl%print_rcs) call print_rcs(sum_spectrum,i,static,&
         &ctrl%theta,out)
      end do
    end if
    deallocate(sum_spectrum)
    write(out,'(2(/),152(''#''),2(/))') 
  end if
  !-------------------------------------
  ! thermochemistry
  !-------------------------------------
  molekel%mass=0.0d0
  do k=1,ubound(atom_index,1)
     molekel%mass=molekel%mass+molecule(k)%mass
  enddo
  mol_mass=molekel%mass/(1000.0d0*avogadro)
  if(ctrl%print_zpe) then
     call zeropoint_energy(global_spectrum,zeropoint)
     write(out,*)
     write(out,'('' General molecular quantities'')')
     write(out,'('' ----------------------------'')')
     write(out,'(''   Zero point energy (ZPE) [kJ/mol]   :'',F10.1)') zeropoint
     write(out,'(''   Molecular mass [amu]               :'',F10.1)') molekel%mass
     write(out,'(''   Principal momenta and eigenvectors :'')')
     write(out,*)
     write(out,'(3F15.5)')(molekel%p_inertia(i),i=1,3)
     write(out,*)
     do j=1,3
        write(out,'(3F15.5)')(molekel%p_axes(i,j),i=1,3)
     enddo
  endif

  !-------------------------------------------------------------------
  ! checking symmetry number (some are not sufficiently tested)
  !-------------------------------------------------------------------
  select case(ctrl%symmetry)
    case('th ','o  ','i  ','ih ')
    call serror('snf',-1,'Note that the symmetry numbers have&
     & not been##sufficiently checked for this point group.',out)
  end select
  if(testlin(molecule)) then
    if(index(char_table%point_group(1:3),'h').ne.0) then
      char_table%sigma=2 
    else 
      char_table%sigma=1 
    end if
  end if
  if(ctrl%print_thermo) then
     allocate(freqs(ubound(global_spectrum,1)))
     do i=1,ubound(global_spectrum,1)
        freqs(i)=global_spectrum(i)%freq
     enddo
     call get_tdvalues(zeropoint,ctrl%elec_energy,freqs,ctrl%temp,ctrl%press,out,&
      & mol_mass,molekel%p_inertia,char_table%sigma,testlin(molecule),ctrl%spin)
     deallocate(freqs)
  endif
  !cmr
  write(out,*)
  write(out,'('' Frequencies and mass-weighted normal coordinates'')')
  write(out,'('' ------------------------------------------------'')')
  call preig(out,ubound(global_spectrum,1),global_spectrum,'eigenvalues  in  cm^-1')
  call print_norm_modes(global_spectrum,molecule,unit=out)
  !----------------------------------------------------------------
  ! warning message in case of wrong hessian
  !----------------------------------------------------------------
  if(new_restart) then
     write(out,*)'*********************************************************'
     write(out,*)'                     W A R N I N G'
     write(out,*)'*********************************************************'
     write(out,*)' Calculation is not valid due to an unsymmetric hessian.'
     write(out,*)' Decide, whether the data obtained here are sufficiently'
     write(out,*)' accurate for further use. In case of doubt, restart the'
     write(out,*)' calculation with the new restart file. The original one'
     write(out,*)' is stored on '//trim(adjustl(ctrl%restart))//'.bak.'
     write(out,*)'*********************************************************'
  endif
  !----------------------------------------------------------------
  ! write mopac and gaussian frequency output
  !----------------------------------------------------------------
  call calc_mode_properties(molecule,global_spectrum)
  if(ctrl%print_mopac.or.ctrl%print_gaussian.or.ctrl%print_xmol) then
     write(out,*)
     write(out,'('' Generate fake outputs for normal mode presentation'')')
     write(out,'('' --------------------------------------------------'')')
     if(ctrl%raman.and.ctrl%dynpol.and.associated(ctrl%dynfreq)) then
       if(ubound(ctrl%dynfreq,1).ge.2) call serror('snf',-1,'Files for&
         & normal mode presentation contain##Raman data only for the first&
         & selected frequency.',out)
     end if
     if(ctrl%print_mopac) &
          &  call print_mopac_frequencies(molecule,global_spectrum,&
          &            ctrl%print_mopxmol,'mopac.out',unit=out)
     if(ctrl%print_gaussian) &
          &  call print_gaussian_frequencies(molecule,global_spectrum,&
          &                            'g98.out',ctrl%raman,unit=out)       
     if(ctrl%print_xmol) &
      & call print_xmol_norm(molecule,global_spectrum,'xmol.XYZ.out',&
      &ctrl%xmolsc1,unit=out)
  endif
  !----------------------------------------------------------------
  ! generate data-, gnuplot-, and eps-files for spectrum plotting
  !----------------------------------------------------------------
  call generate_plots(ctrl,global_spectrum,1,unit=out)
  !----------------------------------------------------------------
  ! clean up memory 
  !----------------------------------------------------------------
  call destroy_spectrum(global_spectrum)
  deallocate(global_spectrum)
  call destroy_char_table(char_table)
  deallocate(representation)
  deallocate(character)
  deallocate(irreps)
  deallocate(proj_op)
  deallocate(dipol_dxyz)
  deallocate(hessian_xyz)
  deallocate(hessian_m)
  call destroy_molecule(molecule)
  call destroy_irred_atoms(atom_index,symm_index,irred_atom_index)
  call destroy_control(ctrl)
  deallocate(distortion)
  close(out)
end program snf

subroutine print_header(out)
  implicit none
  integer,intent(in) :: out
  write(out,'(A)')
!                   +1234567890123456789012345678+8765432109876543210987654321+'
  write(out,'(A)')' +---------------------------------------------------------+'
  write(out,'(A)')' |                                                         |'
  write(out,'(A)')' |                          S N F                          |'
  write(out,'(A)')' |                                                         |'
  write(out,'(A)')' |                         V 2.1.4                         |' !VERSION
  write(out,'(A)')' |                                                         |'
  write(out,'(A)')' |                   (TURBOMOLE VERSION)                   |'
  write(out,'(A)')' |                                                         |'
  write(out,'(A)')' |   A program package for numerical frequency analyses    |'
  write(out,'(A)')' |                                                         |'
  write(out,'(A)')' |                       written by                        |'
  write(out,'(A)')' |            C. Kind, M. Reiher, J. Neugebauer            |'
  write(out,'(A)')' |                      and B. A. Hess                     |'
  write(out,'(A)')' |  Universitaet Erlangen, Egerlandstr. 3, 91058 Erlangen  |'
  write(out,'(A)')' |                        1999-2002                        |'
  write(out,'(A)')' |                                                         |'
  write(out,'(A)')' +---------------------------------------------------------+'
!                   +1234567890123456789012345678+8765432109876543210987654321+'
  write(out,'(A)')
end subroutine print_header

subroutine last_mod(out)
  implicit none
  integer,intent(in) :: out
  write(out,'(A)')
  write(out,'(A)') ' Last modification: 18.02.2002 (JN) '
  write(out,'(A)')
end subroutine last_mod
