
!********************************************************************************
!*   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
!*   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
!*   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
!*
!*
!*   This file is part of the package SNF.
!*
!*   SNF is free software; you can redistribute it and/or modify
!*   it under the terms of the GNU General Public License as published by
!*   the Free Software Foundation; either version 2 of the License, or
!*   (at your option) any later version.
!*
!*   This program is distributed in the hope that it will be useful,
!*   but WITHOUT ANY WARRANTY; without even the implied warranty of
!*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*   GNU General Public License for more details.
!*
!*   You should have received a copy of the GNU General Public License
!*   along with this program; if not, write to the Free Software
!*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!*
!********************************************************************************
module snf_char_table
  USE f90_kind
  !----------------------------------------------------------------
  ! this module contains all subroutines concerning character tables,
  !
  ! written by Carsten Kind, Uni Koeln, 1994
  ! modified by J. Neugebauer, Uni Erlangen, 05/2001
  !----------------------------------------------------------------
  private
  public :: create_char_table, print_char_table,destroy_char_table
  public :: test_char_table,type_char_table,checkpg
  public :: create_product_table,print_product_table
  type type_char_table
     integer                   :: nsymel,nirrep
     character(len=3)          :: point_group
     character (len=5),pointer :: mulliken(:)
     character (len=7),pointer :: symel(:)
     real(double) ,pointer :: l(:,:)
     integer ,pointer          :: g(:),ordnung(:)
     integer :: sigma     ! symmetry number
  end type type_char_table

  !***********************************************************************

contains

  !***********************************************************************

  function mul_ind(mulliken,char_table,unit)
    !----------------------------------------------------------------
    ! this function returns the place of the mullikan symbol inside the 
    ! character table.
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------

    implicit none
    character(len=5),intent(in) :: mulliken
    type(type_char_table),intent(in) :: char_table
    integer,optional :: unit
    integer :: mul_ind,ir,out

    if(present(unit)) then
       out=unit
    else
       out=6
    endif
    ir = 1
    do while (( trim(char_table%mulliken(ir)) .ne. trim(mulliken) ) .and. &
         ( ir .lt. ubound(char_table%mulliken,1)))
       ir = ir + 1
    enddo
    if ( trim(char_table%mulliken(ir)) .ne. trim(mulliken)) then
       write(out,*)' unknown mulliken symbol: ',mulliken
       ir=0
    endif
    mul_ind = ir
  end function mul_ind

  !***********************************************************************

  subroutine create_char_table(char_table,point_group,unit) 
    !-----------------------------------------------------------------
    ! this subroutine calls the subroutines corresponding to the
    ! specific point groups, which set up the character table.
    !
    ! Note: Symmetry numbers sigma are taken from:
    !       
    !       G. Herzberg, Molecular Spectra and Molecular Structure,
    !       II. Infrared and Raman Spectra of Polyatomic Molecules,
    !       Van Nostrand, Princeton, 1962, p. 508.
    !
    ! However, symmetry numbers for T_h, O, I, and I_h are not
    ! given in this reference. They have to be checked !
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    ! extension for icosahedral and cubic point groups
    ! by J. Neugebauer, Universitaet Erlangen-Nuernberg, 05/2001
    !-----------------------------------------------------------------

    implicit none
    type(type_char_table),intent(out) :: char_table
    character(len=3),  intent(in)  :: point_group
    integer,optional :: unit
    integer i,out

    if(present(unit)) then
       out=unit
    else
       out=6
    endif
    !----------------------------------------------------------------
    ! assign character tables
    !----------------------------------------------------------------
    char_table%point_group=point_group
    select case (char_table%point_group)
    case ('ci ') 
       call init_ci(char_table) 
    case ('cs ') 
       call init_cs(char_table)
    case ('c2 ') 
       call init_c2(char_table) 
    case ('c3 ') 
       call init_c3(char_table) 
    case ('c4 ') 
       call init_c4(char_table) 
    case ('c5 ') 
       call init_c5(char_table)
    case ('c6 ') 
       call init_c6(char_table) 
    case ('c7 ') 
       call init_c7(char_table) 
    case ('c8 ') 
       call init_c8(char_table) 
    case ('c2v') 
       call init_c2v(char_table) 
    case ('c3v') 
       call init_c3v(char_table) 
    case ('c4v') 
       call init_c4v(char_table) 
    case ('c5v') 
       call init_c5v(char_table) 
    case ('c6v') 
       call init_c6v(char_table) 
    case ('c2h') 
       call init_c2h(char_table) 
    case ('c3h') 
       call init_c3h(char_table) 
    case ('c4h') 
       call init_c4h(char_table) 
    case ('c5h') 
       call init_c5h(char_table) 
    case ('c6h') 
       call init_c6h(char_table) 
    case ('d2 ') 
       call init_d2(char_table) 
    case ('d3 ') 
       call init_d3(char_table) 
    case ('d4 ') 
       call init_d4(char_table) 
    case ('d5 ') 
       call init_d5(char_table) 
    case ('d6 ') 
       call init_d6(char_table) 
    case ('d2h')
       call init_d2h(char_table)  
    case ('d3h') 
       call init_d3h(char_table) 
    case ('d4h') 
       call init_d4h(char_table) 
    case ('d5h') 
       call init_d5h(char_table) 
    case ('d6h') 
       call init_d6h(char_table) 
    case ('d2d') 
       call init_d2d(char_table) 
    case ('d3d') 
       call init_d3d(char_table) 
    case ('d4d') 
       call init_d4d(char_table) 
    case ('d5d') 
       call init_d5d(char_table) 
    case ('d6d') 
       call init_d6d(char_table) 
    case ('s4 ') 
       call init_s4(char_table) 
    case ('s6 ') 
       call init_s6(char_table) 
    case ('s8 ') 
       call init_s8(char_table) 
    case ('c1 ')
       call init_c1(char_table)
    case ('o  ')
       call init_o(char_table)
    case ('oh ')
       call init_oh(char_table)
    case ('t  ')
       call init_t(char_table)
    case ('th ')
       call init_th(char_table)
    case ('td ')
       call init_td(char_table)
    case ('i  ')
       call init_i(char_table)
    case ('ih ')
       call init_ih(char_table)
    case default
       write(out,*) 
       write(out,*) '****************************************'
       write(out,*) 'ERROR! Unknown point group:',point_group
       write(out,*) '****************************************'
       write(out,*) 
       stop
    end select

    allocate(char_table%g(char_table%nirrep), &
         char_table%ordnung(char_table%nirrep))
    do i=1,char_table%nirrep
       select case (char_table%mulliken(i)(1:1))
       case ('a','b')
          char_table%g(i)=1
       case ('e')
          char_table%g(i)=2
       case ('t')
          char_table%g(i)=3
       case ('g','f')
          char_table%g(i)=4
       case ('h')
          char_table%g(i)=5
       end select
       char_table%ordnung(i)=nint(dot_product(char_table%l(:,i),char_table%l(:,i)))
    enddo

  end subroutine create_char_table

  !***********************************************************************

  subroutine create_product_table(char_table,table)
    !----------------------------------------------------------------
    ! create product table
    !
    ! written by Carsten Kind, Uni Erlangen, 2001
    ! modified by J. Neugebauer, Uni Erlangen, 06/2001
    !----------------------------------------------------------------
    use snf_symmat
    implicit none
    type(type_char_table),intent(in) :: char_table
    character(LEN=7),pointer :: table(:,:)
    integer :: i,j,k
    real(double) :: mat(3,3),tmp(3,3)
    allocate(table(char_table%nsymel,char_table%nsymel))
    do i=1,char_table%nsymel
       do j=1,char_table%nsymel
          mat=matmul(symel_mat(char_table%symel(i)) , symel_mat(char_table%symel(j)))
          k=1
          loop3: do
             tmp=mat-symel_mat(char_table%symel(k))
             if(sum(tmp*tmp) .lt. 0.01) then
                table(i,j) = char_table%symel(k)
                exit loop3
             endif
             k=k+1
             if (k.gt. char_table%nsymel) then
                write(*,*)'ERROR: Multiplication table is not complete !'
                write(*,*)'Unexisting product:  ', char_table%symel(i) ,'  ', char_table%symel(j)
                table(i,j) = '---- '
                exit loop3
             end if
          enddo loop3
       enddo
    enddo
  end subroutine create_product_table

  !***********************************************************************

  subroutine print_product_table(char_table,table,unit)
    !----------------------------------------------------------------
    ! write product table
    !
    ! written by Carsten Kind, Uni Erlangen, 2001
    !----------------------------------------------------------------
    implicit none
    type(type_char_table),intent(in) :: char_table
    character(LEN=7),intent(in) :: table(:,:)
    integer,optional :: unit
    integer i,j,out,base,cols

    if(present(unit)) then
       out=unit
    else
       out=6
    endif
    write(out,*)
    write(out,'('' Product table for '',A3)')char_table%point_group
    write(out,'('' ---------------------'')')
    do base=1,char_table%nsymel,12
       write(out,*)
       cols=min(base+11,char_table%nsymel)
       write(out,'(8X,''| '',12(1X,A7))') &
            & (char_table%symel(j),j=base,cols)
       write(out,'(1X,A7,''|-'',12(A8))') &
            & '-------',('--------',j=base,cols)
       do i=1,char_table%nsymel
          write(out,'(1X,A7,''| '',12(1X,A7))') char_table%symel(i),&
               &(table(i,j),j=base,cols)
       enddo
    enddo
  end subroutine print_product_table

  !***********************************************************************

  subroutine print_char_table(char_table,unit)
    !----------------------------------------------------------------
    ! write character table
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------

    implicit none
    type(type_char_table),intent(in) :: char_table
    integer,optional :: unit
    integer i,j,out,col,cols

    if(present(unit)) then
       out=unit
    else
       out=6
    endif
    write(out,*)
    write(out,'('' Character table for '',A3)')char_table%point_group
    write(out,'('' -----------------------'')') 
    write(out,'(''   Number symmetry elements             :'',I3)') char_table%nsymel
    write(out,'(''   Number of irreducible representations:'',I3)') char_table%nirrep
    do col=0,char_table%nsymel/8-1
       write(out,*)
       write(out,'(8x,8a10)')(char_table%symel(j),j=col*8+1,(col+1)*8)
       do i=1,char_table%nirrep
          write(out,'(3X,a5,8f10.4)') char_table%mulliken(i), & 
               &  (char_table%l(j,i),j=col*8+1,(col+1)*8)
       enddo
    enddo
    cols=mod(char_table%nsymel,8)
    if(cols.ne.0) then
       write(out,*)
       write(out,'(8x,8a10)')(char_table%symel(j),j=char_table%nsymel-cols+1,char_table%nsymel)
       do i=1,char_table%nirrep
          write(out,'(3x,a5,8f10.4)') char_table%mulliken(i), & 
               &  (char_table%l(j,i),j=char_table%nsymel-cols+1,char_table%nsymel)
       enddo
    endif
  end subroutine print_char_table

  !************************************************************************

  subroutine destroy_char_table(char_table)

    !----------------------------------------------------------------
    ! deallocate char_table
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------

    implicit none
    type(type_char_table) :: char_table

    if(associated(char_table%mulliken)) deallocate(char_table%mulliken)
    if(associated(char_table%symel)) deallocate(char_table%symel)
    if(associated(char_table%l)) deallocate(char_table%l)
    if(associated(char_table%g)) deallocate(char_table%g)
    if(associated(char_table%ordnung)) deallocate(char_table%ordnung)
  end subroutine destroy_char_table

  !************************************************************************

  subroutine test_char_table(symgrp,unit)
    !--------------------------------------------------------
    ! checks conformity of character tables with the 
    ! little orthogonality theorem (LOT)
    !
    ! written by C. Kind, Uni Erlangen, 2001
    !----------------------------------------------------------------
    use snf_error
    implicit none
    character(len=3),intent(in) :: symgrp
    integer,optional :: unit
    type(type_char_table) :: char_table
    integer :: out,i,j
    real(double) :: err
    real(double),pointer :: matrix(:,:)
    if(present(unit)) then
       out=unit
    else
       out=6
    endif
    write(out,*)'begin test'
    call create_char_table(char_table,symgrp)
    write(out,*)'successfully created'
!    call print_char_table(char_table,unit=out)
    allocate(matrix(char_table%nirrep,char_table%nirrep))
    matrix=0.d0
    write(out,'(1X,24(I7))')(i,i=1,char_table%nirrep)
    do i=1,char_table%nirrep
       do j=1,i
          matrix(i,j) = dot_product(char_table%l(:,i),char_table%l(:,j))
       enddo
       write(out,'(I3,24(F7.2))')i,(matrix(i,j),j=1,i)
    enddo
    err=0.d0
    do i=1,char_table%nirrep
       do j=1,i-1
          err=err+abs(matrix(i,j))
       enddo
    enddo
    deallocate(matrix)
    if(err.gt. 1.D-4) call serror('test_char_table',2,'character &
      &table for '//char_table%point_group//' is wrong!!',out)
    call destroy_char_table(char_table)
    write(out,*)'test finished'
  end subroutine test_char_table

  !************************************************************************
  ! the following subroutines set up the character tables and several
  ! variables containing information about the point group
  ! char_table%nsymel   : number of symmetry elements
  ! char_table%nirrep   : number of irreducible representations
  ! char_table%symel    : symbols for symmetrie elements
  ! char_table%mulliken : mulliken symbols for irreps
  ! char_table%l   	: characters 
  ! 
  ! Note: Characters are given for ALL SYMMETRY ELEMENTS, not only for
  !       all classes of symmetry elements.
  !************************************************************************

  subroutine init_c1(char_table)
    !----------------------------------------------------------------
    ! init Char_table and related values
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------

    implicit none
    type(type_char_table),intent(inout) :: char_table
    real(double) c(1,1)
    integer i,j

    allocate(char_table%mulliken(1),char_table%symel(1), &
         char_table%l(1,1))

    data ((c(i,j),i=1,1),j=1,1) &
         /1.d0/

    char_table%nsymel=1 
    char_table%nirrep=1

    char_table%symel(1)='cz1_1' 

    char_table%mulliken (1)='a' 

    char_table%l=c
    char_table%sigma = 1
  end subroutine init_c1


  !***********************************************************************

  subroutine init_c2(char_table)
    !----------------------------------------------------------------
    ! init Char_table and related values
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------

    implicit none
    type(type_char_table),intent(inout) :: char_table
    real(double) c(2,2)
    integer i,j

    allocate(char_table%mulliken(2),char_table%symel(2), &
         char_table%l(2,2))

    data ((c(i,j),i=1,2),j=1,2) &
         /1.d0, 1.d0, &
         1.d0,-1.d0/ 

    char_table%nsymel=2 
    char_table%nirrep=2

    char_table%symel(1)='cz1_1' 
    char_table%symel(2)='cz2_1' 

    char_table%mulliken (1)='a' 
    char_table%mulliken (2)='b' 

    char_table%l=c
    char_table%sigma = 2

  end subroutine init_c2

  !***********************************************************************

  subroutine init_c3(char_table)
    !----------------------------------------------------------------
    ! init Char_table and related values
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------

    implicit none
    type(type_char_table),intent(inout) :: char_table
    integer i,j
    real(double) c(3,2)

    allocate(char_table%mulliken(2),char_table%symel(3), &
         char_table%l(3,2))

    data ((c(i,j),i=1,3),j=1,2) &
         /1.d0, 1.d0, 1.d0, &
         2.d0,-1.d0,-1.d0/
    char_table%nsymel=3
    char_table%nirrep=2

    char_table%symel(1)='cz1_1'
    char_table%symel(2)='cz3_1'
    char_table%symel(3)='cz3_2'

    char_table%mulliken(1)='a'
    char_table%mulliken(2)='e'

    char_table%l=c
    char_table%sigma = 3

  end subroutine init_c3

  !***********************************************************************

  subroutine init_c4(char_table)
    !----------------------------------------------------------------
    ! init Char_table and related values
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------

    implicit none
    type(type_char_table),intent(inout) :: char_table
    integer i,j
    real(double) c(4,3)

    allocate(char_table%mulliken(3),char_table%symel(4), &
         char_table%l(4,3))

    data ((c(i,j),i=1,4),j=1,3) &
         /1.d0, 1.d0, 1.d0, 1.d0, &
         1.d0,-1.d0, 1.d0,-1.d0, &
         2.d0, 0.d0,-2.d0, 0.d0/

    char_table%nsymel=4
    char_table%nirrep=3

    char_table%symel(1)='cz1_1'
    char_table%symel(2)='cz4_1'
    char_table%symel(3)='cz2_1'
    char_table%symel(4)='c34'

    char_table%mulliken(1)='a'
    char_table%mulliken(2)='b'
    char_table%mulliken(3)='e'

    char_table%l=c
    char_table%sigma = 4

  end subroutine init_c4

  !***********************************************************************

  subroutine init_c5(char_table)
    !----------------------------------------------------------------
    ! init Char_table and related values
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------

    implicit none
    type(type_char_table),intent(inout) :: char_table
    integer i,j
    real(double) c(5,3),pi
    parameter (pi = 3.1415926535897932384626433832795d0)

    allocate(char_table%mulliken(3),char_table%symel(5), &
         char_table%l(5,3))

    data ((c(i,j),i=1,5),j=1,3) &
         /1.d0, 1.d0, 1.d0, 1.d0, 1.d0, &
         2.d0, 0.d0, 0.d0, 0.d0, 0.d0, &
         2.d0, 0.d0, 0.d0, 0.d0, 0.d0/
    c(2,2)=2*cos(2*pi/5)
    c(3,2)=2*cos(4*pi/5)
    c(4,2)=c(3,2)
    c(5,2)=c(2,2)
    c(2,3)=c(3,2)
    c(3,3)=c(2,2)
    c(4,3)=c(2,2)
    c(5,3)=c(3,2)

    char_table%nsymel=5
    char_table%nirrep=3

    char_table%symel(1)='cz1_1'
    char_table%symel(2)='cz5_1'
    char_table%symel(3)='cz5_2'
    char_table%symel(4)='cz5_3'
    char_table%symel(5)='cz5_4'

    char_table%mulliken(1)='a'
    char_table%mulliken(2)='e1'
    char_table%mulliken(3)='e2'

    char_table%l=c
    char_table%sigma = 5

  end subroutine init_c5

  !***********************************************************************

  subroutine init_c6(char_table)
    !----------------------------------------------------------------
    ! init Char_table and related values
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------

    implicit none
    type(type_char_table),intent(inout) :: char_table
    integer i,j
    real(double) c(6,4)

    allocate(char_table%mulliken(4),char_table%symel(6), &
         char_table%l(6,4))

    data ((c(i,j),i=1,6),j=1,4) &
         /1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, &
         1.d0,-1.d0, 1.d0,-1.d0, 1.d0,-1.d0, &
         2.d0, 1.d0,-1.d0,-2.d0,-1.d0, 1.d0, &
         2.d0,-1.d0,-1.d0, 2.d0,-1.d0,-1.d0/

    char_table%nsymel=6
    char_table%nirrep=4

    char_table%symel(1)='cz1_1'
    char_table%symel(2)='cz6_1'
    char_table%symel(3)='cz3_1'
    char_table%symel(4)='cz2_1'
    char_table%symel(5)='cz3_2'
    char_table%symel(6)='cz6_5'

    char_table%mulliken(1)='a'
    char_table%mulliken(2)='b'
    char_table%mulliken(3)='e1'
    char_table%mulliken(4)='e2'

    char_table%l=c
    char_table%sigma = 6

  end subroutine init_c6

  !***********************************************************************

  subroutine init_c7(char_table)
    !----------------------------------------------------------------
    ! init Char_table and related values
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------

    implicit none
    type(type_char_table),intent(inout) :: char_table
    integer i,j
    real(double) c(7,4),pi
    parameter (pi = 3.1415926535897932384626433832795d0)

    allocate(char_table%mulliken(4),char_table%symel(7), &
         char_table%l(7,4))

    data ((c(i,j),i=1,7),j=1,4) &
         /1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, &
         2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, &
         2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, &
         2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0/
    c(2,2)=2*cos(2*pi/7)
    c(3,2)=2*cos(4*pi/7)
    c(4,2)=2*cos(6*pi/7)
    c(5,2)=c(4,2)
    c(6,2)=c(3,2)
    c(7,2)=c(2,2)
    c(3,2)=c(3,2)

    c(2,3)=c(3,2)
    c(3,3)=c(4,2)
    c(4,3)=c(2,2)
    c(5,3)=c(2,2)
    c(6,3)=c(4,2)
    c(7,3)=c(3,2)

    c(2,4)=c(4,2)
    c(3,4)=c(2,2)
    c(4,4)=c(3,2)
    c(5,4)=c(3,2)
    c(6,4)=c(2,2)
    c(7,4)=c(4,2)

    char_table%nsymel=7
    char_table%nirrep=4

    char_table%symel(1)='cz1_1'
    char_table%symel(2)='cz7_1'
    char_table%symel(3)='cz7_2'
    char_table%symel(4)='cz7_3'
    char_table%symel(5)='cz7_4'
    char_table%symel(6)='cz7_5'
    char_table%symel(7)='cz7_6'

    char_table%mulliken(1)='a'
    char_table%mulliken(2)='e1'
    char_table%mulliken(3)='e2'
    char_table%mulliken(4)='e3'

    char_table%l=c
    char_table%sigma = 7

  end subroutine init_c7

  !***********************************************************************

  subroutine init_c8(char_table)
    !----------------------------------------------------------------
    ! init Char_table and related values
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------

    implicit none
    type(type_char_table),intent(inout) :: char_table
    integer i,j
    real(double) c(8,5),pi
    parameter (pi = 3.1415926535897932384626433832795d0)

    allocate(char_table%mulliken(5),char_table%symel(8), &
         char_table%l(8,5))

    data ((c(i,j),i=1,8),j=1,5) &
         /1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, &
         1.d0,-1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0, &
         2.d0, 0.d0, 0.d0,-2.d0, 0.d0, 0.d0, 0.d0, 0.d0, &
         2.d0, 0.d0,-2.d0, 2.d0,-2.d0, 0.d0, 0.d0, 0.d0, &
         2.d0, 0.d0, 0.d0,-2.d0, 0.d0, 0.d0, 0.d0, 0.d0/
    c(2,3)=2*cos(2*pi/8)
    c(6,3)=-c(2,3)
    c(7,3)=-c(2,3)
    c(8,3)=c(2,3)
    c(2,5)=-c(2,3)
    c(6,5)=c(2,3)
    c(7,5)=c(2,3)
    c(8,5)=-c(2,3)

    char_table%nsymel=8
    char_table%nirrep=5

    char_table%symel(1)='cz1_1'
    char_table%symel(2)='cz8_1'
    char_table%symel(3)='cz4_1'
    char_table%symel(4)='cz2_1'
    char_table%symel(5)='cz4_3'
    char_table%symel(6)='cz8_3'
    char_table%symel(6)='cz8_5'
    char_table%symel(6)='cz8_7'

    char_table%mulliken(1)='a'
    char_table%mulliken(2)='b'
    char_table%mulliken(3)='e1'
    char_table%mulliken(4)='e2'
    char_table%mulliken(5)='e3'

    char_table%l=c
    char_table%sigma = 8

  end subroutine init_c8

  !***********************************************************************

  subroutine init_d2(char_table)
    !----------------------------------------------------------------
    ! init Char_table and related values
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------

    implicit none
    type(type_char_table),intent(inout) :: char_table
    integer i,j
    real(double) c(4,4)

    allocate(char_table%mulliken(4),char_table%symel(4), &
         char_table%l(4,4))

    data ((c(i,j),i=1,4),j=1,4) &
         /1.d0, 1.d0, 1.d0, 1.d0, &
         1.d0, 1.d0,-1.d0,-1.d0, &
         1.d0,-1.d0, 1.d0,-1.d0, &
         1.d0,-1.d0,-1.d0, 1.d0/ 

    char_table%nsymel=4 
    char_table%nirrep=4

    char_table%symel(1)='cz1_1' 
    char_table%symel(2)='cz2_1' 
    char_table%symel(3)='cx2_0' 
    char_table%symel(4)='cx2_1' 

    char_table%mulliken (1)='a' 
    char_table%mulliken (2)='b1' 
    char_table%mulliken (3)='b2' 
    char_table%mulliken (4)='b3' 

    char_table%l=c
    char_table%sigma = 4

  end subroutine init_d2

  !***********************************************************************

  subroutine init_d3(char_table)
    !----------------------------------------------------------------
    ! init Char_table and related values
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------

    implicit none
    type(type_char_table),intent(inout) :: char_table
    integer i,j
    real(double) c(6,3)

    allocate(char_table%mulliken(3),char_table%symel(6), &
         char_table%l(6,3))

    data ((c(i,j),i=1,6),j=1,3)  &
         /1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, &
         1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0, &
         2.d0,-1.d0,-1.d0, 0.d0, 0.d0, 0.d0/

    char_table%nsymel=6
    char_table%nirrep=3

    char_table%symel(1)='cz1_1' 
    char_table%symel(2)='cz3_1' 
    char_table%symel(3)='cz3_2' 
    char_table%symel(4)='cx3_0'
    char_table%symel(5)='cx3_1'
    char_table%symel(6)='cx3_2'


    char_table%mulliken (1)='a1' 
    char_table%mulliken (2)='a2' 
    char_table%mulliken (3)='e' 

    char_table%l=c
    char_table%sigma = 6

  end subroutine init_d3

  !***********************************************************************

  subroutine init_d4(char_table)
    !----------------------------------------------------------------
    ! init Char_table and related values
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------

    implicit none
    type(type_char_table),intent(inout) :: char_table
    integer i,j
    real(double) c(8,5)

    allocate(char_table%mulliken(5),char_table%symel(8), &
         char_table%l(8,5))

    data ((c(i,j),i=1,8),j=1,5)  &
         /1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, &
         1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,-1.d0, &
         1.d0,-1.d0,-1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0, &
         1.d0,-1.d0,-1.d0, 1.d0,-1.d0,-1.d0, 1.d0, 1.d0, &
         2.d0, 0.d0, 0.d0,-2.d0, 0.d0, 0.d0, 0.d0, 0.d0/

    char_table%nsymel=8
    char_table%nirrep=5

    char_table%symel(1)='cz1_1' 
    char_table%symel(2)='cz4_1' 
    char_table%symel(3)='cz4_3' 
    char_table%symel(4)='cz2_1' 
    char_table%symel(5)='cx4_0' 
    char_table%symel(6)='cx4_2' 
    char_table%symel(7)='cx4_1' 
    char_table%symel(8)='cx4_3' 

    char_table%mulliken (1)='a1' 
    char_table%mulliken (2)='a2' 
    char_table%mulliken (3)='b1' 
    char_table%mulliken (4)='b2' 
    char_table%mulliken (5)='e' 

    char_table%l=c
    char_table%sigma = 8

  end subroutine init_d4

  !***********************************************************************

  subroutine init_d5(char_table)
    !----------------------------------------------------------------
    ! init Char_table and related values
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------

    implicit none
    type(type_char_table),intent(inout) :: char_table
    integer i,j
    real(double) c(10,4),pi
    parameter (pi = 3.1415926535897932384626433832795d0)

    allocate(char_table%mulliken(4),char_table%symel(10), &
         char_table%l(10,4))

    data ((c(i,j),i=1,10),j=1,4) &
         /1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, &
         1.d0, 1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0, &
         2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, &
         2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0/
    c(2,3)=2*cos(2*pi/5)
    c(3,3)=c(2,3)
    c(4,3)=2*cos(4*pi/5)
    c(5,3)=c(4,3)
    c(2,4)=c(4,3)
    c(3,4)=c(4,3)
    c(4,4)=c(2,3)
    c(5,4)=c(2,3)

    char_table%nsymel=10
    char_table%nirrep=4

    char_table%symel(1)='cz1_1' 
    char_table%symel(2)='cz5_1' 
    char_table%symel(3)='cz5_4' 
    char_table%symel(4)='cz5_2'
    char_table%symel(5)='cz5_3'
    char_table%symel(6)='cx5_0'
    char_table%symel(7)='cx5_1'
    char_table%symel(8)='cx5_2'
    char_table%symel(9)='cx5_3'
    char_table%symel(10)='cx5_4'


    char_table%mulliken (1)='a1' 
    char_table%mulliken (2)='a2' 
    char_table%mulliken (3)='e1' 
    char_table%mulliken (4)='e2' 

    char_table%l=c
    char_table%sigma = 10

  end subroutine init_d5

  !***********************************************************************

  subroutine init_d6(char_table)
    !----------------------------------------------------------------
    ! init Char_table and related values
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------

    implicit none
    type(type_char_table),intent(inout) :: char_table
    integer i,j
    real(double) c(12,6)

    allocate(char_table%mulliken(6),char_table%symel(12), &
         char_table%l(12,6))

    data ((c(i,j),i=1,12),j=1,6)  &
         /1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, &
         1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0, &
         1.d0,-1.d0,-1.d0, 1.d0, 1.d0,-1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0, &
         1.d0,-1.d0,-1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,-1.d0, 1.d0, 1.d0, 1.d0, &
         2.d0, 1.d0, 1.d0,-1.d0,-1.d0,-2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, &
         2.d0,-1.d0,-1.d0,-1.d0,-1.d0, 2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0/


    char_table%nsymel=12
    char_table%nirrep=6

    char_table%symel(1)='cz1_1' 
    char_table%symel(2)='cz6_1' 
    char_table%symel(3)='cz6_5' 
    char_table%symel(4)='cz3_1' 
    char_table%symel(5)='cz3_2' 
    char_table%symel(6)='cz2_1' 
    char_table%symel(7)='cx6_0' 
    char_table%symel(8)='cx6_2' 
    char_table%symel(9)='cx6_4' 
    char_table%symel(10)='cx6_1' 
    char_table%symel(11)='cx6_3' 
    char_table%symel(12)='cx6_5' 

    char_table%mulliken (1)='a1' 
    char_table%mulliken (2)='a2' 
    char_table%mulliken (3)='b1' 
    char_table%mulliken (4)='b2' 
    char_table%mulliken (5)='e1' 
    char_table%mulliken (6)='e2' 

    char_table%l=c
    char_table%sigma = 12

  end subroutine init_d6

  !***********************************************************************

  subroutine init_c2v(char_table)
    !----------------------------------------------------------------
    ! init Char_table and related values
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------

    implicit none
    type(type_char_table),intent(inout) :: char_table
    integer i,j
    real(double) c(4,4)

    allocate(char_table%mulliken(4),char_table%symel(4), &
         char_table%l(4,4))

    data ((c(i,j),i=1,4),j=1,4)  &
         /1.d0, 1.d0, 1.d0, 1.d0, &
         1.d0, 1.d0,-1.d0,-1.d0, &
         1.d0,-1.d0, 1.d0,-1.d0, &
         1.d0,-1.d0,-1.d0, 1.d0/ 

    char_table%nsymel=4 
    char_table%nirrep=4

    char_table%symel(1)='cz1_1' 
    char_table%symel(2)='cz2_1' 
    char_table%symel(3)='sv2_0' 
    char_table%symel(4)='sd2_0' 

    char_table%mulliken (1)='a1' 
    char_table%mulliken (2)='a2' 
    char_table%mulliken (3)='b1' 
    char_table%mulliken (4)='b2' 

    char_table%l=c
    char_table%sigma = 2

  end subroutine init_c2v

  !***********************************************************************

  subroutine init_c3v(char_table)
    !----------------------------------------------------------------
    ! init Char_table and related values
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------

    implicit none
    type(type_char_table),intent(inout) :: char_table
    integer i,j
    real(double) c(6,3)

    allocate(char_table%mulliken(3),char_table%symel(6), &
         char_table%l(6,3))

    data ((c(i,j),i=1,6),j=1,3) &
         /1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, &
         1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0, &
         2.d0,-1.d0,-1.d0, 0.d0, 0.d0, 0.d0/ 

    char_table%nsymel=6
    char_table%nirrep=3

    char_table%symel(1)='cz1_1' 
    char_table%symel(2)='cz3_1' 
    char_table%symel(3)='cz3_2' 
    char_table%symel(4)='sv3_0' 
    char_table%symel(5)='sv3_1' 
    char_table%symel(6)='sv3_2' 

    char_table%mulliken (1)='a1' 
    char_table%mulliken (2)='a2' 
    char_table%mulliken (3)='e'

    char_table%l=c
    char_table%sigma = 3

  end subroutine init_c3v

  !***********************************************************************

  subroutine init_c4v(char_table)
    !----------------------------------------------------------------
    ! init Char_table and related values
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------

    implicit none
    type(type_char_table),intent(inout) :: char_table
    integer i,j
    real(double) c(8,5)

    allocate(char_table%mulliken(5),char_table%symel(8), &
         char_table%l(8,5))

    data ((c(i,j),i=1,8),j=1,5) &
         /1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, &
         1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,-1.d0, &
         1.d0,-1.d0,-1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0, &
         1.d0,-1.d0,-1.d0, 1.d0,-1.d0,-1.d0, 1.d0, 1.d0, &
         2.d0, 0.d0, 0.d0,-2.d0, 0.d0, 0.d0, 0.d0, 0.d0/

    char_table%nsymel=8
    char_table%nirrep=5

    char_table%symel(1)='cz1_1' 
    char_table%symel(2)='cz4_1' 
    char_table%symel(3)='cz4_3' 
    char_table%symel(4)='cz2_1' 
    char_table%symel(5)='sv4_0' 
    char_table%symel(6)='sv4_1' 
    char_table%symel(7)='sd4_0' 
    char_table%symel(8)='sd4_1' 

    char_table%mulliken (1)='a1' 
    char_table%mulliken (2)='a2' 
    char_table%mulliken (3)='b1' 
    char_table%mulliken (4)='b2' 
    char_table%mulliken (5)='e' 

    char_table%l=c
    char_table%sigma = 4

  end subroutine init_c4v

  !***********************************************************************

  subroutine init_c5v(char_table)
    !----------------------------------------------------------------
    ! init Char_table and related values
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------

    implicit none
    type(type_char_table),intent(inout) :: char_table
    integer i,j
    real(double) c(10,4),pi
    parameter (pi = 3.1415926535897932384626433832795d0)

    allocate(char_table%mulliken(4),char_table%symel(10), &
         char_table%l(10,4))

    data ((c(i,j),i=1,10),j=1,4)  &
         /1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, & 
         1.d0, 1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0, &
         2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, &
         2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0/
    c(2,3)=2.d0*cos(2.d0*pi/5.d0)
    c(2,4)=2.d0*cos(4.d0*pi/5.d0)
    c(3,:)=c(2,:)
    c(4,3)=c(2,4)
    c(4,4)=c(2,3)
    c(5,:)=c(4,:)

    char_table%nsymel=10
    char_table%nirrep=4

    char_table%symel(1)='cz1_1' 
    char_table%symel(2)='cz5_1' 
    char_table%symel(3)='cz5_4' 
    char_table%symel(4)='cz5_2' 
    char_table%symel(5)='cz5_3' 
    char_table%symel(6)='sv5_0' 
    char_table%symel(7)='sv5_1' 
    char_table%symel(8)='sv5_2' 
    char_table%symel(9)='sv5_3' 
    char_table%symel(10)='sv5_4' 

    char_table%mulliken (1)='a1' 
    char_table%mulliken (2)='a2' 
    char_table%mulliken (3)='e1'
    char_table%mulliken (4)='e2'

    char_table%l=c
    char_table%sigma = 5

  end subroutine init_c5v

  !***********************************************************************

  subroutine init_c6v(char_table)
    !----------------------------------------------------------------
    ! init Char_table and related values
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------

    implicit none
    type(type_char_table),intent(inout) :: char_table
    integer i,j
    real(double) c(12,6)

    allocate(char_table%mulliken(6),char_table%symel(12), &
         char_table%l(12,6))

    data ((c(i,j),i=1,12),j=1,6)  &
         /1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, &
         1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0, &
         1.d0,-1.d0,-1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,-1.d0, 1.d0, 1.d0, 1.d0, &
         1.d0,-1.d0,-1.d0, 1.d0, 1.d0,-1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0, &
         2.d0, 1.d0, 1.d0,-1.d0,-1.d0,-2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, &
         2.d0,-1.d0,-1.d0,-1.d0,-1.d0, 2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0/

    char_table%nsymel=12
    char_table%nirrep=6

    char_table%symel(1)='cz1_1' 
    char_table%symel(2)='cz6_1' 
    char_table%symel(3)='cz6_5' 
    char_table%symel(4)='cz3_1' 
    char_table%symel(5)='cz3_2' 
    char_table%symel(6)='cz2_1' 
    char_table%symel(7)='sv6_0' 
    char_table%symel(8)='sv6_1' 
    char_table%symel(9)='sv6_2' 
    char_table%symel(10)='sd6_0' 
    char_table%symel(11)='sd6_1' 
    char_table%symel(12)='sd6_2' 

    char_table%mulliken (1)='a1' 
    char_table%mulliken (2)='a2' 
    char_table%mulliken (3)='b1' 
    char_table%mulliken (4)='b2' 
    char_table%mulliken (5)='e1'
    char_table%mulliken (6)='e2' 

    char_table%l=c
    char_table%sigma = 6

  end subroutine init_c6v

  !***********************************************************************

  subroutine init_c2h(char_table)
    !----------------------------------------------------------------
    ! init Char_table and related values
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------

    implicit none
    type(type_char_table),intent(inout) :: char_table
    integer i,j
    real(double) c(4,4)

    allocate(char_table%mulliken(4),char_table%symel(4), &
         char_table%l(4,4))

    data ((c(i,j),i=1,4),j=1,4)  &
         /1.d0, 1.d0, 1.d0, 1.d0, &
         1.d0,-1.d0, 1.d0,-1.d0, & 
         1.d0, 1.d0,-1.d0,-1.d0, &
         1.d0,-1.d0,-1.d0, 1.d0/ 

    char_table%nsymel=4 
    char_table%nirrep=4

    char_table%symel(1)='cz1_1' 
    char_table%symel(2)='cz2_1'
    char_table%symel(3)='sz2_1' 
    char_table%symel(4)='sh' 

    char_table%mulliken (1)='ag' 
    char_table%mulliken (2)='bg' 
    char_table%mulliken (3)='au' 
    char_table%mulliken (4)='bu' 

    char_table%l=c
    char_table%sigma = 2

  end subroutine init_c2h

  !***********************************************************************

  subroutine init_c3h(char_table)
    !----------------------------------------------------------------
    ! init Char_table and related values
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------

    implicit none
    type(type_char_table),intent(inout) :: char_table
    integer i,j
    real(double) c(6,4)

    allocate(char_table%mulliken(4),char_table%symel(6), &
         char_table%l(6,4))

    data ((c(i,j),i=1,6),j=1,4) &
         /1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, & 
         2.d0,-1.d0,-1.d0, 2.d0,-1.d0,-1.d0, &
         1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0, &
         2.d0,-1.d0,-1.d0,-2.d0, 1.d0, 1.d0/ 

    char_table%nsymel=6
    char_table%nirrep=4

    char_table%symel(1)='cz1_1' 
    char_table%symel(2)='cz3_1'
    char_table%symel(3)='cz3_2'
    char_table%symel(4)='sh' 
    char_table%symel(5)='sz3_1'
    char_table%symel(6)='sz3_2'  

    char_table%mulliken (1)='ai' 
    char_table%mulliken (2)='ei' 
    char_table%mulliken (3)='aii' 
    char_table%mulliken (4)='eii' 

    char_table%l=c
    char_table%sigma = 3

  end subroutine init_c3h

  !***********************************************************************

  subroutine init_c4h(char_table)
    !----------------------------------------------------------------
    ! init Char_table and related values
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------

    implicit none
    type(type_char_table),intent(inout) :: char_table
    integer i,j
    real(double) c(8,6)

    allocate(char_table%mulliken(6),char_table%symel(8), &
         char_table%l(8,6))

    data ((c(i,j),i=1,8),j=1,6) &
         /1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, &
         1.d0,-1.d0, 1.d0,-1.d0, 1.d0,-1.d0, 1.d0,-1.d0, &
         2.d0, 0.d0,-2.d0, 0.d0, 2.d0, 0.d0,-2.d0, 0.d0, &
         1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,-1.d0, &
         1.d0,-1.d0, 1.d0,-1.d0,-1.d0, 1.d0,-1.d0, 1.d0, &
         2.d0, 0.d0,-2.d0, 0.d0,-2.d0, 0.d0, 2.d0, 0.d0/

    char_table%nsymel=8
    char_table%nirrep=6

    char_table%symel(1)='cz1_1' 
    char_table%symel(2)='cz4_1'
    char_table%symel(3)='cz2_1'
    char_table%symel(4)='cz4_3' 
    char_table%symel(5)='sz2_1'
    char_table%symel(6)='sz4_3'  
    char_table%symel(7)='sh'  
    char_table%symel(8)='sz4_1'  

    char_table%mulliken (1)='ag' 
    char_table%mulliken (2)='bg' 
    char_table%mulliken (3)='eg'  
    char_table%mulliken (4)='au' 
    char_table%mulliken (5)='bu' 
    char_table%mulliken (6)='eu'  

    char_table%l=c
    char_table%sigma = 4

  end subroutine init_c4h

  !***********************************************************************

  subroutine init_c5h(char_table)
    !----------------------------------------------------------------
    ! init Char_table and related values
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------

    implicit none
    type(type_char_table),intent(inout) :: char_table
    integer i,j
    real(double) c(10,6),pi
    parameter (pi = 3.1415926535897932384626433832795d0)

    allocate(char_table%mulliken(6),char_table%symel(10), &
         char_table%l(10,6))

    data ((c(i,j),i=1,10),j=1,6) & 
         /1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, &
         2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 2.d0, 0.d0, 0.d0, 0.d0, 0.d0, &
         2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 2.d0, 0.d0, 0.d0, 0.d0, 0.d0, &
         1.d0, 1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0, &
         2.d0, 0.d0, 0.d0, 0.d0, 0.d0,-2.d0, 0.d0, 0.d0, 0.d0, 0.d0, &
         2.d0, 0.d0, 0.d0, 0.d0, 0.d0,-2.d0, 0.d0, 0.d0, 0.d0, 0.d0/
    c(2,2)=2*cos(2*pi/5)
    c(3,2)=2*cos(4*pi/5)
    c(4,2)=c(3,2)
    c(5,2)=c(2,2)
    c(7,2)=c(2,2)
    c(8,2)=c(3,2)
    c(9,2)=c(3,2)
    c(10,2)=c(2,2)
    c(2,3)=c(3,2)
    c(3,3)=c(2,2)
    c(4,3)=c(2,2)
    c(5,3)=c(3,2)
    c(7,3)=c(3,2)
    c(8,3)=c(2,2)
    c(9,3)=c(2,2)
    c(10,3)=c(3,2)
    c(2,5)=c(2,2)
    c(3,5)=c(3,2)
    c(4,5)=c(3,2)
    c(5,5)=c(2,2)
    c(7,5)=-c(2,2)
    c(8,5)=-c(3,2)
    c(9,5)=-c(3,2)
    c(10,5)=-c(2,2)
    c(2,6)=c(3,2)
    c(3,6)=c(2,2)
    c(4,6)=c(2,2)
    c(5,6)=c(3,2)
    c(7,6)=-c(3,2)
    c(8,6)=-c(2,2)
    c(9,6)=-c(2,2)
    c(10,6)=-c(3,2)

    char_table%nsymel=10
    char_table%nirrep=6

    char_table%symel(1)='cz1_1' 
    char_table%symel(2)='cz5_1'
    char_table%symel(3)='cz5_2'
    char_table%symel(4)='cz5_3' 
    char_table%symel(5)='cz5_4'
    char_table%symel(6)='sh'  
    char_table%symel(7)='sz5_1'
    char_table%symel(8)='sz5_7'
    char_table%symel(9)='sz5_3' 
    char_table%symel(10)='sz5_9'

    char_table%mulliken (1)='ai' 
    char_table%mulliken (2)='e1i' 
    char_table%mulliken (3)='e2i'  
    char_table%mulliken (4)='aii' 
    char_table%mulliken (5)='e1i' 
    char_table%mulliken (6)='e2ii'  

    char_table%l=c
    char_table%sigma = 5

  end subroutine init_c5h

  !***********************************************************************

  subroutine init_c6h(char_table)
    !----------------------------------------------------------------
    ! init Char_table and related values
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------

    implicit none
    type(type_char_table),intent(inout) :: char_table
    integer i,j
    real(double) c(12,8)

    allocate(char_table%mulliken(8),char_table%symel(12), &
         char_table%l(12,8))

    data ((c(i,j),i=1,12),j=1,8) & 
         /1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, &
         1.d0,-1.d0, 1.d0,-1.d0, 1.d0,-1.d0, 1.d0,-1.d0, 1.d0,-1.d0, 1.d0,-1.d0, &
         2.d0, 1.d0,-1.d0,-2.d0,-1.d0, 1.d0, 2.d0, 1.d0,-1.d0,-2.d0,-1.d0, 1.d0, &
         2.d0,-1.d0,-1.d0, 2.d0,-1.d0,-1.d0, 2.d0,-1.d0,-1.d0, 2.d0,-1.d0,-1.d0, &
         1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0, &
         1.d0,-1.d0, 1.d0,-1.d0, 1.d0,-1.d0,-1.d0, 1.d0,-1.d0, 1.d0,-1.d0, 1.d0, &
         2.d0, 1.d0,-1.d0,-2.d0,-1.d0, 1.d0,-2.d0,-1.d0, 1.d0, 2.d0, 1.d0,-1.d0, &
         2.d0,-1.d0,-1.d0, 2.d0,-1.d0,-1.d0,-2.d0, 1.d0, 1.d0,-2.d0, 1.d0, 1.d0/

    char_table%nsymel=12
    char_table%nirrep=8

    char_table%symel(1)='cz1_1' 
    char_table%symel(2)='cz6_1'
    char_table%symel(3)='cz3_1'
    char_table%symel(4)='cz2_1' 
    char_table%symel(5)='cz3_2'
    char_table%symel(6)='cz6_5'  
    char_table%symel(7)='sz2_1'  
    char_table%symel(8)='sz3_2'
    char_table%symel(9)='sz6_5'
    char_table%symel(10)='sh'
    char_table%symel(11)='sz6_1'
    char_table%symel(12)='sz3_1'


    char_table%mulliken (1)='ag' 
    char_table%mulliken (2)='bg' 
    char_table%mulliken (3)='e1g' 
    char_table%mulliken (4)='e2g'
    char_table%mulliken (5)='au' 
    char_table%mulliken (6)='bu' 
    char_table%mulliken (7)='e1u'  
    char_table%mulliken (8)='e2u'

    char_table%l=c
    char_table%sigma = 6

  end subroutine init_c6h

  !***********************************************************************

  subroutine init_d2h(char_table)
    !----------------------------------------------------------------
    ! init Char_table and related values
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------

    implicit none
    type(type_char_table),intent(inout) :: char_table
    integer i,j
    real(double) c(8,8)

    allocate(char_table%mulliken(8),char_table%symel(8), &
         char_table%l(8,8))

    data ((c(i,j),i=1,8),j=1,8) &
         /1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, &
         1.d0, 1.d0,-1.d0,-1.d0, 1.d0, 1.d0,-1.d0,-1.d0, &
         1.d0,-1.d0,-1.d0, 1.d0, 1.d0,-1.d0,-1.d0, 1.d0, &
         1.d0,-1.d0, 1.d0,-1.d0, 1.d0,-1.d0, 1.d0,-1.d0, &
         1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,-1.d0, &
         1.d0, 1.d0,-1.d0,-1.d0,-1.d0,-1.d0, 1.d0, 1.d0, &
         1.d0,-1.d0,-1.d0, 1.d0,-1.d0, 1.d0, 1.d0,-1.d0, &
         1.d0,-1.d0, 1.d0,-1.d0,-1.d0, 1.d0,-1.d0, 1.d0/ 

    char_table%nsymel=8 
    char_table%nirrep=8

    char_table%symel(1)='cz1_1' 
    char_table%symel(2)='cz2_1' 
    char_table%symel(3)='cx2_0' 
    char_table%symel(4)='cx2_1' 
    char_table%symel(5)='sz2_1' 
    char_table%symel(6)='sh' 
    char_table%symel(7)='sd2_0'
    char_table%symel(8)='sv2_0'

    char_table%mulliken (1)='ag' 
    char_table%mulliken (2)='b1g' 
    char_table%mulliken (3)='b2g' 
    char_table%mulliken (4)='b3g' 
    char_table%mulliken (5)='au' 
    char_table%mulliken (6)='b1u' 
    char_table%mulliken (7)='b2u' 
    char_table%mulliken (8)='b3u' 

    char_table%l=c
    char_table%sigma = 4

  end subroutine init_d2h

  !***********************************************************************

  subroutine init_d3h(char_table)
    !----------------------------------------------------------------
    ! init Char_table and related values
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------

    implicit none
    type(type_char_table),intent(inout) :: char_table
    integer i,j
    real(double) c(12,6)

    allocate(char_table%mulliken(6),char_table%symel(12), &
         char_table%l(12,6))

    data ((c(i,j),i=1,12),j=1,6) &
         /1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, &
         1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0, &
         2.d0,-1.d0,-1.d0, 0.d0, 0.d0, 0.d0, 2.d0,-1.d0,-1.d0, 0.d0, 0.d0, 0.d0, &
         1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0, &
         1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0, 1.d0, 1.d0, 1.d0, &
         2.d0,-1.d0,-1.d0, 0.d0, 0.d0, 0.d0,-2.d0, 1.d0, 1.d0, 0.d0, 0.d0, 0.d0/

    char_table%nsymel=12
    char_table%nirrep=6

    char_table%symel(1)='cz1_1' 
    char_table%symel(2)='cz3_1' 
    char_table%symel(3)='cz3_2' 
    char_table%symel(4)='cx3_0' 
    char_table%symel(5)='cx3_1' 
    char_table%symel(6)='cx3_2' 
    char_table%symel(7)='sh' 
    char_table%symel(8)='sz3_1' 
    char_table%symel(9)='sz3_2' 
    char_table%symel(10)='sv3_0' 
    char_table%symel(11)='sv3_1' 
    char_table%symel(12)='sv3_2' 

    char_table%mulliken (1)='a1i' 
    char_table%mulliken (2)='a2i' 
    char_table%mulliken (3)='ei' 
    char_table%mulliken (4)='a1ii' 
    char_table%mulliken (5)='a2ii' 
    char_table%mulliken (6)='eii' 

    char_table%l=c
    char_table%sigma = 6

  end subroutine init_d3h

  !***********************************************************************

  subroutine init_d4h(char_table)
    !----------------------------------------------------------------
    ! init Char_table and related values
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------

    implicit none
    type(type_char_table),intent(inout) :: char_table
    integer i,j
    real(double) c(16,10)

    allocate(char_table%mulliken(10),char_table%symel(16), &
         char_table%l(16,10))

    data ((c(i,j),i=1,16),j=1,10)   &
         /1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, &
         1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,-1.d0, 1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,-1.d0, &
         1.d0,-1.d0,-1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0, 1.d0,-1.d0,-1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0, &
         1.d0,-1.d0,-1.d0, 1.d0,-1.d0,-1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0, 1.d0,-1.d0,-1.d0, 1.d0, 1.d0, &
         2.d0, 0.d0, 0.d0,-2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 2.d0, 0.d0, 0.d0,-2.d0, 0.d0, 0.d0, 0.d0, 0.d0, &
         1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0, &
         1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0, 1.d0, 1.d0, 1.d0, 1.d0, &
         1.d0,-1.d0,-1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0, 1.d0, 1.d0, &
         1.d0,-1.d0,-1.d0, 1.d0,-1.d0,-1.d0, 1.d0, 1.d0,-1.d0, 1.d0, 1.d0,-1.d0, 1.d0, 1.d0,-1.d0,-1.d0, &
         2.d0, 0.d0, 0.d0,-2.d0, 0.d0, 0.d0, 0.d0, 0.d0,-2.d0, 0.d0, 0.d0, 2.d0, 0.d0, 0.d0, 0.d0, 0.d0/

    char_table%nsymel=16
    char_table%nirrep=10

    char_table%symel(1)='cz1_1' 
    char_table%symel(2)='cz4_1' 
    char_table%symel(3)='cz4_3' 
    char_table%symel(4)='cz2_1' 
    char_table%symel(5)='cx4_0' 
    char_table%symel(6)='cx4_2' 
    char_table%symel(7)='cx4_1' 
    char_table%symel(8)='cx4_3' 
    char_table%symel(9)='sz2_1' 
    char_table%symel(10)='sz4_1' 
    char_table%symel(11)='sz4_3' 
    char_table%symel(12)='sh' 
    char_table%symel(13)='sv4_0' 
    char_table%symel(14)='sv4_1' 
    char_table%symel(15)='sd4_0' 
    char_table%symel(16)='sd4_1' 

    char_table%mulliken (1)='a1g' 
    char_table%mulliken (2)='a2g' 
    char_table%mulliken (3)='b1g' 
    char_table%mulliken (4)='b2g' 
    char_table%mulliken (5)='eg' 
    char_table%mulliken (6)='a1u' 
    char_table%mulliken (7)='a2u' 
    char_table%mulliken (8)='b1u' 
    char_table%mulliken (9)='b2u' 
    char_table%mulliken (10)='eu' 

    char_table%l=c
    char_table%sigma = 8

  end subroutine init_d4h

  !***********************************************************************

  subroutine init_d5h(char_table)
    !----------------------------------------------------------------
    ! init Char_table and related values
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------

    implicit none
    type(type_char_table),intent(inout) :: char_table
    integer i,j
    real(double) c(20,8),pi
    parameter (pi = 3.1415926535897932384626433832795d0)

    allocate(char_table%mulliken(8),char_table%symel(20),  &
         char_table%l(20,8))

    data ((c(i,j),i=1,20),j=1,8)  &
         /1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, &
         1.d0, 1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,  &
         2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0,  &
         2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0,  &
         1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,  &
         1.d0, 1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0,  &
         2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0,-2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0,  &
         2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0,-2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0/
    c(2,3)=2*cos(2*pi/5)
    c(2,4)=2*cos(4*pi/5)
    c(2,7)=c(2,3)
    c(2,8)=c(2,4)
    c(4,3)=c(2,4)
    c(4,4)=c(2,3)
    c(4,7)=c(4,3)
    c(4,8)=c(4,4)
    c(12,3)=c(2,3)
    c(12,4)=c(2,4)
    c(12,7)=-c(2,3)
    c(12,8)=-c(2,4)
    c(14,3)=c(4,3)
    c(14,4)=c(4,4)
    c(14,7)=-c(4,3)
    c(14,8)=-c(4,4)
    c(3,:)=c(2,:)
    c(5,:)=c(4,:)
    c(13,:)=c(12,:)
    c(15,:)=c(14,:)
    char_table%nsymel=20
    char_table%nirrep=8

    char_table%symel(1)='cz1_1' 
    char_table%symel(2)='cz5_1' 
    char_table%symel(3)='cz5_4' 
    char_table%symel(4)='cz5_2' 
    char_table%symel(5)='cz5_3' 
    char_table%symel(6)='cx5_0' 
    char_table%symel(7)='cx5_1' 
    char_table%symel(8)='cx5_2' 
    char_table%symel(9)='cx5_3' 
    char_table%symel(10)='cx5_4' 
    char_table%symel(11)='sh' 
    char_table%symel(12)='sz5_1' 
    char_table%symel(13)='sz5_4' 
    char_table%symel(14)='sz5_3' 
    char_table%symel(15)='sz5_2' 
    char_table%symel(16)='sv5_0' 
    char_table%symel(17)='sv5_1' 
    char_table%symel(18)='sv5_2' 
    char_table%symel(19)='sv5_3' 
    char_table%symel(20)='sv5_4' 

    char_table%mulliken (1)='a1i' 
    char_table%mulliken (2)='a2i' 
    char_table%mulliken (3)='e1i' 
    char_table%mulliken (4)='e2i' 
    char_table%mulliken (5)='a1ii' 
    char_table%mulliken (6)='a2ii' 
    char_table%mulliken (7)='e1ii' 
    char_table%mulliken (8)='e2ii' 

    char_table%l=c
    char_table%sigma = 10

  end subroutine init_d5h

  !***********************************************************************

  subroutine init_d6h(char_table)
    !----------------------------------------------------------------
    ! init Char_table and related values
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    ! corrected by J. Neugebauer, Uni Erlangen, 2001
    !----------------------------------------------------------------

    implicit none
    type(type_char_table),intent(inout) :: char_table
    integer i,j
    real(double) c(24,12)

    allocate(char_table%mulliken(12),char_table%symel(24),  &
         char_table%l(24,12))

    data ((c(i,j),i=1,24),j=1,12) &
         /1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0,&
         1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0,&
         1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0,&
         -1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0, 1.d0, 1.d0, 1.d0, 1.d0,&
         1.d0, 1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0, 1.d0,-1.d0,&
         -1.d0, 1.d0, 1.d0,-1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,&
         1.d0,-1.d0,-1.d0, 1.d0, 1.d0,-1.d0, 1.d0, 1.d0, 1.d0,-1.d0,&
         -1.d0,-1.d0, 1.d0,-1.d0,-1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,&
         -1.d0, 1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0, 1.d0, 1.d0,-1.d0,&
         -1.d0,-1.d0,-1.d0, 1.d0, 1.d0, 1.d0, 2.d0, 1.d0, 1.d0,-1.d0,&
         -1.d0,-2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 2.d0, 1.d0,&
         1.d0,-1.d0,-1.d0,-2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0,&
         2.d0,-1.d0,-1.d0,-1.d0,-1.d0, 2.d0, 0.d0, 0.d0, 0.d0, 0.d0,&
         0.d0, 0.d0, 2.d0,-1.d0,-1.d0,-1.d0,-1.d0, 2.d0, 0.d0, 0.d0,&
         0.d0, 0.d0, 0.d0, 0.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0,&
         1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,-1.d0,&
         -1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0, 1.d0, 1.d0,&
         1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,&
         -1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0, 1.d0, 1.d0, 1.d0, 1.d0,&
         1.d0, 1.d0, 1.d0,-1.d0,-1.d0, 1.d0, 1.d0,-1.d0, 1.d0, 1.d0,&
         1.d0,-1.d0,-1.d0,-1.d0,-1.d0, 1.d0, 1.d0,-1.d0,-1.d0, 1.d0,&
         -1.d0,-1.d0,-1.d0, 1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0, 1.d0,&
         1.d0,-1.d0,-1.d0,-1.d0,-1.d0, 1.d0, 1.d0, 1.d0,-1.d0, 1.d0,&
         1.d0,-1.d0,-1.d0, 1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,&
         2.d0, 1.d0, 1.d0,-1.d0,-1.d0,-2.d0, 0.d0, 0.d0, 0.d0, 0.d0,&
         0.d0, 0.d0,-2.d0,-1.d0,-1.d0, 1.d0, 1.d0, 2.d0, 0.d0, 0.d0,&
         0.d0, 0.d0, 0.d0, 0.d0, 2.d0,-1.d0,-1.d0,-1.d0,-1.d0, 2.d0,&
         0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0,-2.d0, 1.d0, 1.d0, 1.d0,&
         1.d0,-2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0/

    char_table%nsymel=24
    char_table%nirrep=12

    char_table%symel(1)='cz1_1' 
    char_table%symel(2)='cz6_1' 
    char_table%symel(3)='cz6_5' 
    char_table%symel(4)='cz3_1' 
    char_table%symel(5)='cz3_2' 
    char_table%symel(6)='cz2_1' 
    char_table%symel(7)='cx6_0' 
    char_table%symel(8)='cx6_2' 
    char_table%symel(9)='cx6_4' 
    char_table%symel(10)='cx6_1' 
    char_table%symel(11)='cx6_3' 
    char_table%symel(12)='cx6_5' 
    char_table%symel(13)='sz2_1' 
    char_table%symel(14)='sz3_1' 
    char_table%symel(15)='sz3_2' 
    char_table%symel(16)='sz6_1' 
    char_table%symel(17)='sz6_5' 
    char_table%symel(18)='sh' 
    char_table%symel(19)='sd6_0' 
    char_table%symel(20)='sd6_1' 
    char_table%symel(21)='sd6_2' 
    char_table%symel(22)='sv6_0' 
    char_table%symel(23)='sv6_1' 
    char_table%symel(24)='sv6_2' 

    char_table%mulliken (1)='a1g' 
    char_table%mulliken (2)='a2g' 
    char_table%mulliken (3)='b1g' 
    char_table%mulliken (4)='b2g' 
    char_table%mulliken (5)='e1g' 
    char_table%mulliken (6)='e2g' 
    char_table%mulliken (7)='a1u' 
    char_table%mulliken (8)='a2u' 
    char_table%mulliken (9)='b1u' 
    char_table%mulliken (10)='b2u' 
    char_table%mulliken (11)='e1u' 
    char_table%mulliken (12)='e2u' 

    char_table%l=c
    char_table%sigma = 12

  end subroutine init_d6h

  !  subroutine init_d7h(char_table)
  !    !----------------------------------------------------------------
  !    ! NOT SUFFICIENTLY TESTED
  !    ! init Char_table and related values
  !    !
  !    ! written by Carsten Kind, Uni Erlangen, 2001
  !    !----------------------------------------------------------------
  !
  !    type(type_char_table),intent(inout) :: char_table
  !    integer i,j
  !    real(double) c(28,10)
  !
  !    allocate(char_table%mulliken(10),char_table%symel(28),  &
  !         char_table%l(28,10))
  !
!!!$    data ((c(i,j),i=1,24),j=1,12) &
!!!$        /1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0,  &
!!!$         1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0,  &
!!!$         1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0, 1.d0, 1.d0, &
!!!$         1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,  &
!!!$         1.d0,-1.d0,-1.d0, 1.d0, 1.d0,-1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0, 1.d0,-1.d0,&
!!!$        -1.d0, 1.d0, 1.d0,-1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,  &
!!!$         1.d0,-1.d0,-1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,-1.d0, 1.d0, 1.d0, 1.d0, 1.d0,-1.d0,&
!!!$        -1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,-1.d0, 1.d0, 1.d0, 1.d0,  &
!!!$         2.d0, 1.d0, 1.d0,-1.d0,-1.d0,-2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 2.d0, 1.d0,&
!!!$         1.d0,-1.d0,-1.d0,-2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0,  &
!!!$         2.d0,-1.d0,-1.d0,-1.d0,-1.d0, 2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 2.d0,-1.d0,&
!!!$        -1.d0,-1.d0,-1.d0, 2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0,  &
!!!$         1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,&
!!!$        -1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,  &
!!!$         1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,&
!!!$        -1.d0,-1.d0,-1.d0,-1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0,  &
!!!$         1.d0,-1.d0,-1.d0, 1.d0, 1.d0,-1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,-1.d0, 1.d0, &
!!!$         1.d0,-1.d0,-1.d0, 1.d0,-1.d0,-1.d0,-1.d0, 1.d0, 1.d0, 1.d0,  &
!!!$         1.d0,-1.d0,-1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,-1.d0, 1.d0, 1.d0, 1.d0,-1.d0, 1.d0, &
!!!$         1.d0,-1.d0,-1.d0, 1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,  &
!!!$         2.d0, 1.d0, 1.d0,-1.d0,-1.d0,-2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0,-2.d0,-1.d0,&
!!!$        -1.d0, 1.d0, 1.d0, 2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0,  &
!!!$         2.d0,-1.d0,-1.d0,-1.d0,-1.d0, 2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0,-2.d0, 1.d0,&
!!!$         1.d0, 1.d0, 1.d0,-2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0/
  !    char_table%nsymel=24
  !    char_table%nirrep=12
  !
  !    char_table%symel(1)='cz1_1' 
  !    char_table%symel(2)='cz7_1' 
  !    char_table%symel(3)='cz7_6' 
  !    char_table%symel(4)='cz7_2' 
  !    char_table%symel(5)='cz7_5' 
  !    char_table%symel(6)='cz7_3' 
  !    char_table%symel(7)='cz7_4' 
  !    char_table%symel(8)='cx7_0' 
  !    char_table%symel(9)='cx7_1' 
  !    char_table%symel(10)='cx7_2' 
  !    char_table%symel(11)='cx7_3' 
  !    char_table%symel(12)='cx7_4' 
  !    char_table%symel(13)='cx7_5' 
  !    char_table%symel(14)='cx7_6' 
  !    char_table%symel(15)='sh' 
  !    char_table%symel(16)='sz7_1' 
  !    char_table%symel(17)='sz7_6' 
  !    char_table%symel(18)='sz7_2' 
  !    char_table%symel(19)='sz7_5' 
  !    char_table%symel(20)='sz7_3' 
  !    char_table%symel(21)='sz7_4' 
  !    char_table%symel(22)='sv7_0' 
  !    char_table%symel(23)='sv7_1' 
  !    char_table%symel(24)='sv7_2' 
  !    char_table%symel(25)='sv7_3' 
  !    char_table%symel(26)='sv7_4' 
  !    char_table%symel(27)='sv7_5' 
  !    char_table%symel(28)='sv7_6' 
  !
  !    char_table%mulliken (1)='a1i' 
  !    char_table%mulliken (2)='a2ii'
  !    char_table%mulliken (3)='e1i' 
  !    char_table%mulliken (4)='e2i' 
  !    char_table%mulliken (5)='e3i' 
  !    char_table%mulliken (6)='a1ii' 
  !    char_table%mulliken (7)='a2ii' 
  !    char_table%mulliken (8)='e1ii' 
  !    char_table%mulliken (9)='e2ii' 
  !    char_table%mulliken (10)='e3ii' 
  !
  !    char_table%l=c
  !    char_table%sigma = 14
  !
  !  end subroutine init_d7h
  !
  !***********************************************************************

  subroutine init_d2d(char_table)
    !----------------------------------------------------------------
    ! init Char_table and related values
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------

    implicit none
    type(type_char_table),intent(inout) :: char_table
    integer i,j
    real(double) c(8,5)

    allocate(char_table%mulliken(5),char_table%symel(8),  &
         char_table%l(8,5))

    data ((c(i,j),i=1,8),j=1,5)   &
         /1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0,  &
         1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,-1.d0,  &
         1.d0,-1.d0,-1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,  &
         1.d0,-1.d0,-1.d0, 1.d0,-1.d0,-1.d0, 1.d0, 1.d0,  &
         2.d0, 0.d0, 0.d0,-2.d0, 0.d0, 0.d0, 0.d0, 0.d0/

    char_table%nsymel=8
    char_table%nirrep=5

    char_table%symel(1)='cz1_1' 
    char_table%symel(2)='sz4_1' 
    char_table%symel(3)='sz4_3' 
    char_table%symel(4)='cz2_1' 
    char_table%symel(5)='cx2_0' 
    char_table%symel(6)='cx2_1' 
    char_table%symel(7)='sd4_0'
    char_table%symel(8)='sd4_1'

    char_table%mulliken (1)='a1' 
    char_table%mulliken (2)='a2' 
    char_table%mulliken (3)='b1' 
    char_table%mulliken (4)='b2' 
    char_table%mulliken (5)='e' 

    char_table%l=c
    char_table%sigma = 4

  end subroutine init_d2d

  !***********************************************************************

  subroutine init_d3d(char_table)
    !----------------------------------------------------------------
    ! init Char_table and related values
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------

    implicit none
    type(type_char_table),intent(inout) :: char_table
    integer i,j
    real(double) c(12,6)

    allocate(char_table%mulliken(6),char_table%symel(12),  &
         char_table%l(12,6))

    data ((c(i,j),i=1,12),j=1,6)  &
         /1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0,  &
         1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,  &
         2.d0,-1.d0,-1.d0, 0.d0, 0.d0, 0.d0, 2.d0,-1.d0,-1.d0, 0.d0, 0.d0, 0.d0,  &
         1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,  &
         1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0, 1.d0, 1.d0, 1.d0,  &
         2.d0,-1.d0,-1.d0, 0.d0, 0.d0, 0.d0,-2.d0, 1.d0, 1.d0, 0.d0, 0.d0, 0.d0/

    char_table%nsymel=12
    char_table%nirrep=6

    char_table%symel(1)='cz1_1' 
    char_table%symel(2)='cz3_1' 
    char_table%symel(3)='cz3_2' 
    char_table%symel(4)='cx3_0' 
    char_table%symel(5)='cx3_1' 
    char_table%symel(6)='cx3_2' 
    char_table%symel(7)='sz2_1'
    char_table%symel(8)='sz6_1'
    char_table%symel(9)='sz6_5'
    char_table%symel(10)='sd6_0'
    char_table%symel(11)='sd6_1'
    char_table%symel(12)='sd6_2'

    char_table%mulliken (1)='a1g' 
    char_table%mulliken (2)='a2g' 
    char_table%mulliken (3)='eg' 
    char_table%mulliken (4)='a1u' 
    char_table%mulliken (5)='a2u' 
    char_table%mulliken (6)='eu' 

    char_table%l=c
    char_table%sigma = 6

  end subroutine init_d3d

  !***********************************************************************

  subroutine init_d4d(char_table)
    !----------------------------------------------------------------
    ! init Char_table and related values
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------

    implicit none
    type(type_char_table),intent(inout) :: char_table
    integer i,j
    real(double) c(16,7)

    allocate(char_table%mulliken(7),char_table%symel(16),  &
         char_table%l(16,7))

    data ((c(i,j),i=1,16),j=1,7)   &
         /1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0,  &
         1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,  &
         1.d0,-1.d0,-1.d0, 1.d0, 1.d0,-1.d0,-1.d0, 1.d0,-1.d0,-1.d0,-1.d0,-1.d0, 1.d0, 1.d0, 1.d0, 1.d0,  &
         1.d0,-1.d0,-1.d0, 1.d0, 1.d0,-1.d0,-1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,-1.d0,  &
         2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0,-2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0,  &
         2.d0, 0.d0, 0.d0,-2.d0,-2.d0, 0.d0, 0.d0, 2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0,  &
         2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0,-2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0/

    c(2,5)=sqrt(2.0)
    c(3,5)=c(2,5)
    c(6,5)=-c(2,5)
    c(7,5)=-c(2,5)
    c(2,7)=-c(2,5)
    c(3,7)=-c(2,5)
    c(6,7)=c(2,5)
    c(7,7)=c(2,5)

    char_table%nsymel=16
    char_table%nirrep=7

    char_table%symel(1)='cz1_1' 
    char_table%symel(2)='sz8_1' 
    char_table%symel(3)='sz8_7' 
    char_table%symel(4)='cz4_1' 
    char_table%symel(5)='cz4_3' 
    char_table%symel(6)='sz8_3' 
    char_table%symel(7)='sz8_5'
    char_table%symel(8)='cz2_1'
    char_table%symel(9)='cx4_0'
    char_table%symel(10)='cx4_1'
    char_table%symel(11)='cx4_2'
    char_table%symel(12)='cx4_3'
    char_table%symel(13)='sd8_0'
    char_table%symel(14)='sd8_1'
    char_table%symel(15)='sd8_2'
    char_table%symel(16)='sd8_3'

    char_table%mulliken (1)='a1' 
    char_table%mulliken (2)='a2' 
    char_table%mulliken (3)='b1' 
    char_table%mulliken (4)='b2' 
    char_table%mulliken (5)='e1' 
    char_table%mulliken (6)='e2' 
    char_table%mulliken (7)='e3' 

    char_table%l=c
    char_table%sigma = 8

  end subroutine init_d4d

  !***********************************************************************

  subroutine init_d5d(char_table)
    !----------------------------------------------------------------
    ! init Char_table and related values
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------

    implicit none
    type(type_char_table),intent(inout) :: char_table
    integer i,j
    real(double) c(20,8),pi
    parameter (pi = 3.1415926535897932384626433832795d0)

    allocate(char_table%mulliken(8),char_table%symel(20),  &
         char_table%l(20,8))

    data ((c(i,j),i=1,20),j=1,8)  &
         /1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, &
         1.d0, 1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,  &
         2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0,  &
         2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0,  &
         1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,  &
         1.d0, 1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0,  &
         2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0,-2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0,  &
         2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0,-2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0/  

    c(2,3)=2*cos(2*pi/5)
    c(3,3)=c(2,3)
    c(4,3)=2*cos(4*pi/5)
    c(5,3)=c(4,3)
    c(12,3)=c(2,3)
    c(13,3)=c(2,3)
    c(14,3)=c(4,3)
    c(15,3)=c(4,3)

    c(2,4)=c(4,3)
    c(3,4)=c(4,3)
    c(4,4)=c(2,3)
    c(5,4)=c(2,3)
    c(12,4)=c(4,3)
    c(13,4)=c(4,3)
    c(14,4)=c(2,3)
    c(15,4)=c(2,3)

    c(2,7)=c(2,3)
    c(3,7)=c(2,3)
    c(4,7)=c(4,3)
    c(5,7)=c(4,3)
    c(12,7)=-c(2,3)
    c(13,7)=-c(2,3)
    c(14,7)=-c(4,3)
    c(15,7)=-c(4,3)

    c(2,8)=c(4,3)
    c(3,8)=c(4,3)
    c(4,8)=c(2,3)
    c(5,8)=c(2,3)
    c(12,8)=-c(4,3)
    c(13,8)=-c(4,3)
    c(14,8)=-c(2,3)
    c(15,8)=-c(2,3)

    char_table%nsymel=20
    char_table%nirrep=8

    char_table%symel(1)='cz1_1' 
    char_table%symel(2)='cz5_1' 
    char_table%symel(3)='cz5_4' 
    char_table%symel(4)='cz5_2' 
    char_table%symel(5)='cz5_3' 
    char_table%symel(6)='cx5_0'
    char_table%symel(7)='cx5_1'
    char_table%symel(8)='cx5_2'
    char_table%symel(9)='cx5_3'
    char_table%symel(10)='cx5_4'
    char_table%symel(11)='sz2_1' 
    char_table%symel(12)='sz10_3'
    char_table%symel(13)='sz10_7'
    char_table%symel(14)='sz10_1'
    char_table%symel(15)='sz10_9'
    char_table%symel(16)='sd10_0'
    char_table%symel(17)='sd10_1'
    char_table%symel(18)='sd10_2'
    char_table%symel(19)='sd10_3'
    char_table%symel(20)='sd10_4'

    char_table%mulliken (1)='a1g' 
    char_table%mulliken (2)='a2g' 
    char_table%mulliken (3)='e1g' 
    char_table%mulliken (4)='e2g' 
    char_table%mulliken (5)='a1u' 
    char_table%mulliken (6)='a2u' 
    char_table%mulliken (7)='e1u' 
    char_table%mulliken (8)='e2u' 

    char_table%l=c
    char_table%sigma = 10

  end subroutine init_d5d

  !***********************************************************************

  subroutine init_d6d(char_table)
    !----------------------------------------------------------------
    ! init Char_table and related values
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------

    implicit none
    type(type_char_table),intent(inout) :: char_table
    integer i,j
    real(double) c(24,9)

    allocate(char_table%mulliken(9),char_table%symel(24),  &
         char_table%l(24,9))

    data ((c(i,j),i=1,24),j=1,9)         &
         /1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, &
         1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, &
         1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, &
         1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,-1.d0, &
         -1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0, 1.d0,-1.d0, &
         -1.d0, 1.d0, 1.d0,-1.d0,-1.d0, 1.d0, 1.d0,-1.d0,-1.d0, 1.d0, &
         1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,-1.d0, &
         -1.d0,-1.d0, 1.d0,-1.d0,-1.d0, 1.d0, 1.d0,-1.d0,-1.d0, 1.d0, &
         1.d0,-1.d0,-1.d0, 1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0, &
         1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 2.d0, 0.d0, 0.d0, 1.d0, &
         1.d0, 0.d0, 0.d0,-1.d0,-1.d0, 0.d0, 0.d0,-2.d0, 0.d0, 0.d0, &
         0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, &
         2.d0, 1.d0, 1.d0,-1.d0,-1.d0,-2.d0,-2.d0,-1.d0,-1.d0, 1.d0, &
         1.d0, 2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, &
         0.d0, 0.d0, 0.d0, 0.d0, 2.d0, 0.d0, 0.d0,-2.d0,-2.d0, 0.d0, &
         0.d0, 2.d0, 2.d0, 0.d0, 0.d0,-2.d0, 0.d0, 0.d0, 0.d0, 0.d0, &
         0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 2.d0,-1.d0, &
         -1.d0,-1.d0,-1.d0, 2.d0, 2.d0,-1.d0,-1.d0,-1.d0,-1.d0, 2.d0, &
         0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, &
         0.d0, 0.d0, 2.d0, 0.d0, 0.d0, 1.d0, 1.d0, 0.d0, 0.d0,-1.d0, &
         -1.d0, 0.d0, 0.d0,-2.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, &
         0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0/

    c(2,5)=sqrt(3.0)
    c(3,5)=c(2,5)
    c(10,5)=-c(2,5)
    c(11,5)=-c(2,5)
    c(2,9)=-c(2,5)
    c(3,9)=-c(2,5)
    c(10,9)=c(2,5)
    c(11,9)=c(2,5)

    char_table%nsymel=24
    char_table%nirrep=9

    char_table%symel(1)='cz1_1' 
    char_table%symel(2)='sz12_1' 
    char_table%symel(3)='sz12_11'
    char_table%symel(4)='cz6_1' 
    char_table%symel(5)='cz6_5' 
    char_table%symel(6)='sz4_1' 
    char_table%symel(7)='sz4_3'
    char_table%symel(8)='cz3_1' 
    char_table%symel(9)='cz3_2' 
    char_table%symel(10)='sz12_5' 
    char_table%symel(11)='sz12_7'
    char_table%symel(12)='cz2_1'
    char_table%symel(13)='cx6_0'
    char_table%symel(14)='cx6_1'
    char_table%symel(15)='cx6_2'
    char_table%symel(16)='cx6_3'
    char_table%symel(17)='cx6_4'
    char_table%symel(18)='cx6_5'
    char_table%symel(19)='sd12_0'
    char_table%symel(20)='sd12_1'
    char_table%symel(21)='sd12_2'
    char_table%symel(22)='sd12_3'
    char_table%symel(23)='sd12_4'
    char_table%symel(24)='sd12_5'

    char_table%mulliken (1)='a1' 
    char_table%mulliken (2)='a2' 
    char_table%mulliken (3)='b1' 
    char_table%mulliken (4)='b2' 
    char_table%mulliken (5)='e1' 
    char_table%mulliken (6)='e2' 
    char_table%mulliken (7)='e3' 
    char_table%mulliken (8)='e4' 
    char_table%mulliken (9)='e5' 

    char_table%l=c
    char_table%sigma = 12

  end subroutine init_d6d

  !***********************************************************************

  subroutine init_s4(char_table)
    !----------------------------------------------------------------
    ! init Char_table and related values
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------

    implicit none
    type(type_char_table),intent(inout) :: char_table
    integer i,j
    real(double) c(4,3)

    allocate(char_table%mulliken(3),char_table%symel(4),  &
         char_table%l(4,3))

    data ((c(i,j),i=1,4),j=1,3)  &
         /1.d0, 1.d0, 1.d0, 1.d0,  &
         1.d0,-1.d0, 1.d0,-1.d0,  &
         2.d0, 0.d0,-2.d0, 0.d0/

    char_table%nsymel=4
    char_table%nirrep=3

    char_table%symel(1)='cz1_1'
    char_table%symel(2)='sz4_1'
    char_table%symel(3)='cz2_1'
    char_table%symel(4)='sz4_3'

    char_table%mulliken(1)='a'
    char_table%mulliken(2)='b'
    char_table%mulliken(3)='e'

    char_table%l=c
    char_table%sigma = 2

  end subroutine init_s4

  !***********************************************************************

  subroutine init_s6(char_table)
    !----------------------------------------------------------------
    ! init Char_table and related values
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------
    implicit none
    type(type_char_table),intent(inout) :: char_table
    integer i,j
    real(double) c(6,4)

    allocate(char_table%mulliken(4),char_table%symel(6),  &
         char_table%l(6,4))

    data ((c(i,j),i=1,6),j=1,4) &
         /1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0,  &
         2.d0,-1.d0,-1.d0, 2.d0,-1.d0,-1.d0,  &
         1.d0, 1.d0, 1.d0,-1.d0,-1.d0,-1.d0,  &
         2.d0,-1.d0,-1.d0,-2.d0, 1.d0, 1.d0/

    char_table%nsymel=6
    char_table%nirrep=4

    char_table%symel(1)='cz1_1'
    char_table%symel(2)='cz3_1'
    char_table%symel(3)='cz3_2'
    char_table%symel(4)='sz2_1'
    char_table%symel(5)='sz6_5'
    char_table%symel(6)='sz6_1'

    char_table%mulliken(1)='ag'
    char_table%mulliken(2)='eg'
    char_table%mulliken(3)='au'
    char_table%mulliken(4)='eu'

    char_table%l=c
    char_table%sigma = 3

  end subroutine init_s6

  !***********************************************************************

  subroutine init_s8(char_table)
    !----------------------------------------------------------------
    ! init Char_table and related values
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------
    implicit none
    type(type_char_table),intent(inout) :: char_table
    integer i,j
    real(double) c(8,5),pi
    parameter (pi = 3.1415926535897932384626433832795d0)

    allocate(char_table%mulliken(5),char_table%symel(8),  &
         char_table%l(8,5))

    data ((c(i,j),i=1,8),j=1,5)     &
         /1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0,  &
         1.d0,-1.d0, 1.d0,-1.d0, 1.d0,-1.d0, 1.d0,-1.d0,  &
         2.d0, 0.d0, 0.d0, 0.d0,-2.d0, 0.d0, 0.d0, 0.d0,  &
         2.d0, 0.d0,-2.d0, 0.d0, 2.d0, 0.d0,-2.d0, 0.d0,  &
         2.d0, 0.d0, 0.d0, 0.d0,-2.d0, 0.d0, 0.d0, 0.d0/

    c(2,3)=2*cos(2*pi/8)
    c(4,3)=-c(2,3)
    c(6,3)=-c(2,3)
    c(8,3)=c(2,3)
    c(2,5)=-c(2,3)
    c(4,5)=c(2,3)
    c(6,5)=c(2,3)
    c(8,5)=-c(2,3)

    char_table%nsymel=8
    char_table%nirrep=5

    char_table%symel(1)='cz1_1'
    char_table%symel(2)='sz8_1'
    char_table%symel(3)='cz4_1'
    char_table%symel(4)='sz8_3'
    char_table%symel(5)='cz2_1'
    char_table%symel(6)='sz8_5'
    char_table%symel(7)='cz4_3'
    char_table%symel(8)='sz8_7'

    char_table%mulliken(1)='a'
    char_table%mulliken(2)='b'
    char_table%mulliken(3)='e1'
    char_table%mulliken(4)='e2'
    char_table%mulliken(5)='e3'

    char_table%l=c
    char_table%sigma = 4

  end subroutine init_s8

  !***********************************************************************

  subroutine init_cs(char_table)
    !----------------------------------------------------------------
    ! init Char_table and related values
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------
    implicit none
    type(type_char_table),intent(inout) :: char_table
    integer i,j
    real(double) c(2,2)
    allocate(char_table%mulliken(2),char_table%symel(2), &
         char_table%l(2,2))

    data ((c(i,j),i=1,2),j=1,2) &
         /1.d0, 1.d0, &
         1.d0,-1.d0/ 

    char_table%nsymel=2 
    char_table%nirrep=2

    char_table%symel(1)='cz1_1' 
    char_table%symel(2)='sh' 

    char_table%mulliken (1)='a1' 
    char_table%mulliken (2)='a2' 

    char_table%l=c
    char_table%sigma = 1

  end subroutine init_cs

  !***********************************************************************

  subroutine init_ci(char_table)
    !----------------------------------------------------------------
    ! init Char_table and related values
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------
    implicit none
    type(type_char_table),intent(inout) :: char_table
    integer i,j
    real(double) c(2,2)

    allocate(char_table%mulliken(2),char_table%symel(2), &
         char_table%l(2,2))

    data ((c(i,j),i=1,2),j=1,2) &
         /1.d0, 1.d0, &
         1.d0,-1.d0/ 

    char_table%nsymel=2 
    char_table%nirrep=2

    char_table%symel(1)='cz1_1' 
    char_table%symel(2)='sz2_1'

    char_table%mulliken (1)='ag' 
    char_table%mulliken (2)='au' 

    char_table%l=c
    char_table%sigma = 1

  end subroutine init_ci

  !***********************************************************************

  subroutine init_o(char_table)
    !-------------------------------------------------------------------
    ! init Char_table and related values
    !
    ! written by J. Neugebauer, Universitaet Erlangen-Nuernberg, 05/2001
    !-------------------------------------------------------------------
    implicit none
    type(type_char_table),intent(inout) :: char_table
    integer i,j
    real(double) c(24,5)

    allocate(char_table%mulliken(5),char_table%symel(24), &
         char_table%l(24,5))

    data (c(1,j),j=1,5) / 1.d0, 1.d0, 2.d0, 3.d0, 3.d0 /
    data (c(2,j),j=1,5) / 1.d0, 1.d0, 2.d0,-1.d0,-1.d0 /
    data (c(5,j),j=1,5) / 1.d0, 1.d0,-1.d0, 0.d0, 0.d0 /
    data (c(13,j),j=1,5) / 1.d0,-1.d0, 0.d0, 1.d0,-1.d0 /
    data (c(19,j),j=1,5) / 1.d0,-1.d0, 0.d0,-1.d0, 1.d0 /

    c(3,:)=c(2,:)
    c(4,:)=c(2,:)
    do i=6,12
       c(i,:)=c(5,:)
    end do
    do i=14,18
       c(i,:)=c(13,:)
    end do
    do i=20,24
       c(i,:)=c(19,:)
    end do

    char_table%nsymel=24 
    char_table%nirrep=5

    char_table%symel(1)='cz1_1' 
    char_table%symel(2)='cz2_1'
    char_table%symel(3)='cx2_0'
    char_table%symel(4)='cx2_1'
    char_table%symel(5)='o1301'
    char_table%symel(6)='o1302'
    char_table%symel(7)='o1303'
    char_table%symel(8)='o1304'
    char_table%symel(9)='o2301'
    char_table%symel(10)='o2302'
    char_table%symel(11)='o2303'
    char_table%symel(12)='o2304'
    char_table%symel(13)='o1401'
    char_table%symel(14)='o1402'
    char_table%symel(15)='o1403'
    char_table%symel(16)='o3401'
    char_table%symel(17)='o3402'
    char_table%symel(18)='o3403'
    char_table%symel(19)='o1201'
    char_table%symel(20)='o1202'
    char_table%symel(21)='o1203'
    char_table%symel(22)='o1204'
    char_table%symel(23)='o1205'
    char_table%symel(24)='o1206'

    char_table%mulliken (1)='a1' 
    char_table%mulliken (2)='a2' 
    char_table%mulliken (3)='e' 
    char_table%mulliken (4)='t1' 
    char_table%mulliken (5)='t2' 

    char_table%l=c
    char_table%sigma = 24

  end subroutine init_o

  !***********************************************************************


  subroutine init_oh(char_table)
    !-------------------------------------------------------------------
    ! init Char_table and related values
    !
    ! written by J. Neugebauer, Universitaet Erlangen-Nuernberg, 05/2001
    !-------------------------------------------------------------------
    implicit none
    type(type_char_table),intent(inout) :: char_table
    integer i,j
    real(double) c(48,10)

    allocate(char_table%mulliken(10),char_table%symel(48), &
         char_table%l(48,10))

    data (c(1,j),j=1,5) / 1.d0, 1.d0, 2.d0, 3.d0, 3.d0 /
    data (c(2,j),j=1,5) / 1.d0, 1.d0, 2.d0,-1.d0,-1.d0 /
    data (c(5,j),j=1,5) / 1.d0, 1.d0,-1.d0, 0.d0, 0.d0 /
    data (c(13,j),j=1,5) / 1.d0,-1.d0, 0.d0, 1.d0,-1.d0 /
    data (c(19,j),j=1,5) / 1.d0,-1.d0, 0.d0,-1.d0, 1.d0 /

    c(3,:)=c(2,:)
    c(4,:)=c(2,:)
    do i=6,12
       c(i,:)=c(5,:)
    end do
    do i=14,18
       c(i,:)=c(13,:)
    end do
    do i=20,24
       c(i,:)=c(19,:)
    end do

    do i=1,24
       do j=1,5
          c(i+24,j)=c(i,j)
          c(i,j+5)=c(i,j)
          c(i+24,j+5)=-c(i,j)
       end do
    end do

    char_table%nsymel=48 
    char_table%nirrep=10

    char_table%symel(1)='cz1_1' 
    char_table%symel(2)='cz2_1'
    char_table%symel(3)='cx2_0'
    char_table%symel(4)='cx2_1'
    char_table%symel(5)='o1301'
    char_table%symel(6)='o1302'
    char_table%symel(7)='o1303'
    char_table%symel(8)='o1304'
    char_table%symel(9)='o2301'
    char_table%symel(10)='o2302'
    char_table%symel(11)='o2303'
    char_table%symel(12)='o2304'
    char_table%symel(13)='o1401'
    char_table%symel(14)='o1402'
    char_table%symel(15)='o1403'
    char_table%symel(16)='o3401'
    char_table%symel(17)='o3402'
    char_table%symel(18)='o3403'
    char_table%symel(19)='o1201'
    char_table%symel(20)='o1202'
    char_table%symel(21)='o1203'
    char_table%symel(22)='o1204'
    char_table%symel(23)='o1205'
    char_table%symel(24)='o1206'
    char_table%symel(25)='sz2_1' 
    char_table%symel(26)='sh'
    char_table%symel(27)='sv2_0'
    char_table%symel(28)='sd2_0'
    char_table%symel(29)='p1601'
    char_table%symel(30)='p1602'
    char_table%symel(31)='p1603'
    char_table%symel(32)='p1604'
    char_table%symel(33)='p5601'
    char_table%symel(34)='p5602'
    char_table%symel(35)='p5603'
    char_table%symel(36)='p5604'
    char_table%symel(37)='p1401'
    char_table%symel(38)='p1402'
    char_table%symel(39)='p1403'
    char_table%symel(40)='p3401'
    char_table%symel(41)='p3402'
    char_table%symel(42)='p3403'
    char_table%symel(43)='p1101'
    char_table%symel(44)='p1102'
    char_table%symel(45)='p1103'
    char_table%symel(46)='p1104'
    char_table%symel(47)='p1105'
    char_table%symel(48)='p1106'

    char_table%mulliken (1)='a1g' 
    char_table%mulliken (2)='a2g' 
    char_table%mulliken (3)='eg' 
    char_table%mulliken (4)='t1g' 
    char_table%mulliken (5)='t2g' 
    char_table%mulliken (6)='a1u' 
    char_table%mulliken (7)='a2u' 
    char_table%mulliken (8)='eu' 
    char_table%mulliken (9)='t1u' 
    char_table%mulliken (10)='t2u' 

    char_table%l=c
    char_table%sigma = 24

  end subroutine init_oh

  !***********************************************************************

  subroutine init_td(char_table)
    !-----------------------------------------------------------------
    ! init Char_table and related values
    !
    ! written by J. Neugebauer, University Erlangen-Nuernberg, 05/2001
    !-----------------------------------------------------------------
    implicit none
    type(type_char_table),intent(inout) :: char_table
    integer i,j
    real(double) c(24,5)

    allocate(char_table%mulliken(5),char_table%symel(24), &
         char_table%l(24,5))

    data (c(1,j),j=1,5) / 1.d0, 1.d0, 2.d0, 3.d0, 3.d0 /
    data (c(2,j),j=1,5) / 1.d0, 1.d0, 2.d0,-1.d0,-1.d0 /
    data (c(5,j),j=1,5) / 1.d0, 1.d0,-1.d0, 0.d0, 0.d0 /
    data (c(13,j),j=1,5) / 1.d0,-1.d0, 0.d0, 1.d0,-1.d0 /
    data (c(19,j),j=1,5) / 1.d0,-1.d0, 0.d0,-1.d0, 1.d0 /

    c(3,:)=c(2,:)
    c(4,:)=c(2,:)
    do i=6,12
       c(i,:)=c(5,:)
    end do
    do i=14,18
       c(i,:)=c(13,:)
    end do
    do i=20,24
       c(i,:)=c(19,:)
    end do

    char_table%nsymel=24
    char_table%nirrep=5

    char_table%symel(1)='cz1_1' 
    char_table%symel(2)='cz2_1'
    char_table%symel(3)='cx2_0'
    char_table%symel(4)='cx2_1'
    char_table%symel(5)='o1301'
    char_table%symel(6)='o1302'
    char_table%symel(7)='o1303'
    char_table%symel(8)='o1304'
    char_table%symel(9)='o2301'
    char_table%symel(10)='o2302'
    char_table%symel(11)='o2303'
    char_table%symel(12)='o2304'
    char_table%symel(13)='p1401'
    char_table%symel(14)='p1402'
    char_table%symel(15)='p1403'
    char_table%symel(16)='p3401'
    char_table%symel(17)='p3402'
    char_table%symel(18)='p3403'
    char_table%symel(19)='p1101'
    char_table%symel(20)='p1102'
    char_table%symel(21)='p1103'
    char_table%symel(22)='p1104'
    char_table%symel(23)='p1105'
    char_table%symel(24)='p1106'

    char_table%mulliken (1)='a1' 
    char_table%mulliken (2)='a2' 
    char_table%mulliken (3)='e' 
    char_table%mulliken (4)='t1' 
    char_table%mulliken (5)='t2' 

    char_table%l=c
    char_table%sigma = 12

  end subroutine init_td

  !***********************************************************************

  subroutine init_th(char_table)
    !-----------------------------------------------------------------
    ! init Char_table and related values
    !
    ! written by J. Neugebauer, University Erlangen-Nuernberg, 05/2001
    !-----------------------------------------------------------------
    implicit none
    type(type_char_table),intent(inout) :: char_table
    integer i,j
    real(double) c(24,6)

    allocate(char_table%mulliken(6),char_table%symel(24), &
         char_table%l(24,6))

    data (c(1,j),j=1,6) / 1.d0, 2.d0, 3.d0, 1.d0, 2.d0, 3.d0 /
    data (c(2,j),j=1,6) / 1.d0, 2.d0,-1.d0, 1.d0, 2.d0,-1.d0 /
    data (c(5,j),j=1,6) / 1.d0,-1.d0, 0.d0, 1.d0,-1.d0, 0.d0 /
    data (c(9,j),j=1,6) / 1.d0,-1.d0, 0.d0, 1.d0,-1.d0, 0.d0 /

    c(3,:)=c(2,:)
    c(4,:)=c(2,:)
    do i=6,8
       c(i,:)=c(5,:)
    end do
    do i=10,12
       c(i,:)=c(9,:)
    end do
    do i=1,12
       do j=1,3
          c(i+12,j)=c(i,j)
          c(i+12,j+3)=-c(i,j)
       end do
    end do

    char_table%nsymel=24
    char_table%nirrep=6

    char_table%symel(1)='cz1_1' 
    char_table%symel(2)='cz2_1'
    char_table%symel(3)='cx2_0'
    char_table%symel(4)='cx2_1'
    char_table%symel(5)='o1301'
    char_table%symel(6)='o1302'
    char_table%symel(7)='o1303'
    char_table%symel(8)='o1304'
    char_table%symel(9)='o2301'
    char_table%symel(10)='o2302'
    char_table%symel(11)='o2303'
    char_table%symel(12)='o2304'
    char_table%symel(13)='sz2_1'
    char_table%symel(14)='sh'
    char_table%symel(15)='sv2_0'
    char_table%symel(16)='sd2_0'
    char_table%symel(17)='p1601'
    char_table%symel(18)='p1602'
    char_table%symel(19)='p1603'
    char_table%symel(20)='p1604'
    char_table%symel(21)='p3601'
    char_table%symel(22)='p3602'
    char_table%symel(23)='p3603'
    char_table%symel(24)='p3604'

    char_table%mulliken (1)='ag' 
    char_table%mulliken (2)='eg' 
    char_table%mulliken (3)='tg'
    char_table%mulliken (4)='au' 
    char_table%mulliken (5)='eu' 
    char_table%mulliken (6)='tu' 

    char_table%l=c
    char_table%sigma = 12

  end subroutine init_th

  !***********************************************************************

  subroutine init_t(char_table)
    !-----------------------------------------------------------------
    ! init Char_table and related values
    !
    ! written by J. Neugebauer, University Erlangen-Nuernberg, 05/2001
    !-----------------------------------------------------------------
    implicit none
    type(type_char_table),intent(inout) :: char_table
    integer i,j
    real(double) c(12,3)

    allocate(char_table%mulliken(3),char_table%symel(12), &
         char_table%l(12,3))

    data (c(1,j),j=1,3) / 1.d0, 2.d0, 3.d0 /
    data (c(2,j),j=1,3) / 1.d0, 2.d0,-1.d0 /
    data (c(5,j),j=1,3) / 1.d0,-1.d0, 0.d0 /
    data (c(9,j),j=1,3) / 1.d0,-1.d0, 0.d0 /

    c(3,:)=c(2,:)
    c(4,:)=c(2,:)
    do i=6,8
       c(i,:)=c(5,:)
    end do
    do i=10,12
       c(i,:)=c(9,:)
    end do

    char_table%nsymel=12
    char_table%nirrep=3

    char_table%symel(1)='cz1_1'
    char_table%symel(2)='cz2_1'
    char_table%symel(3)='cx2_0'
    char_table%symel(4)='cx2_1'
    char_table%symel(5)='o1301'
    char_table%symel(6)='o2302'
    char_table%symel(7)='o1303'
    char_table%symel(8)='o2304'
    char_table%symel(9)='o2301'
    char_table%symel(10)='o1302'
    char_table%symel(11)='o2303'
    char_table%symel(12)='o1304'

    char_table%mulliken (1)='a'
    char_table%mulliken (2)='e'
    char_table%mulliken (3)='t'

    char_table%l=c
    char_table%sigma = 12

  end subroutine init_t

  !***********************************************************************

  subroutine init_i(char_table)

    !-----------------------------------------------------------------
    ! init Char_table and related values
    !
    ! iabcd: Cb^a operation around axis No. (c*10+d)
    !
    ! written by J. Neugebauer, University Erlangen-Nuernberg, 05/2001
    !-----------------------------------------------------------------
    implicit none
    type(type_char_table),intent(inout) :: char_table
    integer i,j
    real(double) c(60,5),pi,c5(2)

    parameter (pi = 3.1415926535897932384626433832795d0)
    allocate(char_table%mulliken(5),char_table%symel(60), &
         char_table%l(60,5))

    c5(1)=2.d0*cos(pi/5.d0)
    c5(2)=2.d0*cos(pi/5.d0*3.d0)

    data (c(1,j),j=1,5)  /1.d0, 3.d0, 3.d0, 4.d0, 5.d0/
    c(2,1) = 1.d0
    c(2,2) = c5(1)
    c(2,3) = c5(2)
    data (c(2,j),j=4,5)  /-1.d0,0.d0/
    c(14,1)=1.d0
    c(14,2)=c5(2)
    c(14,3)=c5(1)
    data (c(14,j),j=4,5) /-1.d0,0.d0/
    data (c(26,j),j=1,5) /1.d0,0.d0,0.d0,1.d0,-1.d0/
    data (c(46,j),j=1,5) /1.d0,-1.d0,-1.d0,0.d0,1.d0/

    !------------------------------------------------------------------
    ! copy characters for symmetry elements belonging to the same class
    !------------------------------------------------------------------

    do i=3,13
       c(i,:)=c(2,:)
       c(i+12,:)=c(14,:)
    end do
    do i=27,45
       c(i,:)=c(26,:)
    end do
    do i=47,60
       c(i,:)=c(46,:)
    end do

    char_table%nsymel=60
    char_table%nirrep=5

    char_table%symel(1)='cz1_1' 
    char_table%symel(2)='i1501'
    char_table%symel(3)='i1502'
    char_table%symel(4)='i1503'
    char_table%symel(5)='i1504'
    char_table%symel(6)='i1505'
    char_table%symel(7)='i1506'
    char_table%symel(8)='i4501'
    char_table%symel(9)='i4502'
    char_table%symel(10)='i4503'
    char_table%symel(11)='i4504'
    char_table%symel(12)='i4505'
    char_table%symel(13)='i4506'
    char_table%symel(14)='i2501'
    char_table%symel(15)='i2502'
    char_table%symel(16)='i2503'
    char_table%symel(17)='i2504'
    char_table%symel(18)='i2505'
    char_table%symel(19)='i2506'
    char_table%symel(20)='i3501'
    char_table%symel(21)='i3502'
    char_table%symel(22)='i3503'
    char_table%symel(23)='i3504'
    char_table%symel(24)='i3505'
    char_table%symel(25)='i3506'
    char_table%symel(26)='i1301'
    char_table%symel(27)='i1302'
    char_table%symel(28)='i1303'
    char_table%symel(29)='i1304'
    char_table%symel(30)='i1305'
    char_table%symel(31)='i1306'
    char_table%symel(32)='i1307'
    char_table%symel(33)='i1308'
    char_table%symel(34)='i1309'
    char_table%symel(35)='i1310'
    char_table%symel(36)='i2301'
    char_table%symel(37)='i2302'
    char_table%symel(38)='i2303'
    char_table%symel(39)='i2304'
    char_table%symel(40)='i2305'
    char_table%symel(41)='i2306'
    char_table%symel(42)='i2307'
    char_table%symel(43)='i2308'
    char_table%symel(44)='i2309'
    char_table%symel(45)='i2310'
    char_table%symel(46)='i1201'
    char_table%symel(47)='i1202'
    char_table%symel(48)='i1203'
    char_table%symel(49)='i1204'
    char_table%symel(50)='i1205'
    char_table%symel(51)='i1206'
    char_table%symel(52)='i1207'
    char_table%symel(53)='i1208'
    char_table%symel(54)='i1209'
    char_table%symel(55)='i1210'
    char_table%symel(56)='i1211'
    char_table%symel(57)='i1212'
    char_table%symel(58)='i1213'
    char_table%symel(59)='i1214'
    char_table%symel(60)='i1215'

    char_table%mulliken (1)='a' 
    char_table%mulliken (2)='t1' 
    char_table%mulliken (3)='t2' 
    char_table%mulliken (4)='f' 
    char_table%mulliken (5)='h' 

    char_table%l=c
    char_table%sigma = 15

  end subroutine init_i

  !***********************************************************************

  subroutine init_ih(char_table)

    !-----------------------------------------------------------------
    ! init Char_table and related values
    !
    ! iabcd: Cb^a operation around axis No. (c*10+d)
    ! jabcd: Sb^a operation around axis No. (c*10+d)
    !
    ! written by J. Neugebauer, University Erlangen-Nuernberg, 05/2001
    !-----------------------------------------------------------------

    implicit none
    type(type_char_table),intent(inout) :: char_table
    integer i,j
    real(double) c(120,10),pi,c5(2)
    parameter (pi = 3.1415926535897932384626433832795d0)

    allocate(char_table%mulliken(10),char_table%symel(120), &
         char_table%l(120,10))

    c5(1)=2.d0*cos(pi/5.d0)
    c5(2)=2.d0*cos(pi/5.d0*3.d0)

    data (c(1,j),j=1,5)  /1.d0, 3.d0, 3.d0, 4.d0, 5.d0 /
    c(2,1)=1.d0
    c(2,2)=c5(1)
    c(2,3)=c5(2)
    data (c(2,j),j=4,5)  /-1.d0,0.d0 /
    c(14,1)=1.d0
    c(14,2)=c5(2)
    c(14,3)=c5(1)
    data (c(14,j),j=4,5) /-1.d0,0.d0 /
    data (c(26,j),j=1,5) /1.d0,0.d0,0.d0,1.d0,-1.d0 /
    data (c(46,j),j=1,5) /1.d0,-1.d0,-1.d0,0.d0,1.d0 /

    !------------------------------------------------------------------
    ! copy characters for symmetry elements belonging to the same class
    !------------------------------------------------------------------

    do i=3,13
       c(i,:)=c(2,:)
       c(i+12,:)=c(14,:)
    end do
    do i=27,45
       c(i,:)=c(26,:)
    end do
    do i=47,60
       c(i,:)=c(46,:)
    end do

    !-----------------------------------------------------------------
    ! copy characters for additional symmetry elements and 
    ! analogous representations
    !-----------------------------------------------------------------

    do i=1,60
       do j=6,10
          c(i+60,j-5)=c(i,j-5)
          c(i,j)=c(i,j-5)
          c(i+60,j)=-c(i,j-5)
       end do
    end do

    char_table%nsymel=120
    char_table%nirrep=10

    char_table%symel(1)='cz1_1' 
    char_table%symel(2)='i1501'
    char_table%symel(3)='i1502'
    char_table%symel(4)='i1503'
    char_table%symel(5)='i1504'
    char_table%symel(6)='i1505'
    char_table%symel(7)='i1506'
    char_table%symel(8)='i4501'
    char_table%symel(9)='i4502'
    char_table%symel(10)='i4503'
    char_table%symel(11)='i4504'
    char_table%symel(12)='i4505'
    char_table%symel(13)='i4506'
    char_table%symel(14)='i2501'
    char_table%symel(15)='i2502'
    char_table%symel(16)='i2503'
    char_table%symel(17)='i2504'
    char_table%symel(18)='i2505'
    char_table%symel(19)='i2506'
    char_table%symel(20)='i3501'
    char_table%symel(21)='i3502'
    char_table%symel(22)='i3503'
    char_table%symel(23)='i3504'
    char_table%symel(24)='i3505'
    char_table%symel(25)='i3506'
    char_table%symel(26)='i1301'
    char_table%symel(27)='i1302'
    char_table%symel(28)='i1303'
    char_table%symel(29)='i1304'
    char_table%symel(30)='i1305'
    char_table%symel(31)='i1306'
    char_table%symel(32)='i1307'
    char_table%symel(33)='i1308'
    char_table%symel(34)='i1309'
    char_table%symel(35)='i1310'
    char_table%symel(36)='i2301'
    char_table%symel(37)='i2302'
    char_table%symel(38)='i2303'
    char_table%symel(39)='i2304'
    char_table%symel(40)='i2305'
    char_table%symel(41)='i2306'
    char_table%symel(42)='i2307'
    char_table%symel(43)='i2308'
    char_table%symel(44)='i2309'
    char_table%symel(45)='i2310'
    char_table%symel(46)='i1201'
    char_table%symel(47)='i1202'
    char_table%symel(48)='i1203'
    char_table%symel(49)='i1204'
    char_table%symel(50)='i1205'
    char_table%symel(51)='i1206'
    char_table%symel(52)='i1207'
    char_table%symel(53)='i1208'
    char_table%symel(54)='i1209'
    char_table%symel(55)='i1210'
    char_table%symel(56)='i1211'
    char_table%symel(57)='i1212'
    char_table%symel(58)='i1213'
    char_table%symel(59)='i1214'
    char_table%symel(60)='i1215'
    char_table%symel(61)='sz2_1' 
    char_table%symel(62)='j3001'
    char_table%symel(63)='j3002'
    char_table%symel(64)='j3003'
    char_table%symel(65)='j3004'
    char_table%symel(66)='j3005'
    char_table%symel(67)='j3006'
    char_table%symel(68)='j7001'
    char_table%symel(69)='j7002'
    char_table%symel(70)='j7003'
    char_table%symel(71)='j7004'
    char_table%symel(72)='j7005'
    char_table%symel(73)='j7006'
    char_table%symel(74)='j1001'
    char_table%symel(75)='j1002'
    char_table%symel(76)='j1003'
    char_table%symel(77)='j1004'
    char_table%symel(78)='j1005'
    char_table%symel(79)='j1006'
    char_table%symel(80)='j9001'
    char_table%symel(81)='j9002'
    char_table%symel(82)='j9003'
    char_table%symel(83)='j9004'
    char_table%symel(84)='j9005'
    char_table%symel(85)='j9006'
    char_table%symel(86)='j1601'
    char_table%symel(87)='j1602'
    char_table%symel(88)='j1603'
    char_table%symel(89)='j1604'
    char_table%symel(90)='j1605'
    char_table%symel(91)='j1606'
    char_table%symel(92)='j1607'
    char_table%symel(93)='j1608'
    char_table%symel(94)='j1609'
    char_table%symel(95)='j1610'
    char_table%symel(96)='j5601'
    char_table%symel(97)='j5602'
    char_table%symel(98)='j5603'
    char_table%symel(99)='j5604'
    char_table%symel(100)='j5605'
    char_table%symel(101)='j5606'
    char_table%symel(102)='j5607'
    char_table%symel(103)='j5608'
    char_table%symel(104)='j5609'
    char_table%symel(105)='j5610'
    char_table%symel(106)='j1101'
    char_table%symel(107)='j1102'
    char_table%symel(108)='j1103'
    char_table%symel(109)='j1104'
    char_table%symel(110)='j1105'
    char_table%symel(111)='j1106'
    char_table%symel(112)='j1107'
    char_table%symel(113)='j1108'
    char_table%symel(114)='j1109'
    char_table%symel(115)='j1110'
    char_table%symel(116)='j1111'
    char_table%symel(117)='j1112'
    char_table%symel(118)='j1113'
    char_table%symel(119)='j1114'
    char_table%symel(120)='j1115'

    char_table%mulliken (1)='ag   ' 
    char_table%mulliken (2)='t1g  ' 
    char_table%mulliken (3)='t2g  ' 
    char_table%mulliken (4)='fg   ' 
    char_table%mulliken (5)='hg   ' 
    char_table%mulliken (6)='au   ' 
    char_table%mulliken (7)='t1u  ' 
    char_table%mulliken (8)='t2u  ' 
    char_table%mulliken (9)='fu   ' 
    char_table%mulliken (10)='hu   ' 

    char_table%l=c
    char_table%sigma = 15

  end subroutine init_ih

  !***********************************************************************

  subroutine checkpg(pg,pgstat)
    !-----------------------------------------------------------------------
    ! this subroutine checks, whether the character(len=3) symbol "pg" 
    ! is an allowed point-group symbol.
    ! 
    ! output: 
    !
    ! pgstat =  1 ; point group exists and is implemented in snf
    ! pgstat =  0 ; point group exists, but is not yet implemented in snf
    ! pgstat = -1 ; point group is unknown (and therefore not implemented)
    !
    ! written by J. Neugebauer, Universitaet Erlangen-Nuernberg, 05/2001
    !-----------------------------------------------------------------------
    implicit none
    integer :: i,pgstat,nav
    parameter(nav = 45)
    character (len=3) :: pgav(nav),pg
    data(pgav(i),i=1,nav) / 'ci ','cs ','c2 ','c3 ','c4 ','c5 ','c6 ',&
         'c7 ','c8 ','c2v','c3v','c4v','c5v','c6v','c2h','c3h','c4h','c5h',&
         'c6h','d2 ','d3 ','d4 ','d5 ','d6 ','d2h','d3h','d4h','d5h','d6h',&
         'd2d','d3d','d4d','d5d','d6d','s4 ','s6 ','s8 ','c1 ','i  ','ih ',&
         'o  ','oh ','t  ','td ','th ' /
    pgstat=-1
    do i=1,nav
       if(pg.eq.pgav(i)) then
          pgstat=1
          return
       end if
    end do
  end subroutine checkpg

  !***********************************************************************

end module snf_char_table
