
!********************************************************************************
!*   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
!*   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
!*   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
!*
!*
!*   This file is part of the package SNF.
!*
!*   SNF is free software; you can redistribute it and/or modify
!*   it under the terms of the GNU General Public License as published by
!*   the Free Software Foundation; either version 2 of the License, or
!*   (at your option) any later version.
!*
!*   This program is distributed in the hope that it will be useful,
!*   but WITHOUT ANY WARRANTY; without even the implied warranty of
!*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*   GNU General Public License for more details.
!*
!*   You should have received a copy of the GNU General Public License
!*   along with this program; if not, write to the Free Software
!*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!*
!********************************************************************************
module snf_character
  use f90_kind

  !----------------------------------------------------------------
  ! this module contains all subroutines dealing with characters
  ! of different representations
  !
  ! written by Carsten Kind, Uni Koeln, 1994
  !----------------------------------------------------------------

  private
  public :: create_character, print_character, destroy_character
  public :: create_irreps, print_irreps, destroy_irreps

  !************************************************************************

contains

  !************************************************************************

  subroutine create_character(representation,character)
    !----------------------------------------------------------------
    ! this subroutine creates the characters of representations by
    ! adding the diagonal elements. the first index of "representation"
    ! denotes the symmetry element, the other two indices denote columns
    ! and rows.
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------
    implicit none
    real(double),intent(in) :: representation(:,:,:)
    real(double),pointer:: character(:)
    integer j

    allocate (character(ubound(representation,3)))

    character(:) = 0.0
    do j=1,ubound(representation,2)
       character(:)=character(:)+representation(j,j,:)
    enddo
  end subroutine create_character

  !************************************************************************

  subroutine print_character(character,char_table,unit)
    !----------------------------------------------------------------
    ! this subroutine prints the characters of the reducible
    ! representations
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------
    use snf_char_table,only:type_char_table
    implicit none
    real(double) ,intent(in) :: character(:)
    type(type_char_table),intent(in) :: char_table
    integer,optional :: unit
    integer i,out

    if(present(unit)) then
       out=unit
    else
       out=6
    endif
    write(out,*)
    write(out,'('' Output of characters'')')
    write(out,'('' --------------------'')')
    write(out,'(20(3X,5(a5,a2,F8.4,4X)/))') (char_table%symel(i),' :',character(i),&
         & i=1,ubound(character,1))
  end subroutine print_character

  !************************************************************************

  subroutine destroy_character(character)
    !----------------------------------------------------------------
    ! this subroutine deallocates the vector "character"
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------
    implicit none
    real(double),pointer:: character(:)

    deallocate(character)
  end subroutine destroy_character

  !************************************************************************

  subroutine create_irreps(character,irreps,char_table)
    !----------------------------------------------------------------
    ! this subroutine calculates the number of irreducible 
    ! representations in a reducible representation with given 
    ! character by applying the reduction formula.
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    ! modified by C. Kind and J. Neugebauer, Uni Erlangen, 2001
    !----------------------------------------------------------------
    use snf_char_table,only:type_char_table
    implicit none
    type(type_char_table),intent(in) :: char_table
    real(double) ,intent(in)  :: character(:)
    integer,pointer               :: irreps(:)
    real(double),pointer               :: firreps(:)
    real(double) :: thres
    integer j     
    parameter(thres=0.01d0)

    allocate(irreps(char_table%nirrep),firreps(char_table%nirrep))
    irreps(:) = 0 
    firreps(:) = 0 
    do j=1,char_table%nsymel 
       firreps(:)=firreps(:)+character(j)*char_table%l(j,:) 
    enddo
    do j=1,char_table%nirrep
       if(mod(firreps(j)+thres,dble(char_table%ordnung(j))).gt.2*thres) then
          write(*,*) '**********************************' 
          write(*,*) 'ERROR! Could not perform reduction' 
          write(*,*) 'of reducible representation!'
          write(*,'(a,i2,a,f8.4)') ' g * a(irrep No. ',j,') = ',firreps(j)
          write(*,'(a,f12.4)') ' g =', dble(char_table%ordnung(j))
          write(*,*) '**********************************' 
       end if
       irreps(j)=nint(firreps(j)/char_table%ordnung(j))
    end do
    deallocate(firreps)
  end subroutine create_irreps

  !************************************************************************

  subroutine print_irreps(irreps,char_table,unit)
    !----------------------------------------------------------------
    ! this subroutine prints the number of irreducible 
    ! representations.
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------
    use snf_char_table,only:type_char_table
    implicit none
    integer,intent(in) :: irreps(:)
    type(type_char_table),intent(in) :: char_table
    integer,optional :: unit
    integer i,out

    if(present(unit)) then
       out=unit
    else
       out=6
    endif
    write(out,*)
    write(out,'('' Occurance of the irreducible representations'')')
    write(out,'('' --------------------------------------------'')')
    write(out,'(10(5X,6(a5,a2,i4,6x)/))') (char_table%mulliken(i),' :',irreps(i),&
         &  i=1,ubound(irreps,1))
  end subroutine print_irreps

  !************************************************************************

  subroutine destroy_irreps(irreps)
    !----------------------------------------------------------------
    ! this routine deallocates the vector "irreps"
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------
    implicit none
    integer,pointer :: irreps(:)
    deallocate(irreps)
  end subroutine destroy_irreps

  !************************************************************************

end module snf_character
