
!********************************************************************************
!*   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
!*   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
!*   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
!*
!*
!*   This file is part of the package SNF.
!*
!*   SNF is free software; you can redistribute it and/or modify
!*   it under the terms of the GNU General Public License as published by
!*   the Free Software Foundation; either version 2 of the License, or
!*   (at your option) any later version.
!*
!*   This program is distributed in the hope that it will be useful,
!*   but WITHOUT ANY WARRANTY; without even the implied warranty of
!*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*   GNU General Public License for more details.
!*
!*   You should have received a copy of the GNU General Public License
!*   along with this program; if not, write to the Free Software
!*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!*
!********************************************************************************
module snf_control
  !----------------------------------------------------------------
  ! module to initialze and control the frequency calculation
  ! 
  ! written by Carsten Kind, Uni Erlangen, 2000-2001
  ! modified by J. Neugebauer, Uni Erlangen, 2001
  ! modified by Markus Reiher, Uni Erlangen, Aug. and Nov. 2001
  !----------------------------------------------------------------
  use f90_kind
  use snf_error, only : serror
  use snf_turbomole,only : type_tm_ctrl
  use snf_dalton,       only : type_dal_ctrl
  use snf_molecule,only  : type_atom

  private
  public :: read_control, write_control, copy_tm_to_ctrl,copy_ctrl_to_tm
  public :: type_ctrl,print_ctrl,check_ctrl,init_ctrl,destroy_control
  public :: getdfreq,get_energy,get_energy_mp2,copy_ctrl_to_dal,get_spin

  !************************************************************************

  !----------------------------------------------------------------
  ! Note: There is now a variable "mode" in type "type_ctrl", but
  !       also in the subtype "type_tm_ctrl" for compatibility with
  !       former parts of the program. 
  !       However, there is a common mode number for every
  !       program mode, since SNFDC is not able to differentiate
  !       between type_dal_ctrl%mode (which does not exist)
  !       and type_tm_ctrl%mode.
  !       The modes have the following meanings:
  !
  !       mode  | program       | type of calculation
  !       ------|---------------|----------------------------------
  !         1   | TURBOMOLE     | dscf (HF or DFT) {, escf}
  !       ------|---------------|----------------------------------
  !         2   | TURBOMOLE     | ridft (DFT, BP86)
  !       ------|---------------|----------------------------------
  !         3   | TURBOMOLE     | rimp2  DC, No Raman
  !       ------|---------------|----------------------------------
  !         4   | TURBOMOLE     | mpgrad, No Raman
  !       ------|---------------|----------------------------------
  !         5   | TURBOMOLE     | dscf, using escf.intel working 
  !             |               | version          
  !       ------|---------------|----------------------------------
  !         6   | TURBOMOLE     | dscf, using escf.athlon working 
  !             |               | version           
  !       ------|---------------|----------------------------------
  !         7   | TURBOMOLE     | ridft, using escf.intel working 
  !             |               | version
  !       ------|---------------|----------------------------------
  !         8   | TURBOMOLE     | ridft, using escf.athlon working 
  !             |               | version
  !       ------|---------------|----------------------------------
  !        20   | DALTON        | SCF   
  !       ------|---------------|----------------------------------
  !        21   | DALTON        | MP2  No Raman
  !       ------|---------------|----------------------------------
  !        22   | DALTON        | CC2  
  !       ------|---------------|----------------------------------
  !        23   | DALTON        | CCS 
  !       ------|---------------|----------------------------------
  !        24   | DALTON        | CCSD
  !       ------|---------------|----------------------------------
  !        24   | DALTON        | MCSCF
  !----------------------------------------------------------------

  type type_ctrl
     character(len=132) :: title     ! title of calculation
     type(type_tm_ctrl),pointer :: tm_ctrl
     type(type_dal_ctrl),pointer :: dal_ctrl
     logical :: raman                ! calculate raman intensities
     logical :: sumdeg               ! sum intensities for degenerate modes
     logical :: maxsym               ! use symmetry to determine redundant
                                     ! distortions for non-redundant atoms
     logical :: dynpol               ! calculate raman intensities with dynamic
                                     ! polarizabilities
     integer :: version, &           ! version number  
          &     nvar, &
          &     scfconv, &           ! scfconv parameter
          &     mode,  &             ! program mode
          &     numderiv             ! flag for first-derivative 
                                     ! (1: 3-point formula; 2: 5-point formula)
     real(double) :: cstep           ! derivation of cartesian coordinates
     character(len=3) :: symmetry    ! schoenfliess symbol of point group
     character(len=3) :: isosym      ! schoenfliess symbol of point group 
                                     ! after selection of isotopes
     character(len=80) :: restart    ! filename for restart file
     character(len=80) :: control    ! filename for snf_control file
     real(double) :: minfreq,maxfreq,width,theta   ! theta = scattering angle
     integer :: npoint
     real(double) :: temp,press,elec_energy,spin
     logical :: print_gaussian,&     ! print gaussian fake output
          &     print_mopac, &       ! print mopac fake output
          &     print_mopxmol, &     ! print mopac fake output for use in xmol
          &     print_xmol, &        ! print xmol XYZ input file
          &     print_symmetry, &    ! print detailed symmetry-information
          &     print_irred_at, &    ! print infos about irreducible atoms
          &     prirdist,       &    ! print infos about irreducible distortions
          &     print_cioslowski, &  ! print cioslowski charge decomposition
          &     print_force, &       ! print force constant matrix
          &     print_hessian, &     ! print cartesian hessian
          &     print_dip, &         ! print cartesian dipol derivates
          &     print_pol, &         ! print cartesian polarizability derivates
          &     print_rcs, &         ! print raman cross sections
          &     print_transrot, &    ! print transl. and rot. contributions
          &     print_zpe, &         ! print  zeropoint energy 
          &     print_thermo         ! print thermodynamics
     real(double) :: xmolsc1         ! scale factor for xmol output
     real(double),pointer :: dynfreq(:) ! frequencies for the calculation
                                        ! of dynamic polarizabilities
  end type type_ctrl

  !************************************************************************

contains

  !************************************************************************

  subroutine init_ctrl(ctrl)
    implicit none
    !----------------------------------------------------------------
    ! initialize default ctrl values
    ! 
    ! written by Carsten Kind, Uni Erlangen, 2000-2001
    ! modified by J. Neugebauer, Uni Erlangen, 2001
    ! modified by Markus Reiher, Uni Erlangen, Aug. and Nov. 2001
    !----------------------------------------------------------------
    type(type_ctrl),intent(out) :: ctrl
    ctrl%title= 'snf calculation'
    nullify(ctrl%tm_ctrl)
    nullify(ctrl%dynfreq)
    ctrl%version= 170
    ctrl%raman=.false.
    ctrl%sumdeg=.false.
    ctrl%dynpol=.false.
    ctrl%scfconv = 8
    ctrl%symmetry='c1 '
    ctrl%isosym='def'
    ctrl%numderiv=1
    ctrl%control='snf_control'
    ctrl%restart='restart'
    ctrl%cstep=1.0d-2
    ctrl%mode=1
    ctrl%xmolsc1=2.0d0
    ctrl%theta= 90.0d0
    ctrl%minfreq=200
    ctrl%maxfreq=3500
    ctrl%width=10
    ctrl%npoint=2000
    ctrl%temp=273.15d0+25.0d0
    ctrl%press=101325.0d0
    ctrl%print_gaussian=.true.
    ctrl%print_mopac=.false.
    ctrl%print_mopxmol=.false.
    ctrl%print_xmol=.false.
    ctrl%print_symmetry=.false.
    ctrl%print_irred_at=.false.
    ctrl%prirdist=.false.
    ctrl%print_cioslowski=.true.
    ctrl%print_force=.false.
    ctrl%print_hessian=.false.
    ctrl%print_dip=.false.
    ctrl%print_rcs=.false.
    ctrl%print_pol=.false.
    ctrl%print_transrot=.true.
    ctrl%print_zpe=.true.
    ctrl%print_thermo=.true.
    ctrl%maxsym=.true.
    ctrl%elec_energy=0.0d0
    ctrl%spin=0.0d0
  end subroutine init_ctrl

  !************************************************************************

  subroutine read_control(ctrl,selatom,massatom)
    !----------------------------------------------------------------
    ! read all avaible data from snf_control
    ! 
    ! written by Carsten Kind, Uni Erlangen, 2000-2001
    ! modified by Markus Reiher, Uni Erlangen, Aug. and Nov. 2001
    ! modified by J. Neugebauer, Uni Erlangen, 2001 
    !----------------------------------------------------------------
    use snf_turbomole, only : read_tm_ctrl, init_tm_ctrl
    use snf_tools, only: string2integer,string2real
    implicit none
    type(type_ctrl),intent(inout) :: ctrl
    character(len=132)          :: buf
    real(double),pointer :: massatom(:)
    integer, pointer :: selatom(:)
    integer :: ucf,nchange
    logical :: ex,foundmaxsym
    ! --------- read control file completely -------------------------
    ucf = 12
    buf='  '
    inquire(file=ctrl%control,exist=ex)
    if(.not. ex) then
       call serror('read_control',2,'snf_control does not exist.##&
         &If you want to evaluate restart-files from former##&
         &versions of snf, please first run snf_define to get##&
         &the snf_control file, and then run snf again.')
    end if

    open(unit=ucf,file=ctrl%control,status='unknown')

    nchange=0
    do while(index(buf,'$end').eq.0)
       read(ucf,'(a)') buf
       if(index(buf,'$changemass').ne.0) nchange=nchange+1
    end do
    rewind(ucf)
    if(nchange.ge.1) then
       allocate(selatom(nchange),massatom(nchange))
       write(*,*) '>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<'
       write(*,*) ' Found modified isotopes for the following atoms:'
       write(*,*) 
       write(*,*) ' No.    atomic mass [amu]:'
       write(*,*) 
    end if
    nchange=1
    foundmaxsym=.false.
    buf=''

    do while(index(buf,'$end').eq.0)
       read(ucf,'(a)') buf
       if(index(buf,'$title ').ne.0) then
          read(ucf,'(a)') buf
          ctrl%title=trim(buf)
       else if( index(buf,'$turbomole').ne.0) then
          if(associated(ctrl%tm_ctrl)) deallocate(ctrl%tm_ctrl)
          allocate(ctrl%tm_ctrl)
          call init_tm_ctrl(ctrl%tm_ctrl)
          ctrl%tm_ctrl%control=trim(adjustl(buf(index(buf,'$turbomole')+10:)))
          call read_tm_ctrl(ctrl%tm_ctrl)
          call copy_tm_to_ctrl(ctrl)
       else if( index(buf,'$symmetry ').ne.0) then
          ctrl%symmetry=trim(buf(11:))
       else if( index(buf,'$isosym ').ne.0) then
          ctrl%isosym=trim(buf(9:))
       else if( index(buf,'$raman').ne.0) then
          if(index(buf,'on' ).ne.0) ctrl%raman = .true.
          if(index(buf,'off').ne.0) ctrl%raman = .false.
       else if( index(buf,'$dynpol').ne.0) then
          if(index(buf,'on' ).ne.0) ctrl%dynpol= .true.
          if(index(buf,'off').ne.0) ctrl%dynpol= .false.
       else if( index(buf,'$dynfreq').ne.0) then
          if(ctrl%dynpol) then
            call getdfreq(buf(10:),ctrl)
          else
            call serror('read_control',2,'Found $dynfreq but no (or &
             &before) $dynpol')
          end if
       else if( index(buf,'$cstep').ne.0) then
          ctrl%cstep=string2real(buf)
       else if( index(buf,'$numderiv').ne.0) then
          ctrl%numderiv=string2integer(buf)
       else if( index(buf,'$restart').ne.0) then
          ctrl%restart=trim(buf(index(buf,'=')+1:))
       else if( index(buf,'$minfreq').ne.0) then
          ctrl%minfreq=string2real(buf(index(buf,'$minfreq')+8:))
       else if( index(buf,'$maxfreq').ne.0) then
          ctrl%maxfreq=string2real(buf(index(buf,'$maxfreq')+8:))
       else if( index(buf,'$width').ne.0) then
          ctrl%width=string2real(buf(index(buf,'$width')+6:))
       else if( index(buf,'$npoint').ne.0) then
          ctrl%npoint=string2integer(buf(index(buf,'$npoint')+7:))
       else if( index(buf,'$temp').ne.0) then
          ctrl%temp=string2real(buf(index(buf,'$temp')+5:))
       else if( index(buf,'$theta').ne.0) then
          ctrl%theta=string2real(buf(index(buf,'$theta')+6:))
       else if( index(buf,'$press').ne.0) then
          ctrl%press=string2real(buf(index(buf,'$press')+6:))
       else if( index(buf,'$xmolsc1').ne.0) then
          ctrl%xmolsc1=string2real(buf(index(buf,'$xmolsc1')+8:))
       else if( index(buf,'$print_gaussian').ne.0) then
          if(index(buf,'on' ).ne.0) ctrl%print_gaussian = .true.
          if(index(buf,'off').ne.0) ctrl%print_gaussian  = .false.
       else if( index(buf,'$print_mopac').ne.0) then      
          if(index(buf,'on' ).ne.0) ctrl%print_mopac = .true.
          if(index(buf,'off').ne.0) ctrl%print_mopac  = .false.
       else if( index(buf,'$print_mopxmol').ne.0) then      
          if(index(buf,'on' ).ne.0) ctrl%print_mopxmol = .true.
          if(index(buf,'off').ne.0) ctrl%print_mopxmol  = .false.
       else if( index(buf,'$print_xmol').ne.0) then      
          if(index(buf,'on' ).ne.0) ctrl%print_xmol = .true.
          if(index(buf,'off').ne.0) ctrl%print_xmol  = .false.
       else if( index(buf,'$print_symmetry').ne.0) then   
          if(index(buf,'on' ).ne.0) ctrl%print_symmetry = .true.
          if(index(buf,'off').ne.0) ctrl%print_symmetry  = .false.
       else if( index(buf,'$print_irred_at').ne.0) then   
          if(index(buf,'on' ).ne.0) ctrl%print_irred_at = .true.
          if(index(buf,'off').ne.0) ctrl%print_irred_at = .false.
       else if( index(buf,'$prirdist').ne.0) then   
          if(index(buf,'on' ).ne.0) ctrl%prirdist = .true.
          if(index(buf,'off').ne.0) ctrl%prirdist = .false.
       else if( index(buf,'$print_cioslowski').ne.0) then 
          if(index(buf,'on' ).ne.0) ctrl%print_cioslowski = .true.
          if(index(buf,'off').ne.0) ctrl%print_cioslowski = .false.
       else if( index(buf,'$print_force').ne.0) then      
          if(index(buf,'on' ).ne.0) ctrl%print_force = .true.
          if(index(buf,'off').ne.0) ctrl%print_force = .false.
       else if( index(buf,'$print_hessian').ne.0) then    
          if(index(buf,'on' ).ne.0) ctrl%print_hessian = .true.
          if(index(buf,'off').ne.0) ctrl%print_hessian = .false.
       else if( index(buf,'$print_dip').ne.0) then        
          if(index(buf,'on' ).ne.0) ctrl%print_dip = .true.
          if(index(buf,'off').ne.0) ctrl%print_dip = .false.
       else if( index(buf,'$sumdeg').ne.0) then        
          if(index(buf,'on' ).ne.0) ctrl%sumdeg = .true.
          if(index(buf,'off').ne.0) ctrl%sumdeg = .false.
       else if( index(buf,'$maxsym').ne.0) then        
          foundmaxsym=.true.
          if(index(buf,'on' ).ne.0) ctrl%maxsym = .true.
          if(index(buf,'off').ne.0) ctrl%maxsym = .false.
       else if( index(buf,'$print_pol').ne.0) then        
          if(index(buf,'on' ).ne.0) ctrl%print_pol = .true.
          if(index(buf,'off').ne.0) ctrl%print_pol = .false.
       else if( index(buf,'$print_rcs').ne.0) then        
          if(index(buf,'on' ).ne.0) ctrl%print_rcs = .true.
          if(index(buf,'off').ne.0) ctrl%print_rcs = .false.
       else if( index(buf,'$print_transrot').ne.0) then   
          if(index(buf,'on' ).ne.0) ctrl%print_transrot = .true.
          if(index(buf,'off').ne.0) ctrl%print_transrot = .false.
       else if( index(buf,'$print_zpe').ne.0) then        
          if(index(buf,'on' ).ne.0) ctrl%print_zpe = .true.
          if(index(buf,'off').ne.0) ctrl%print_zpe = .false.
       else if( index(buf,'$print_thermo').ne.0) then        
          if(index(buf,'on' ).ne.0) ctrl%print_thermo = .true.
          if(index(buf,'off').ne.0) ctrl%print_thermo = .false.
       else if( index(buf,'$changemass').ne.0) then        
          read(buf(12:),*) selatom(nchange),massatom(nchange)
          write(*,'(i5,f24.16)') selatom(nchange),massatom(nchange)
          if(nchange.eq.ubound(selatom,1)) &
               write(*,*) '>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<'
          nchange=nchange+1
       else if( index(buf,'$total electronic energy ').ne.0) then 
          ctrl%elec_energy=string2real(buf(index(buf,&
            &'$total electronic energy ')+25:))
       else if( index(buf,'$total spin ').ne.0) then
          ctrl%spin=string2real(buf(index(buf,& 
            &'$total spin ')+10:)) 
       endif
    enddo
    if(.not.foundmaxsym) ctrl%maxsym=.false.
    if(ctrl%raman) then
      if(ctrl%mode.eq.2.or.ctrl%mode.eq.7) then
        ctrl%raman=.false.
      end if
    endif
    close(ucf)    
  end subroutine read_control

  !************************************************************************

  subroutine write_control(ctrl,molecule,unit)
    implicit none
    !----------------------------------------------------------------
    ! write all available data to snf_control
    !
    ! written by Carsten Kind, Uni Erlangen, 2000-2001
    ! modified by Markus Reiher, Uni Erlangen, Aug. and Nov. 2001
    !----------------------------------------------------------------
    type(type_ctrl),intent(in) :: ctrl
    type(type_atom),pointer    :: molecule(:)
    integer,optional,intent(in) :: unit
    integer :: ucf,out,i
    ! --------- read whole control file  -------------------------
    if(present(unit)) then
       out=unit
    else
       out=6
    endif
    ucf = 43
    open(unit=ucf,file=ctrl%control,status='unknown')
    write(ucf,'(''$snf_control'')') 
    write(ucf,'(''$title'')')
    write(ucf,'(a)')trim(ctrl%title)
    if(associated(ctrl%tm_ctrl)) then
       write(ucf,'(''$turbomole '',(a))') trim(ctrl%tm_ctrl%control)
    endif
    write(ucf,'(''$cstep    '',f12.3)') ctrl%cstep
    write(ucf,'(''$symmetry '',a3)') ctrl%symmetry
    if(ctrl%isosym.ne.'def') &
         write(ucf,'(''$isosym '',a3)') ctrl%isosym
    write(ucf,'(''$restart ='',a)') trim(ctrl%restart)
    write(ucf,'(''$minfreq '',f10.2)') ctrl%minfreq
    write(ucf,'(''$maxfreq '',f10.2)') ctrl%maxfreq
    write(ucf,'(''$width   '',f10.2)') ctrl%width
    if(ctrl%print_xmol) write(ucf,'(''$xmolsc1 '',f14.6)') ctrl%xmolsc1
    write(ucf,'(''$npoint  '',i7)') ctrl%npoint
    write(ucf,'(''$temp    '',f10.2)') ctrl%temp
    write(ucf,'(''$theta   '',f10.3)') ctrl%theta
    write(ucf,'(''$press   '',f10.2)') ctrl%press
    if(ctrl%print_gaussian) then
       write(ucf,'(''$print_gaussian on'')')
    else                                     
       write(ucf,'(''$print_gaussian off'')')
    endif
    if(ctrl%print_mopac) then
       write(ucf,'(''$print_mopac on'')')
    else
       write(ucf,'(''$print_mopac off'')')
    endif
    if(ctrl%print_mopxmol) then
       write(ucf,'(''$print_mopxmol on'')')
    else
       write(ucf,'(''$print_mopxmol off'')')
    endif
    if(ctrl%print_xmol) then
       write(ucf,'(''$print_xmol on'')')
    else
       write(ucf,'(''$print_xmol off'')')
    endif
    if(ctrl%print_symmetry) then
       write(ucf,'(''$print_symmetry on'')')
    else
       write(ucf,'(''$print_symmetry off'')')
    endif
    if(ctrl%print_irred_at) then
       write(ucf,'(''$print_irred_at on'')')
    else
       write(ucf,'(''$print_irred_at off'')')
    endif
    if(ctrl%prirdist) then
       write(ucf,'(''$prirdist on'')')
    else
       write(ucf,'(''$prirdist off'')')
    endif
    if(ctrl%print_cioslowski) then
       write(ucf,'(''$print_cioslowski on'')')
    else
       write(ucf,'(''$print_cioslowski off'')')
    endif
    if(ctrl%print_force) then
       write(ucf,'(''$print_force on'')')
    else
       write(ucf,'(''$print_force off'')')
    endif
    if(ctrl%print_hessian) then
       write(ucf,'(''$print_hessian on'')')
    else                                    
       write(ucf,'(''$print_hessian off'')')
    endif
    if(ctrl%print_dip) then
       write(ucf,'(''$print_dip on'')')
    else                                
       write(ucf,'(''$print_dip off'')')
    endif
    if(ctrl%maxsym) then
       write(ucf,'(''$maxsym on'')')
    else                                
       write(ucf,'(''$maxsym off'')')
    endif
    if(ctrl%sumdeg) then
       write(ucf,'(''$sumdeg on'')')
    else                                
       write(ucf,'(''$sumdeg off'')')
    endif
    if(ctrl%print_pol) then
       write(ucf,'(''$print_pol on'')')
    else                                
       write(ucf,'(''$print_pol off'')')
    endif
    if(ctrl%print_rcs) then
       write(ucf,'(''$print_rcs on'')')
    else                                
       write(ucf,'(''$print_rcs off'')')
    endif
    if(ctrl%print_transrot) then
       write(ucf,'(''$print_transrot on'')')
    else
       write(ucf,'(''$print_transrot off'')')
    endif
    if(ctrl%print_zpe) then
       write(ucf,'(''$print_zpe on'')')
    else
       write(ucf,'(''$print_zpe off'')')
    endif
    if(ctrl%print_thermo) then
       write(ucf,'(''$print_thermo on'')')
    else
       write(ucf,'(''$print_thermo off'')')
    endif
    if(ctrl%raman) then
       write(ucf,'(''$raman on'')')
    else
       write(ucf,'(''$raman off'')')
    endif
    if(ctrl%dynpol) then
       write(ucf,'(''$dynpol on'')')
    else
       write(ucf,'(''$dynpol off'')')
    endif
    if(ctrl%dynpol.and.associated(ctrl%dynfreq)) then
       write(ucf,'(''$dynfreq '',10f16.8)') (ctrl%dynfreq(i),&
       i=1,ubound(ctrl%dynfreq,1))
    endif

    write(ucf,'(''$total electronic energy '',f20.10)') ctrl%elec_energy
    write(ucf,'(''$total spin '',f20.10)') ctrl%spin

    do i=1,ubound(molecule,1)
       if(molecule(i)%changem) then
          write(ucf,'(''$changemass'',1x,i5,f24.16)') i,molecule(i)%mass
       end if
    end do
    write(ucf,'(''$end'')')
    close(ucf)
    write(out,*)
    write(out,'(''  Created new snf control file : '',A)') trim(adjustl(ctrl%control))
  end subroutine write_control

  !************************************************************************

  subroutine copy_ctrl_to_dal(ctrl)
    !----------------------------------------------------------------
    !
    ! written by J. Neugebauer, Uni Erlangen, 2001
    !----------------------------------------------------------------
    implicit none
    integer :: i
    type(type_ctrl),intent(inout) :: ctrl
    if(ctrl%raman) then 
      if(.not.ctrl%dynpol) then
        ctrl%dal_ctrl%inputpol=1
      else
        ctrl%dal_ctrl%inputpol=2
        if(associated(ctrl%dynfreq)) then
          if(associated(ctrl%dal_ctrl%dynfreq)) deallocate(ctrl%dal_ctrl%dynfreq)
          allocate(ctrl%dal_ctrl%dynfreq(ubound(ctrl%dynfreq,1)))
          do i=1,ubound(ctrl%dynfreq,1)
            ctrl%dal_ctrl%dynfreq(i)=ctrl%dynfreq(i)
          end do
        end if
      end if
      if(ctrl%mode.eq.2) then
        call serror('copy_ctrl_to_tm',-1,'No ridft available##&
         &for Raman calculations')
        ctrl%tm_ctrl%mode=1
        ctrl%mode=1
      end if
    else
       ctrl%dal_ctrl%inputpol=0
    endif
    if(ctrl%dal_ctrl%inputpol.ne.ctrl%tm_ctrl%inputpol) &
     & call serror('copy_ctrl_to_dal',2,'inputpol values for DALTON##&
     &and TURBOMOLE differ !')
    ctrl%dal_ctrl%mode=ctrl%mode
  end subroutine copy_ctrl_to_dal

  !************************************************************************

  subroutine copy_ctrl_to_tm(ctrl)
    !----------------------------------------------------------------
    !
    ! written by Carsten Kind, Uni Erlangen, 2000-2001
    !----------------------------------------------------------------
    implicit none
    integer :: i
    type(type_ctrl),intent(inout) :: ctrl
    if(ctrl%raman) then 
      if(.not.ctrl%dynpol) then
        ctrl%tm_ctrl%inputpol=1
      else
        ctrl%tm_ctrl%inputpol=2
        if(associated(ctrl%dynfreq)) then
          if(associated(ctrl%tm_ctrl%dynfreq)) deallocate(ctrl%tm_ctrl%dynfreq)
          allocate(ctrl%tm_ctrl%dynfreq(ubound(ctrl%dynfreq,1)))
          do i=1,ubound(ctrl%dynfreq,1)
            ctrl%tm_ctrl%dynfreq(i)=ctrl%dynfreq(i)
          end do
        end if
      end if
      if(ctrl%mode.eq.2) then
        call serror('copy_ctrl_to_tm',-1,'No ridft available##&
         &for Raman calculations')
        ctrl%tm_ctrl%mode=1
        ctrl%mode=1
      end if
    else
       ctrl%tm_ctrl%inputpol=0
    endif
    ctrl%tm_ctrl%scfconv = ctrl%scfconv
    ctrl%tm_ctrl%numderiv = ctrl%numderiv
    ctrl%tm_ctrl%cstep = ctrl%cstep
    ctrl%tm_ctrl%mode= ctrl%mode
  end subroutine copy_ctrl_to_tm

  !************************************************************************

  subroutine copy_tm_to_ctrl(ctrl)
    !----------------------------------------------------------------
    !
    ! written by Carsten Kind, Uni Erlangen, 2000-2001
    ! modified by J. Neugebauer, Uni Erlangen, 2001
    !----------------------------------------------------------------
    implicit none
    integer i
    type(type_ctrl),intent(inout) :: ctrl
    ctrl%raman=ctrl%tm_ctrl%inputpol .ge. 1
    ctrl%dynpol=ctrl%tm_ctrl%inputpol .eq. 2
    if(ctrl%dynpol.and.ctrl%raman.and.associated(ctrl%tm_ctrl%dynfreq)) then
      if(associated(ctrl%dynfreq)) deallocate(ctrl%dynfreq)
      allocate(ctrl%dynfreq(ubound(ctrl%tm_ctrl%dynfreq,1)))
      do i=1,ubound(ctrl%dynfreq,1)
        ctrl%dynfreq(i)=ctrl%tm_ctrl%dynfreq(i)
      end do
    end if
    if(abs(ctrl%tm_ctrl%cstep+1.d0) .gt. 1.d-10) then
       ctrl%cstep=ctrl%tm_ctrl%cstep
    endif
    if(ctrl%tm_ctrl%numderiv.ne.-1)  then
       ctrl%numderiv=ctrl%tm_ctrl%numderiv
    endif
    if(ctrl%tm_ctrl%scfconv .ne.-1)  then
       ctrl%scfconv=ctrl%tm_ctrl%scfconv
    endif
    if(ctrl%tm_ctrl%symmetry .ne. 'c1 ') then
       ctrl%symmetry=ctrl%tm_ctrl%symmetry
    endif
  end subroutine copy_tm_to_ctrl

  !************************************************************************

  subroutine print_ctrl(ctrl,unit)
    !----------------------------------------------------------------
    ! output routine
    !
    ! written by Carsten Kind, Uni Erlangen, 2000-2001
    ! modified by Johannes Neugebauer, Uni Erlangen, 2001
    !----------------------------------------------------------------
    implicit none
    type(type_ctrl),intent(in) :: ctrl
    integer,optional :: unit
    integer :: out
    if(present(unit)) then
       out=unit
    else
       out=6
    endif
    write(out,*)
    write(out,'('' SNF Control file information'')')
    write(out,'('' ----------------------------'')')
    write(out,'(''   snf control file          : '',A)') trim(adjustl(ctrl%control))
    write(out,'(''   title                     : '',A)') trim(adjustl(ctrl%title))
    write(out,'(''   snf restart file          : '',A)') trim(adjustl(ctrl%restart))
    write(out,'(''   restart file format vers. :'',F6.2)') dble(ctrl%version)/100.D0
    write(out,'(''   symmetry                  :   '',A3)') ctrl%symmetry
    write(out,'(''   Number of atoms           :'',I6)') &
                                             & ctrl%nvar/3
    write(out,'(''   Number of steps           :'',I6)') ctrl%numderiv*2
    write(out,'(''   distortion [bohr]         : '',F6.3)') ctrl%cstep
    write(out,'(''   SCF convergence criterion :'',I6)') ctrl%scfconv
    if(ctrl%raman) then
       write(out,'(''   Raman intensities switched on'')')
       if(ctrl%dynpol) write(out,'(''   Dynamic polarizabilities will be &
         &used'')')
    end if
    if(associated(Ctrl%tm_ctrl)) &
         & write(out,'(''   Energy and gradient source is turbomole'')')
    if(ctrl%print_gaussian) &
         & write(out,'(''   print gaussian fake output'')')
    if(ctrl%print_mopac) then
      write(out,'(''   print mopac fake output'')')
      if(ctrl%print_mopxmol) &
        write(out,'(''   for use in xmol'')')
    end if
    if(ctrl%print_xmol) &
         & write(out,'(''   print xmol input file'')')
    if(ctrl%print_symmetry) &
         & write(out,'(''   print detailed symmetry-information'')')
    if(ctrl%print_irred_at) &
         & write(out,'(''   print information about symmetry redundant atoms'')')
    if(ctrl%prirdist) &
         & write(out,'(''   print information about symmetry redundant&
         & distortions'')')
    if(ctrl%print_cioslowski) &
         & write(out,'(''   print cioslowski charge decomposition'')')
    if(ctrl%print_force) &
         & write(out,'(''   print force constant matrix'')')
    if(ctrl%print_hessian) &
         & write(out,'(''   print cartesian hessian'')')
    if(ctrl%print_dip) &
         & write(out,'(''   print cartesian dipol derivates'')')
    if(ctrl%print_pol) &
         & write(out,'(''   print cartesian polarizability derivates'')')
    if(ctrl%print_rcs) &
         & write(out,'(''   print raman scattering cross sections'')')
    if(ctrl%print_transrot) &
         & write(out,'(''   print transl. and rot. contributions'')')
    if(ctrl%print_zpe) &
         & write(out,'(''   print  zeropoint energy '')')
    if(ctrl%print_thermo) &
         & write(out,'(''   print thermodynamics'')')
    write(out,'(''   Plot range for spectra  [cm^-1]       : '',I4,''-'',I4)')&
         & int(Ctrl%minfreq),int(Ctrl%maxfreq)
    write(out,'(''   Gaussian broadening peak width [cm^-1]:'',I4)') int(ctrl%width)
  end subroutine print_ctrl

  !************************************************************************

  subroutine check_ctrl(ctrl1,ctrl2,unit)
    implicit none
    !----------------------------------------------------------------
    ! original validation checks. not used in this version.
    !
    ! written by Carsten Kind, Uni Erlangen, 2000-2001
    !----------------------------------------------------------------
    type(type_ctrl),intent(in) :: ctrl1,ctrl2
    integer,optional :: unit
    integer :: out
    if(present(unit)) then
       out=unit
    else
       out=6
    endif
    if (ctrl1%version/10.ne.ctrl2%version/10) then
       write (out,*) 'ERROR: incompatible versions '
       write (out,*) '   --> version on restart file :',ctrl1%version
       write (out,*) '   --> current program version :',ctrl2%version
       stop 'incompatible restart file'
    endif
    if (ctrl1%mode .ne. ctrl2%mode) then
       write (out,*) 'ERROR: <control> file has been changed:'
       write (out,*) '   --> mode on restart file :',ctrl1%mode
       write (out,*) '   --> current program mode :',ctrl2%mode
       stop 'incompatible restart file'
    endif
    if (dabs(ctrl1%cstep-ctrl2%cstep).gt.1.d-6) then
       write (out,*) 'ERROR: cstep has been changed:'
       write (out,*) '   --> cstep on restart file     :',ctrl1%cstep
       write (out,*) '   --> current program cstepmode :',ctrl2%cstep
       stop 'incompatible restart file'
    endif
    if (ctrl1%numderiv.ne.ctrl2%numderiv) then
       write (out,*) 'ERROR: numderiv has been changed !'
       write (out,*) '   --> numderiv on restart file     :',ctrl1%numderiv
       write (out,*) '   --> current program numderiv :',ctrl2%numderiv
       stop 'incompatible restart file'
    endif
    if (iabs(ctrl1%scfconv-ctrl2%scfconv).gt.0) then
       write (out,*) 'ERROR: scfconv has been changed:'
       write (out,*) '   --> scfconv on restart file     :',ctrl1%scfconv
       write (out,*) '   --> current program scfconv :',ctrl2%scfconv
       stop 'incompatible restart file'
    endif
  end subroutine check_ctrl

  !************************************************************************

  subroutine destroy_control(ctrl)
    implicit none
    type(type_ctrl),intent(inout) :: ctrl
    if(associated(ctrl%tm_ctrl)) deallocate(ctrl%tm_ctrl)
  end subroutine destroy_control

  !************************************************************************

        subroutine get_spin(dir,spin,out)
!
!       written by Markus Reiher, University of Erlangen, Nov. 2001
!
        use snf_tools,only     : string2real,clean
        implicit none
        character(len=*)     :: dir
        real(double)         :: spin,si,thres
        character(len=132)   :: buf
        character(len=256)   :: errbuf
        character(len=20)    :: cdumb
        integer              :: out,i
        logical              :: ex
        parameter(thres=0.25d0)
        inquire(file=dir,exist=ex)
        if(.not.ex) goto 999
        open(unit=45,file=dir,status='unknown')
10      read(45,'(a)',err=999,end=999) buf
        if (index(buf(1:32),'<S*S>').ne.0) then
          call clean(cdumb)
          cdumb(1:20)=buf(7:26)
          spin=string2real(cdumb)
!	*** <S*S> approx S(S+1) => extract S
	  do i=1,10
            si=0.5d0*dble(i)
 	    if (abs(si*(si+1.0d0)-spin).le.thres) then
	      spin=si
              close(45)
              return
            endif
          enddo
	  write(errbuf,'(3a)') ' total spin > 5 ?!,&
            &total spin will be set to zero.'
          errbuf=trim(adjustl(errbuf))
	  call serror('get_spin',0,errbuf,out) 
	  close(45)
	  return
        endif
        goto 10
999     errbuf=''
        write(errbuf,'(3a)') 'Reading total spin from file ''',&
          &trim(adjustl(dir)),''' failed.## Total spin will be set to zero.'
        errbuf=trim(adjustl(errbuf))
        call serror('get_spin',0,errbuf,out)
        close(45)
        return
        end subroutine get_spin

  !************************************************************************

        subroutine get_energy(dir,e,out)
!
!       written by Markus Reiher, University of Erlangen, Aug. 2001
!       modified by J. Neugebauer, University of Erlangen, Sep. 2001
!
        use snf_tools,only     : string2real,clean
        implicit none
        character(len=*)     :: dir
        double precision     :: e
        character(len=132)   :: buf
        character(len=256)   :: errbuf
        character(len=20)    :: cdumb
        integer              :: out
        logical              :: ex
        inquire(file=dir,exist=ex)
        if(.not.ex) goto 999
        open(unit=45,file=dir,status='unknown')
10      read(45,'(a)',err=999,end=999) buf
        if (buf(1:32).eq.'                 |  total energy') then
          call clean(cdumb)
          cdumb(1:20)=buf(index(buf,'=')+1:index(buf,'=')+19)
          e=string2real(cdumb)
          close(45)
          return
        endif
        goto 10
999     errbuf=''
        write(errbuf,'(3a)') 'Reading total energy from file ''',&
          &trim(adjustl(dir)),''' failed.## Total energy will be set to zero.' 
        errbuf=trim(adjustl(errbuf))
        call serror('get_energy',0,errbuf,out)
        close(45)
        return
        end subroutine get_energy 

  !************************************************************************
  
        subroutine get_energy_mp2(dir,e,out)
!
!       written by Markus Reiher, University of Erlangen, Aug. 2001
!       modified by J. Neugebauer, University of Erlangen, Sep. 2001
!
        use snf_tools,only     : string2real,clean
        implicit none
        character(len=*)  :: dir
        double precision  :: e
        character(len=132):: buf
        character(len=256):: errbuf
        character(len=22) :: cdumb
        character(len=14) :: cdumc
        integer out
        logical           :: ex
        inquire(file=dir,exist=ex)
        if(.not.ex) goto 999
        open(unit=45,file=dir,status='unknown')
10      read(45,'(a)',err=999,end=999) buf
        cdumc(1:14)=buf(21:35)
        if (cdumc.eq.'total        :') then
          call clean(cdumb)
          cdumb(1:22)=buf(index(buf,':')+1:index(buf,':')+21)
          e=string2real(cdumb)
          close(45)
          return
        endif
        goto 10
999     errbuf=''
        write(errbuf,'(3a)') 'Reading total energy from file ''',&
          &trim(adjustl(dir)),''' failed.## Total energy will be set to zero.' 
        errbuf=trim(adjustl(errbuf))
        call serror('get_energy_mp2',0,errbuf,out)
        close(45)
        return
        end subroutine get_energy_mp2

  !************************************************************************
  
  subroutine getdfreq(row, ctrl)
    !----------------------------------------------------------------
    ! read angular frequencies for dynamical polarizabilities from "row"
    ! up to <nfreq> frequencies are considered
    !
    ! written by Johannes Neugebauer, Uni Erlangen, 2001
    !----------------------------------------------------------------
    use snf_param
    implicit none
    integer :: i,j,iend,nfreq
    type(type_ctrl),intent(inout) :: ctrl
    character(len=*) :: row
    real(double), pointer :: dynfreq(:)
    parameter(nfreq=10)
    allocate(dynfreq(nfreq))
    if(associated(ctrl%dynfreq)) deallocate(ctrl%dynfreq)
    dynfreq=-1.0d0
    read(row(1:),*,end=987,err=789) (dynfreq(i),i=1,nfreq)
987 continue
    j=0
    iend=0
    do i=1,nfreq
      if(dynfreq(i).lt.0.0d0) then
        iend=iend+1
        do j=i,nfreq-1
          dynfreq(j)=dynfreq(j+1)
        end do
      end if
    end do
    j=0
    do i=1,nfreq-iend
      if(dynfreq(i).ge.0.0d0) j=j+1
    end do
    if(j.gt.0) then
      allocate (ctrl%dynfreq(j))
      do i=1,ubound(ctrl%dynfreq,1)
        ctrl%dynfreq(i)=dynfreq(i)
      end do
    else
      goto 789
    end if
 
    deallocate(dynfreq)
    return
 
789 call serror('getdfreq',1,'Could not properly read frequencies##&
     &Setting default value (514.5 nm)')
    if(associated(ctrl%dynfreq)) deallocate(ctrl%dynfreq)
    deallocate(dynfreq)
 
  end subroutine getdfreq
 
  !************************************************************************  

end module snf_control

