
!********************************************************************************
!*   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
!*   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
!*   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
!*
!*
!*   This file is part of the package SNF.
!*
!*   SNF is free software; you can redistribute it and/or modify
!*   it under the terms of the GNU General Public License as published by
!*   the Free Software Foundation; either version 2 of the License, or
!*   (at your option) any later version.
!*
!*   This program is distributed in the hope that it will be useful,
!*   but WITHOUT ANY WARRANTY; without even the implied warranty of
!*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*   GNU General Public License for more details.
!*
!*   You should have received a copy of the GNU General Public License
!*   along with this program; if not, write to the Free Software
!*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!*
!********************************************************************************
module snf_dalton
  use f90_kind
  use snf_error
  use snf_tools
  use snf_param
  
  private
  public :: type_dal_ctrl, init_dal_ctrl,read_dal_coord,write_dal_ctrl
  
  type type_dal_ctrl
    real(double),pointer :: dynfreq(:) ! angular frequencies for dyn. pol.
    logical  :: tmmos              ! use Turbomole MOs as start orbitals
    character(len=132) :: coord    ! coord file in DALTON format
    character(len=132) :: control  ! control file for DALTON
    character(len=132) :: mos      ! use DALTON MOs form file <mos>
    integer :: mode,  &            ! program mode
         &     inputpol,       &   ! flag for polarizabilities
         &     hfocc,          &   ! HF occupation in case of MCSCF
         &     cas,            &   ! CAS SPACE
         &     caselec,        &   ! ELECTRONS (IN CAS SPACE)
         &     inactive,       &   ! No. of inactive orbitals (CASSCF)
         &     multiplicity        ! SPIN MULTIPLICITY for MCSCF
  end type type_dal_ctrl

  !*********************************************************************
 
contains
 
  !*********************************************************************
   
  subroutine init_dal_ctrl(ctrl)
    !----------------------------------------------------------------
    ! initialize default ctrl values for dalton.inp
    !
    ! written by J. Neugebauer, Uni Erlangen, 2001
    !---------------------------------------------------------------- 
    implicit none
    type(type_dal_ctrl), intent(out) :: ctrl
    nullify(ctrl%dynfreq)
    ctrl%mode= 20
    ctrl%inputpol= 0
    ctrl%tmmos = .false.
    ctrl%mos='----------'
    ctrl%coord='molecule.mol'
    ctrl%control='molecule.dal'
    ctrl%hfocc=0
    ctrl%cas=0
    ctrl%caselec=0
    ctrl%inactive=0
    ctrl%multiplicity=1
  end subroutine init_dal_ctrl
 
  !*********************************************************************
   
  subroutine read_dal_coord(tmpname)
    !---------------------------------------------------------------- 
    ! written by J. Neugebauer, Uni Erlangen, 2001
    !---------------------------------------------------------------- 
    implicit none
    character(len=132)             ::  tmpname
  end subroutine read_dal_coord
 
  !*********************************************************************
   
  subroutine write_dal_ctrl(ctrl)
    !---------------------------------------------------------------- 
    ! writes the file ctrl%control and includes the MO start vector
    ! from file "mos", if(ctrl%tmmos). In the latter case, also the
    ! file molecule.mol will be created, which is not necessary for
    ! the Snfdc calculation (this file has to be created for every
    ! slave process, but not for the master process).
    ! It is also possible to supply MO start vectors in the 
    ! DALTON.MOPUN format from file <ctrl%mos>
    ! Note: keyword '.DIRECT' might be introduced after 
    ! '**DALTON INPUT' to perform integral direct calculations.
    ! However, these seem to be slower than integral non-direct 
    ! calculations !?
    !
    ! written by J. Neugebauer, Uni Erlangen, 2001
    !---------------------------------------------------------------- 
      implicit none
      type(type_dal_ctrl), intent(in) :: ctrl
      integer info,system,inout,i
      character(len=132) buf
      character(len=10) progtype,calctype
      logical ex
 
      inout=25
      call rmfile('snf.tmp',1)
      open(inout,file='snf.tmp')
      write(inout,'(a)') '**DALTON INPUT'
      write(inout,'(a)') '.RUN WAVE FUNCTION'
      if(ctrl%mode.eq.20.or.ctrl%mode.eq.25) then
        write(inout,'(a)') '.RUN PROPERTIES'
        write(inout,'(a)') '**PROPERTIES'
        write(inout,'(a)') '.MOLGRA'
        if(ctrl%inputpol.eq.1) then
          write(inout,'(a)') '.POLARI'
        else if(ctrl%inputpol.eq.2) then
          write(inout,'(a)') '.ABALNR'
          write(inout,'(a)') '*ABALNR'
          write(inout,'(a)') '.ALFA  '
          write(inout,'(a)') '.FREQUE'
          if(associated(ctrl%dynfreq)) then
            write(inout,'(i2)') ubound(ctrl%dynfreq)
            write(inout,'(10f16.8)') (ctrl%dynfreq(i),i=1,&
             &ubound(ctrl%dynfreq,1))
          else
            write(inout,'(a)') '1'
            write(inout,'(f16.8)') nm2freq/514.5d0
          end if
        end if
        write(inout,'(a)') '**WAVE FUNCTIONS'
        write(inout,'(a)') '.HF'
        if(ctrl%mode.eq.25) then
          call casinput(ctrl%hfocc,ctrl%multiplicity,ctrl%inactive, &
     & ctrl%cas,ctrl%caselec)
          write(inout,'(a)') '.MCSCF'
          write(inout,'(a)') '*OPTIMIZATION'
          write(inout,'(a)') '.MAX MACRO ITERATIONS'
          write(inout,'(a)') '40'
          write(inout,'(a)') '.MAX MICRO ITERATIONS'
          write(inout,'(a)') '30'
          write(inout,'(a)') '*HF INPUT'
          write(inout,'(a)') '.HF OCCUPATION'
          write(inout,'(i2)') ctrl%hfocc
          write(inout,'(a)') '*CONFIGURATION INPUT'
          write(inout,'(a)') '.SYMMETRY'
          write(inout,'(a)') '1'
          write(inout,'(a)') '.SPIN MULTIPLICITY'
          write(inout,'(i2)') ctrl%multiplicity
          write(inout,'(a)') '.INACTIVE'
          write(inout,'(i2)') ctrl%inactive
          write(inout,'(a)') '.CAS SPACE'
          write(inout,'(i2)') ctrl%cas
          write(inout,'(a)') '.ELECTRONS'
          write(inout,'(i2)') ctrl%caselec
        end if
      else if(ctrl%mode.ge.21.and.ctrl%mode.le.24) then
        call prog_type(ctrl%mode,progtype,calctype)
        write(inout,'(a)') '**INTEGRALS'
        write(inout,'(a)') '.DIPLEN'
        write(inout,'(a)') '.DEROVL'
        write(inout,'(a)') '.DERHAM'
        write(inout,'(a)') '**WAVE FUNCTIONS'
        write(inout,'(a)') '.CC'
        write(inout,'(a)') '*CC INPUT'
        write(inout,'(a)') '.'//trim(adjustl(calctype))
        write(inout,'(a)') '*CCFOP'
        write(inout,'(a)') '.DIPMOM'
        if(ctrl%inputpol.ge.1.and.ctrl%inputpol.le.2) then
          write(inout,'(a)') '*CCLR'
          write(inout,'(a)') '.DIPOLE'
          write(inout,'(a)') '.FREQUE'
          if(associated(ctrl%dynfreq)) then
            write(inout,'(i2)') ubound(ctrl%dynfreq)
            write(inout,'(10f16.8)') (ctrl%dynfreq(i),i=1,&
             &ubound(ctrl%dynfreq,1))
          else
            write(inout,'(a)') '1'
            if(ctrl%inputpol.eq.1) then
              write(inout,'(a)') '0.0d0'
            else if(ctrl%inputpol.eq.2) then
              write(inout,'(f16.8)') nm2freq/514.5d0
            end if
          end if
        end if
        write(inout,'(a)') '*DERIVATIVES'
      end if
      write(inout,'(a)') '*ORBITAL INPUT'
      write(inout,'(a)') '.NOSUPSYM'
      call rmfile('molecule.dal',1)
      !--------------------------------------------------------------
      ! supply start MOs, if desired
      !--------------------------------------------------------------
      if(index(ctrl%mos,'--------').eq.0) then
        write(inout,'(a)') '.MOSTART'
        write(inout,'(a)') ' FORM18'
        close(inout)
        info=system('cat snf.tmp '//trim(adjustl(ctrl%mos))//' > molecule.dal')
        call crdalmol('molecule')
        if(info.ne.0) goto 900
        call rmfile('snf.tmp',0)
      else if(ctrl%tmmos) then
        write(inout,'(a)') '.MOSTART'
        write(inout,'(a)') ' FORM18'
        close(inout)
        call crdalmos('molecule')
        inquire(file='molecule.MOPUN',exist=ex)
        if(.not.ex) goto 900
        info=system('cat snf.tmp molecule.MOPUN > molecule.dal')
        if(info.ne.0) goto 900
        call rmfile('snf.tmp',0)
      else
        close(inout)
!--------------------------------------------------------------
!        call crdalmol('molecule')
! better use crdalmos, otherwise a molecular charge of zero
! is always assumed !
!--------------------------------------------------------------
        call crdalmos('molecule')
        info=system('mv snf.tmp molecule.dal')
      end if
      open(inout,file='molecule.dal')
      do
        read(inout,'(a)',end=800) buf
      end do
800   write(inout,'(a)') '*END OF INPUT'
      close(inout)
      return
900   call serror('write_dal_ctrl',2,'Error while preparing&
       & DALTON input file.')
 
  end subroutine write_dal_ctrl
 
  !*********************************************************************

  subroutine crdalmos(dalfile)
    !---------------------------------------------------------------- 
    ! creates a dalton.mol and a dalton.MOPUN file 
    ! using the basis set editor 
    ! "based" by B.A. Hess.
    ! Note: based must be available in $PATH
    !
    ! written by J. Neugebauer, Uni Erlangen, 2001
    !---------------------------------------------------------------- 
      implicit none
      integer ounit,info,system
      character(len=*) dalfile

      ounit=40
      call testfile('coord')
      call testfile('control')
      call testfile('basis')
      call testfile('mos')
      call rmfile(trim(adjustl(dalfile))//'.MOPUN',1)
      call rmfile(trim(adjustl(dalfile))//'.dalton',1)
      call rmfile('bscript.tmp',1)
      open(ounit,file='bscript.tmp')
        write(ounit,'(a)') 'based << !'
        write(ounit,'(a)') 're control'
        write(ounit,'(a)') 'wr '//trim(adjustl(dalfile))//'.dalton'
        write(ounit,'(a)') 'x'
        write(ounit,'(a)') '!'
      close(ounit)
      call exscript('bscript.tmp')
      call rmfile('bscript.tmp',0)
      call rmfile('TMP.LOG',0)
      call rmfile('fort.41',0)
      call rmfile('fort.42',0)
      info=system('mv '//trim(adjustl(dalfile))//'.dalton '&
       &//trim(adjustl(dalfile))//'.mol')
      if(info.ne.0) goto 900
      return
900   call serror('crdalmos',2,'Error while preparing&
       & DALTON input file.')

      end subroutine crdalmos
   
  !*********************************************************************

  subroutine crdalmol(dalfile)
    !---------------------------------------------------------------- 
    ! creates a dalton.mol file using the basis set editor 
    ! "based" by B.A. Hess.
    ! Note: based must be available in $PATH
    !
    ! written by J. Neugebauer, Uni Erlangen, 2001
    !---------------------------------------------------------------- 
      implicit none
      integer ounit,info,system
      character(len=*) dalfile
      logical ex

      ounit=40
      call testfile('coord')
      call testfile('control')
      call testfile('basis')
      inquire(file='mos',exist=ex)
      if(ex) then
        call rmfile('mos.tmp',1)
        info=system('mv mos mos.tmp')
        if(info.ne.0) goto 900
      end if
      call rmfile(trim(adjustl(dalfile))//'.MOPUN',1)
      call rmfile(trim(adjustl(dalfile))//'.dalton',1)
      call rmfile(trim(adjustl(dalfile))//'.mol',1)
      call rmfile('bscript.tmp',1)
      open(ounit,file='bscript.tmp')
        write(ounit,'(a)') 'based << !'
        write(ounit,'(a)') 're control'
        write(ounit,'(a)') 'wr '//trim(adjustl(dalfile))//'.dalton'
        write(ounit,'(a)') 'x'
        write(ounit,'(a)') '!'
      close(ounit)
      call exscript('bscript.tmp')
      call rmfile('bscript.tmp',0)
      call rmfile('TMP.LOG',0)
      call rmfile('fort.41',0)
      call rmfile('fort.42',0)
      info=system('mv '//trim(adjustl(dalfile))//'.dalton '&
       &//trim(adjustl(dalfile))//'.mol')
      if(info.ne.0) goto 900
      if(ex) then
        info=system('mv mos.tmp mos')
        if(info.ne.0) goto 900
      end if
      return
900   call serror('crdalmos',2,'Error while preparing&
       & DALTON input file.')

      end subroutine crdalmol
   
  !*********************************************************************

      subroutine casinput(hfocc,mult,inac,cas,ecas)
      !---------------------------------------------------------------- 
      ! interactive input of data required for DALTON CASSCF 
      ! calculation
      !
      ! written by J. Neugebauer, Uni Erlangen, 2002
      !---------------------------------------------------------------- 
      implicit none
      integer :: hfocc,mult,inac,cas,ecas,i
      character(len=80) :: zeile
      logical :: valid
 
      valid=.false.
100   do while(.not.valid)
        write(*,*)
        write(*,*)
        write(*,*) ' Additional input is necessary for CASSCF calculations.'
        write(*,*) ' Syntax: parameter <parameter_value>'
        write(*,*) 
        write(*,*) ' Please enter the following parameters:'
        write(*,*) 
        write(*,'(a)') '    parameter: meaning                  : current setting'
        write(*,'(a)') '    -----------------------------------------------------'
        write(*,'(a,i3)') '    hfoc     : no. of occ. HF orbitals  : ',hfocc
        write(*,'(a,i3)') '    mult     : spin multiplicity        : ',mult
        write(*,'(a,i3)') '    inac     : no. of inactive orbitals : ',inac
        write(*,'(a,i3)') '    ocas     : no. of orbitals in CAS   : ',cas
        write(*,'(a,i3)') '    ecas     : no. of electrons in CAS  : ',ecas
        write(*,*)
        write(*,'(a)') '    >return< : quit this menu'
        write(*,*)
        read(*,'(a)') zeile
        zeile=trim(adjustl(zeile))
        if(len_trim(zeile).eq.0) then
          valid=.true.
        else 
          if(len(zeile).le.4) goto 100
          read(zeile(5:),*,err=100,end=100) i
          select case(zeile(1:4))
           case ('hfoc')
             hfocc=i
           case ('mult')
             mult=i
           case ('inac')
             inac=i
           case ('ocas')
             cas=i
           case ('ecas')
             ecas=i
           case default
          end select
        end if
      end do      
      end subroutine casinput
   
  !*********************************************************************
 
end module snf_dalton
