
!********************************************************************************
!*   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
!*   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
!*   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
!*
!*
!*   This file is part of the package SNF.
!*
!*   SNF is free software; you can redistribute it and/or modify
!*   it under the terms of the GNU General Public License as published by
!*   the Free Software Foundation; either version 2 of the License, or
!*   (at your option) any later version.
!*
!*   This program is distributed in the hope that it will be useful,
!*   but WITHOUT ANY WARRANTY; without even the implied warranty of
!*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*   GNU General Public License for more details.
!*
!*   You should have received a copy of the GNU General Public License
!*   along with this program; if not, write to the Free Software
!*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!*
!********************************************************************************
module snf_gaussian
  use f90_kind

  private
  public :: print_gaussian_frequencies

  !************************************************************************

contains

  !************************************************************************

  subroutine print_gaussian_frequencies(molecule,spectrum,gaussian_file,&
    & ramanfl,unit)
    !----------------------------------------------------------------
    ! Driver for gaussian fake output
    !
    ! written by Carsten Kind, Uni Erlangen, 2001
    !----------------------------------------------------------------
    use snf_molecule, only : type_atom
    use snf_spectrum, only : type_peak
    implicit none
    type(type_atom),intent(in) :: molecule(:)
    type(type_peak),intent(inout) :: spectrum(:)
    character(len=*),intent(in) :: gaussian_file
    integer,optional,intent(in) :: unit
    integer :: gauss_unit,out
    logical :: ramanfl
    if(present(unit)) then
       out=unit
    else
       out=6
    endif
    gauss_unit=55
    open(unit=gauss_unit,file=gaussian_file,status='unknown')
    write (gauss_unit,'('' Entering Gaussian System'')') 
    write (gauss_unit,'('' *********************************************'')') 
    write (gauss_unit,'('' Gaussian 98:'')') 
    write (gauss_unit,'('' frequency fake output'')') 
    write (gauss_unit,'('' *********************************************'')') 
    call print_gaussian_molecule(molecule,unit=gauss_unit)
    call transform_to_cartesian(molecule,spectrum)
    call print_gaussian_norm_modes(molecule,spectrum,ramanfl,unit=gauss_unit)
    write(gauss_unit,*)'  '
    close(gauss_unit)
    write(out,'(''   Created gaussian fake output file : '',A)') gaussian_file
  end subroutine print_gaussian_frequencies

  !************************************************************************

  subroutine transform_to_cartesian(molecule,spectrum)
    !----------------------------------------------------------------
    ! generate normalized cartesian representation of normal modes 
    !
    ! written by Carsten Kind, Uni Erlangen, 2001
    !----------------------------------------------------------------
    use snf_molecule, only : type_atom
    use snf_spectrum, only : type_peak
    implicit none
    type(type_atom),intent(in) :: molecule(:)
    type(type_peak),intent(inout) :: spectrum(:)
    integer i,j,nvar,npeak
    real(double) :: fac
    npeak=ubound(spectrum,1)
    nvar=ubound(spectrum(1)%norm_mode,1)
    do i=1,nvar
       fac=1.d0/sqrt(molecule((i+2)/3)%mass)
       do j=1,npeak
          spectrum(j)%norm_mode(i)=fac*spectrum(j)%norm_mode(i)
       enddo
    enddo
    !----------------------------------------------------------
    ! renormalize
    !----------------------------------------------------------
    do j=1,npeak
       fac=dot_product(spectrum(j)%norm_mode,spectrum(j)%norm_mode)
       if(fac .gt. 0.0001) then
          fac=1.d0/sqrt(fac)
       else
          fac=1.d0
       endif
       spectrum(j)%norm_mode(:)=fac*spectrum(j)%norm_mode(:)
    enddo
  end subroutine transform_to_cartesian

  !************************************************************************

  subroutine print_gaussian_molecule(molecule,unit)
    !----------------------------------------------------------------
    ! gaussian fake output of geometry information
    !
    ! written by Carsten Kind, Uni Erlangen, 2001
    !----------------------------------------------------------------
    use snf_molecule, only : type_atom
    use snf_param
    implicit none
    type(type_atom),intent(in) :: molecule(:)
    integer,optional :: unit
    integer         :: i,natoms,out
    if(present(unit)) then
       out=unit
    else
       out=6
    endif
    natoms= ubound(molecule,1)
    write (out,*) '                        Standard orientation:'
    write (out,*) '--------------------------------------------------------------------'
    write (out,*) ' Center     Atomic     Atomic              Coordinates (Angstroms)'
    write (out,*) ' Number     Number      Type              X           Y           Z'
    write (out,*) '--------------------------------------------------------------------'
    do i=1,natoms
       write(out,110) i,&
            & molecule(i)%nucl_charge,&
            & 0, &
            & molecule(i)%cartcoord(1)*bohr2ang,&
            & molecule(i)%cartcoord(2)*bohr2ang,&
            & molecule(i)%cartcoord(3)*bohr2ang
    enddo
    write (out,*) '--------------------------------------------------------------------'
    write (out,*) '    1 basis functions        1 primitive gaussians'
    write (out,*) '    1 alpha electrons        1 beta electrons'
    write (out,*) 
110 format(i5,i11,i14,4x,3f12.6)
  end subroutine print_gaussian_molecule

  !************************************************************************

  subroutine  print_gaussian_norm_modes(molecule,spectrum,ramanfl,unit)
    !----------------------------------------------------------------
    ! gaussian fake output of frequency informations
    !
    ! written by Carsten Kind, Uni Erlangen, 2001
    !----------------------------------------------------------------
    use snf_molecule, only : type_atom
    use snf_spectrum, only : type_peak
    use snf_param
    implicit none

    type(type_atom),intent(in) :: molecule(:)
    type(type_peak),intent(in) :: spectrum(:)
    integer,optional :: unit
    integer ka,kb,kc,la,lb,nr,nc, i,j,out
    logical,intent(in) :: ramanfl
    if(present(unit)) then
       out=unit
    else
       out=6
    endif
    write (out,*) 'Harmonic frequencies (cm**-1), IR intensities (KM/Mole),'
    write (out,*) 'Raman scattering activities (A**4/amu), Raman depolarization ratios,'
    write (out,*) 'reduced masses (AMU), force constants (mDyne/A) and normal coordinates:'
    nc=ubound(spectrum,1)
    nr=ubound(molecule,1)

    ka=1
    kc=3
60  kb=min0(kc,nc)
    write (out,100) (j,j=ka,kb)
    write (out,105) (spectrum(j)%irrep,j=ka,kb)
    write (out,110) ' Frequencies --',(spectrum(j)%freq,j=ka,kb)
    write (out,110) ' Red. masses --',(spectrum(j)%red_mass,j=ka,kb)
    write (out,110) ' Frc consts  --',(spectrum(j)%force,j=ka,kb)
    write (out,110) ' IR Inten    --',(spectrum(j)%ir_int,j=ka,kb)
    if(ramanfl) then 
      write (out,110) ' Raman Activ --',(spectrum(j)%raman(1)%int,j=ka,kb)
      write (out,110) ' Depolar     --',(spectrum(j)%raman(1)%depol_ratio,j=ka,kb)
    else
      write (out,110) ' Raman Activ --',(0.0d0,j=ka,kb)
      write (out,110) ' Depolar     --',(0.d0,j=ka,kb)
    end if
    write (out,'(a75)') 'Atom AN      X      Y      Z        X      Y      Z        X      Y      Z'
    la=1
70  lb=nr
    do  i=la,lb
       write (out,130) i,molecule(i)%nucl_charge , &
            &       (spectrum(j)%norm_mode(i*3-2), &
            &        spectrum(j)%norm_mode(i*3-1), &
            &        spectrum(j)%norm_mode(i*3),j=ka,kb)
    enddo
    if (lb.eq.nr) go to 90
    go to 70
90  if (kb.eq.nc) then
       return
    endif
    ka=kc+1
    kc=kc+3
    go to 60

100 format (3(20x,i3))
105 format (3x,3(18x,a5))
110 format (a15,f11.4,12x,f11.4,12x,f11.4)
130 format (2i4,3(2x,3f7.2))
  end subroutine print_gaussian_norm_modes

  !************************************************************************

end module snf_gaussian
