
!********************************************************************************
!*   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
!*   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
!*   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
!*
!*
!*   This file is part of the package SNF.
!*
!*   SNF is free software; you can redistribute it and/or modify
!*   it under the terms of the GNU General Public License as published by
!*   the Free Software Foundation; either version 2 of the License, or
!*   (at your option) any later version.
!*
!*   This program is distributed in the hope that it will be useful,
!*   but WITHOUT ANY WARRANTY; without even the implied warranty of
!*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*   GNU General Public License for more details.
!*
!*   You should have received a copy of the GNU General Public License
!*   along with this program; if not, write to the Free Software
!*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!*
!********************************************************************************
module snf_gnuplot
  use f90_kind
  use snf_error

  private
  public :: generate_plots

  !************************************************************************

contains

  !************************************************************************

  subroutine generate_plots(ctrl,spectrum,intno,unit)
    !----------------------------------------------------------------
    ! Driver routine to generate all possible plots
    ! After this call gnuplot files, data files and Postscript files 
    ! reside on disk.
    !
    ! written by Carsten Kind, Uni Erlangen, 2001
    !----------------------------------------------------------------
    use snf_control,only : type_ctrl
    use snf_spectrum,only : type_peak
    implicit none
    type(type_ctrl),intent(in) :: ctrl
    type(type_peak), intent(in) :: spectrum(:)
    integer,optional,intent(in) :: unit
    integer :: info,system,out
    integer,intent(in) :: intno
    if(present(unit)) then
       out=unit
    else
       out=6
    endif
    !----------------------------------------------------------------
    ! write gnuplot and data files
    !----------------------------------------------------------------
    write(out,*)
    write(out,'('' Generate spectra plots'')')
    write(out,'('' ----------------------'')')
    if(    spectrum(1)%freq .le. ctrl%maxfreq .and. &
         & spectrum(ubound(spectrum,1))%freq.ge. ctrl%minfreq) then
       call write_gauss_data(ctrl,spectrum,intno,out)
       call write_line_data(ctrl,spectrum,intno,out)
       call write_gnuplot_file(ctrl,'line',title='ir',unit=out)
       call write_gnuplot_file(ctrl,'gaus',title='ir',unit=out)
       if (ctrl%raman) then
          call write_gnuplot_file(ctrl,'line',title='raman',unit=out)
          call write_gnuplot_file(ctrl,'gaus',title='raman',unit=out)
       endif
       !----------------------------------------------------------------
       ! create encapsulated postscript files with gnuplot
       !----------------------------------------------------------------
       info = system('gnuplot ir_line.gpin')
       if(info .ne. 0) then
          call serror('generate_plots',1,'Could not execute gnuplot properly##&
           &for IR line spectrum !')
       else
          write(out,'(''   Created spectrum eps file  : '',A)')'ir_line.eps'
       endif
       info = system('gnuplot ir_gauss.gpin')
       if(info .ne. 0) then
          call serror('generate_plots',1,'Could not execute gnuplot properly##&
           &for IR gauss spectrum !')
       else
          write(out,'(''   Created spectrum eps file  : '',A)') 'ir_gauss.eps'
       endif
       if (ctrl%raman) then
          info = system('gnuplot raman_line.gpin')
          if(info .ne. 0) then
             call serror('generate_plots',1,'Could not execute gnuplot properly##&
              &for Raman line spectrum !')
          else
             write(out,'(''   Created spectrum eps file  : '',A)') 'raman_line.eps'
          endif
          info = system('gnuplot raman_gauss.gpin')
          if(info .ne. 0) then
             call serror('generate_plots',1,'Could not execute gnuplot properly##&
              &for Raman gauss spectrum !')
          else
             write(out,'(''   Created spectrum eps file  : '',A)') 'raman_gauss.eps'
          endif
       endif
    else
       call serror('generate_plots',2,'No plots created since all peaks are##&
        &out of the selected wavenumber range.')
    endif
  end subroutine generate_plots

  !************************************************************************

  subroutine write_gnuplot_file(ctrl,typ,title,unit)
    !----------------------------------------------------------------
    ! write gnuplot files according to type and title
    ! possible values for typ: 'line' 'gaus'
    ! possible values for title: 'ir' 'raman'
    !
    ! written by Carsten Kind, Uni Erlangen, 2001
    ! modified by Johannes Neugebauer, Uni Erlangen, 2001
    !----------------------------------------------------------------
    use snf_control,only : type_ctrl
    implicit none
    type(type_ctrl),intent(in) :: ctrl
    character(len=4),intent(in) :: typ
    character(len=*),optional,intent(in) :: title
    integer,optional,intent(in) :: unit
    character(len=30) :: filename
    integer :: gpin,out
    if(present(unit)) then
       out=unit
    else
       out=6
    endif
    gpin=10
    if(title.eq.'ir' .and. typ.eq.'line') then
       filename='ir_line'
    elseif(title.eq.'ir' .and. typ.eq.'gaus') then
       filename='ir_gauss'
    elseif(title.eq.'raman' .and. typ.eq.'line') then
       filename='raman_line'
    elseif(title.eq.'raman' .and. typ.eq.'gaus') then
       filename='raman_gauss'
    else
       call serror('write_gnuplot_file',2,'Invalid types or titles')
    endif
    open (unit=gpin,file=trim(filename)//'.gpin',form='formatted')
    write(gpin,*)'set term postscript color'
    write(gpin,*)'set term postscript eps'
    write(gpin,*)'set output "',trim(filename),'.eps"'
    if(title.eq.'ir') then
       write(gpin,*)'set title ''Infrared spectrum'''
    else
       write(gpin,*)'set title ''Raman spectrum'''
    endif
    write(gpin,*)'set xlabel ''wavenumber [1/cm]'''
    write(gpin,'(''set xrange ['',f6.1,'':'',f6.1,'']'')')ctrl%minfreq,ctrl%maxfreq
    if(title.eq.'ir') then
       write(gpin,*)'set ylabel ''Absorption coefficient [km/mol]'''
       if(typ.eq.'line') then
          write(gpin,'((a))')'plot "line_spectrum.dat"  u 1:2 w l'
       else
          write(gpin,'((a))')'plot "gauss_spectrum.dat"  u 1:2 w l'
       endif
    else
       write(gpin,*)'set ylabel ''Raman intensity [A^4/amu]'''
       if(typ.eq.'line') then
          write(gpin,'((a))')'plot "line_spectrum.dat"  u 1:3 w l'
       else
          write(gpin,'((a))')'plot "gauss_spectrum.dat"  u 1:3 w l'
       endif
    endif
    close(gpin)
    write(out,'(''   Created gnuplot input file : '',A)') &
!CJN         &    filename//'.gpin'
         &    trim(filename)//'.gpin'
!CBAH     &    trim(trim(filename)//'.gpin')
  end subroutine write_gnuplot_file

  !************************************************************************

  subroutine write_line_data(ctrl,spectrum,intno,unit)
    !----------------------------------------------------------------
    ! write data file for line version of spectra
    ! 1. column are frequencies
    ! 2. column are IR intensities
    ! 3. column are Raman intensities
    !
    ! Note: IR and Raman intensities are summed for all lines which
    !       differ by less than >thres< in their frequencies.
    !
    ! written by Johannes Neugebauer, Uni Erlangen, 2001
    ! based on the former version by Carsten Kind, Uni Erlangen, 2001
    ! (can be found as write_line_data_old_and_wrong)
    !----------------------------------------------------------------
    use snf_spectrum,only : type_peak
    use snf_control,only : type_ctrl
    implicit none
    type(type_peak), intent(in) :: spectrum(:)
    type(type_ctrl),intent(in) :: ctrl
    real(double) :: ir_int,raman_int,thres,relthres
    integer,optional,intent(in) :: unit
    integer :: i,n,dat,out
    integer,intent(in) :: intno
    logical :: flag
    parameter(relthres=1.5d-3)
    if(present(unit)) then
       out=unit
    else
       out=6
    endif
    dat=45
    n = ubound(spectrum,1)
    open (unit=dat,file='line_spectrum.dat',form='formatted')
    thres=abs(ctrl%maxfreq-ctrl%minfreq)*relthres
    ir_int=0.0d0
    raman_int=0.0d0
    flag=.true.
    do i=1,n
       if(flag) then
         ir_int=spectrum(i)%ir_int
         if(ctrl%raman) raman_int=spectrum(i)%raman(intno)%int
       end if
       if(i.lt.n) then
         if(abs(spectrum(i)%freq-spectrum(i+1)%freq).le.thres) then
           ir_int=ir_int+spectrum(i+1)%ir_int
           if(ctrl%raman) raman_int=raman_int+spectrum(i+1)%raman(intno)%int
           flag=.false.
           cycle
         end if
       end if
       if(ctrl%raman) then
          write(dat,'(3f10.4)') spectrum(i)%freq,0.d0,0.d0
          write(dat,'(3f10.4)') spectrum(i)%freq,ir_int,raman_int
          write(dat,*)
       else
          write(dat,'(2f10.4)') spectrum(i)%freq,0.d0
          write(dat,'(2f10.4)') spectrum(i)%freq,ir_int
          write(dat,*)
       endif
       ir_int=0.0d0
       raman_int=0.0d0
       flag=.true.
    end do
    close(dat)
    write(out,'(''   Created gnuplot data file  : '',A)')'line_spectrum.dat'
  end subroutine write_line_data

  !************************************************************************

!  subroutine write_line_data_old_and_wrong(ctrl,spectrum,unit)
!    !----------------------------------------------------------------
!    ! write data file for line version of spectra
!    ! 1. column are frequencies
!    ! 2. column are IR intensities
!    ! 3. column are Raman intensities
!    !
!    ! written by Carsten Kind, Uni Erlangen, 2001
!    !----------------------------------------------------------------
!    use snf_spectrum,only : type_peak
!    use snf_control,only : type_ctrl
!    implicit none
!    type(type_peak), intent(in) :: spectrum(:)
!    type(type_ctrl),intent(in) :: ctrl
!    integer,optional,intent(in) :: unit
!    integer :: i,n,dat,out
!    if(present(unit)) then
!       out=unit
!    else
!       out=6
!    endif
!    dat=45
!    n = ubound(spectrum,1)
!    open (unit=dat,file='line_spectrum.dat',form='formatted')
!    do i=1,n
!       if(ctrl%raman) then
!          write(dat,'(3f10.4)') spectrum(i)%freq,0.d0,0.d0
!          write(dat,'(3f10.4)') spectrum(i)%freq,spectrum(i)%ir_int,&
!               & spectrum(i)%raman_int
!          write(dat,*)
!       else
!          write(dat,'(2f10.4)') spectrum(i)%freq,0.d0
!          write(dat,'(2f10.4)') spectrum(i)%freq,spectrum(i)%ir_int
!          write(dat,*)
!       endif
!    end do
!    close(dat)
!    write(out,'(''   Created gnuplot data file  : '',A)')'line_spectrum.dat'
!  end subroutine write_line_data_old_and_wrong

  !************************************************************************

  subroutine write_gauss_data(ctrl,spectrum,intno,unit)
    !----------------------------------------------------------------
    ! write data file for gaussian broaded version of spectra
    ! 1. column are frequencies
    ! 2. column are IR intensities
    ! 3. column are Raman intensities
    !
    ! written by Carsten Kind, Uni Erlangen, 2001
    !----------------------------------------------------------------
    use snf_spectrum,only : type_peak
    use snf_control,only : type_ctrl
    implicit none
    type(type_peak), intent(in) :: spectrum(:)
    type(type_ctrl),intent(in) :: ctrl
    integer,optional,intent(in) :: unit
    integer :: i,mode,nmodes,dat,out
    integer,intent(in) :: intno
    real(double) :: increment,xval,ir_val,raman_val,tmpfac,alpha,fac2p
    if(present(unit)) then
       out=unit
    else
       out=6
    endif
    dat=45
    alpha=1.d0/(2*ctrl%width**2)
    fac2p=1.d0/sqrt(4*acos(0.d0))
    nmodes = ubound(spectrum,1)
    increment=(ctrl%maxfreq-ctrl%minfreq)/(ctrl%npoint-1)
    open (unit=dat,file='gauss_spectrum.dat',form='formatted')
    xval=ctrl%minfreq
    do i=1,ctrl%npoint
       ir_val=0.d0
       raman_val=0.d0
       do mode=1,nmodes
          tmpfac=exp(-alpha*(xval-spectrum(mode)%freq)**2)
          ir_val=ir_val+spectrum(mode)%ir_int*tmpfac
          if(ctrl%raman) raman_val=raman_val+spectrum(mode)%raman(intno)%int*tmpfac
       enddo
       write(dat,'(3f10.4)') xval,fac2p/ctrl%width*ir_val,fac2p/ctrl%width*raman_val
       xval=xval+increment
    enddo
    close(dat)
    write(out,'(''   Created gnuplot data file  : '',A)')'gauss_spectrum.dat'
  end subroutine write_gauss_data

  !************************************************************************

end module snf_gnuplot
