
!********************************************************************************
!*   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
!*   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
!*   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
!*
!*
!*   This file is part of the package SNF.
!*
!*   SNF is free software; you can redistribute it and/or modify
!*   it under the terms of the GNU General Public License as published by
!*   the Free Software Foundation; either version 2 of the License, or
!*   (at your option) any later version.
!*
!*   This program is distributed in the hope that it will be useful,
!*   but WITHOUT ANY WARRANTY; without even the implied warranty of
!*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*   GNU General Public License for more details.
!*
!*   You should have received a copy of the GNU General Public License
!*   along with this program; if not, write to the Free Software
!*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!*
!********************************************************************************
module snf_hessian
  use f90_kind
  use snf_error

  private
  public :: create_hessian_xyz, print_hessian
  public :: destroy_hessian_xyz,expand_hessian, check_hessian
  public :: balance_hessian,symmetrize_hessian
  !************************************************************************


contains

  !************************************************************************

  subroutine create_hessian_xyz(hessian_xyz,gradients,distortion)
    implicit none
    real(double),intent(in) ::gradients(:,:,:),distortion(:)
    real(double), pointer :: hessian_xyz(:,:)
    integer :: i,j,nvar,idum
    !----------------------------------------------------------------
    ! calculation of second derivative of ernergy with following formula:
    ! 
    !  d^2e      de      de  
    ! ------ =   ---  -  --- 
    ! dx1dxi     dxi     dxi 
    !           --------------
    !           2*distortion(x1)
    !
    ! unit is hartree/bohr^2
    !
    ! written by Carsten Kind and Markus Reiher, Uni Erlangen, 1999
    !----------------------------------------------------------------
    nvar=ubound(gradients,1)
    idum=ubound(gradients,3)
    allocate(hessian_xyz(nvar,nvar))
    if (idum.eq.2) then
       do i=1,nvar
          do j=1,nvar
             hessian_xyz(j,i)=0.5d0/distortion(i)* &
                  &           (gradients(j,i,1)-gradients(j,i,2))
          enddo
       enddo
    else if (idum.eq.4) then
       do i=1,nvar
          do j=1,nvar
             hessian_xyz(j,i)=1.0d0/(24.0d0*distortion(i))* &
                  &           ( -2.0d0*gradients(j,i,1) &
                  &            +16.0d0*gradients(j,i,2) &
                  &            -16.0d0*gradients(j,i,3) &
                  &             +2.0d0*gradients(j,i,4))
          enddo
       enddo
    else
       call serror('create_hessian_xyz',2,'Wrong ctrl%numderiv value!')
    end if
  end subroutine create_hessian_xyz

  !************************************************************************

  subroutine print_hessian(hessian_xyz,filename)
    !----------------------------------------------------------------
    !     output routine
    !----------------------------------------------------------------
    implicit none
    real(double),intent(in) :: hessian_xyz(:,:)
    character*(*) filename
    integer :: i,j,nvar
    nvar=ubound(hessian_xyz,1)
    open(unit=45,file=filename,status='unknown')
    write(45,*) &
         & 'hessian matrix in cartesian coordinates'
    write(45,*)'nvar=',nvar    
    write(45,'(4d18.10)')((hessian_xyz(i,j),j=1,i),i=1,nvar)
    close (45)
  end subroutine print_hessian

  !************************************************************************

  subroutine expand_hessian(hessian_xyz,rep,symm_index)
    use snf_symmat,only : symel_mat
    use snf_tools
    !----------------------------------------------------------------
    ! copy hessian calculated on irreducible atoms to the 
    ! reducible atoms. this is done by multiplying symmetry representations in 
    ! the derivative basis from the left and from the right with the dipol 
    ! moment derivates and selecting only the required columns.
    !
    ! written by Carsten Kind, Uni Erlangen, 1999
    !----------------------------------------------------------------
    implicit none
    real(double),pointer :: hessian_xyz(:,:),rot_hessian(:,:),expanded_hessian(:,:),tmpmat(:,:)
    real(double),intent(in) :: rep(:,:,:)
    integer, intent(in) :: symm_index(:)
    integer :: s,i,offset,nvar,nsymel
    nsymel=ubound(rep,3)
    nvar=ubound(hessian_xyz,2)
    allocate(rot_hessian(nvar,nvar),expanded_hessian(nvar,nvar),tmpmat(nvar,nvar))
    do s=1,nsymel

!-------------------------------------------------------------
! matrix multiplication might be included in if-statement ?!
! (JN, 08/2001)
!-------------------------------------------------------------

       call dgemm('N','N',nvar,nvar,nvar,1.D0,hessian_xyz(1,1),nvar,rep(1,1,s),nvar,0.D0,tmpmat(1,1),nvar)
       call dgemm('T','N',nvar,nvar,nvar,1.D0,rep(1,1,s),nvar,tmpmat(1,1),nvar,0.D0,rot_hessian(1,1),nvar)
       do i=1,ubound(symm_index,1)
          if(symm_index(i) .eq. s) then
             offset=(i-1)*3
             expanded_hessian(:,offset+1:offset+3)=rot_hessian(:,offset+1:offset+3)
          endif
       enddo
    enddo
    deallocate(hessian_xyz,rot_hessian,tmpmat)
    hessian_xyz => expanded_hessian
  end subroutine expand_hessian

  !************************************************************************

  subroutine symmetrize_hessian(h_xyz,rep)
    !----------------------------------------------------------------
    ! written by Carsten Kind, Uni Erlangen, 1999
    ! symmetry check included 
    ! by Johannes Neugebauer, Uni Erlangen, 2001
    !----------------------------------------------------------------
    use snf_tools
    implicit none
    real(double),intent(inout) :: h_xyz(:,:)
    real(double) :: fac,errsum
    real(double),intent(in) :: rep(:,:,:)
    real(double),pointer :: h_rot(:,:),tmpmat(:,:),tmpmat2(:,:)
    integer :: s,nvar,nsymel,i,j
    character(len=60) :: errbuff
    nvar=ubound(h_xyz,2)
    nsymel=ubound(rep,3)
    fac=1.D0/dble(nsymel)
    allocate(h_rot(nvar,nvar),tmpmat(nvar,nvar),tmpmat2(nvar,nvar))
    h_rot=0.0D0
    do s=1,nsymel
       call dgemm('N','N',nvar,nvar,nvar,1.D0,h_xyz(1,1),nvar,rep(1,1,s),nvar,0.D0,tmpmat(1,1),nvar)
!CJN >>>>>>>>>>>>>
       tmpmat2=0.0d0
!       call dgemm('T','N',nvar,nvar,nvar,1.D0,rep(1,1,s),nvar,tmpmat(1,1),nvar,1.D0,h_rot(1,1),nvar)
       call dgemm('T','N',nvar,nvar,nvar,1.D0,rep(1,1,s),nvar,tmpmat(1,1),nvar,0.D0,tmpmat2(1,1),nvar)
       errsum=0.0d0
       do i=1,nvar
         do j=1,nvar
           errsum=errsum+abs(h_xyz(i,j)-tmpmat2(i,j))
         end do
       end do
       errsum=errsum/dble(nvar)/dble(nvar)
       if(errsum.gt.1.0d-3) then
         write(errbuff,'(a,f8.4,a,i3)') 'average error = ',errsum, ' for s = ',s
         call serror('symmetrize_hessian',0,trim(adjustl(errbuff)))
       end if
       h_rot=h_rot + tmpmat2
!CJN <<<<<<<<<<<<<
    enddo
    h_xyz=fac*h_rot
    deallocate(h_rot,tmpmat,tmpmat2)
  end subroutine symmetrize_hessian

  !************************************************************************

  subroutine destroy_hessian_xyz(hessian_xyz)
    implicit none
    !----------------------------------------------------------------
    ! clean up memory
    !
    ! written by Carsten Kind, Uni Erlangen, 1999
    !----------------------------------------------------------------
    real(double), pointer :: hessian_xyz(:,:)
    if(associated(hessian_xyz)) deallocate(hessian_xyz)
  end subroutine destroy_hessian_xyz

  !************************************************************************

  subroutine check_hessian(hessian_xyz,ctrl,symm_index,istepfl,new_restart,unit)
    !----------------------------------------------------------------
    ! Determine which atoms are to be recalculated based on the asymmetry of 
    ! the Hessian matrix.
    !
    ! written by Carsten Kind, Uni Erlangen, 1999
    !----------------------------------------------------------------
    use snf_control, only: type_ctrl
    implicit none

    type type_error
       integer :: index,number
       real(double) :: value
    end type type_error

    type(type_ctrl) :: ctrl      ! essential control flags and parameters
    real(double),pointer :: hessian_xyz(:,:)
    integer,optional :: unit
    character(len=3) :: xyz
    integer :: i,j,nvar,atom_i,xyz_i,max_diff,index_max_diff,out
    type(type_error) :: tmp_error
    type(type_error),pointer :: error_vec(:)
    real(double) :: threshold,maxerr,meanerr,err
    integer, pointer :: istepfl(:),symm_index(:)
    logical,intent(out) :: new_restart
    if(present(unit)) then
       out=unit
    else
       out=6
    endif
    new_restart = .false.
    nvar=ubound(hessian_xyz,1)
    allocate(error_vec(nvar))
    xyz='xyz'
    threshold = 6.d01 * ctrl%cstep**2
    !----------------------------------------------------------------
    ! count all differences between h(i,j) and h(j,i) larger than 100*cstep**2
    !----------------------------------------------------------------
    maxerr=0.0d0
    do i=1,nvar
       error_vec(i)%number=0
       error_vec(i)%value=0.d0
       do j=1,nvar
          err=abs(hessian_xyz(i,j)-hessian_xyz(j,i))
          if(err.gt.threshold) then
             error_vec(i)%value = error_vec(i)%value + err
             error_vec(i)%number=error_vec(i)%number+1
          endif
          maxerr=max(maxerr,err)
          meanerr=meanerr+err*err
       enddo
    enddo
    meanerr=sqrt(meanerr/(nvar*nvar-1))
    write(out,*)
    write(out,'('' Hessian symmetry check information   '')')
    write(out,'('' ------------------------------------ '')')
    write(out,'(''   Maximal absolute error |hij-hji| : '',G9.2)')maxerr
    write(out,'(''   Standard deviation               : '',G9.2)')meanerr
    !----------------------------------------------------------------
    ! save indices
    !----------------------------------------------------------------
    do i=1,nvar
       error_vec(i)%index=i
    enddo
    !----------------------------------------------------------------
    ! sort error-vector with decreasing order of error
    !----------------------------------------------------------------
    do i=1,nvar-1
       do j=i+1,nvar
          if(error_vec(i)%number .lt. error_vec(j)%number) then
             tmp_error = error_vec(i)
             error_vec(i) = error_vec(j)
             error_vec(j)=tmp_error
          endif
       enddo
    enddo
    if(error_vec(1)%number .gt. 0) then
       !----------------------------------------------------------------
       ! determine index of the vector with the largest decrease of 
       ! of errors according to the next vector
       !----------------------------------------------------------------
       write(out,*)
       write(out,'(''******************************************************'')')
       write(out,'(''ERROR: Hessian is not symmetric'')')
       write(out,*)
       write(out,'(''Error criterion: |hij-hji| > threshold='',f15.6)') &
            &  threshold
       write(out,*)
       write(out,'('' atom  | coord.|  error    | sum of |hij-hji|'')')
       write(out,'(''number | [xyz] |  counts   | for all errors.'')')
       write(out,'(''-------|-------|-----------|-----------------'')')
       do i=1,nvar
          atom_i=(error_vec(i)%index+2)/3
          xyz_i=error_vec(i)%index-(atom_i-1)*3
          write(out,'(i5,2x,''|   '',a1,''   |'',i6,''     |'',f15.6)') &
               &    atom_i,xyz(xyz_i:xyz_i),&
               &    error_vec(i)%number, error_vec(i)%value
       enddo
       write(out,'(''******************************************************'')')
       write(out,*)
       do i=2,nvar
          if((error_vec(i-1)%number-error_vec(i)%number).gt.max_diff) then
             max_diff = error_vec(i-1)%number-error_vec(i)%number
             index_max_diff=i-1
          endif
       enddo
       write(out,*)
       !----------------------------------------------------------------
       ! determine all irreducible vectors with the largest number of errors
       ! and print warning
       !----------------------------------------------------------------
       do i=1,index_max_diff
          if(symm_index((error_vec(i)%index+2)/3) .eq. 1) then
             atom_i=(error_vec(i)%index+2)/3
             xyz_i=error_vec(i)%index-(atom_i-1)*3
             istepfl(error_vec(i)%index*2-1) = 0
             istepfl(error_vec(i)%index*2) = 0
             write(out,'(''atom '',i3,'' coordinate '',a1,'' has to be recalculated'')') &
                  & atom_i,xyz(xyz_i:xyz_i)
             new_restart = .true.
          endif
       enddo
       write(out,*)
    endif
    deallocate(error_vec)
  end subroutine check_hessian

  !************************************************************************

  subroutine balance_hessian(hessian)
    !----------------------------------------------------------------
    ! symmetrize Hessian
    !
    ! written by Carsten Kind, Uni Erlangen, 1999
    !----------------------------------------------------------------
    implicit none
    real(double),intent(inout) :: hessian(:,:)
    integer i,j,nvar
    nvar=ubound(hessian,1)
    do j=1,nvar
       do i=1,j-1
          hessian(i,j)=0.5d0*(hessian(i,j)+hessian(j,i))
          hessian(j,i) = hessian(i,j)
       enddo
    enddo
  end subroutine balance_hessian

  !************************************************************************

  subroutine matrix_bfgs_update(hessian,step,gradient,old_gradient)
    !----------------------------------------------------------------
    ! BFGS update (not tested and used!)
    !
    ! written by Carsten Kind, Uni Erlangen, 2000
    !----------------------------------------------------------------
    implicit none
    real(double),intent(inout) :: hessian(:,:)
    real(double),intent(in) ::  step(:),gradient(:),old_gradient(:)
    real(double),pointer :: yvec(:)
    real(double) :: fac,ddot
    integer i,j,nvar
    nvar=ubound(gradient,1)
    allocate(yvec(nvar))
    call dcopy(nvar,gradient,1,yvec,1)
    call daxpy(nvar,-1,old_gradient,1,yvec)
    fac = 1.d0/ddot(nvar,yvec,1,step,1)
    do i=1,nvar
       do j=1,i
          hessian(j,1) = hessian(j,1) + fac*yvec(i)*yvec(j)
       enddo
    enddo
    call dsymv('u',nvar,1.d0,hessian,nvar,step,1,0.d0,yvec,1)
    fac = -1.d0/ddot(nvar,step,1,yvec,1)
    call dsyr('u',nvar,fac,yvec,1,hessian,nvar)
    deallocate(yvec,yvec)
  end subroutine matrix_bfgs_update

  !************************************************************************

  subroutine matrix_psb_update(hessian,step,gradient,old_gradient)
    !----------------------------------------------------------------
    ! PSB update (not tested and used!)
    !
    ! written by Carsten Kind, Uni Erlangen, 2000
    !----------------------------------------------------------------
    real(double),intent(inout) :: hessian(:,:)
    real(double),intent(in) ::  step(:),gradient(:),old_gradient(:)
    real(double),pointer :: tvec(:)
    real(double) :: fac,ddot
    integer nvar
    nvar=ubound(gradient,1)
    allocate(tvec(nvar))
    call dcopy(nvar,gradient,1,tvec,1)
    call daxpy(nvar,-1.d0,old_gradient,1,tvec)
    call dsymv('u',nvar,-1.d0,hessian,1,nvar,step,1,1.d0,tvec,1)
    fac=1.d0/sqrt(ddot(nvar,step,1,step,1))
    call dsyr2('u',nvar,fac,tvec,1,step,1,hessian,nvar)
    fac=-fac*fac * ddot(nvar,tvec,1,step,1)
    call dsyr('u',nvar,fac,step,1,hessian,nvar)
    deallocate(tvec)
  end subroutine matrix_psb_update

  !************************************************************************

  subroutine bfgs_update(hessian,step,gradient,old_gradient)
    !----------------------------------------------------------------
    ! BFGS update (not tested and used!)
    !
    ! written by Carsten Kind, Uni Erlangen, 2000
    !----------------------------------------------------------------
    implicit none
    real(double),intent(inout) :: hessian(:,:)
    real(double),intent(in) ::  step(:),gradient(:),old_gradient(:)
    real(double),pointer :: yvec(:),tvec(:)
    real(double) :: norm_ts,dds
    integer i,j,nvar
    nvar=ubound(gradient,1)
    allocate(yvec(nvar),tvec(nvar))
    tvec(:)=0.d0
    do  j=1,nvar
       do  i=1,nvar
          tvec(i)=tvec(i) + hessian(i,j)*step(j)
       enddo
    enddo
    yvec(:)=gradient(:)-old_gradient(:)
    dds=dot_product(yvec,step)
    !----------------------------------------------------------------
    !  if dds is negative, retention of positive definiteness is not
    !  guaranteed. print a warning and skip update this cycle.
    !----------------------------------------------------------------
    norm_ts=dot_product(tvec,step)
    do  i=2,nvar
       do  j=1,i-1
          hessian(i,j)=hessian(i,j) + &
               & (yvec(i)*yvec(j))/dds - (tvec(i)*tvec(j))/norm_ts
          hessian(j,i)=hessian(i,j)
       enddo
    enddo
    do  i=1,nvar
       hessian(i,i)=hessian(i,i)+(yvec(i)*yvec(i))/dds - (tvec(i)*tvec(i))/norm_ts
    enddo
    deallocate(yvec,tvec)
  end subroutine bfgs_update

  !************************************************************************

  subroutine psb_update(hessian,step,gradient,old_gradient)
    !----------------------------------------------------------------
    ! PSB update (not tested and used!)
    !
    ! written by Carsten Kind, Uni Erlangen, 2000
    !----------------------------------------------------------------
    real(double),intent(inout) :: hessian(:,:)
    real(double),intent(in) ::  step(:),gradient(:),old_gradient(:)
    real(double),pointer :: tvec(:),yvec(:)
    real(double) :: step_norm2,norm_ts,temp
    integer nvar,i,j
    nvar=ubound(gradient,1)
    allocate(tvec(nvar),yvec(nvar))
    tvec(:)=0.d0
    do  j=1,nvar
       do  i=1,nvar
          tvec(i)=tvec(i) + hessian(i,j)*step(j)
       enddo
    enddo

    do i=1,nvar
       yvec(i)=gradient(i)-old_gradient(i)
       tvec(i)=yvec(i)-tvec(i)
    enddo
    step_norm2=dot_product(step,step)
    norm_ts=dot_product(tvec,step)
    norm_ts=norm_ts/step_norm2
    do i=2,nvar
       do j=1,i-1
          temp=tvec(i)*step(j) + step(i)*tvec(j) - step(i)*norm_ts*step(j)
          hessian(i,j)=hessian(i,j)+temp/step_norm2
          hessian(j,i)=hessian(i,j)
       enddo
    enddo
    do  i=1,nvar
       temp=step(i)*(2.0d0*tvec(i) - step(i)*norm_ts)
       hessian(i,i)=hessian(i,i)+temp/step_norm2
    enddo
    deallocate(yvec,tvec)
  end subroutine psb_update

  !************************************************************************

end module snf_hessian
