
!********************************************************************************
!*   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
!*   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
!*   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
!*
!*
!*   This file is part of the package SNF.
!*
!*   SNF is free software; you can redistribute it and/or modify
!*   it under the terms of the GNU General Public License as published by
!*   the Free Software Foundation; either version 2 of the License, or
!*   (at your option) any later version.
!*
!*   This program is distributed in the hope that it will be useful,
!*   but WITHOUT ANY WARRANTY; without even the implied warranty of
!*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*   GNU General Public License for more details.
!*
!*   You should have received a copy of the GNU General Public License
!*   along with this program; if not, write to the Free Software
!*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!*
!********************************************************************************
module snf_irred_dist
  !--------------------------------------------------------------
  ! this module allows to generate all data of distortions of 
  ! non-redundant atoms, which are related to other distortions
  ! of the non-redundant atoms by symmtry operations.
  ! (e.g., in H2O lying in the zx-plane, the distortion d(-y) is 
  ! redundant because the results can be obtained by applying
  ! reflections on the results of d(y) ).
  !
  ! written by J. Neugebauer, Uni Erlangen, 2001
  !--------------------------------------------------------------
  use f90_kind
  use snf_error

  private :: origdist,atomofdist
  public :: create_irred_dist,destroy_irred_dist,print_irred_dist
  public :: create_red_grad,prepare_restart2,create_red_dip
  public :: create_red_pol

  !************************************************************************

contains

  subroutine create_irred_dist(molecule,char_table,dist_index,&
     & dsymm_index,irred_dist_index,nirred_dist,nirred_atoms,&
     & irred_atom_index,naus,distortion,printf,unit)
    !----------------------------------------------------------------
    ! determination of "number of irreducible distortion structures",
    ! only distortions of the non-redundant atoms are considered.
    !
    ! written by Johannes Neugebauer, Uni Erlangen, 2001
    !----------------------------------------------------------------
    use snf_molecule,only:type_atom,atom_equal,coord_equal
    use snf_symmat,only:symel_mat
    use snf_char_table,only: type_char_table

    implicit none
    type(type_atom),intent(in) :: molecule(:)
    type(type_char_table),intent(in) :: char_table
    integer,pointer :: dist_index(:),dsymm_index(:),irred_dist_index(:)
    integer,pointer :: irred_atom_index(:)
    integer i,dist1,dist2,atom1,out
    integer natoms,nirred_dist,ndist,nirred_atoms,naus,odist,odist2,count
    real(double),intent(in) :: distortion(:)
    real(double),pointer :: dcoord(:,:),d2coord(:,:),transcoord(:,:)
    real(double) :: threshold
    logical :: isfine,printf
    integer,optional :: unit
    parameter(threshold = 1.0d-4)

    if(present(unit)) then
       out=unit
    else
       out=6
    endif
    !----------------------------------------------------------------
    ! determination of non-redundant distortions ("irreducible dist.")
    !----------------------------------------------------------------
    natoms = ubound(molecule,1)
    ndist = nirred_atoms*3*naus
    allocate(dist_index(3*nirred_atoms*naus),dsymm_index(3*nirred_atoms*naus),&
         & irred_dist_index(3*nirred_atoms*naus),transcoord(3,natoms))
    irred_dist_index(:)=0
    dist_index(:) = -1
    dsymm_index(:) = 0
    nirred_dist = 0
    count=0
    do dist1=1,ndist
       if(dist_index(dist1) .eq. -1) then
          !-------------------------------------------------------------
          ! neglect dummy atoms
          !-------------------------------------------------------------
          odist=origdist(irred_atom_index,naus,dist1)
          atom1=atomofdist(naus,odist)
          if(molecule(atom1)%element.eq.'Q') cycle
          nirred_dist=nirred_dist+1
          irred_dist_index(nirred_dist)=origdist(irred_atom_index,naus,dist1)
          !-------------------------------------------------------------
          ! create distorted structure 1
          !-------------------------------------------------------------
          call distcoord(molecule,distortion,odist,dcoord,naus)
          !----------------------------------------------------------------
          ! transform dist1 with all symmetry elements of the point group
          !----------------------------------------------------------------
          do i=1,char_table%nsymel
             transcoord(:,atom1)=matmul(symel_mat(char_table%symel(i)), &
                 &  dcoord(:,atom1))
             isfine = .false.
             !-------------------------------------------------------------
             ! compare coordinates of transformed dcoord1 and d2coord
             !-------------------------------------------------------------
             dist2=dist1-mod(dist1-1,3*naus)
             odist2=origdist(irred_atom_index,naus,dist2)
             call distcoord(molecule,distortion,odist2,d2coord,naus)
             do while (.not.coord_equal(transcoord(:,atom1),&
              &d2coord(:,atom1),threshold))
                !-------------------------------------------------------------
                ! create distorted structure 2
                !-------------------------------------------------------------
                count=count + 1
                dist2=dist2+1
                if (dist2.gt.dist1-mod(dist1-1,3*naus)+3*naus-1) exit
                odist2=origdist(irred_atom_index,naus,dist2)
                call distcoord(molecule,distortion,odist2,d2coord,naus)
             enddo 
             if(dist2.gt.dist1-mod(dist1-1,3*naus)+3*naus-1) then
                isfine = .false.
             else 
                isfine=.true.
             endif
             !----------------------------------------------------------------
             ! assign symmetry equivalent dist. to irred. dist. and 
             ! correspondig symmetry operation if this dist. has not been 
             ! assigned to another dist. yet
             !----------------------------------------------------------------
             if(isfine) then
               if(dist_index(dist2) .eq. -1) then
                 dist_index(dist2) = dist1
                 dsymm_index(dist2) = i
               endif
             end if
           enddo
       endif
    enddo
    if(printf) call print_irred_dist(dsymm_index,irred_atom_index,&
    &nirred_atoms,nirred_dist,dist_index,irred_dist_index,naus,out)
  end subroutine create_irred_dist

  !************************************************************************

  subroutine print_irred_dist(dsymm_index,irred_atom_index,&
   &nirred_atoms,nirred_dist,dist_index,irred_dist_index,naus,unit)
    !----------------------------------------------------------------
    ! print information about symmetry redundant distortions
    !
    ! written by Johannes Neugebauer, Uni Erlangen, 2001
    !----------------------------------------------------------------
    implicit none
    integer,intent(in) :: dsymm_index(:),irred_atom_index(:)
    integer,intent(in) :: dist_index(:),irred_dist_index(:)
    integer :: nirred_atoms,i,out,nirred_dist,oi,ooi,naus,ai
    integer :: atomdist,acoordnr,cnaus
    integer,optional :: unit
    character(len=1) :: ci,csym(3)
    character(len=2) :: disf
    data csym /'x','y','z' /

    if(present(unit)) then
       out=unit
    else
       out=6
    endif
    write(out,*)
    write(out,'('' Information about symmetry redundant distortions'')')
    write(out,'('' ----------------------------------------------'')')
    write(out,'(''   Total number of distortions'')')
    write(out,'(''   for non-redundant atoms            :'',i3)') nirred_atoms*3*naus
    write(out,'(''   Number of non-redundant distortions:'',i3)') nirred_dist
    write(out,*)
    write(out,'(''   List of non-redundant distortions:'')')
    write(out,'(3X,16i4)')(irred_dist_index(i),i=1,nirred_dist)
    write(out,*)
    write(out,'(''   dist No. | atom | coord. | rel. nonred. dist. | symmetry op.'')')
    write(out,'(''   ---------|------|--------|--------------------|-------------'')')
    do i=1,ubound(dist_index,1)
       oi=origdist(irred_atom_index,naus,i)
       ooi=origdist(irred_atom_index,naus,dist_index(i))
       ai=atomofdist(naus,ooi)
       atomdist=mod(oi-1,3*naus)+1
       acoordnr=(atomdist-1)/naus +1
       cnaus=mod(atomdist-1,naus)+1
       ci=csym(acoordnr)
       if(nint(dfac(naus,cnaus)).eq.-1) then
         disf=' -'
       else if(nint(dfac(naus,cnaus)).eq.1) then
         disf=' +'
       else if(nint(dfac(naus,cnaus)).eq.-2) then
         disf='-2'
       else if(nint(dfac(naus,cnaus)).eq.2) then
         disf='+2'
       else
         disf='  '
       end if
       write(out,'(3X,i7,a3,i5,a3,3x,a2,a1,a3,i18,a2,i11)') oi,'  |',&
         &ai,' | ',disf,ci,' | ',ooi,' |',dsymm_index(i)
    enddo
    write(out,*)   
  end subroutine print_irred_dist

  !************************************************************************

  subroutine destroy_irred_dist(dist_index,dsymm_index,irred_dist_index)
    !----------------------------------------------------------------
    !
    ! written by Johannes Neugebauer, Uni Erlangen, 2001
    !----------------------------------------------------------------
    implicit none
    integer,pointer :: dist_index(:),dsymm_index(:),irred_dist_index(:)

    if (associated(dist_index)) deallocate(dist_index)
    if (associated(dsymm_index)) deallocate(dsymm_index)
    if (associated(irred_dist_index)) deallocate(irred_dist_index)
  end subroutine destroy_irred_dist

  !************************************************************************

  integer function atomofdist(naus,dist)
    !----------------------------------------------------------------
    ! given the number of a particular distortion, this function 
    ! yields the number of the distorted atom.
    !
    ! written by Johannes Neugebauer, Uni Erlangen, 2001
    !----------------------------------------------------------------
    implicit none
    integer dist,naus
  
    atomofdist=(dist-1)/(3*naus) + 1
  end function atomofdist

  !************************************************************************

  integer function origdist(irred_atom_index,naus,dist)
    !----------------------------------------------------------------
    ! given the distortion number "dist" in the list of distortions
    ! of the non-redundant atoms, this function yields the distortion 
    ! number in the list of all distortions of all atoms
    !
    ! written by Johannes Neugebauer, Uni Erlangen, 2001
    !----------------------------------------------------------------

    implicit none
    integer, intent(in) :: irred_atom_index(:)
    integer dist,naus,nat
    nat=irred_atom_index(atomofdist(naus,dist))
    origdist=(nat-1)*3*naus+mod(dist-1,3*naus)+1
  end function origdist

  !************************************************************************

  subroutine distcoord(molecule,distortion,ndist,coord,naus)
    !----------------------------------------------------------------
    ! ndist:  distortion no. (total no. of dist.: natoms*3*naus)
    ! atomi:  atom for which coordinates are distorted
    ! atomdist: distortion no. of atom atomi
    ! acoordnr: coordinate of atom atomi to be distorted (1,2,3) = (x,y,z)
    ! cnaus: distortion no. of coordinate acoordnr
    ! varnr: variable number to be distorted
    ! dfac: distortion prefactor, e.g. (+1,-1) for naus = 2
    !       or (+2,+1,-1,-2) for naus = 4
    ! 
    ! written by Johannes Neugebauer, Uni Erlangen, 2001
    !----------------------------------------------------------------
    use snf_molecule, only: type_atom

    implicit none
    type(type_atom),intent(in) :: molecule(:)
    real(double),intent(in) :: distortion(:)
    real(double),pointer :: coord(:,:)
    integer,intent(in) :: ndist,naus
    integer :: atomdist,acoordnr,varnr,cnaus
    integer :: i,atomi,natoms

    if(associated(coord)) deallocate(coord)
    natoms=ubound(molecule,1)
    allocate(coord(3,natoms))
    do i=1,natoms
      coord(:,i)=molecule(i)%cartcoord(:)
    end do
    atomi=atomofdist(naus,ndist)
    atomdist=mod(ndist-1,3*naus)+1
    acoordnr=(atomdist-1)/naus +1
    cnaus=mod(atomdist-1,naus)+1
    varnr=(atomi-1)*3+acoordnr
    coord(acoordnr,atomi)=coord(acoordnr,atomi)+dfac(naus,cnaus)&
     &*distortion(varnr)
  end subroutine distcoord

  !************************************************************************
  function dfac(naus,cnaus)
    !----------------------------------------------------------------
    ! calculate prefactor for cstep value for a particular distortion
    !
    ! written by J. Neugebauer, Uni Erlangen, 2001
    !----------------------------------------------------------------
    implicit none 
    integer,intent(in) :: naus,cnaus
    real(double) :: dfac
    dfac=naus/2-cnaus+1
    if(dfac.le.0) dfac=dfac-1
  end function dfac

  !************************************************************************
 
  subroutine print_dcoord(molecule,coord,ndist,naus,unit)
    !----------------------------------------------------------------
    ! output routine, used only for debugging
    !
    ! written by J. Neugebauer, Uni Erlangen, 2001
    !----------------------------------------------------------------
    use snf_molecule,only: type_atom
 
    implicit none
    type(type_atom) :: molecule(:)
    real(double),intent(in) :: coord(:,:)
    integer,optional :: unit
    integer         :: atom,out,ndist,naus,atomi,atomdist,acoordnr
    integer         :: cnaus,varnr
    character(len=1) :: csym(3)
    data csym /'x','y','z' /
    if(present(unit)) then
       out=unit
    else
       out=6
    endif
    atomi=atomofdist(naus,ndist)
    atomdist=mod(ndist-1,3*naus)+1
    acoordnr=(atomdist-1)/naus +1
    cnaus=mod(atomdist-1,naus)+1
    varnr=(atomi-1)*3+acoordnr
    write(out,*)
    write(out,'('' Coordinates for dist. No.: '',i4)') ndist
    write(out,'('' -------------------------------'')')
    write(out,*) 
    write(out,'(a,i4)') 'No. of distorted atom:           ',atomi
    write(out,'(a,i4)') 'No. of distortion for this atom: ',atomdist
    write(out,'(a,a)') 'Distorted coordinate:               ',csym(acoordnr)
    write(out,'(a,i4)') 'No. of dist. for this coord.:    ',cnaus
    write(out,'(a,i4)') 'No. of this degree of freedom:   ',varnr
    write(out,'(a,f16.8)') 'Distortion prefactor: ',dfac(naus,cnaus)
    write(out,'("   No El  OZ",12x,"m",15x,"x",15x,"y",15x,"z")')
    do atom = 1, ubound(molecule,dim=1)
       write(out,'(''   '',i2,1x,a2,1x,i3,f13.6,3e16.8)')atom,&
            molecule(atom)%element,&
            molecule(atom)%nucl_charge,&
            molecule(atom)%mass,&
            coord(1,atom),&
            coord(2,atom),&
            coord(3,atom)
    end do
  end subroutine print_dcoord
 
  !************************************************************************
  logical function dist_equal(molecule,coord1,coord2,thres,compcount)
    !----------------------------------------------------------------
    ! .true., if molecules coord1 and coord2 are equal.
    !
    ! written by J. Neugebauer, Uni Erlangen, 2001
    !----------------------------------------------------------------
    use snf_molecule,only: type_atom,coord_equal
    implicit none
    type(type_atom) :: molecule(:)
    real(double),intent(in) :: coord1(:,:),coord2(:,:),thres
    integer,pointer :: equal(:)
    integer :: natoms,i,j,compcount

    natoms=ubound(molecule,1)
    allocate(equal(natoms))
    equal=0
    do i=1,natoms
      do j=1,natoms
        if(equal(i).eq.1) cycle
        compcount=compcount+1
        if(coord_equal(coord1(:,i),coord2(:,j),thres).and.&
         &molecule(i)%element.eq.molecule(j)%element) equal(i)=equal(i)+1
      end do
      if(equal(i).eq.0) then
        dist_equal=.false.
        return
      end if
    end do
    dist_equal=.true.
    if((any(equal.gt.1)).or.(any(equal.eq.0))) dist_equal=.false.
  end function dist_equal
 
  !************************************************************************
  subroutine create_red_grad(grad,dist_index, &
     & dsymm_index,nirred_atoms,irred_atom_index,naus,rep)   
     !--------------------------------------------------------------
     ! create gradients for symmetry redundand distorted structures
     ! of symmetry non-redundant atoms.
     !
     ! written by J. Neugebauer, Uni Erlangen, 2001
     !--------------------------------------------------------------
     use snf_char_table,only: type_char_table

     implicit none
     real(double),intent(in) :: rep(:,:,:)
     real(double),pointer :: grad(:,:,:)
     integer,pointer :: dist_index(:),dsymm_index(:)
     integer,pointer :: irred_atom_index(:)
     integer i,odist,atomi,atomdist,acoordnr,cnaus,varnr,odisto
     integer nirred_atoms,naus,ovarnr,ocnaus
     integer oatomdist,oacoordnr,sni
 
     do i=1,nirred_atoms*3*naus
       if(i.ne.dist_index(i)) then
         !-----------------------------------------------------
         ! found redundant distortion. copy entry of related
         ! non-redundant distortion.
         !-----------------------------------------------------
         sni=dsymm_index(i)
         odist=origdist(irred_atom_index,naus,i)
         atomi=atomofdist(naus,odist)
         atomdist=mod(odist-1,3*naus)+1
         acoordnr=(atomdist-1)/naus +1
         cnaus=mod(atomdist-1,naus)+1
         varnr=(atomi-1)*3+acoordnr
         odisto=origdist(irred_atom_index,naus,dist_index(i))
         oatomdist=mod(odisto-1,3*naus)+1
         oacoordnr=(oatomdist-1)/naus +1
         ocnaus=mod(oatomdist-1,naus)+1
         ovarnr=(atomi-1)*3+oacoordnr
         grad(:,varnr,cnaus)=matmul(grad(:,ovarnr,ocnaus),rep(:,:,sni))
       end if
     end do
  end subroutine create_red_grad

  !************************************************************************

 subroutine prepare_restart2(naus,istepfl,irred_dist_index,natoms, &
      &                      nirred_dist)
    !----------------------------------------------------------------
    ! fill up istepfl array according to irreducible distortions
    ! to be calculated
    !
    ! written by J. Neugebauer, Uni Erlangen, 2001
    !----------------------------------------------------------------
    implicit none
    integer,pointer :: istepfl(:)
    integer i,nstept,natoms,naus
    integer,pointer :: irred_dist_index(:)
    integer nirred_dist
    nstept=naus*natoms*3
    if(.not. associated(istepfl)) allocate(istepfl(nstept))
    istepfl(:)=1
    do i=1,nirred_dist
      istepfl(irred_dist_index(i))=0
    end do
  end subroutine prepare_restart2
 
  !************************************************************************

  subroutine create_red_dip(dip,dist_index, &
     & dsymm_index,nirred_atoms,irred_atom_index,naus,char_table)   
     !--------------------------------------------------------------
     ! create dipole moments for symmetry redundand distorted structures
     ! of symmetry non-redundant atoms.
     !
     ! written by J. Neugebauer, Uni Erlangen, 2001
     !--------------------------------------------------------------
     use snf_char_table,only: type_char_table
     use snf_symmat,only:symel_mat

     implicit none
     type(type_char_table),intent(in) :: char_table
     real(double),pointer :: dip(:,:,:)
     integer,pointer :: dist_index(:),dsymm_index(:)
     integer,pointer :: irred_atom_index(:)
     integer i,odist,atomi,atomdist,acoordnr,cnaus,varnr,odisto
     integer nirred_atoms,naus,ovarnr,ocnaus
     integer oatomdist,oacoordnr,sni
 
     do i=1,nirred_atoms*3*naus
       if(i.ne.dist_index(i)) then
         !-----------------------------------------------------
         ! found redundant distortion. copy entry of related
         ! non-redundant distortion.
         !-----------------------------------------------------
         sni=dsymm_index(i)
         odist=origdist(irred_atom_index,naus,i)
         atomi=atomofdist(naus,odist)
         atomdist=mod(odist-1,3*naus)+1
         acoordnr=(atomdist-1)/naus +1
         cnaus=mod(atomdist-1,naus)+1
         varnr=(atomi-1)*3+acoordnr
         odisto=origdist(irred_atom_index,naus,dist_index(i))
         oatomdist=mod(odisto-1,3*naus)+1
         oacoordnr=(oatomdist-1)/naus +1
         ocnaus=mod(oatomdist-1,naus)+1
         ovarnr=(atomi-1)*3+oacoordnr
         dip(:,varnr,cnaus)=matmul(symel_mat(char_table%symel(sni))&
          &,dip(:,ovarnr,ocnaus))
       end if
     end do
  end subroutine create_red_dip
 
  !************************************************************************
     
  subroutine create_red_pol(pol,dist_index, &
     & dsymm_index,nirred_atoms,irred_atom_index,naus,char_table)   
     !--------------------------------------------------------------
     ! create polarizability tensors for symmetry redundand distorted 
     ! structures of symmetry non-redundant atoms.
     !
     ! written by J. Neugebauer, Uni Erlangen, 2001
     !--------------------------------------------------------------
     use snf_char_table,only: type_char_table
     use snf_symmat,only:symel_mat

     implicit none
     type(type_char_table),intent(in) :: char_table
     real(double),intent(inout) :: pol(:,:,:)
     real(double) :: tmpol1(3,3),tmpol2(3,3)
     integer,pointer :: dist_index(:),dsymm_index(:)
     integer,pointer :: irred_atom_index(:)
     integer i,odist,atomi,atomdist,acoordnr,cnaus,varnr,odisto
     integer nirred_atoms,naus,ovarnr,ocnaus
     integer oatomdist,oacoordnr,sni
 
     do i=1,nirred_atoms*3*naus
       if(i.ne.dist_index(i)) then
         !-----------------------------------------------------
         ! found redundant distortion. copy entry of related
         ! non-redundant distortion.
         !-----------------------------------------------------
         sni=dsymm_index(i)
         odist=origdist(irred_atom_index,naus,i)
         atomi=atomofdist(naus,odist)
         atomdist=mod(odist-1,3*naus)+1
         acoordnr=(atomdist-1)/naus +1
         cnaus=mod(atomdist-1,naus)+1
         varnr=(atomi-1)*3+acoordnr
         odisto=origdist(irred_atom_index,naus,dist_index(i))
         oatomdist=mod(odisto-1,3*naus)+1
         oacoordnr=(oatomdist-1)/naus +1
         ocnaus=mod(oatomdist-1,naus)+1
         ovarnr=(atomi-1)*3+oacoordnr
         tmpol1(1,1)=pol(1,ovarnr,ocnaus)
         tmpol1(1,2)=pol(2,ovarnr,ocnaus)
         tmpol1(1,3)=pol(3,ovarnr,ocnaus)
         tmpol1(2,1)=pol(2,ovarnr,ocnaus)
         tmpol1(2,2)=pol(4,ovarnr,ocnaus)
         tmpol1(2,3)=pol(5,ovarnr,ocnaus)
         tmpol1(3,1)=pol(3,ovarnr,ocnaus)
         tmpol1(3,2)=pol(5,ovarnr,ocnaus)
         tmpol1(3,3)=pol(6,ovarnr,ocnaus)
         tmpol2=matmul(symel_mat(char_table%symel(sni))&
          &,matmul(tmpol1,transpose(symel_mat(char_table%symel(sni)))))
         pol(1,varnr,cnaus)=tmpol2(1,1)
         pol(2,varnr,cnaus)=tmpol2(1,2)
         pol(3,varnr,cnaus)=tmpol2(1,3)
         pol(4,varnr,cnaus)=tmpol2(2,2)
         pol(5,varnr,cnaus)=tmpol2(2,3)
         pol(6,varnr,cnaus)=tmpol2(3,3)
       end if
     end do
  end subroutine create_red_pol
 
  !************************************************************************
     
end module snf_irred_dist
