
!********************************************************************************
!*   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
!*   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
!*   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
!*
!*
!*   This file is part of the package SNF.
!*
!*   SNF is free software; you can redistribute it and/or modify
!*   it under the terms of the GNU General Public License as published by
!*   the Free Software Foundation; either version 2 of the License, or
!*   (at your option) any later version.
!*
!*   This program is distributed in the hope that it will be useful,
!*   but WITHOUT ANY WARRANTY; without even the implied warranty of
!*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*   GNU General Public License for more details.
!*
!*   You should have received a copy of the GNU General Public License
!*   along with this program; if not, write to the Free Software
!*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!*
!********************************************************************************
module snf_molecule
  use f90_kind
  use snf_error

  private
  public :: type_atom,type_molecule,print_molecule,coord_equal,destroy_molecule
  public :: hauptax_trans,calc_delta_i,print_isomass
  public :: find_nucl_charge,find_mass,fiso_mass
  public :: atom_equal,isotope_equal,setisomass,testlin
  !************************************************************************

  type type_atom
     real(double)     :: cartcoord(3) ! carthesian coordinates
     integer          :: nucl_charge  ! nuclear charge
     real(double)     :: mass         ! mass of the selected isotope
     character(len=2) :: element      ! 'h','c','o','cl' etc.
     logical          :: changem      ! special isotopes have been 
                                      ! selected if changem=.true.
  end type type_atom

  type type_molecule
     real(double)     ::  mass        ! total mass in amu
     real(double)     ::  inertia(3,3),  & ! moments of inertia in bohr^2 amu &
          &               p_inertia(3),  & ! principal moments of inertia in bohr^2 amu
          &               p_axes(3,3)        ! principal rotation axes
  end type type_molecule

  !************************************************************************

contains

  !************************************************************************

  subroutine print_molecule(molecule,unit)
    implicit none
    !----------------------------------------------------------------
    ! output routine
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------
    type(type_atom) :: molecule(:)
    integer,optional :: unit
    integer         :: atom,out
    if(present(unit)) then
       out=unit
    else
       out=6
    endif
    write(out,*)
    write(out,'('' Molecule of snf run'')')
    write(out,'('' -------------------'')')
    write(out,*)
    write(out,'("   No El  OZ",12x,"m",15x,"x",15x,"y",15x,"z")')
    do atom = 1, ubound(molecule,dim=1)
       write(out,'(''   '',i2,1x,a2,1x,i3,f13.6,3e16.8)')atom,&
            molecule(atom)%element,&
            molecule(atom)%nucl_charge,&
            molecule(atom)%mass,&
            molecule(atom)%cartcoord(1),&
            molecule(atom)%cartcoord(2),&
            molecule(atom)%cartcoord(3)
    end do
  end subroutine print_molecule

  !************************************************************************

  subroutine print_isomass(molecule,unit)
    !----------------------------------------------------------------
    ! output routine
    !
    ! written by Johannes Neugebauer, Uni Erlangen, 05/2001
    !----------------------------------------------------------------
    implicit none
    type(type_atom),intent(in) :: molecule(:)
    integer,optional :: unit
    integer         :: atom,out
    if(present(unit)) then
       out=unit
    else
       out=6
    endif
    write(out,*)
    write(out,*) '>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<'
    write(out,*) ' Found modified isotopes for the following atoms:' 
    write(out,*)
    write(out,*) ' No.    atomic mass [amu]:'
    write(out,*)
    do atom = 1, ubound(molecule,dim=1)
       if(molecule(atom)%changem) &
            write(out,'(i5,f24.16)') atom, molecule(atom)%mass
    end do
    write(out,*)
    write(out,*)   ' NOTE THAT THE DETERMINATION OF THE FREQUENCIES '
    write(out,*)   ' MAY BE PERFORMED IN A MODIFIED SYMMETRY !!!'
    write(out,*) '>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<'
    write(out,*)
  end subroutine print_isomass

  !************************************************************************

  subroutine setisomass(molecule,selatom,massatom)
    !----------------------------------------------------------------
    ! output routine
    !
    ! written by Johannes Neugebauer, Uni Erlangen, 07/2001
    !----------------------------------------------------------------
    implicit none
    type(type_atom),intent(inout) :: molecule(:)
    real(double), pointer :: massatom(:)
    integer, pointer :: selatom(:)
    integer :: atom

    molecule(:)%changem=.false.
    if(associated(selatom)) then
       do atom=1,ubound(selatom,1)
          molecule(selatom(atom))%mass=massatom(atom)
          molecule(selatom(atom))%changem=.true.
       end do
    end if
  end subroutine setisomass

  !************************************************************************

  subroutine destroy_molecule(molecule)
    !----------------------------------------------------------------
    ! Destructor
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------
    implicit none
    type(type_atom),pointer :: molecule(:)
    if(associated(molecule)) deallocate(molecule)
  end subroutine destroy_molecule

  !************************************************************************

  function find_element(nucl_charge)
    !----------------------------------------------------------------
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    ! corrected by J. Neugebauer, Universitaet Erlangen-Nuernberg,
    ! 05/2001
    !----------------------------------------------------------------
    implicit none
    integer :: nucl_charge
    character(len=2) :: find_element,el(110)
    data el/'H','HE',  &
         'LI','BE','B' ,'C' ,'N' ,'O' ,'F' ,'NE',  &
         'NA','MG','AL','SI','P' ,'S' ,'CL','AR', &
         'K' ,'CA','SC','TI','V' ,'CR','MN','FE','CO','NI',&
         'CU','ZN','GA','GE','AS','SE','BR','KR', &
         'RB','SR','Y' ,'ZR','NB','MO','TC','RU','RH','PD',&
         'AG','CD','IN','SN','SB','TE','I' ,'XE', &
         'CS','BA','LA','CE','PR','ND','PM','SM','EU','GD',&
         'TB','DY','HO','ER','TM','YB','LU', &
         'HF','TA','W' ,'RE','OS','IR','PT','AU',&
         'HG','TL','PB','BI','PO','AT','RN', &
         'FR','RA','AC','TH','PA','U' ,'NP','PU',&
         'AM','CM','BK','CF','ES','FM','MD','NO',&
         'LR','RF','DB','SG','BH','HS','MT','D'/
    IF (nucl_charge.LE.0) THEN
       find_element = 'Q'
    else if(nucl_charge.le. 110) then
       find_element=el(nucl_charge)
    else
       find_element = '  '
    endif
  end function find_element

  !************************************************************************

  function find_nucl_charge(el)
    !------------------------------------------------------------------
    ! scan all atomic numbers until right element symbol is found.
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !
    ! modified by J. Neugebauer, Universitaet Erlangen-Nuernberg, 2001.
    !------------------------------------------------------------------
    implicit none
    character(len=2),intent(in) :: el
    character(len=2) :: element
    integer :: an, find_nucl_charge
    !
    element=el
    call upper(element)
    ! include dummy atoms (Q)
    do an=0,110
       if(element .eq. find_element(an)) exit
    enddo
    ! deuterium
    if (an .eq. 110) an = 1
    ! deuterium
    if (an .eq. 0) an = 0
    ! error
    if (an .eq. 111) an = -1
    find_nucl_charge=an
  end function find_nucl_charge

  !************************************************************************

  function find_mass(nucl_charge)
    !----------------------------------------------------------------
    ! evaluate atomic mass
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !
    ! update by Johannes Neugebauer,
    ! Universitaet Erlangen-Nuernberg, 05/2001, using the following
    !
    ! source: Coursey, J.S. and Dragoset, R. A. (2000). Atomic 
    !         Weights and Isotopic Compositions (version 2.2), [Online].
    !         Available: http://physics.nist.gov/Comp [2001, May 29].
    !         National Institute of Standards and Technology,
    !         Gaithersburg, MD.
    !----------------------------------------------------------------
    implicit none
    !
    !
    integer,intent(in) :: nucl_charge
    real(double) :: mass(111)
    real(double) :: find_mass
    data mass/1.007825032d0,4.0026032497d0,7.0160040d0,9.0121821d0, &
         11.0093055d0,12.0d0,14.0030740052d0,15.9949146221d0,18.99840320d0, &
         19.9924401759d0,22.98976967d0,23.98504190d0,26.98153844d0, &
         27.9769265327d0,30.97376151d0,31.97207069d0,34.96885271d0, &
         39.962383123d0,38.9637069d0,39.9625912d0,44.9559102d0,47.9479471d0, &
         50.9439637d0,51.9405119d0,54.9380496d0,55.9349421d0,58.9332002d0, &
         57.9353479d0,62.9296011d0,63.9291466d0,68.925581d0,73.9211782d0, &
         74.9215964d0,79.9165218d0,78.9183376d0,83.911507d0,84.9117893d0, &
         87.9056143d0,88.9058479d0,89.9047037d0,92.9063775d0,97.9054078d0, &
         97.907216d0,101.9043495d0,102.905504d0,105.903483d0,106.905093d0, &
         113.9033581d0,114.903878d0,119.9021966d0,120.9038180d0,129.9062228d0, &
         126.904468d0,131.9041545d0,132.905447d0,137.905241d0,138.906348d0, &
         139.905434d0,140.907648d0,141.907648d0,144.912744d0,151.919728d0, &
         152.921226d0,157.924101d0,158.925343d0,163.929171d0,164.930319d0, &
         165.930290d0,168.934211d0,173.9388581d0,174.9407679d0,179.9465488d0, &
         180.947996d0,183.9509326d0,186.9557508d0,191.961479d0,192.962924d0, &
         194.964774d0,196.966552d0,201.970626d0,204.974412d0,207.976636d0, &
         208.980383d0,208.982416d0,209.987131d0,222.0175705d0,223.0197307d0, &
         226.0254026d0,227.0277470d0,232.0380504d0,231.0358789d0,238.0507826d0, &
         237.0481673d0,244.064198d0,243.0613727d0,247.070347d0,247.070299d0, &
         251.079580d0,252.082970d0,257.095099d0,258.098425d0,259.101020d0, &
         262.109690d0,261.108750d0,262.114150d0,266.121930d0,264.124730d0, &
         269.134110d0,268.138820d0,271.146080d0,272.153480d0 /
    if(nucl_charge.gt.0 .and. nucl_charge .le. 111) then
       find_mass=mass(nucl_charge)
    else
       find_mass=0.d0
    endif
  end function find_mass

  !************************************************************************

  function coord_equal(p1,p2,tolerance)
    !----------------------------------------------------------------
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------

    implicit none
    real(double), intent(in) :: p1(3),p2(3)
    real(double),intent(in) :: tolerance
    logical coord_equal

    if (sqrt((p1(1)-p2(1))**2+(p1(2)-p2(2))**2+(p1(3)-p2(3))**2)  &
         < tolerance) then
       coord_equal = .true.
    else
       coord_equal = .false.
    endif
  end function coord_equal

  !************************************************************************

  function atom_equal(a1,a2,tolerance)
    !----------------------------------------------------------------
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------

    implicit none
    type(type_atom), intent(in) :: a1,a2
    real(double),intent(in) :: tolerance
    logical atom_equal
    real(double) :: tol,vec(3)
    tol=tolerance*tolerance
    atom_equal = .false.
    if(a1%element .eq. a2%element) then
       vec=a1%cartcoord-a2%cartcoord
       if (dot_product(vec,vec) < tol) then
          atom_equal = .true.
       endif
    end if
  end function atom_equal

  !************************************************************************

  function isotope_equal(a1,a2,tolerance)
    !----------------------------------------------------------------
    ! written by J. Neugebauer, Uni Erlangen, 06/2001
    !----------------------------------------------------------------

    implicit none
    type(type_atom), intent(in) :: a1,a2
    real(double),intent(in) :: tolerance
    logical isotope_equal
    real(double) :: tol,vec(3),thres
    parameter(thres=1.0d-2)
    tol=tolerance*tolerance
    isotope_equal = .false.
    if(a1%element .eq. a2%element) then
       if(abs(a1%mass-a2%mass).le.thres) then
          vec=a1%cartcoord-a2%cartcoord
          if (dot_product(vec,vec) < tol) then
             isotope_equal = .true.
          endif
       end if
    end if
  end function isotope_equal

  !************************************************************************

  subroutine hauptax_trans(mol,molekel)
    !----------------------------------------------------------------
    ! determine moments of inertia
    ! 
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------
    implicit none
    type(type_atom) :: mol(:)
    real(double) :: mkoord(3)
    type(type_molecule),intent(out)  :: molekel  ! information about the whole molecule  
    integer :: atom,zeile,spalte,info
    real(double) :: ematrix(3,3),work(8)
    data ematrix /1,0,0, &
         &        0,1,0, &
         &        0,0,1 /
    !-----------------------------------------------------------------------------
    ! determine center of gravity
    !-----------------------------------------------------------------------------

    mkoord = 0
    molekel%mass = 0
    do atom=1,ubound(mol,1)
       mkoord = mkoord+mol(atom)%cartcoord*mol(atom)%mass
       molekel%mass = molekel%mass+mol(atom)%mass
    enddo
    mkoord = mkoord/molekel%mass
    !-----------------------------------------------------------------------------
    ! set nuclear coordinates to center of gravity
    !-----------------------------------------------------------------------------
    do atom=1,ubound(mol,1)
       mol(atom)%cartcoord = mol(atom)%cartcoord-mkoord
    enddo
    !-----------------------------------------------------------------------------
    ! determine matrix of moments of inertia
    !-----------------------------------------------------------------------------
    do spalte=1,3
       do zeile=1,3
          molekel%inertia(zeile,spalte) = sum( mol(:)%mass*(ematrix(spalte,zeile)* &
               (mol(:)%cartcoord(1)**2+mol(:)%cartcoord(2)**2+mol(:)%cartcoord(3)**2) &
               -mol(:)%cartcoord(spalte)*mol(:)%cartcoord(zeile)) )
       enddo
    enddo
    molekel%p_axes=molekel%inertia
    !-----------------------------------------------------------------------------
    ! solve sekular determinat und determine eigenvectors
    !-----------------------------------------------------------------------------
    call dsyev('V','L',3,molekel%p_axes,3,molekel%p_inertia,work,8,info)
    if(info.ne.0) then 
       write(*,*) "ERROR! Could not calculate eigenvectors"
       write(*,*) "of the hessian."
       stop
    end if
    !-----------------------------------------------------------------------------
    ! multiply nuclear coordinates with transpose(imatrix)
    !-----------------------------------------------------------------------------
    do atom=1,ubound(mol,1)
       mol(atom)%cartcoord = matmul(transpose(molekel%p_axes),mol(atom)%cartcoord)
    enddo
  end subroutine hauptax_trans

  !*****************************************************************************

  subroutine calc_delta_i(mol,delta_i,toleranz,molekel)
    !----------------------------------------------------------------
    ! this routine evaluates the mean error of the moments of inertia
    ! based on the error in the cartesian coordinates of the molecule
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------
    implicit none
    type(type_atom),intent(in) :: mol(:)
    integer :: spalte,zeile
    real(double),intent(out) :: delta_i(3)
    real(double) :: delta_imatrix(3,3), &
         &              deltai_trans(3,3), toleranz
    type(type_molecule),intent(in)  :: molekel  ! information about the whole molecule  
    real(double) :: ematrix(3,3)
    data ematrix /1,0,0, &
         &        0,1,0, &
         &        0,0,1 /

    toleranz=1.0d-3
    do spalte=1,3
       do zeile=1,3
          delta_imatrix(spalte,zeile) = sum( mol(:)%mass* &
               (ematrix(spalte,zeile)*toleranz*sqrt &
               (mol(:)%cartcoord(1)**2+mol(:)%cartcoord(2)**2+mol(:)%cartcoord(3)**2) &
               + abs(mol(:)%cartcoord(spalte)*toleranz/2) &
               + abs(mol(:)%cartcoord(zeile)*toleranz/2)))
       enddo
    enddo
    deltai_trans = matmul(matmul(transpose(molekel%p_axes),delta_imatrix),molekel%p_axes)
    do zeile=1,3
       delta_i(zeile)=deltai_trans(zeile,zeile)
    enddo
  end subroutine calc_delta_i

  !*****************************************************************************

      function fiso_mass(nch,mnr)
!     !----------------------------------------------------------------
!     ! evaluate mass for atom with nuclear charge number "nch"
!     ! and number of nucleones "mnr"
!     !
!     ! written by Johannes Neugebauer, 
!     ! Universitaet Erlangen-Nuernberg, 05/2001
!     !
!     ! source: Coursey, J.S. and Dragoset, R. A. (2000). Atomic 
!     !         Weights and Isotopic Compositions (version 2.2), [Online].
!     !         Available: http://physics.nist.gov/Comp [2001, May 29].
!     !         National Institute of Standards and Technology,
!     !         Gaithersburg, MD.
!     !----------------------------------------------------------------
      implicit none
      integer,intent(in) :: nch,mnr
      integer      :: massind(10,111),isoind,i,j
      real(double) :: mass(10,111)
      real(double) :: fiso_mass

      data ((massind(i,j),i=1,10),j=1,70) / &
     & 1, 2, 3, 0, 0, 0, 0, 0, 0, 0 ,  3, 4, 0, 0, 0, 0, 0, 0, 0, 0 ,  &
     & 6, 7, 0, 0, 0, 0, 0, 0, 0, 0 ,  9, 0, 0, 0, 0, 0, 0, 0, 0, 0 ,  &
     & 10,11, 0, 0, 0, 0, 0, 0, 0, 0 , 12,13, 0, 0, 0, 0, 0, 0, 0, 0 , &
     & 14,15, 0, 0, 0, 0, 0, 0, 0, 0 , 16,17,18, 0, 0, 0, 0, 0, 0, 0 , &
     & 19, 0, 0, 0, 0, 0, 0, 0, 0, 0 , 20,21,22, 0, 0, 0, 0, 0, 0, 0 , &
     & 23, 0, 0, 0, 0, 0, 0, 0, 0, 0 , 24,25,26, 0, 0, 0, 0, 0, 0, 0 , &
     & 27, 0, 0, 0, 0, 0, 0, 0, 0, 0 , 28,29,30, 0, 0, 0, 0, 0, 0, 0 , &
     & 31, 0, 0, 0, 0, 0, 0, 0, 0, 0 , 32,33,34,36, 0, 0, 0, 0, 0, 0 , &
     & 35,37, 0, 0, 0, 0, 0, 0, 0, 0 , 36,38,40, 0, 0, 0, 0, 0, 0, 0 , &
     & 39,40,41, 0, 0, 0, 0, 0, 0, 0 , 40,42,43,44,46,48, 0, 0, 0, 0 , &
     & 45, 0, 0, 0, 0, 0, 0, 0, 0, 0 , 46,47,48,49,50, 0, 0, 0, 0, 0 , &
     & 50, 0, 0, 0, 0, 0, 0, 0, 0, 0 , 50,52,53,54, 0, 0, 0, 0, 0, 0 , &
     & 55, 0, 0, 0, 0, 0, 0, 0, 0, 0 , 54,56,57,58, 0, 0, 0, 0, 0, 0 , &
     & 59, 0, 0, 0, 0, 0, 0, 0, 0, 0 , 58,60,61,62,64, 0, 0, 0, 0, 0 , &
     & 63,65, 0, 0, 0, 0, 0, 0, 0, 0 , 64,66,67,68,70, 0, 0, 0, 0, 0 , &
     & 69,70, 0, 0, 0, 0, 0, 0, 0, 0 , 70,72,73,74,76, 0, 0, 0, 0, 0 , &
     & 75, 0, 0, 0, 0, 0, 0, 0, 0, 0 , 74,76,77,78,80,82, 0, 0, 0, 0 , &
     & 79,81, 0, 0, 0, 0, 0, 0, 0, 0 , 78,80,82,83,84,86, 0, 0, 0, 0 , &
     & 85,86, 0, 0, 0, 0, 0, 0, 0, 0 , 84,86,87,88, 0, 0, 0, 0, 0, 0 , &
     & 89, 0, 0, 0, 0, 0, 0, 0, 0, 0 , 90,91,92,94,96, 0, 0, 0, 0, 0 , &
     & 93, 0, 0, 0, 0, 0, 0, 0, 0, 0 , 92,94,95,96,97,98,100, 0, 0, 0 ,&
     & 97,98,99, 0, 0, 0, 0, 0, 0, 0 , 96,98,99,100,101,102,104,0,0,0 ,&
     &103,0,0,0,0,0,0, 0, 0, 0 , 102,104,105,106,108,110,  0, 0, 0, 0 ,&
     &107,109,0,0,0,0,0,0,0,0 , 106,108,110,111,112,113,114,116, 0, 0 ,&
     &113,115,0,0,0,0,0,0,0,0, 112,114,115,116,117,118,119,120,122,124,&
     &121,123,0,0,0,0,0,0,0,0, 120,122,123,124,125,126,128,130,  0,  0,&
     &127,0,0,0,0,0,0,0,0, 0 , 124,126,128,129,130,131,132,134,136,  0,&
     &133,0,0,0,0,0,0,0,0, 0 , 130,132,134,135,136,137,138,  0,  0,  0,&
     &138,139,0,0,0,0,0,0,0,0, 136,138,140,142,  0,  0,  0,  0,  0,  0,&
     &141,0,0,0,0,0,0,0,0, 0 , 142,143,144,145,146,148,150,  0,  0,  0,&
     &145,147,0,0,0,0,0,0,0,0, 144,147,148,149,150,152,154,  0,  0,  0,&
     &151,153,0,0,0,0,0,0,0,0, 152,154,155,156,157,158,160,  0,  0,  0,&
     &159,  0,0,0,0,0,0,0,0,0, 156,158,160,161,162,163,164,  0,  0,  0,&
     &165,  0,0,0,0,0,0,0,0,0, 162,164,166,167,168,170,  0,  0,  0,  0,&
     &169,  0,0,0,0,0,0,0,0,0, 168,170,171,172,173,174,176,  0,  0,  0 /

    data ((massind(i,j),i=1,10),j=71,111) / &
     &175,176, 0, 0, 0, 0, 0, 0, 0, 0 , 174,176,177,178,179,180,  0,  0,  0,  0,&
     &180,181, 0, 0, 0, 0, 0, 0, 0, 0 , 180,182,183,184,186,  0,  0,  0,  0,  0,&
     &185,187, 0, 0, 0, 0, 0, 0, 0, 0 , 184,186,187,188,189,190,192,  0,  0,  0,&
     &191,193, 0, 0, 0, 0, 0, 0, 0, 0 , 190,192,194,195,196,198,  0,  0,  0,  0,&
     &197,  0, 0, 0, 0, 0, 0, 0, 0, 0 , 196,198,199,200,201,202,204,  0,  0,  0,&
     &203,205, 0, 0, 0, 0, 0, 0, 0, 0 , 204,206,207,208,  0,  0,  0,  0,  0,  0,&
     &209,  0, 0, 0, 0, 0, 0, 0, 0, 0 , 209,210,  0,  0,  0,  0,  0,  0,  0,  0,&
     &210,211, 0, 0, 0, 0, 0, 0, 0, 0 , 211,220,222,  0,  0,  0,  0,  0,  0,  0,&
     &223,  0, 0, 0, 0, 0, 0, 0, 0, 0 , 223,224,226,228,  0,  0,  0,  0,  0,  0,&
     &227,  0, 0, 0, 0, 0, 0, 0, 0, 0 , 230,232,  0,  0,  0,  0,  0,  0,  0,  0,&
     &231,  0, 0, 0, 0, 0, 0, 0, 0, 0 , 233,234,235,236,238,  0,  0,  0,  0,  0,&
     &237,  0, 0, 0, 0, 0, 0, 0, 0, 0 , 238,239,240,241,242,244,  0,  0,  0,  0,&
     &241,243, 0, 0, 0, 0, 0, 0, 0, 0 , 243,244,245,246,247,248,  0,  0,  0,  0,&
     &247,249, 0, 0, 0, 0, 0, 0, 0, 0 , 249,250,251,252,  0,  0,  0,  0,  0,  0,&
     &252,  0, 0, 0, 0, 0, 0, 0, 0, 0 , 257,  0,  0,  0,  0,  0,  0,  0,  0,  0,&
     &256,258, 0, 0, 0, 0, 0, 0, 0, 0 , 259,  0,  0,  0,  0,  0,  0,  0,  0,  0,&
     &262,  0, 0, 0, 0, 0, 0, 0, 0, 0 , 261,  0,  0,  0,  0,  0,  0,  0,  0,  0,&
     &262,  0, 0, 0, 0, 0, 0, 0, 0, 0 , 266,  0,  0,  0,  0,  0,  0,  0,  0,  0,&
     &264,  0, 0, 0, 0, 0, 0, 0, 0, 0 , 269,  0,  0,  0,  0,  0,  0,  0,  0,  0,&
     &268,  0, 0, 0, 0, 0, 0, 0, 0, 0 , 271,  0,  0,  0,  0,  0,  0,  0,  0,  0,&
     &272,  0, 0, 0, 0, 0, 0, 0, 0, 0 /
      
    data ((mass(i,j),i=1,10),j=1,15) / &
     &1.0078250321d0,2.0141017780d0,3.0160492675d0,0.000d0,0.000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.000d0,0.000d0, &
     &3.0160293097d0,4.0026032497d0,0.0000000000d0,0.000d0,0.000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.000d0,0.000d0, &
     &6.0151223d0   ,7.0160040d0   ,0.0000000000d0,0.000d0,0.000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.000d0,0.000d0, &
     &9.0121821d0   ,0.0000000000d0,0.0000000000d0,0.000d0,0.000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.000d0,0.000d0, &
     &10.0129370d0  ,11.0093055d0  ,0.0000000000d0,0.000d0,0.000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.000d0,0.000d0, &
     &12.000000000d0,13.0033548378d0,0.000000000d0,0.000d0,0.000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.000d0,0.000d0, &
     &14.0030740052d0,15.0001088984d0,0.00000000d0,0.000d0,0.000d0, &
     &0.0000000000d0,0.0000000000d0,0.00000d0,0.00000d0,0.000000d0, &
     &15.9949146221d0,16.99913150d0 ,17.9991604d0,0.000d0,0.0000d0, &
     &0.00d0,0.0000d0,0.0000000000d0,0.0000000000d0,0.0000000000d0, &
     &18.99840320d0,0.000d0,0.000d0,0.000000000d0,0.000000000000d0, &
     &0.000d0,0.000d0,0.0000000000d0,0.0000000000d0,0.0000000000d0, &
     &19.9924401759d0,20.99384674d0,21.99138551d0,0.00d0,0.00000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.00d0,0.0000d0, &
     &22.98976967d0,0.00d0,0.00000d0,0.00000000d0,0.000000000000d0, &
     &0.0000000000d0,0.00d0,0.0000d0,0.0000000000d0,0.0000000000d0, &
     &23.98504190d0,24.98583702d0,25.98259304d0,0.0d0,0.00000000d0, &
     &0.0000000000d0,0.0000000000d0,0.000d0,0.000d0,0.0000000000d0, &
     &26.98153844d0,0.000d0,0.0000d0,0.00000000d0,0.000000000000d0, &
     &0.0000000000d0,0.000d0,0.000d0,0.0000000000d0,0.0000000000d0, &
     &27.9769265327d0,28.97649472d0,29.97377022d0,0.0d0,0.000000d0, &
     &0.0000000000d0,0.0000000000d0,0.00d0,0.0000d0,0.0000000000d0, &
     &30.97376151d0,0.0000d0,0.000d0,0.00000000d0,0.000000000000d0, &
     &0.0000000000d0,0.000d0,0.0000d0,0.0000000000d0,0.0000000000d0 /

      data ((mass(i,j),i=1,10),j=16,30) / &
     &31.97207069d0,32.97145850d0,33.96786683d0,35.96708088d0,0.000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.00d0,0.0000000d0, &
     &34.96885271d0,36.96590260d0,0.00000000d0,0.00000000d0,0.00000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.0000d0,0.00000d0, &
     &35.96754628d0,37.9627322d0,39.962383123d0,0.0000000d0,0.00000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.000d0,0.000000d0, &
     &38.9637069d0,39.96399867d0,40.96182597d0,0.0000000d0,0.000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.000d0,0.000000d0, &
     &39.9625912d0,41.9586183d0,42.9587668d0,43.9554811d0,45.9536928d0,&
     &47.952534d0,0.00000d0,0.0000000d0,0.0000000000d0,0.0000000000d0, &
     &44.9559102d0,0.00000d0,0.000000d0,0.00000000d0,0.000000000000d0, &
     &0.0000000000d0,0.00d0,0.0000000d0,0.0000000000d0,0.0000000000d0, &
     &45.9526295d0,46.9517638d0,47.9479471d0,48.9478708d0,49.9447921d0,&
     &0.000000d0,0.0000000000d0,0.0000000000d0,0.000d0,0.0000000000d0, &
     &49.9471628d0,50.9439637d0,0.00000000d0,0.000d0,0.000000000000d0, &
     &0.0000000000d0,0.0000000000d0,0.00d0,0.0000000d0,0.0000000000d0, &
     &49.9460496d0,51.9405119d0,52.9406538d0,53.9388849d0,0.0000000d0, &
     &0.000000d0,0.0000000000d0,0.0000000000d0,0.000d0,0.0000000000d0, &
     &54.9380496d0,0.0000000d0,0.00000000d0,0.0000d0,0.000000000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0d0,0.00000000d0,0.0000000000d0, &
     &53.9396148d0,55.9349421d0,56.9353987d0,57.9332805d0,0.0000000d0, &
     &0.000000d0,0.0000000000d0,0.0000000000d0,0.000d0,0.0000000000d0, &
     &58.9332002d0,0.0000000d0,0.00000000d0,0.0000d0,0.000000000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0d0,0.00000000d0,0.0000000000d0, &
     &57.9353479d0,59.9307906d0,60.9310604d0,61.9283488d0,63.9279696d0,&
     &0.000000d0,0.0000000000d0,0.0000000000d0,0.0000000000d0,0.000d0, &
     &62.9296011d0,64.9277937d0,0.00000000d0,0.00000000d0,0.0000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.00d0,0.0000000d0, &
     &63.9291466d0,65.9260368d0,66.9271309d0,67.9248476d0,69.925325d0, &
     &0.000000d0,0.0000000000d0,0.0000000000d0,0.0000d0,0.0000000000d0 /
     
      data ((mass(i,j),i=1,10),j=31,45) / &
     &68.925581d0,70.9247050d0,0.00000000d0,0.00000000d0,0.00000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.000d0,0.000000d0, &
     &69.9242504d0,71.9220762d0,72.9234594d0,73.9211782d0,75.9214027d0,&
     &0.000000d0,0.0000000000d0,0.0000000000d0,0.0000000000d0,0.000d0, &
     &74.9215964d0,0.0000000d0,0.00000000d0,0.00000000d0,0.00000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.00000d0,0.0000d0, &
     &73.9224766d0,75.9192141d0,76.9199146d0,77.9173095d0,79.9165218d0,&
     &81.9167000d0,0.0000000000d0,0.0000000000d0,0.000000000d0,0.00d0, &
     &78.9183376d0,80.916291d0,0.00000000d0,0.0000d0,0.000000000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.00d0,0.0000000d0, &
     &77.920386d0,79.916378d0,81.9134846d0,82.914136d0,83.911507d0,    &
     &85.9106103d0,0.0000000000d0,0.0000000000d0,0.0000000000d0,0.0d0, &
     &84.9117893d0,86.9091835d0,0.00000000d0,0.00000000d0,0.0000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.00d0,0.0000000d0, &
     &83.913425d0,85.9092624d0,86.9088793d0,87.9056143d0,0.000000d0,   &
     &0.0000000d0,0.0000000000d0,0.0000000000d0,0.0000000000d0,0.00d0, &
     &88.9058479d0,0.0000000d0,0.00000000d0,0.00000000d0,0.00000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.0000000d0,0.00d0, &
     &89.9047037d0,90.9056450d0,91.9050401d0,93.9063158d0,95.908276d0, &
     &0.0000000d0,0.0000000000d0,0.0000000000d0,0.0000000000d0,0.00d0, &
     &92.9063775d0,0.0000000d0,0.00000000d0,0.00000000d0,0.00000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.00000d0,0.0000d0, &
     &91.906810d0,93.9050876d0,94.9058415d0,95.9046789d0,96.9060210d0, &
     &97.9054078d0,99.907477d0,0.0000000000d0,0.0000000000d0,0.0000d0, &
     &96.906365d0,97.907216d0,98.9062546d0,0.00000000d0,0.000000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.0000000d0,0.00d0, &
     &95.907598d0,97.905287d0,98.9059393d0,99.9042197d0,100.9055822d0, &
     &101.9043495d0,103.905430d0,0.0000000000d0,0.000000d0,0.000000d0, &
     &102.905504d0,0.0000000d0,0.00000000d0,0.00000000d0,0.00000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.0000d0,0.000000d0 /

      data ((mass(i,j),i=1,10),j=46,60) / &
     &101.905608d0,103.904035d0,104.905084d0,105.903483d0,107.903894d0,&
     &109.905152d0,103.905430d0,0.0000000000d0,0.0000000000d0,0.000d0, &
     &106.905093d0,108.904756d0,0.00000000d0,0.00000000d0,0.0000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.0000d0,0.00000d0, &
     &105.906458d0,107.904183d0,109.903006d0,110.904182d0,111.9027572d0,&
     &112.9044009d0,113.9033581d0,115.904755d0,0.000000d0,0.0000000d0, &
     &112.904061d0,114.903878d0,0.00000000d0,0.00000000d0,0.0000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.0d0,0.0000000d0 , &
     &111.904821d0,113.902782d0,114.903346d0,115.901744d0,116.902954d0,&
     &117.901606d0,118.903309d0,119.9021966d0,121.9034401d0,           &
     &123.9052746d0,                                                   &
     &120.9038180d0,122.9042157d0,0.00000000d0,0.00000000d0,0.00000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.0000000d0,0.00d0, &
     &119.904020d0,121.9030471d0,122.9042730d0,123.9028195d0,          &
     &124.9044247d0,                                                   &
     &125.9033055d0,127.9044614d0,129.9062228d0,0.0000000000d0,0.00d0, &
     &126.904468d0,0.0000000d0,0.00000000d0,0.00000000d0,0.00000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.00000d0,0.0000d0, &
     &123.9058958d0,125.904269d0,127.9035304d0,128.9047795d0,          &
     &129.9035079d0,                                                   &
     &130.9050819d0,131.9041545d0,133.9053945d0,135.907220d0,0.0000d0, &
     &132.905447d0,0.0000000d0,0.00000000d0,0.00000000d0,0.00000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.000d0,0.000000d0, &
     &129.906310d0,131.905056d0,133.904503d0,134.905683d0,135.904570d0,&
     &136.905821d0,137.905241d0,0.000000000d0,0.00000000d0,0.000000d0, &
     &137.907107d0,138.906348d0,0.00000000d0,0.00000000d0,0.0000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.00d0,0.0000000d0, &
     &135.907140d0,137.905986d0,139.905434d0,141.909240d0,0.0000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.000d0,0.000000d0, &
     &140.907648d0,0.00000000d0,0.00000000d0,0.00000000d0,0.0000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.00d0,0.0000000d0, &
     &141.907719d0,142.909810d0,143.910083d0,144.912569d0,145.913112d0,&
     &147.916889d0,149.920887d0,0.0000000000d0,0.0000000000d0,0.0000d0 /

      data ((mass(i,j),i=1,10),j=61,75) / &
     &144.912744d0,146.915134d0,0.00000000d0,0.00000000d0,0.0000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.00000d0,0.0000d0, &
     &143.911995d0,146.914893d0,147.914818d0,148.917180d0,149.917271d0,&
     &151.919728d0,153.922205d0,0.0000000000d0,0.0000000000d0,0.000d0, &
     &150.919846d0,152.921226d0,0.00000000d0,0.00000000d0,0.0000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.0000d0,0.00000d0, &
     &151.919788d0,153.920862d0,154.922619d0,155.922120d0,156.923957d0,&
     &157.924101d0,159.927051d0,0.0000000000d0,0.0000000000d0,0.000d0, &
     &158.925343d0,0.000000d0,0.00000000d0,0.00000000d0,0.000000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.000d0,0.000000d0, &
     &155.924278d0,157.924405d0,159.925194d0,160.926930d0,161.926795d0,&
     &162.928728d0,163.929171d0,0.0000000000d0,0.0000000000d0,0.000d0, &
     &164.930319d0,0.000000d0,0.00000000d0,0.00000000d0,0.000000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.000d0,0.000000d0, &
     &161.928775d0,163.929197d0,165.930290d0,166.932045d0,167.932368d0,&
     &169.935460d0,0.00000000d0,0.0000000000d0,0.0000000000d0,0.000d0, &
     &168.934211d0,0.000000d0,0.00000000d0,0.00000000d0,0.000000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.000d0,0.000000d0, &
     &167.933894d0,169.934759d0,170.936322d0,171.9363777d0,            &
     &172.9382068d0,                                                   &
     &173.9388581d0,175.942568d0,0.0000d0,0.00000000d0,0.0000000000d0, &
     &174.9407679d0,175.9426824d0,0.00d0,0.0000000d0,0.000000000000d0, &
     &0.0000000000d0,0.0000000000d0,0.000d0,0.000000d0,0.0000000000d0, &
     &173.940040d0,175.9414018d0,176.9432200d0,177.9436977d0,          &
     &178.9458151d0,                                                   &
     &179.9465488d0,0.00000000d0,0.0000000000d0,0.0000000000d0,0.00d0, &
     &179.947466d0,180.947996d0,0.00000000d0,0.00000000d0,0.0000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.00d0,0.0000000d0, &
     &179.946706d0,181.948206d0,182.9502245d0,183.9509326d0,           &
     &185.954362d0,                                                    &
     &0.000000000d0,0.00000000d0,0.0000000000d0,0.0000000000d0,0.00d0, &
     &184.9529557d0,186.9557508d0,0.00000000d0,0.00000000d0,0.00000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.00d0,0.00000000d0 /

      data ((mass(i,j),i=1,10),j=76,90) / &
     &183.952491d0,185.953838d0,186.9557479d0,187.9558360d0,           &
     &188.9581449d0, &
     &189.958445d0,191.961479d0,0.0000000000d0,0.0000000000d0,0.000d0, &
     &190.960591d0,192.962924d0,0.00000000d0,0.00000000d0,0.0000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.000d0,0.000000d0, &
     &189.959930d0,191.961035d0,193.962664d0,194.964774d0,195.964935d0,&
     &197.967876d0,0.00000000d0,0.0000000000d0,0.0000000000d0,0.000d0, &
     &196.966552d0,0.00000000d0,0.00000000d0,0.00000000d0,0.0000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.0000d0,0.00000d0, &
     &195.965815d0,197.966752d0,198.968262d0,199.968309d0,200.970285d0,&
     &201.970626d0,203.973476d0,0.0000000000d0,0.0000000000d0,0.000d0, &
     &202.972329d0,204.974412d0,0.00000000d0,0.00000000d0,0.0000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.000d0,0.000000d0, &
     &203.973029d0,205.974449d0,206.975881d0,207.976636d0,0.0000000d0, &
     &0.00000000d0,0.00000000d0,0.0000000000d0,0.000d0,0.0000000000d0, &
     &208.980383d0,0.00000000d0,0.00000000d0,0.00000000d0,0.0000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.00d0,0.0000000d0, &
     &208.982416d0,209.982857d0,0.00000000d0,0.00000000d0,0.0000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.00d0,0.0000000d0, &
     &209.987131d0,210.987481d0,0.00000000d0,0.00000000d0,0.0000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.0d0,0.00000000d0, &
     &210.990585d0,220.0113841d0,222.0175705d0,0.00000000d0,0.00000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.000d0,0.000000d0, &
     &223.0197307d0,0.00000000d0,0.00000000d0,0.00000000d0,0.000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.000d0,0.000000d0, &
     &223.018497d0,224.0202020d0,226.0254026d0,228.0310641d0,0.0000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.00d0,0.0000000d0, &
     &227.0277470d0,0.00000000d0,0.00000000d0,0.00000000d0,0.000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.00d0,0.0000000d0, &
     &230.03312660,232.0380504d0,0.00000000d0,0.00000000d0,0.000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.000d0,0.0000000d0 /

      data ((mass(i,j),i=1,10),j=91,105) / &
     &231.0358789d0,0.00000000d0,0.00000000d0,0.00000000d0,0.000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.0000000d0,0.00d0, &
     &233.039628d0,234.0409456d0,235.0439231d0,236.0455619d0,          &
     &238.0507826d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.000000d0,0.000d0, &
     &237.0481673d0,239.0529314d0,0.00000000d0,0.000d0,0.0000000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.000d0,0.000000d0, &
     &238.0495534d0,239.0521565d0,240.0538075d0,241.0568453d0,         &
     &242.0587368d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.00d0,0.0000000d0, &
     &241.0568229d0,243.0613727d0,0.00000000d0,0.00000000d0,0.00000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.00d0,0.0000000d0, &
     &243.0613822d0,244.0627463d0,245.0654856d0,246.0672176d0,         &
     &247.070347d0, &
     &248.072342d0,0.0000000000d0,0.0000000000d0,0.000d0,0.00000000d0, &
     &247.070299d0,249.074980d0,0.00000000d0,0.00000000d0,0.0000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.000d0,0.000000d0, &
     &249.074847d0,250.0764000d0,251.079580d0,252.081620d0,0.000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.0d0,0.00000000d0, &
     &252.082970d0,0.00000000d0,0.00000000d0,0.00000000d0,0.0000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.00000d0,0.0000d0, &
     &257.095099d0,0.00000000d0,0.00000000d0,0.00000000d0,0.0000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.00d0,0.0000000d0, &
     &256.094050d0,258.098425d0,0.00000000d0,0.00000000d0,0.0000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.00d0,0.0000000d0, &
     &259.101020d0,0.00000000d0,0.00000000d0,0.00000000d0,0.0000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.00d0,0.0000000d0, &
     &262.109690d0,0.00000000d0,0.00000000d0,0.00000000d0,0.0000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.00d0,0.0000000d0, &
     &261.108750d0,0.00000000d0,0.00000000d0,0.00000000d0,0.0000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.000d0,0.000000d0, &
     &262.114150d0,0.00000000d0,0.00000000d0,0.00000000d0,0.0000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.000d0,0.0000000d0 /

      data ((mass(i,j),i=1,10),j=106,111) / &
     &266.121930d0,0.00000000d0,0.00000000d0,0.00000000d0,0.0000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.000000d0,0.000d0, &
     &264.124730d0,0.00000000d0,0.00000000d0,0.00000d0,0.0000000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.000000d0,0.000d0, &
     &269.134110d0,0.00000000d0,0.00000000d0,0.00000000d0,0.0000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.000d0,0.000000d0, &
     &268.138820d0,0.00000000d0,0.00000000d0,0.00000000d0,0.0000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.0000d0,0.00000d0, &
     &271.146080d0,0.00000000d0,0.00000000d0,0.00000000d0,0.0000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.00d0,0.0000000d0, &
     &272.153480d0,0.00000000d0,0.00000000d0,0.00000000d0,0.0000000d0, &
     &0.0000000000d0,0.0000000000d0,0.0000000000d0,0.00d0,0.00000000d0 /


    isoind=0
    if(nch.gt.0.and.nch.le.111) then
       do i=1,10
          if(massind(i,nch).eq.mnr) isoind=i
       end do
    else
       fiso_mass=0.d0
    endif
    if(isoind.ne.0) then
       fiso_mass=mass(isoind,nch)
    else
       fiso_mass=0.d0
    end if
  end function fiso_mass

  !************************************************************************

    function testlin(molecule)
    !----------------------------------------------------------------
    ! .true., if molecule is linear
    !
    ! written by Johannes Neugebauer, 
    ! Universitaet Erlangen-Nuernberg, 2001
    !----------------------------------------------------------------
    implicit none
    logical :: testlin
    integer :: i
    type(type_atom),intent(in) :: molecule(:)
    real(double) :: orig(3),slope(3),a(3)
    real(double),parameter :: thres=1.0d-8
    testlin=.false.
    if(ubound(molecule,1).eq.2) then
      testlin=.true.
      return
    else if(ubound(molecule,1).eq.1) then
      call serror('testlin',2,'You can''t calculate vibrational spectra for atoms')
    end if
    orig(:)=molecule(1)%cartcoord(:)
    slope(:)=molecule(2)%cartcoord(:)-molecule(1)%cartcoord(:)
    if(.not.(any(abs(slope).ge.thres))) call serror('testlin',2,'Detected two &
     & atoms with identical coordinates')
    do i=3,ubound(molecule,1)
      if(.not.slope(1).le.thres) then
        a(1)=(molecule(i)%cartcoord(1)-orig(1))/slope(1)
      else
        if(abs(molecule(i)%cartcoord(1)-orig(1)).gt.thres) return
      end if
      if(.not.slope(2).le.thres) then
        a(2)=(molecule(i)%cartcoord(2)-orig(2))/slope(2)
        if(.not.slope(1).le.thres) then
          if(abs(a(1)-a(2)).gt.thres) return
        end if
      else
        if(abs(molecule(i)%cartcoord(2)-orig(2)).gt.thres) return
      end if
      if(.not.slope(3).le.thres) then
        a(3)=(molecule(i)%cartcoord(3)-orig(3))/slope(3)
        if(.not.slope(1).le.thres) then
          if(abs(a(1)-a(3)).gt.thres) return
        end if
        if(.not.slope(2).le.thres) then
          if(abs(a(2)-a(3)).gt.thres) return
        end if
      else
        if(abs(molecule(i)%cartcoord(3)-orig(3)).gt.thres) return
      end if
    end do
    testlin=.true.
    end function testlin

  !************************************************************************

end module snf_molecule

