
!********************************************************************************
!*   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
!*   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
!*   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
!*
!*
!*   This file is part of the package SNF.
!*
!*   SNF is free software; you can redistribute it and/or modify
!*   it under the terms of the GNU General Public License as published by
!*   the Free Software Foundation; either version 2 of the License, or
!*   (at your option) any later version.
!*
!*   This program is distributed in the hope that it will be useful,
!*   but WITHOUT ANY WARRANTY; without even the implied warranty of
!*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*   GNU General Public License for more details.
!*
!*   You should have received a copy of the GNU General Public License
!*   along with this program; if not, write to the Free Software
!*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!*
!********************************************************************************
module snf_norm_modes
  use f90_kind
  use snf_error
  !----------------------------------------------------------------
  ! this module creates normal modes in matrix formulation
  ! for output routines look at module "snf_spectrum" 
  !
  ! written by Carsten Kind, Uni Erlangen, 1999-2001
  !----------------------------------------------------------------

  !************************************************************************

contains

  !************************************************************************

  subroutine project_off_transrot(molecule,c2p,mode)
    !----------------------------------------------------------------
    ! Project off all translational and rotational modes from C2P matrix
    ! mode determines which modes are to be projected out
    !   'trans': projects out only translational modes
    !   'rot'  : projects out only rotational modes
    !   'all'  : projects out translational and rotational modes
    !
    ! written by Carsten Kind, Uni Erlangen, 2000-2001
    !----------------------------------------------------------------
    use snf_molecule, only: type_atom
    use snf_tools,    only: gram_schmidt_procedure
    use snf_symmat
    implicit none
    type(type_atom), intent(in) :: molecule(:)
    type(type_atom), pointer :: tmp_mol(:)
    character (len=*) :: mode
    real(double),pointer :: c2p(:,:)
    real(double),allocatable :: trans_modes(:,:),rot_modes(:,:),transp(:,:),rotp(:,:),c2ptmp(:,:)
    integer :: i,j,nvar,irrep_dim,natoms,modes_to_omit
    natoms=ubound(molecule,1)
    nvar=ubound(c2p,1)
    irrep_dim=ubound(c2p,2)
    allocate(tmp_mol(natoms),trans_modes(nvar,3),rot_modes(nvar,3),&
         &   c2ptmp(nvar,irrep_dim),transp(irrep_dim,3),rotp(irrep_dim,3))
    tmp_mol=molecule
    call move_to_center_of_mass(tmp_mol)
    call generate_trans_modes(tmp_mol,trans_modes)    
    call generate_rot_modes(tmp_mol,rot_modes)
    !----------------------------------------------------------------
    ! examining the number of modes to omit by transforming 
    ! translational and rotational modes to symmetry-adapted
    ! basis, and counting nonvanishing scalarproducts in that basis.
    !----------------------------------------------------------------
    call dgemm('T','N',irrep_dim,3,nvar,1.D0,c2p(1,1),nvar,trans_modes(1,1),nvar,0.D0,transp(1,1),irrep_dim)
    call dgemm('T','N',irrep_dim,3,nvar,1.D0,c2p(1,1),nvar,rot_modes(1,1),nvar,0.D0,rotp(1,1),irrep_dim)
    modes_to_omit=0
    do i=1,3
       modes_to_omit=modes_to_omit+ &
            & nint(dot_product(transp(:,i),transp(:,i))) + &
            & nint(dot_product(rotp(:,i),rotp(:,i)))
    enddo
    deallocate(transp,rotp,tmp_mol)
    !----------------------------------------------------------------
    ! eliminate translational components from projection operator
    !----------------------------------------------------------------
    do i=1,irrep_dim
       if(mode .eq. 'trans' .or. mode .eq. 'all') then
          do j=1,3
             c2p(:,i) = c2p(:,i)-trans_modes(:,j) * &
                  &         dot_product(c2p(:,i),trans_modes(:,j))
          enddo
       endif
       !----------------------------------------------------------------
       ! eliminate rotational components from projection operator
       !----------------------------------------------------------------
       if(mode .eq. 'rot' .or. mode .eq. 'all') then
          do j=1,3
             c2p(:,i) = c2p(:,i)-rot_modes(:,j) * &
                  &         dot_product(c2p(:,i),rot_modes(:,j))
          enddo
       endif
    enddo
    call gram_schmidt_procedure(c2p,c2ptmp, 1.d-5)

    !----------------------------------------------------------------
    ! reallocate projection operators and separate off zero vectors.
    ! this procedure relies on the capabilities of this special
    ! gram-schmidt-procedure. it moves the least accurate and therefore
    ! significant basis vectors to the end. when you keep in mind that
    ! theoretically zero vectors should occur with zero norm, but in
    ! practice also small norms occur, you understand the reason for 
    ! this more complicated way.
    !----------------------------------------------------------------
    irrep_dim = irrep_dim - modes_to_omit
    allocate(c2p(nvar,irrep_dim))
    c2p(:,:) = c2ptmp(:,1:irrep_dim)
    deallocate(c2ptmp,rot_modes,trans_modes)
  end subroutine project_off_transrot

  !************************************************************************

  subroutine move_to_center_of_mass(molecule)
    !-----------------------------------------------------------------------------
    ! determine center of mass and mass of the molecule
    ! and move molecule to center of mass
    !
    ! written by Carsten Kind, Uni Erlangen, 2000
    !-----------------------------------------------------------------------------
    use snf_molecule, only: type_atom
    implicit none
    type(type_atom), intent(inout) :: molecule(:)
    integer :: atom,natoms
    real(double) :: molecule_mass,mkoord(3)
    natoms=ubound(molecule,1)
    mkoord = 0.d0
    molecule_mass = 0.d0
    do atom=1,natoms
       mkoord = mkoord+molecule(atom)%cartcoord*molecule(atom)%mass
       molecule_mass = molecule_mass+molecule(atom)%mass
    enddo
    mkoord = mkoord/molecule_mass

    do atom=1,natoms
       molecule(atom)%cartcoord = molecule(atom)%cartcoord-mkoord
    enddo
  end subroutine move_to_center_of_mass

  !************************************************************************

  subroutine generate_trans_modes(molecule,trans_modes)
    !----------------------------------------------------------------
    ! generate translational modes
    !
    ! written by Carsten Kind, Uni Erlangen, 2000
    !----------------------------------------------------------------
    use snf_molecule, only: type_atom
    implicit none
    type(type_atom), intent(in) :: molecule(:)
    real(double),intent(out) :: trans_modes(:,:)
    integer i,j,natoms
    real(double) :: normfac
    trans_modes = 0.d0
    natoms=ubound(molecule,1)
    !----------------------------------------------------------------
    ! construct translational normalmodes
    !----------------------------------------------------------------     
    do j=1,3
       do i=1,natoms
          trans_modes((i-1)*3+j,j)=sqrt(molecule(i)%mass)
       enddo
    enddo
    !----------------------------------------------------------------
    ! normalize translational normalmodes
    !----------------------------------------------------------------     
    do j=1,3
       normfac=1.d0/sqrt(dot_product(trans_modes(:,j),trans_modes(:,j)))
       trans_modes(:,j)=normfac*trans_modes(:,j)
    enddo
  end subroutine generate_trans_modes

  !************************************************************************

  subroutine generate_rot_modes(molecule,rot_modes)
    !----------------------------------------------------------------
    ! generate rotational modes
    !
    ! written by Carsten Kind, Uni Erlangen, 2000
    ! slightly modified by Johannes Neugebauer, Uni Erlangen, 2001
    !----------------------------------------------------------------
    use snf_molecule, only: type_atom
    use snf_symmat,only : vec_product
    implicit none
    type(type_atom), intent(in) :: molecule(:)
    real(double), intent(out) :: rot_modes(:,:)
    real(double) :: x_a(3),ewert(3),imatrix(3,3),ematrix(3,3)
    integer i,j,natoms,col,row,info
    real(double) :: normfac,work(8)
    data ematrix &
         &/1.d0, 0.d0, 0.d0,  &
         & 0.d0, 1.d0, 0.d0,  &
         & 0.d0, 0.d0, 1.d0/ 
    !-----------------------------------------------------------------------------
    ! construct inertia matrix
    !-----------------------------------------------------------------------------
    natoms=ubound(molecule,1)
    do col=1,3
       do row=1,3
          imatrix(row,col) = sum( molecule(:)%mass*(ematrix(col,row)* &
               & (molecule(:)%cartcoord(1)**2  + &
               &  molecule(:)%cartcoord(2)**2  + &
               &  molecule(:)%cartcoord(3)**2) - &
               &  molecule(:)%cartcoord(col)*molecule(:)%cartcoord(row)) )
       enddo
    enddo
    !-----------------------------------------------------------------------------
    ! diagonalize inertia matrix
    !-----------------------------------------------------------------------------
    call dsyev('V','L',3,imatrix(1,1),3,ewert(1),work,8,info)
    if(info.ne.0) then
       call serror('generate_rot_modes',2,'Could not calculate eigenvectors of the##&
        &moments of inertia matrix')
    end if
    !----------------------------------------------------------------
    ! construct rotational normalmodes
    !----------------------------------------------------------------     
    rot_modes=0.d0
    do j=1,3
       if(ewert(j).gt. 1.d0) then
          do i=1,natoms
             !----------------------------------------------------------------
             ! the threshold 1.0 results from the assumption, that the vanishing 
             ! principal inertia of a linear molecule should be below the principal 
             ! inertia of the h2-molecule (=1.45)
             !----------------------------------------------------------------
             x_a = molecule(i)%cartcoord - &
                  & dot_product(molecule(i)%cartcoord,imatrix(:,j)) / &
                  & dot_product(imatrix(:,j),imatrix(:,j))*imatrix(:,j)
             rot_modes(i*3-2:i*3,j) = sqrt(molecule(i)%mass) * &
                  &                           vec_product(x_a,imatrix(:,j))
          enddo
       else
          rot_modes(:,j) = 0.d0
       endif
    enddo
    !----------------------------------------------------------------
    ! normalize rotational normalmodes
    !----------------------------------------------------------------     
    do j=1,3
       normfac=sqrt(dot_product(rot_modes(:,j),rot_modes(:,j)))
       if(normfac.gt. 1.d-5) then
          normfac=1.d0/normfac
          rot_modes(:,j) = normfac*rot_modes(:,j)
       endif
    enddo
  end subroutine generate_rot_modes

  !************************************************************************

end module snf_norm_modes
