
!********************************************************************************
!*   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
!*   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
!*   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
!*
!*
!*   This file is part of the package SNF.
!*
!*   SNF is free software; you can redistribute it and/or modify
!*   it under the terms of the GNU General Public License as published by
!*   the Free Software Foundation; either version 2 of the License, or
!*   (at your option) any later version.
!*
!*   This program is distributed in the hope that it will be useful,
!*   but WITHOUT ANY WARRANTY; without even the implied warranty of
!*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*   GNU General Public License for more details.
!*
!*   You should have received a copy of the GNU General Public License
!*   along with this program; if not, write to the Free Software
!*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!*
!********************************************************************************
module snf_polar
  use f90_kind
  use snf_error

  private
  public :: create_polar_dxyz,print_polar_dxyz,expand_polar
  public :: destroy_polar_dxyz, create_polar_dq, symm_polar


  !************************************************************************

contains

  !************************************************************************

  subroutine create_polar_dxyz(polar_dxyz,pols,distortion)
    implicit none
    !----------------------------------------------------------------
    ! calculation of first derivative of polarizability according to 
    ! following 3-point formula:
    ! 
    !  d alpha_ij     pols_ij(xi+e(xi)) - pols_ij(xi-e(xi))  
    ! ------------ = ---------------------------------------   
    !    dxi                     2*e(xi) 
    !           
    ! Note: polarizabilities are given as polarizability volumes
    !       [bohr**3]. Derivatives are obtained in [Angstroem**2].
    !
    ! written by m. reiher, uni erlangen, sept. and dec. 2000
    ! modified by J. Neugebauer, Uni Erlangen, 2001
    !----------------------------------------------------------------
    real(double),intent(inout) :: polar_dxyz(:,:)
    real(double),intent(in) :: pols(:,:,:)
    real(double),intent(in) :: distortion(:)
    real(double) bohr2a
    parameter (bohr2a=0.52917720d0)
    ! note: step size in bohr, not Angstroem
    !    parameter (bohr2a=1.0d0)
    integer :: idum
    polar_dxyz=0.0d0
    idum=ubound(pols,3)
    if (idum.eq.2) then
       ! alpha' = 1/2   (alpha_-1 - alpha_+1) [bohr**3] / cstep [bohr] 
       ! =>  factor: bohr2a**3/bohr2a =>  * bohr2a**2
       polar_dxyz(1,:) = 0.5d0/distortion(:)*(pols(1,:,1)-pols(1,:,2))*bohr2a*bohr2a
       polar_dxyz(2,:) = 0.5d0/distortion(:)*(pols(2,:,1)-pols(2,:,2))*bohr2a*bohr2a
       polar_dxyz(3,:) = 0.5d0/distortion(:)*(pols(3,:,1)-pols(3,:,2))*bohr2a*bohr2a
       polar_dxyz(4,:) = 0.5d0/distortion(:)*(pols(4,:,1)-pols(4,:,2))*bohr2a*bohr2a
       polar_dxyz(5,:) = 0.5d0/distortion(:)*(pols(5,:,1)-pols(5,:,2))*bohr2a*bohr2a
       polar_dxyz(6,:) = 0.5d0/distortion(:)*(pols(6,:,1)-pols(6,:,2))*bohr2a*bohr2a
    else if (idum.eq.4) then   
       polar_dxyz(1,:) = 1.0d0/(24.0d0*distortion(:))*(-2.0d0*pols(1,:,1)&
        &+16.0d0*pols(1,:,2)-16.0d0*pols(1,:,3)+2.0d0*pols(1,:,4))*bohr2a*bohr2a
       polar_dxyz(2,:) = 1.0d0/(24.0d0*distortion(:))*(-2.0d0*pols(2,:,1)&
        &+16.0d0*pols(2,:,2)-16.0d0*pols(2,:,3)+2.0d0*pols(2,:,4))*bohr2a*bohr2a
       polar_dxyz(3,:) = 1.0d0/(24.0d0*distortion(:))*(-2.0d0*pols(3,:,1)&
        &+16.0d0*pols(3,:,2)-16.0d0*pols(3,:,3)+2.0d0*pols(3,:,4))*bohr2a*bohr2a
       polar_dxyz(4,:) = 1.0d0/(24.0d0*distortion(:))*(-2.0d0*pols(4,:,1)&
        &+16.0d0*pols(4,:,2)-16.0d0*pols(4,:,3)+2.0d0*pols(4,:,4))*bohr2a*bohr2a
       polar_dxyz(5,:) = 1.0d0/(24.0d0*distortion(:))*(-2.0d0*pols(5,:,1)&
        &+16.0d0*pols(5,:,2)-16.0d0*pols(5,:,3)+2.0d0*pols(5,:,4))*bohr2a*bohr2a
       polar_dxyz(6,:) = 1.0d0/(24.0d0*distortion(:))*(-2.0d0*pols(6,:,1)&
        &+16.0d0*pols(6,:,2)-16.0d0*pols(6,:,3)+2.0d0*pols(6,:,4))*bohr2a*bohr2a
    else
       call serror('create_polar_dxyz',2,'Wrong ctrl%numderiv value!') 
    end if
  end subroutine create_polar_dxyz

  !************************************************************************

  subroutine expand_polar(polar_dxyz,representation,symm_index,char_table)
    use snf_char_table, only : type_char_table
    use snf_symmat,only : symel_mat
    !----------------------------------------------------------------
    ! copy polarizability derivatives calculated on irreducible atoms to the 
    ! reducible atoms. this is done by multiplying symmetry representations 
    ! in the pol basis from the left and symmetry representations in the 
    ! derivate basis from the right with the polarizability derivatives 
    ! and selecting only the required columns.
    !
    ! written by Carsten Kind and Markus Reiher, Uni Erlangen, 2000
    ! modified by Johannes Neugebauer, Uni Erlangen, 2001
    !----------------------------------------------------------------
    implicit none
    type(type_char_table), intent(in) :: char_table
    real(double),intent(inout) :: polar_dxyz(:,:)
    real(double),pointer :: rot_polar(:,:),expanded_polar(:,:),polar_tmp(:,:)
    real(double),intent(in) :: representation(:,:,:)
    integer, intent(in) :: symm_index(:)
    integer :: s,i,offset,nvar,counter,j,k,natoms
    real(double) :: pol(3,3),transpol(3,3)
    nvar=ubound(polar_dxyz,2)
    natoms=nvar/3
    allocate(rot_polar(6,nvar),polar_tmp(6,nvar),expanded_polar(6,nvar))
    do s=1,char_table%nsymel
       do i=1,nvar
          ! copy part of long packed matrix to square matrix
          counter=0
          do j=1,3
             do k=j,3
                counter=counter+1
                pol(k,j) = polar_dxyz(counter,i)
                pol(j,k) = pol(k,j)
             enddo
          enddo
          ! rotate square matrix
          transpol = matmul(symel_mat(char_table%symel(s)),&
               &            matmul(pol,transpose(symel_mat(char_table%symel(s)))))
          ! copy back to packed long matrix
          counter=0
          do j=1,3
             do k=j,3
                counter=counter+1
                polar_tmp(counter,i)=transpol(k,j)
             enddo
          enddo
       enddo

       ! perform transformation of gradient
       rot_polar=matmul(polar_tmp,representation(:,:,s))
       ! (6 x nvar ) = (6 x nvar) * (nvar x nvar)
       ! move selected data per atom to final place
       do i=1,natoms
          if(symm_index(i) .eq. s) then
             offset=(i-1)*3
             expanded_polar(:,offset+1:offset+3)=rot_polar(:,offset+1:offset+3)
             !              6,3                            6,3
          endif
       enddo
    enddo
    do i=1,6
      do j=1,nvar
        polar_dxyz(i,j) = expanded_polar(i,j)
      end do
    end do
    deallocate(rot_polar,polar_tmp,expanded_polar)
  end subroutine expand_polar

  !************************************************************************

  subroutine symm_polar(polar_dxyz,rep,char_table)
    use snf_char_table, only : type_char_table
    use snf_symmat,only : symel_mat
    !----------------------------------------------------------------
    ! this subroutine transforms the polarizability derivatives
    ! according to all symmetry operations of the point group.
    ! all resulting matrices are added and then diveded by the
    ! number of symmetry operations. a symmetry check is performed.
    !                                                               
    ! written by Johannes Neugebauer, Uni Erlangen, 2001
    !----------------------------------------------------------------
    implicit none
    type(type_char_table), intent(in) :: char_table
    real(double),intent(inout) :: polar_dxyz(:,:)
    real(double),pointer :: rot_polar(:,:),polar_tmp(:,:),tmpmat(:,:)
    real(double),intent(in) :: rep(:,:,:)
    real(double) :: errsum
    integer :: s,i,nvar,counter,j,k
    character(len=60) :: errbuff 
    real(double) :: pol(3,3),transpol(3,3)
    nvar=ubound(polar_dxyz,2)
    allocate(rot_polar(6,nvar),polar_tmp(6,nvar),tmpmat(6,nvar))
    rot_polar=0.0d0
    do s=1,char_table%nsymel
      do i=1,nvar
        ! copy part of long packed matrix to square matrix
        counter=0
        do j=1,3
           do k=j,3
              counter=counter+1
              pol(k,j) = polar_dxyz(counter,i)
              pol(j,k) = pol(k,j)
           enddo
        enddo
        ! rotate square matrix
        transpol = matmul(symel_mat(char_table%symel(s)),&
             &            matmul(pol,transpose(symel_mat(char_table%symel(s)))))
        ! copy back to packed long matrix
        counter=0
        do j=1,3
           do k=j,3
              counter=counter+1
              polar_tmp(counter,i)=transpol(k,j)
           enddo
        enddo
      enddo
      ! perform transformation of gradient
      tmpmat = matmul(polar_tmp,rep(:,:,s))
      rot_polar=rot_polar + tmpmat
      do i=1,6
        do j=1,nvar
          errsum=errsum+abs(tmpmat(i,j)-polar_dxyz(i,j))
        end do
      enddo
      errsum=errsum/dble(nvar)/6.0d0
      if(errsum.gt.1.0d-3) then
        write(errbuff,'(a,f8.4,a,i3)') 'average error = ',errsum, ' for s = ',s
        call serror('symm_polar',0,trim(adjustl(errbuff)))
      end if
    enddo
    rot_polar=rot_polar/dble(char_table%nsymel)
    do i=1,6
      do j=1,nvar
        polar_dxyz(i,j) = rot_polar(i,j)
      end do
    end do
    deallocate(rot_polar,polar_tmp,tmpmat)
  end subroutine symm_polar

  !************************************************************************

  subroutine print_polar_dxyz(polar_dxyz,filename)
    implicit none
    !----------------------------------------------------------------
    ! output routine
    !----------------------------------------------------------------
    character*(*) :: filename
    integer :: i,k
    real(double),intent(in) :: polar_dxyz(:,:)

    open(unit=45,file=filename,status='unknown')
    write(45,*) 'cartesian polarizability derivatives'
    write(45,'(3d18.10)') ((polar_dxyz(i,k),i=1,ubound(polar_dxyz,1)),k=1,ubound(polar_dxyz,2))
    close (45)
  end subroutine print_polar_dxyz

  !************************************************************************

  subroutine destroy_polar_dxyz(polar_dxyz)
    implicit none
    !----------------------------------------------------------------
    ! clean up memory
    !
    ! written by Markus Reiher, Uni Erlangen, 2000
    !----------------------------------------------------------------
    real(double),pointer ::  polar_dxyz(:,:)
    if(associated(polar_dxyz)) deallocate(polar_dxyz)
  end subroutine destroy_polar_dxyz

  !************************************************************************

  subroutine create_polar_dq(polar_dxyz,norm_modes,molecule,polar_dq)
    !----------------------------------------------------------------
    ! written by M. Reiher, Uni Erlangen, 2000, 2001
    ! modified by Johannes Neugebauer, Uni Erlangen, 2001
    !
    ! Note: cartesian polarizability derivatives are given 
    !       in [Angstroem**2]. 
    !       derivatives with respect to normal coordinates are 
    !       obtained in [Angstroem**2/sqrt(amu)]
    !----------------------------------------------------------------
    use snf_molecule, only: type_atom
    implicit none
    real(double),intent(in) :: polar_dxyz(:,:),norm_modes(:,:)
    type(type_atom), pointer :: molecule(:)
    real(double),intent(inout):: polar_dq(:,:)
    integer :: i,j,k,nvar,irrep_dim

    nvar=ubound(norm_modes,1)
    irrep_dim=ubound(norm_modes,2)
    polar_dq=0.d0
    do k=1,6
       do i=1,nvar
          do j=1,irrep_dim
             polar_dq(k,j)= polar_dq(k,j) + polar_dxyz(k,i)*norm_modes(i,j) &
                  &        / dsqrt(molecule((i+2)/3)%mass)
          enddo
       enddo
    enddo
  end subroutine create_polar_dq

  !************************************************************************

end module snf_polar
