
!********************************************************************************
!*   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
!*   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
!*   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
!*
!*
!*   This file is part of the package SNF.
!*
!*   SNF is free software; you can redistribute it and/or modify
!*   it under the terms of the GNU General Public License as published by
!*   the Free Software Foundation; either version 2 of the License, or
!*   (at your option) any later version.
!*
!*   This program is distributed in the hope that it will be useful,
!*   but WITHOUT ANY WARRANTY; without even the implied warranty of
!*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*   GNU General Public License for more details.
!*
!*   You should have received a copy of the GNU General Public License
!*   along with this program; if not, write to the Free Software
!*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!*
!********************************************************************************
module snf_representation
  use f90_kind

  private
  public :: create_representation_for_vib, print_representation, &
       & destroy_representation

  !************************************************************************

contains

  !************************************************************************

  subroutine create_representation_for_vib(representation,molecule,char_table)
    !----------------------------------------------------------------
    ! written by Carsten Kind, Uni Koeln, 1994
    ! modified by J. Neugebauer, Uni Erlangen, 05/2001
    !----------------------------------------------------------------
    use snf_molecule, only:type_atom,coord_equal
    use snf_char_table,only:type_char_table
    use snf_symmat,only:symel_mat

    implicit none
    type(type_atom) :: molecule(:)
    type(type_char_table),intent(in) :: char_table
    real(double),pointer :: representation(:,:,:)
    integer i,atom1_ind,atom2_ind,natoms
    real(double) :: product(3), threshold
    logical :: isfine

    natoms=ubound(molecule,1)
    allocate(representation(natoms*3,natoms*3,char_table%nsymel))
    !----------------------------------------------------------------
    ! initialisialize representations
    !----------------------------------------------------------------
    representation(:,:,:) = 0.0

    do i=1,char_table%nsymel
       do atom1_ind=1,natoms

          ! get new atom
          isfine = .false.
          threshold = 1.0d-7
          do while(.not. isfine .and. threshold .lt. 1.d-3)
             atom2_ind = 1
             product=matmul(symel_mat(char_table%symel(i)), molecule(atom1_ind)%cartcoord)
             do while (.not. (coord_equal(product,molecule(atom2_ind)%cartcoord,threshold).and. &
                  molecule(atom2_ind)%element.eq.molecule(atom1_ind)%element) &
                  .and. atom2_ind .ne. natoms)
                atom2_ind = atom2_ind+1
             enddo
             if(atom2_ind .eq. natoms .and. .not.(coord_equal(product,molecule(atom2_ind)%cartcoord,threshold) &
                  .and.molecule(atom2_ind)%element.eq.molecule(atom1_ind)%element)) then
                isfine = .false.
                threshold = threshold * 2.d0
             else 
                isfine=.true.
             endif
          enddo
          if(.not. isfine) then
             write(*,*)' *************************************************'
             write(*,*)' ERROR: Molecule has not desired symmetry'
             write(*,*)' Could definitly not map symmetry equivalent atoms'
             write(*,*)' *************************************************'
             stop
          endif
          !----------------------------------------------------------------
          ! assign representation values
          !----------------------------------------------------------------
          representation(atom1_ind*3-2:atom1_ind*3,atom2_ind*3-2:atom2_ind*3,i) &
               = transpose(symel_mat(char_table%symel(i)))
       enddo! atom1 index
    enddo! symmetry elements
  end subroutine create_representation_for_vib

  !************************************************************************

  subroutine print_representation(representation,char_table,unit)
    !----------------------------------------------------------------
    ! print out representation matrizes 
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------
    use snf_char_table,only:type_char_table
    use snf_tools,only:printmat
    implicit none

    real(double) ,intent(in) :: representation(:,:,:)
    type(type_char_table),intent(in) :: char_table
    integer,optional :: unit
    integer i,out

    if(present(unit)) then
       out=unit
    else
       out=6
    endif
    write(out,*)
    write(out,*)' list of representations'
    write(out,*)' -----------------------'
    do i=1,ubound(representation,3)
       write(out,*)
       call printmat(representation(:,:,i), unit=out,&
            text='symmetry element: '//char_table%symel(i),fmt='F8.3')
    enddo
  end subroutine print_representation

  !************************************************************************

  subroutine destroy_representation(representation)
    !----------------------------------------------------------------
    ! Destructor
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------
    implicit none
    real(double) , pointer :: representation(:,:,:)

    deallocate(representation)
  end subroutine destroy_representation

  !************************************************************************

end module snf_representation
