
!********************************************************************************
!*   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
!*   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
!*   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
!*
!*
!*   This file is part of the package SNF.
!*
!*   SNF is free software; you can redistribute it and/or modify
!*   it under the terms of the GNU General Public License as published by
!*   the Free Software Foundation; either version 2 of the License, or
!*   (at your option) any later version.
!*
!*   This program is distributed in the hope that it will be useful,
!*   but WITHOUT ANY WARRANTY; without even the implied warranty of
!*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*   GNU General Public License for more details.
!*
!*   You should have received a copy of the GNU General Public License
!*   along with this program; if not, write to the Free Software
!*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!*
!********************************************************************************
module snf_restart
  use f90_kind
  use snf_error
  use snf_tools

  private
  public :: read_restart,write_restart,prepare_dummy_restart
  public :: backup_restart,check_restart_status

  !************************************************************************

contains

  !************************************************************************

  subroutine read_restart(filename,ctrl,&
       &      gradients,dipols,pols,istepfl,nvmach,status,ramanf,unit)
    !----------------------------------------------------------------
    ! read restart file, and check against actual control flags
    !
    ! written by Carsten Kind, Uni Erlangen, Autumn 1999
    ! modified by J. Neugebauer, Uni Erlangen, 2001
    !----------------------------------------------------------------
    use snf_control,only: type_ctrl,check_ctrl
    implicit none
    character(LEN=*),intent(in) :: filename
    character(len=10)  :: progtype
    character(len=10)  :: calctype
    type(type_ctrl), intent(inout) :: ctrl
    integer,intent(inout) :: status
    integer,optional :: unit
    real(double), pointer :: &
         & dipols(:,:,:),gradients(:,:,:), &
         & pols(:,:,:,:)
    character(len=8),pointer :: nvmach(:)
    character(len=9) :: form
    integer,pointer :: istepfl(:)
    logical ex,ramanf
    integer :: nvar,i,j,m,isum,nstept,idum,k,naus,natoms,l,in1,out
    integer :: inputpol,mode,freqn
    if(present(unit)) then
       out=unit
    else
       out=0
    endif
    ctrl%restart=trim(filename)
    inquire (file=ctrl%restart,exist=ex)
    if (.not.ex) then
       status=-9
       return
    endif
    in1=88
    if(out.ne.0) then
       write(out,*)
       write(out,'('' Restart file information'')')
       write(out,'('' ------------------------'')')
       write(out,'(''   restart file name         :'',1x,a)') trim(ctrl%restart)
    end if
    open (unit=in1,file=ctrl%restart,status='old')
    rewind(in1)

    read(in1,'(a)',end=444,err=444) ctrl%title
    read(in1,'(a)',end=444,err=444) form
    read(in1,'(10i5)',end=444,err=444) ctrl%version,  &
         &            mode,     &
         &            inputpol, &
         &            freqn, & 
         &            ctrl%nvar,     &
         &            ctrl%scfconv,  &
         &            naus
    if(.not.ramanf.and.(inputpol.ge.1)) &
      & call serror('read_restart',0,'inputpol.ne.0##&
      & in file restart, but Raman is switched off!',out)
    if(ramanf.and.(inputpol.eq.0)) call serror('read_restart',1,'inputpol=0&
      & in file restart, but Raman is switched on!',out)
    if(ramanf.and.ctrl%dynpol.and.(inputpol.ne.2)) &
      & call serror('read_restart',1,'inputpol.ne.2&
      & in file restart, but Raman##is switched on using dynamic&
      & polarizabilities!',out)
    if(.not.ctrl%dynpol.and.(inputpol.eq.2)) &
      & call serror('read_restart',1,'inputpol=2&
      & in file restart, but Raman##is switched on using static&
      & polarizabilities!',out)
    if(mode.ge.1.and. mode.le.25) then
       allocate(ctrl%tm_ctrl)
       ctrl%mode=mode
       ctrl%tm_ctrl%mode=mode
       ctrl%tm_ctrl%scfconv=ctrl%scfconv
       ctrl%tm_ctrl%inputpol=inputpol
    else 
      call serror('read_restart',2,'Unknown calculation mode.')
    endif
    if (mod(naus,2).ne.0) then
       deallocate(ctrl%tm_ctrl)
       call serror('read_restart',2,'odd number of steps for&
        & numerical derivative !',out)
    endif
    if (mod(ctrl%nvar,3).ne.0) then
       deallocate(ctrl%tm_ctrl)
       call serror('read_restart',2,'nvar is not a multiple of 3 !',out)
    endif
    nvar=ctrl%nvar
    ctrl%numderiv=naus/2
    read (in1,form) ctrl%cstep
    !    call check_ctrl(ctrl,ctrl)
    nstept=nvar*naus
    natoms=nvar/3
    allocate(nvmach(nstept),istepfl(nstept))
    read (in1,'(a)') 
    read (in1,'(a)') 
    do i=1,natoms
       j=(i-1)*naus*3
       read (in1,'(24i3)') idum,(istepfl(k),k=j+1,j+naus*3)
       read (in1,'(12a8)') (nvmach(k),k=j+1,j+naus*3)
    enddo
    call check_restart_status(istepfl,status)
    if(status.eq.1.or.status.eq.0) then
       allocate(dipols(3,nvar,2*ctrl%numderiv),&
            &   gradients(nvar,nvar,2*ctrl%numderiv))
       if(ramanf) then
         if(ctrl%dynpol.and.associated(ctrl%dynfreq)) then
           if(ubound(ctrl%dynfreq,1).ne.freqn) then
             call serror('read_restart',1,'No. of dyn. frequencies in restart&
              & differs##from that in snf_control !')
             goto 225
           end if
           allocate(pols(8,nvar,2*ctrl%numderiv,ubound(ctrl%dynfreq,1)))
         else
           allocate(pols(8,nvar,2*ctrl%numderiv,1))
         end if
       end if
       do i=1,natoms
          read (in1,'(a)',err=222,end=222)
          read (in1,form,err=222,end=222) (((dipols(k,(i-1)*3+l,j),&
           &k=1,3),j=1,naus),l=1,3) 
       enddo
       do i=1,natoms
          read (in1,'(a)',err=223,end=223)
          read (in1,form,err=223,end=223) (((gradients(k,(i-1)*3+l,j),&
           &k=1,nvar),j=1,naus),l=1,3) 
       enddo
       if (ramanf) then
         if(ctrl%dynpol.and.associated(ctrl%dynfreq)) then
           do m=1,ubound(ctrl%dynfreq,1)
             do i=1,natoms
!JN              read (in1,'(a43,form)',err=224,end=224)
              read (in1,'(a)',err=224,end=224)
              read (in1,form,err=224,end=224) (((pols(k,(i-1)*3+l,j,m),k=1,8),&
               j=1,naus),l=1,3)
             end do
           end do
         else
           do i=1,natoms
             read (in1,'(a)',err=224,end=224)
             read (in1,form,err=224,end=224) (((pols(k,(i-1)*3+l,j,1),k=1,8),&
              j=1,naus),l=1,3)
           end do
         end if
       endif
       !----------------------------------------------------------------
       ! data conservation for partly completed restart files without 
       ! "-1" entries
       !-----------------------------------------------------------------
       if(status.eq.0) then
         if(any(abs(dipols).gt.1.0d-10)) status = -1
         if(any(abs(gradients).ge.1.0d-10)) status = -1
         if(ramanf) then
           if(any(abs(pols).ge.1.0d-10)) status = -1
         end if
       end if
    endif
    close(in1)
    goto 333
222 call serror('read_restart',1,'Error while reading dipole moments from restart') 
    goto 225
223 call serror('read_restart',1,'Error while reading gradients from restart') 
    goto 225
224 call serror('read_restart',1,'Error while reading polarizabilities from restart') 
    goto 225
225 close(in1)
    status=-2
333 continue
    !----------------------------------------------------------------
    ! Print restart information
    !----------------------------------------------------------------
    if(out.ne.0) then
       isum=0
       do i=1,nstept
          if(istepfl(i).gt.0) isum=isum+istepfl(i) 
       enddo
       if(status.ne.1.and.status.ne.-1) isum=0
       call prog_type(ctrl%mode,progtype,calctype)
       write(out,'(''   title                     : '',A)') &
            & trim(adjustl(ctrl%title))
       write(out,'(''   program for single points : '',a)') &
        & progtype
       write(out,'(''   type of calculation       : '',a)') &
        & calctype
!---------------------------------------------------------------------
! THIS IS ONLY FOR THE TURBOMOLE VERSION
!---------------------------------------------------------------------
       if(index(progtype,'TURBOMOLE').eq.0) &
      & call serror('read_restart',2,'ONLY TURBOMOLE CAN BE USED&
      & IN THIS VERSION OF SNF !')
!---------------------------------------------------------------------
       write(out,'(''   file format version       :'',F6.2)') &
            &  dble(ctrl%version)/100.D0
       write(out,'(''   SCF convergence criterion :'',I6)')ctrl%scfconv
       write(out,'(''   Number of atoms           :'',I6)') natoms
       write(out,'(''   Number of steps           :'',I6)') naus
       write(out,'(''   distortion [bohr]         : '',F6.3)') ctrl%cstep
       if(inputpol.eq.1) &
            & write(out,'(''   Polarizabilities are available.'')')
       if(inputpol.eq.2) &
            & write(out,'(''   Dynamic polarizabilities are available.'')')
       if(mode.eq.2) &
            & write(out,'(''   RI approximation is applied.'')')
       write(out,*)
    end if
    write (out,*) 
    write(out,'(f8.2,a)') 100.*dble(isum)/dble(nstept),&
         & ' % of all calculations have been done yet'
    write(out,'(a)') '  (or are redundant because of the molecular symmetry).'
    return

444 close(in1)
    call serror('read_restart',2,'Error while reading basic restart-file&
     & information')
  end subroutine read_restart

  !************************************************************************

  subroutine write_restart(gradients,dipols,pols,istepfl,nvmach,ctrl,unit)
    !----------------------------------------------------------------
    ! write restart file, if data are not available fill up with zeros.
    !
    ! written by Carsten Kind, Uni Erlangen, 2000
    ! modified by J. Neugebauer, Uni Erlangen, 05/2001
    !----------------------------------------------------------------
    use snf_control,only: type_ctrl
    implicit none
    type(type_ctrl), intent(in) :: ctrl
    real(double), pointer :: &
         & dipols(:,:,:),gradients(:,:,:), &
         & pols(:,:,:,:)
    character(len=8),pointer :: nvmach(:)
    integer,pointer :: istepfl(:)
    integer,optional :: unit
    integer :: nvar,i,j,m,isum,nstept,k,naus,natoms,l,out1,out,inputpol
    integer :: freqn !former iomeg variable
    character(len=9) :: form
    external system
    data form /'(4d20.10)'/

    if(present(unit)) then
       out=unit
    else
       out=0
    endif
    out1=88
    open (unit=out1,file=ctrl%restart,status='unknown')
    write(out1,'(a)') ctrl%title
    write(out1,'(a9)') form
    freqn=0
    if(ctrl%raman.and.(.not.ctrl%dynpol)) then
       inputpol=1
       freqn=1
    else if(ctrl%raman.and.(ctrl%dynpol)) then
       inputpol=2
       freqn=1
       if(associated(ctrl%dynfreq)) freqn=ubound(ctrl%dynfreq,1)
    else
       inputpol=0
    endif
    write(out1,'(10i5)') ctrl%version,  &
         &            ctrl%mode,     &
         &            inputpol, &
         &            freqn, & 
         &            ctrl%nvar,     &
         &            ctrl%scfconv,  &
         &            2*ctrl%numderiv   
    write (out1,form) ctrl%cstep
    naus=2*ctrl%numderiv   
    nvar=ctrl%nvar
    nstept=nvar*naus
    natoms=nvar/3
    write (out1,'(a)') 'mydir'
    write (out1,'(a)') 'tempdir'
    do i=1,natoms
       j=(i-1)*naus*3
       if(associated(istepfl)) then
          write (out1,'(24i3)') i,(istepfl(k),k=j+1,j+naus*3)
       else
          write (out1,'(24i3)') i,(0,k=j+1,j+naus*3)
       endif
       if(associated(nvmach)) then
          write (out1,'(12a8)') (nvmach(k),k=j+1,j+naus*3)
       else
          write (out1,'(12a8)') ('------',k=j+1,j+naus*3)
       endif
    enddo
    if(associated(dipols)) then
       do i=1,natoms
          write (out1,*) 'dipoles atom',i
          write (out1,form) (((dipols(k,(i-1)*3+l,j),k=1,3),j=1,naus),l=1,3) 
       enddo
    else
       do i=1,natoms
          write (out1,*) 'dipoles atom',i
          write (out1,form) (((0.d0,k=1,3),j=1,naus),l=1,3) 
       enddo
    endif
    if(associated(gradients)) then
       do i=1,natoms
          write (out1,*) 'gradients atom',i
          write (out1,form) (((gradients(k,(i-1)*3+l,j),k=1,nvar),j=1,naus),l=1,3) 
       enddo
    else
       do i=1,natoms
          write (out1,*) 'gradients atom',i
          write (out1,form) (((0.d0,k=1,nvar),j=1,naus),l=1,3) 
       enddo
    endif
    if (ctrl%raman) then
      if(associated(pols)) then
        if(ctrl%dynpol.and.associated(ctrl%dynfreq)) then
          do m=1,ubound(ctrl%dynfreq,1)
            do i=1,natoms
              write (out1,'(a22,i8,a13,d20.10)') ' polarizabilities atom',i,&
               &' frequency = ',ctrl%dynfreq(m)
              write (out1,form) (((pols(k,(i-1)*3+l,j,m),k=1,8),j=1,naus),l=1,3) 
            enddo
          end do
        else
          do i=1,natoms
            write (out1,*) 'polarizabilities atom',i
            write (out1,form) (((pols(k,(i-1)*3+l,j,1),k=1,8),j=1,naus),l=1,3) 
          enddo
        end if
      else
        if(ctrl%dynpol.and.associated(ctrl%dynfreq)) then
          do m=1,ubound(ctrl%dynfreq,1)
            do i=1,natoms
              write (out1,'(a22,i8,a13,d20.10)') ' polarizabilities atom',i,&
               &' frequency = ',ctrl%dynfreq(m)
              write (out1,form) (((0.0d0,k=1,8),j=1,naus),l=1,3)
            enddo
          end do
        else
          do i=1,natoms
            write (out1,*) 'polarizabilities atom',i
            write (out1,form) (((0.d0,k=1,8),j=1,naus),l=1,3) 
          enddo
        end if
      endif
    endif
    close (out1)
    if(out.ne.0) then
       write(out,*)
       write(out,'(''  Created new restart file : '',A)') &
            &    trim(adjustl(ctrl%restart))

       isum=0
       do i=1,nstept
          if(istepfl(i).gt.0) isum=isum+istepfl(i) 
       enddo
       write(out,*)
       write(out,'(f8.2,a)') 100.*dble(isum)/dble(nstept),&
            & ' % of all calculations have been done yet'
       write(out,'(a)') ' (or are redundant because of the molecular symmetry).'
    endif
  end subroutine write_restart

  !************************************************************************


  subroutine prepare_dummy_restart(irred_atom_index,anzahl_irred_atome,&
     & ctrl,istepfl)
    !----------------------------------------------------------------
    ! fill up istepfl array according to irreducible atoms to be calculated
    !
    ! written by Carsten Kind, Uni Erlangen, 2000
    ! modified by J. Neugebauer, Uni Erlangen, 2001
    !----------------------------------------------------------------
    use snf_control,only : type_ctrl
    implicit none
    type(type_ctrl) :: ctrl
    integer,pointer :: irred_atom_index(:)
    integer :: idum
    integer,pointer :: istepfl(:)
    integer i,nstept,anzahl_irred_atome,start,ende
    nstept =ctrl%nvar*2*ctrl%numderiv   
    if(associated(istepfl)) deallocate(istepfl)
    allocate(istepfl(nstept))
! this might cause problems if ctrl%numderiv is changed ...
!    if(.not. associated(istepfl)) allocate(istepfl(nstept))
    istepfl(:)=1
    idum=ctrl%numderiv
    do i=1,anzahl_irred_atome
       start=(irred_atom_index(i)-1)*6*idum+1
       ende=  irred_atom_index(i)   *6*idum
       istepfl(start:ende) = 0
    end do
  end subroutine prepare_dummy_restart

  !************************************************************************

  subroutine backup_restart(restart,unit)
    !----------------------------------------------------------------
    ! move restart-file to files restart.1, restart.2 ...
    !
    ! written by Carsten Kind, Uni Erlangen, 2001
    !----------------------------------------------------------------
    implicit none
    character(len=80),intent(in) :: restart
    integer,optional,intent(in) :: unit
    character(len=80) :: backup,errbuff
    integer :: counter,info,system,out
    character(len=3) :: count_string
    logical :: ex
    if(present(unit)) then
       out=unit
    else
       out=6
    endif
    counter=0
    backup = trim(adjustl(restart))
    do 
       inquire (file=backup,exist=ex)
       if(.not. ex) exit
       counter=counter+1
       write(count_string,'(i3)') counter
       backup = trim(adjustl(restart)) //'.' //trim(adjustl(count_string))
    enddo
    if(counter.ne.0) then
       info = system('mv '//trim(restart)//' '//trim(backup))
       if (info .ne. 0) then
         write(errbuff,'(a,a,a,a)') 'error while moving ',trim(restart),&
          &' to ',trim(backup)
         call serror('backup_restart',2,errbuff)
       else
          write(out,'(''  Moved old restart file to '',A)') &
               &    trim(adjustl(backup))
       endif
    endif
  end subroutine backup_restart

  !************************************************************************

  subroutine check_restart_status(istepfl,status)
    !----------------------------------------------------------------
    ! check status f restart file
    !  Input:
    !    filename of restart file
    !  Output:
    !    status:
    !     -1  running or aborted calculation
    !      0  restart file is brand new and empty
    !      1  calculation finished
    !
    ! written by Carsten Kind, Uni Erlangen, 2001
    !----------------------------------------------------------------
    implicit none
    integer,intent(out) :: status
    integer,intent(in)  :: istepfl(:)
    integer :: i,nstept,zeros,minusones,plusones
    nstept=ubound(istepfl,1)
    zeros=0
    minusones=0
    plusones=0
    do i=1,nstept
       select case(istepfl(i))
       case(0)
          zeros=zeros+1
       case(1) 
          plusones=plusones+1
       case(-1) 
          minusones=minusones+1
       end select
    enddo
    if(plusones.eq.nstept) then 
       status= 1     ! finished calculation
    else if(minusones .ne. 0) then
       status=-1     ! running or aborted job
    else 
       status= 0     ! fresh restart
    endif
  end subroutine check_restart_status

  !************************************************************************

end module snf_restart
