
!********************************************************************************
!*   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
!*   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
!*   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
!*
!*
!*   This file is part of the package SNF.
!*
!*   SNF is free software; you can redistribute it and/or modify
!*   it under the terms of the GNU General Public License as published by
!*   the Free Software Foundation; either version 2 of the License, or
!*   (at your option) any later version.
!*
!*   This program is distributed in the hope that it will be useful,
!*   but WITHOUT ANY WARRANTY; without even the implied warranty of
!*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*   GNU General Public License for more details.
!*
!*   You should have received a copy of the GNU General Public License
!*   along with this program; if not, write to the Free Software
!*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!*
!********************************************************************************
module snf_spectrum
    !----------------------------------------------------------------
    !
    ! written by Carsten Kind and Markus Reiher
    ! changed by J. Neugebauer, 06/2001
    !----------------------------------------------------------------
  use f90_kind
  use snf_error


  private
  public :: type_peak, print_ir, destroy_spectrum, &
       &    print_raman, calc_raman_intensities, &
       &    calc_frequencies, calc_ir_intensities, &
       &    print_norm_modes,sort_spectrum, &
       &    get_translation_and_rotation,zeropoint_energy,assign_irrep_number, &
       &    calc_mode_properties,get_relcross,type_raman,print_rcs, &
       &    get_relir,sum_degen

  !************************************************************************
  type type_raman
    real(double) :: cperu,cparu,cperper, cparpar, int, depol_ratio,&
       &             normcross, relcross, relint, cperpar, cnu, relac
  end type type_raman

  type type_peak
     real(double) :: freq,ir_int,rel_ir_int,red_mass,force
     type(type_raman),pointer :: raman(:)
     character(len=5) :: irrep
     integer :: number_in_irrep,g
     real(double),pointer :: norm_mode(:)
  end type type_peak

  !************************************************************************

contains

  !************************************************************************

  subroutine calc_frequencies(hessian_q,spectrum)
    !----------------------------------------------------------------
    ! Calculation of wave numbers in cm^-1 
    ! with conversion factors based on SI units
    !
    ! written by Carsten Kind, Uni Erlangen, 1999
    !----------------------------------------------------------------
    use snf_param
    implicit none
    type(type_peak),intent(inout) :: spectrum(:)
    real(double),intent(in) :: hessian_q(:)
    real(double) :: atu2sec
    integer :: i
    !----------------------------------------------------------------
    ! frequency[1/s]=1/(2*pi)*sqrt(d^2 e/ dq^2)
    ! wavenumber[cm-1] = 1/c * frequency
    !
    ! factor 1.0d-06 results from conversion of kj-->j and g-->kg
    ! factor 1.d0+2 results from conversion of c[m/s] -->c[cm/s]
    !
    ! note that mass-weighting was performed using [amu], not [m_e] !
    ! => multiply by Avogadro's number
    !----------------------------------------------------------------
    atu2sec = 1.0d-06*(bohr**2)/hartree2kj_mol
    do i=1,ubound(hessian_q,1)
       if (hessian_q(i).lt.0.0d0) then
          spectrum(i)%freq=hessian_q(i)/atu2sec
          spectrum(i)%freq=-( 1.d0/(2.0d0*pi)*sqrt(-spectrum(i)%freq ) /(1.d+02*speedoflight))
       else
          spectrum(i)%freq=hessian_q(i)/atu2sec
          spectrum(i)%freq=( 1.d0/(2.0d0*pi)*sqrt(spectrum(i)%freq )/(1.d+02*speedoflight))
       endif
    enddo
  end subroutine calc_frequencies

  !************************************************************************

  subroutine calc_mode_properties(molecule,spectrum)
    !----------------------------------------------------------------
    ! Calculation of reduced mass and force constants
    ! with conversion factors based on SI units
    !
    ! written by Carsten Kind, Uni Erlangen, 2001
    !----------------------------------------------------------------
    use snf_molecule,only : type_atom
    use snf_param
    implicit none
    type(type_atom), intent(in) :: molecule(:)
    type(type_peak),intent(inout) :: spectrum(:)
    integer :: i,j,nmode,mode,nvar
    real(double),allocatable :: cart(:,:)
    real(double) :: amu2kg,fac,omega
    nmode=ubound(spectrum,1)
    nvar=ubound(spectrum(1)%norm_mode,1)
    amu2kg=1.d0/1000.d0/avogadro
    allocate(cart(nvar,nmode))
    do i=1,nvar
       fac=1.d0/sqrt(molecule((i+2)/3)%mass)
       do j=1,nmode
          cart(i,j)=fac*spectrum(j)%norm_mode(i)
       enddo
    enddo
    do mode=1,nmode
       spectrum(mode)%red_mass=1.d0/dot_product(cart(:,mode),cart(:,mode))
       omega=(2*pi*speedoflight*spectrum(mode)%freq*100.d0)
       ! 1 dyne  =10^-5 n
       ! 1 ang   = 10^-10 m
       ! ==> 1 n/m = 10^-2 mdyne/ang == last factor
       spectrum(mode)%force=amu2kg*spectrum(mode)%red_mass * omega**2 /100.d0
    enddo
    deallocate(cart)
  end subroutine calc_mode_properties

  !************************************************************************

  subroutine calc_intensities_cgs(dipol_dq,spectrum)
    use snf_param
    implicit none
    !----------------------------------------------------------------
    ! calculates IR intensities and transforms them to cgs units.
    ! finally, the result will be divided by 100000 to obtain km/mol
    !
    !     n_a * pi   / ddipol \ 2
    ! i = -------- * | ------ |
    !     3 * c**2   \   dq   /
    !
    ! unit is first m/mol and then km/mol.
    !----------------------------------------------------------------
    type(type_peak),intent(inout) :: spectrum(:)
    real(double),intent(in) :: dipol_dq(:,:)
    real(double) :: avo,c,toe,umrechnung
    integer :: i

    avo=6.02213670d23 ! avogadro constant (inverse is amu in cgs)
    c=2.99792458d10   ! speed of light in cgs units
    toe=4.803242d-10  ! electron charge in electrostatic units (esu)
    umrechnung = toe*dsqrt((avo*avo*pi)/(3.0d0*c*c))

    do i = 7, ubound(dipol_dq,1)
       spectrum(i)%ir_int = 1.d-5*umrechnung*umrechnung* &
            & (dot_product(dipol_dq(i,:),dipol_dq(i,:)))
    enddo
  end subroutine calc_intensities_cgs

  !************************************************************************

  subroutine calc_ir_intensities(dipol_dq,spectrum)
    use snf_param
    implicit none
    !----------------------------------------------------------------
    ! calculates ir intensities 
    ! with conversion factors based on SI units
    ! finally, the result will be divided by 1000 to obtain km/mol
    !
    !     1                 n_a * pi   / ddipol \ 2
    ! i = -------------- *  -------- * | ------ |
    !     4*pi*epsilon_0    3 * c**2   \   dq   /
    !
    ! unit is first m/mol then km/mol.
    !
    ! written by Carsten Kind, Uni Erlangen, 1999
    !----------------------------------------------------------------
    type(type_peak),intent(inout) :: spectrum(:)
    real(double),intent(in) :: dipol_dq(:,:)
    real(double) :: dipol_dq2mks , prefactor
    integer :: i
    dipol_dq2mks= el_charge/sqrt(atomic_mass_unit)
    prefactor = 1/(4*pi*epsilon_0)*(avogadro*pi)/(3.0d0*speedoflight**2)
    prefactor = 1.0d-3 * prefactor
    !     auf debye/(angstrom*amu)

    do i = 1, ubound(dipol_dq,2)
       spectrum(i)%ir_int = prefactor * dipol_dq2mks**2 * &
            & (dot_product(dipol_dq(:,i),dipol_dq(:,i)))
    enddo
  end subroutine calc_ir_intensities

  !************************************************************************

  subroutine calc_raman_intensities(polar_dq,spectrum,temperature,theta,&
   &intno,wvn)
    use snf_param
    implicit none
    !----------------------------------------------------------------
    ! calculates Raman intensities
    !
    ! units: polarizability derivatives are given in 
    !        [Angstroem**2/sqrt(amu)], intensities (Raman scattering
    !        factors) are therefore in [Angstroem**4/amu]
    ! 
    ! written by C. Kind and M. Reiher
    ! modified by J. Neugebauer 2001
    !----------------------------------------------------------------
    type(type_peak),intent(inout) :: spectrum(:)
    real(double),intent(in) :: polar_dq(:,:),theta
    real(double),optional :: wvn
    real(double) :: alpha_p, gamma_p,dummy,conver,tempfac,temperature,vbyt, &
         &   lambda_0,lambda_z,lambda_t,thetarad,gscr,ascr
    integer :: i,nvar
    integer,intent(in) :: intno
    parameter(conver=2.d0*planck_h*pi*pi/speedoflight*1.d-40/atomic_mass_unit)
    !----------------------------------------------------------------
    ! note: lambda_... is a wavenumber
    !----------------------------------------------------------------
    ! vbyt = planck_h*speedoflight*10^2(cm/m) / boltz
    ! vbyt = 1.43877522095016535323970161017601633d0
    !----------------------------------------------------------------
    parameter(vbyt=planck_h*speedoflight*100.0d0/boltz)

    thetarad=theta*2.d0*pi/360.d0
    if(present(wvn)) then
      lambda_0=wvn/nm2freq*1.0d7
    else
      !-------------------------------------------------------------------
      ! set default wave number (Ar ion laser, wavelength =  514.5 nm 
      ! => wavenumber lambda_0 = 19436.3459669582118561710d0 1/cm)
      !-------------------------------------------------------------------
      lambda_0=19436.3459669582118561710d0
    end if


    nvar=ubound(polar_dq,2) 
    do i = 1,nvar 
       !-----------------------------
       ! store actual mode wavenumber
       !-----------------------------
       lambda_z=spectrum(i)%freq

       !=====================================================================
       ! Note: 	The formulae for alpha_p and gamma_p require
       ! 	the following order of the elements of the
       !	polarizability tensor in matrix polar_dq:
       !	
       ! alpha_xx=polar_dq(1,)  alpha_xy=polar_dq(2,)  alpha_xz=polar_dq(3,)
       !                        alpha_yy=polar_dq(4,)  alpha_yz=polar_dq(5,)
       !                                               alpha_zz=polar_dq(6,)
       !
       !        NUMFREQ reads the components exactly in this manner and
       !        writes them to the restart file. Additionally, a and 
       !        gamma are stored.
       !
       !        Might there be an error in the TURBOMOLE-code ???
       !        
       !        (J. Neugebauer, 05/2001)
       !=====================================================================

       alpha_p=(polar_dq(1,i)+polar_dq(4,i)+polar_dq(6,i))/3.0d0
       alpha_p=alpha_p*alpha_p

       gamma_p=abs(1.5d0*(polar_dq(1,i)**2+polar_dq(4,i)**2+polar_dq(6,i)**2) &
     &    +3.0d0*(polar_dq(2,i)**2+polar_dq(3,i)**2+polar_dq(5,i)**2)-4.5d0*alpha_p)
       spectrum(i)%raman(intno)%int = 45.0d0 * alpha_p + 7.0d0 * gamma_p
       dummy= 45.0d0*alpha_p+4.0d0*gamma_p
       if (dummy.ge.5.0d-7) then
          spectrum(i)%raman(intno)%depol_ratio = 3.0d0 * gamma_p /dummy
       else
          spectrum(i)%raman(intno)%depol_ratio = -1.0d0
       end if

       !------------------------------------------------------------------------
       ! calculate absolute differential raman scattering cross section 
       !
       !   d(sigma)/d(omega) = h/(8*(epsilon0)^2 *c) *(lambda_0-lambda_z)^4/lambda_z
       !                       /(1-exp(-h*c*lambda_z/(kT)) 
       !                       * (polar_dq's and spherically averaged)^2
       !------------------------------------------------------------------------

       if (temperature.le.1.0d-10) then
          tempfac=1.0d0
       else
          tempfac=1.0d0/(1.0d0-exp(-vbyt*lambda_z/temperature))
       end if
       !------------------------------------------------------------------------
       ! Q-Branch is considered
       !------------------------------------------------------------------------
       lambda_t=tempfac*(lambda_0-lambda_z)**4/lambda_z *1.0d6
       
       !------------------------------------------------------------------------
       ! 1.0d+6 changes from cm^(-3) to m^(-3)
       !------------------------------------------------------------------------
       ! conversion of squared polarizability derivative from [a.u.] 
       ! (this is no longer used, since escf yields polarizability volumes
       !  in [bohr**3], not polarizablities in [e**2*bohr/sqrt(me)/Eh])
       ! into SI units:
       !      e**4*bohr**2/[(me)*(Eh)**2] = 1.0656941d-31 C**4*m**2/kg/J**2
       ! 
       ! differential cross sections:
       ! - constant pre-factor for differential scattering cross sections:
       !        h/(8*(epsilon0)**2*c) = 3.524100053d-21 J**3*s**2*m/C**4
       ! - lambda_t is given in [1/m**3]
       ! - squared polarizability derivatives have to be given in 
       !   [C**2*m**2/V**2/kg], but they are given as squared derivatives of 
       !   polarizability volumes in [Angstroem**4/amu].
       !   conversion factor: (4*pi*epsilon0)**2*10**(-40){Angstroem**4/m**4}
       !                      /atomic_mass_unit{amu/kg}
       !
       ! => total conversion factor: 
       !   conver = 2*planck_h*pi**2/c * 10**40/atomic_mass_unit
       ! 
       ! variable:	incident beam pol.	analyzer pol.
       !--------------------------------------------------------------
       ! cperu		perp. to scatt. plane	none
       ! cparu		parall.			none
       ! cperper 	perp.			perp.
       ! cperpar	perp.			parall.
       ! cparper        parall.			perp.           = cperpar
       ! cparpar	parall.			parall.
       ! cnper		natural (unpolarized)	perp.           = cperu * 0.5d0
       ! cnpar		natural (unpolarized)	parall.         = cparu * 0.5d0
       ! cnu		natural (unpolarized)   none
       !
       ! For theta = 90 deg. : cparpar = cperpar = cparper
       !
       ! standard geometry corresponds to cperu(theta = 90 deg., incident
       !  beam perp. to scatt. plane, no analyzer)
       !                 
       ! normcross : cperu * (lambda_0-lambda_z)**(-4)
       ! ("normalized differential Raman scattering cross sections")
       ! 
       ! relint    : cperu/max(cperu)
       ! (not calculated in this subroutine, but in get_relcross)
       !------------------------------------------------------------------------
       gscr = gamma_p/15.0d0*conver*lambda_t
       ascr = (alpha_p+4.0d0/45.0d0*gamma_p)*conver*lambda_t
       spectrum(i)%raman(intno)%cperpar=gscr
       spectrum(i)%raman(intno)%cperper=ascr
       spectrum(i)%raman(intno)%cperu=gscr+ascr
       spectrum(i)%raman(intno)%cparpar=gscr*dsin(thetarad)*dsin(thetarad) &
     &   + ascr*dcos(thetarad)*dcos(thetarad)
       spectrum(i)%raman(intno)%cparu=gscr+spectrum(i)%raman(intno)%cparpar
       spectrum(i)%raman(intno)%cnu=ascr*0.5d0 + gscr &
     &   + spectrum(i)%raman(intno)%cparpar*0.5d0

       spectrum(i)%raman(intno)%normcross=spectrum(i)%raman(intno)%cperu*&
         &(lambda_0-lambda_z)**(-4)*1.0d-8
    enddo
  end subroutine calc_raman_intensities

  !************************************************************************
    
  subroutine get_relir(spectrum)
    !----------------------------------------------------------------
    ! calculate relative IR intensities rel_ir_int
    ! 
    ! written by J. Neugebauer, Uni Erlangen, 2001
    !----------------------------------------------------------------
    implicit none
    type(type_peak) :: spectrum(:)
    real(double) :: maxir
    integer :: i

    maxir=0.0d0
    do i=1,ubound(spectrum,1)
      if(spectrum(i)%ir_int.gt.maxir) &
       & maxir = spectrum(i)%ir_int
    end do
    if(maxir.gt.1.d-8) then
      do i=1,ubound(spectrum,1)
        spectrum(i)%rel_ir_int=spectrum(i)%ir_int/maxir
      end do
    else
      do i=1,ubound(spectrum,1)
        spectrum(i)%rel_ir_int=0.0d0
      end do
    end if

  end subroutine get_relir
  !************************************************************************
    
  subroutine get_relcross(spectrum,intno)
    !----------------------------------------------------------------
    ! calculate relative normalized Raman scattering cross sections
    ! raman%relint = raman%cperu/max(raman%cperu)
    ! raman%relac = raman%int/max(raman%int)
    ! raman%relcross = raman%normcross/raman%normcross(dinitrogen,Q-Branch)
    ! 
    ! For details of parameter n_n2, see:
    !  H.W. Schroetter, H.W. Kloeckner, Topics in Current Physics 11, 
    !  Springer 1979, p. 123 - 166.
    ! 
    !  exp. value :  n_n2 = 5.05d-60 m^6/sr,
    !  TZVP/BP86  :  n_n2 = 6.61d-60 m^6/sr 
    !                (using SNF, Numfreq and TURBOMOLE)
    !
    ! written by J. Neugebauer, Uni Erlangen, 2001
    !----------------------------------------------------------------
    implicit none
    type(type_peak) :: spectrum(:)
    integer :: i,intno
    real(double) :: maxcperu,n_n2,maxac
    parameter(n_n2=5.05d-60)
    maxcperu=0.0d0
    maxac=0.0d0
    do i=1,ubound(spectrum,1)
      if(spectrum(i)%raman(intno)%cperu.gt.maxcperu) &
       & maxcperu = spectrum(i)%raman(intno)%cperu
      if(spectrum(i)%raman(intno)%int.gt.maxac) &
       & maxac = spectrum(i)%raman(intno)%int
    end do
    if(maxac.gt.1.d-10) then
      do i=1,ubound(spectrum,1)
        spectrum(i)%raman(intno)%relint=spectrum(i)%raman(intno)%cperu/maxcperu
        spectrum(i)%raman(intno)%relac=spectrum(i)%raman(intno)%int/maxac
        spectrum(i)%raman(intno)%relcross=spectrum(i)%raman(intno)%normcross/n_n2
      end do
    else
      do i=1,ubound(spectrum,1)
        spectrum(i)%raman(intno)%relint=0.0d0
        spectrum(i)%raman(intno)%relac=0.0d0
        spectrum(i)%raman(intno)%relcross=0.0d0
      end do
    end if
  end subroutine get_relcross

  !************************************************************************

  subroutine sum_degen(spectrum,sum_spectrum,ramanfl)
    !----------------------------------------------------------------
    ! sum intensities for degenerate modes
    !
    ! written by J. Neugebauer, Uni Erlangen, 2001
    !----------------------------------------------------------------
    implicit none
    type(type_peak),pointer :: sum_spectrum(:)
    type(type_peak),intent(in) :: spectrum(:)
    integer, pointer :: in(:)
    integer :: i,j,nmodes,nfreq,nnorm,jj
    real(double) :: ggg
    logical,intent(in) :: ramanfl
    nmodes=0
    i=0
    do 
      nmodes=nmodes+1
      i=i+spectrum(i+1)%g
     if(i.eq.ubound(spectrum,1)) exit
      if(i.gt.ubound(spectrum,1)) call serror('sum_degen',2,&
        &'Wrong number of normal modes.')
    end do
    allocate(sum_spectrum(nmodes),in(nmodes))
    if(ramanfl) then
      nfreq=ubound(spectrum(1)%raman,1)
      nnorm=ubound(spectrum(1)%norm_mode,1)
      do i=1,nmodes
        allocate(sum_spectrum(i)%raman(nfreq))
        allocate(sum_spectrum(i)%norm_mode(nnorm))
      end do
    end if
    j=0
    i=0
    do 
      i=i+spectrum(i+1)%g
      j=j+1
      in(j)=i
      if(j.eq.nmodes) exit
    end do
    do i=1,nmodes
      j=in(i)
      sum_spectrum(i)%freq=spectrum(j)%freq
      sum_spectrum(i)%irrep=spectrum(j)%irrep
      sum_spectrum(i)%g=spectrum(j)%g
      ggg=dble(spectrum(j)%g)
      if(ramanfl) then
!---------------------------------------------------------------------------
! the brain-damaged pgf90v3.3-1 is not able to perform this implicit loops
! in contrast to the former version pgf90v3.2-4
!---------------------------------------------------------------------------
!        sum_spectrum(i)%raman(:)%int=spectrum(j)%raman(:)%int*ggg
!        sum_spectrum(i)%raman(:)%cperu=spectrum(j)%raman(:)%cperu*ggg
!        sum_spectrum(i)%raman(:)%cperper=spectrum(j)%raman(:)%cperper*ggg
!        sum_spectrum(i)%raman(:)%cperpar=spectrum(j)%raman(:)%cperpar*ggg
!        sum_spectrum(i)%raman(:)%cparpar=spectrum(j)%raman(:)%cparpar*ggg
!        sum_spectrum(i)%raman(:)%cparu=spectrum(j)%raman(:)%cparu*ggg
!        sum_spectrum(i)%raman(:)%cnu=spectrum(j)%raman(:)%cnu*ggg
!        sum_spectrum(i)%raman(:)%normcross=spectrum(j)%raman(:)%normcross*ggg
!        sum_spectrum(i)%raman(:)%depol_ratio=spectrum(j)%raman(:)%depol_ratio
        do jj=1,nfreq
          sum_spectrum(i)%raman(jj)%int=spectrum(j)%raman(jj)%int*ggg
          sum_spectrum(i)%raman(jj)%cperu=spectrum(j)%raman(jj)%cperu*ggg
          sum_spectrum(i)%raman(jj)%cperper=spectrum(j)%raman(jj)%cperper*ggg
          sum_spectrum(i)%raman(jj)%cperpar=spectrum(j)%raman(jj)%cperpar*ggg
          sum_spectrum(i)%raman(jj)%cparpar=spectrum(j)%raman(jj)%cparpar*ggg
          sum_spectrum(i)%raman(jj)%cparu=spectrum(j)%raman(jj)%cparu*ggg
          sum_spectrum(i)%raman(jj)%cnu=spectrum(j)%raman(jj)%cnu*ggg
          sum_spectrum(i)%raman(jj)%normcross=spectrum(j)%raman(jj)%normcross*ggg
          sum_spectrum(i)%raman(jj)%depol_ratio=spectrum(j)%raman(jj)%depol_ratio
        end do
      end if
      sum_spectrum(i)%ir_int=spectrum(j)%ir_int*ggg
      sum_spectrum(i)%number_in_irrep=0
    end do
    deallocate(in)
  end subroutine sum_degen
  
  !************************************************************************

  subroutine print_ir(spectrum,unit)
    !----------------------------------------------------------------
    ! output routine
    ! written by Carsten Kind, Uni Erlangen, 1999
    ! modified by J. Neugebauer, Uni Erlangen, 06/2001
    !----------------------------------------------------------------
    implicit none
    type(type_peak),intent(in) :: spectrum(:)
    integer,optional :: unit
    integer :: i,out
    if(present(unit)) then
       out=unit
    else
       out=6
    endif
    write(out,*)
    write(out,'('' Infrared intensities:'')')
    write(out,'('' ---------------------'')')
    write(out,*)
    write(out,'(3x,5(a,'' | ''),a)') 'mode','irrep','mode no. in',&
     &'wavenumber','ir intensity','rel. int.'
    write(out,'(3x,5(a,'' | ''),a)') 'no. ','     ','this irrep ',&
     &'  [1/cm]  ','  [km/mol]  ','         '
    write(out,'(3x,5(''-''),''|'',7(''-''),''|'',13(''-''),''|'',&
     & 12(''-''),''|'',14(''-''),''|'',11(''-''))')
    do i = 1, ubound(spectrum,1)
       write (out,200) i, &
            &          spectrum(i)%irrep,&
            &          spectrum(i)%number_in_irrep, &
            &          spectrum(i)%freq, &
            &          spectrum(i)%ir_int, &
            &          spectrum(i)%rel_ir_int

    enddo
!200 format (' mode',i3,':',f14.6,'  at',f10.2,' cm^-1  ',i3,' ',a5)
200 format (2x,i5,' | ',a5,' | ',i11,' | ', f10.2,' | ', f12.6,' | ',f9.4)
  end subroutine print_ir

  !************************************************************************

  subroutine print_raman(spectrum,intno,unit)
    !----------------------------------------------------------------
    ! output routine
    ! written by Carsten Kind, Uni Erlangen
    ! modified by J. Neugebauer, Uni Erlangen, 2001
    !----------------------------------------------------------------
    implicit none
    type(type_peak),intent(in) :: spectrum(:)
    integer,optional :: unit
    integer :: i,out
    integer,intent(in) :: intno
    if(present(unit)) then
       out=unit
    else
       out=6
    endif

    write(out,*)
    write(out,'('' Raman activities, and depolarization ratios:'')')
    write(out,'('' --------------------------------------------'')')
    write(out,*)
    write(out,'(3x,6(a,'' | ''),a)') 'mode','irrep','mode no. in',&
     &'wavenumber','raman activity','rel. act.','depol. ratio'
    write(out,'(3x,6(a,'' | ''),a)') 'no. ','     ','this irrep ',&
     &'  [1/cm]  ','  [A^4/amu]  ','         ',' '
    write(out,'(3x,5(''-''),''|'',7(''-''),''|'',13(''-''),''|'',&
     & 12(''-''),''|'',16(''-''),''|'',11(''-''),''|'',14(''-''))')
    do i = 1, ubound(spectrum,1)
      if((spectrum(i)%raman(intno)%depol_ratio.ge.0.0d0).and.&
        &(spectrum(i)%raman(intno)%depol_ratio.lt.0.7500005d0)) then
        write (out,200) i, &
            &          spectrum(i)%irrep, &
            &          spectrum(i)%number_in_irrep, &
            &          spectrum(i)%freq, &
            &          spectrum(i)%raman(intno)%int,&
            &          spectrum(i)%raman(intno)%relac,&
            &          spectrum(i)%raman(intno)%depol_ratio
      else if(spectrum(i)%raman(intno)%depol_ratio.lt.0.0d0) then
        write (out,201) i, &
            &          spectrum(i)%irrep, &
            &          spectrum(i)%number_in_irrep, &
            &          spectrum(i)%freq, &
            &          spectrum(i)%raman(intno)%int,&
            &          spectrum(i)%raman(intno)%relac,&
            &          '(Raman inactive)'
      else
        write (out,201) i, &
            &          spectrum(i)%irrep, &
            &          spectrum(i)%number_in_irrep, &
            &          spectrum(i)%freq, &
            &          spectrum(i)%raman(intno)%int,&
            &          spectrum(i)%raman(intno)%relac,&
            &          'ERROR! Depol. ratio > 0.75 !'
      end if
    enddo
200 format (2x,i5,' | ',a5,' | ',i11,' | ', f10.2,' | ', f14.6, &
     & ' | ',f9.4,' | ', f12.6)
201 format (2x,i5,' | ',a5,' | ',i11,' | ', f10.2,' | ', f14.6, &
     & ' | ',f9.4,' | ', a)
    end subroutine print_raman

  !************************************************************************

  subroutine print_rcs(spectrum,intno,static,theta,unit)
    !----------------------------------------------------------------
    ! output routine for scattering cross sections etc.
    ! written by J. Neugebauer, Uni Erlangen, 2001
    !----------------------------------------------------------------
    implicit none
    type(type_peak),intent(in) :: spectrum(:)
    real(double),intent(in) :: theta
    integer,optional :: unit
    integer :: i,out
    integer,intent(in) :: intno
    logical :: static
    if(present(unit)) then
       out=unit
    else
       out=6
    endif
    write(out,'(3(/),'' Raman intesities, &
      &and differential Raman scattering cross sections:'')')
    write(out,'('' ------------------&
      &-------------------------------------------------'')')
    write(out,*)
    if(static) then
      write(out,*) '   For static polarizabilities, a wavelength of'
      write(out,*) '   the incident laser beam of 514.5 nm will be'
      write(out,*) '   used to calculate scattering cross sections'
      write(out,*)
    end if
    write(out,'((/),a,f5.1,a,(/))') '  Scattering angle theta = ',theta,' deg'
    write(out,*)  '    quantity  | pol. incident beam   | analyzer'
    write(out,*)  '   -----------|----------------------|---------------------'
    write(out,*)  '    (per,u)   | perp. to scat. plane | none    '
    write(out,*)  '    (per,per) | perp.                | perp. to scat. plane'
    write(out,*)  '    (per,par) | perp.                | parallel'
    write(out,*)  '    (par,par) | parallel             | parallel'
    write(out,*)  '    (par,u)   | parallel             | none'
    write(out,*)  '    (nat,u)   | none (''natural'')     | none'
    write(out,*)  
    write(out,*)  '    not calculated:'
    write(out,*)  '    (par,per) | parallel             | perp.    | = (per,par)'
    write(out,*)  '    (nat,per) | none                 | perp.    | = (per,u)/2'
    write(out,*)  '    (nat,par) | none                 | parallel | = (par,u)/2'
    write(out,*)  
    write(out,*)  '    For theta = 90.0 deg : (par,par) = (per,par) = (par,per)'
    write(out,*)  '                           (par,u)   = 2*(per,par) '
    write(out,*)  
    write(out,'(''   norm. cr. sect. : normalized differential cross section &
      &= (per,u) * (wavenumber_0 - wavenumber_i)^(-4)'')')
    write(out,'(''   relcross        : relative normalized differential &
     &cross section, compared to the Q-branch of Dinitrogen (5.05d-60 m^6/sr)'')')
    write(out,'(''                     (See H.W. Schroetter, H.W. Kloeckner,&
     & Topics in Current Physics 11, Springer 1979, p. 123 - 166.)'')')
    write(out,'(''   rel. int        : relative intensity = (per,u)/&
     &max(per,u)'')')
    write(out,*)
    write(out,'(3x,6(a,'' | ''),a)') 'mode','irrep','wavenumber',&
   &'        absolute differential cross sections [10^-36 m^2/sr]         ',&
     &'norm. cr. sect.','rel. int','relcross'
    write(out,'(3x,11(a,'' | ''),a)') 'no. ','     ',&
     &'  [1/cm]  ',' (per,u) ','(per,per)','(per,par)','(par,par)',&
     & ' (par,u) ',' (nat,u) ','[10^-60 m^6/sr]','        ','          '
    write(out,'(3x,5(''-''),''|'',7(''-''),''|'',&
     & 12(''-''),''|'',6(11(''-''),''|''),17(''-''),&
     &''|'',10(''-''),''|'',12(''-''))')
    do i = 1, ubound(spectrum,1)
        write (out,202) i, &
            &          spectrum(i)%irrep, &
            &          spectrum(i)%freq, &
            &          spectrum(i)%raman(intno)%cperu*1.0d36,&
            &          spectrum(i)%raman(intno)%cperper*1.0d36, &
            &          spectrum(i)%raman(intno)%cperpar*1.0d36, &
            &          spectrum(i)%raman(intno)%cparpar*1.0d36, &
            &          spectrum(i)%raman(intno)%cparu*1.0d36, &
            &          spectrum(i)%raman(intno)%cnu*1.0d36, &
            &          spectrum(i)%raman(intno)%normcross*1.0d60, &
            &          spectrum(i)%raman(intno)%relint, &
            &          spectrum(i)%raman(intno)%relcross
    enddo
202 format (2x,i5,' | ',a5,' | ', f10.2,' | ', 6(f9.3, (' | '))&
     & ,f15.4,' | ',f8.4,' | ',f10.4)
  end subroutine print_rcs

  !************************************************************************ 

  subroutine destroy_spectrum(spectrum)
    !----------------------------------------------------------------
    ! deallocates "spectrum%norm_mode"
    ! written by Carsten Kind, Uni Erlangen, 1999
    !----------------------------------------------------------------
    implicit none
    type(type_peak),intent(inout) :: spectrum(:)
    integer :: i
    do i=1,ubound(spectrum,1)
       if(associated(spectrum(i)%norm_mode)) &
            & deallocate(spectrum(i)%norm_mode)
    enddo
  end subroutine destroy_spectrum

  !************************************************************************

  subroutine print_norm_modes(spectrum,molecule,unit)
    use snf_molecule,       only: type_atom
    implicit none
    !----------------------------------------------------------------
    ! matout prints a rectangular matrix of eigenvectors and eigenvalues
    !----------------------------------------------------------------
    type(type_atom),intent(in) :: molecule(:)
    type(type_peak),intent(in) :: spectrum(:)
    integer,optional :: unit
    character(len=2),pointer ::  itext(:), jtext(:)
    character(len=2) :: atorbs(3)
    integer,pointer ::  natom(:)
    integer ka,kb,kc,la,lb,lc,nr,nc, i,j,nvar,out
    data atorbs /'x ','y ','z '/
    if(present(unit)) then
       out=unit
    else
       out=6
    endif
    nc=ubound(spectrum,1)
    nr=ubound(spectrum(1)%norm_mode,1)
    nvar=nr

    allocate(natom(nvar),itext(nvar),jtext(nvar))
    do i=1,nvar
       natom(i)=(i+2)/3
       itext(i)=atorbs(i-3*(natom(i)-1))
       jtext(i)=molecule((i+2)/3)%element
    enddo

    ka=1
    kc=6
60  kb=min0(kc,nc)
    write (out,100) (i,i=ka,kb)
    if(spectrum(1)%freq.ge.1.d-10)write (out,110) (spectrum(i)%freq,i=ka,kb)
    write (out,120)
    la=1
    lc=40
70  lb=min0(lc,nr)
    do  i=la,lb
       !       if(itext(i).eq.' s')write(out,120)
       write (out,130) itext(i),jtext(i),natom(i),(spectrum(j)%norm_mode(i),j=ka,kb)
    enddo
    if (lb.eq.nr) go to 90
    la=lc+1
    lc=lc+40
    write (out,140)
    go to 70
90  if (kb.eq.nc) then
       deallocate(natom,itext,jtext)       
       return
    endif
    ka=kc+1
    kc=kc+6
    if (nr.gt.25) write (out,140)
    go to 60

100 format (////,3x,9h root no. ,i5,9i12)
110 format (/8x,10f12.5)
120 format (2h  )
130 format (2(1x,a2),i4,f10.5,10f12.5)
140 format (1h1)

  end subroutine print_norm_modes

  !************************************************************************

  subroutine  sort_spectrum(spectrum)
    implicit none
    type(type_peak),pointer :: spectrum(:)
    type(type_peak) :: tmp_peak
    integer i,j,min_freq,nvar
    !----------------------------------------------------------------
    ! sort 
    !----------------------------------------------------------------
    nvar=ubound(spectrum,1)
    do i=1,nvar
       min_freq=i
       do j=i+1,nvar
          if(spectrum(j)%freq.lt.spectrum(min_freq)%freq) min_freq = j
       enddo
       if(min_freq .ne.i) then
          tmp_peak=spectrum(min_freq)
          spectrum(min_freq) = spectrum(i)
          spectrum(i)= tmp_peak
       endif
    enddo
  end subroutine sort_spectrum

  !************************************************************************

  subroutine get_translation_and_rotation(spectrum,molecule,number_modes_to_omit,unit)
    !--------------------------------------------------------------------------
    ! this routine examines rotational and translational modes.
    ! since rotational/translational and vibrational normal modes could 
    ! mix, there could be more 
    ! than three modes with translational components. due to this fact,
    ! first the translational modes are examined by calculation of the 
    ! center of gravity movement. then the rotational modes without translational 
    ! components are identified by calculation of the angular momentum. if there
    ! is a nonnegligible angular momentum the mode is omitted.
    !
    ! written by Carsten Kind, Uni Erlangen, 2000-2001
    ! extension for linear molecules by Johannes Neugebauer, Uni Erlangen, 2001
    !--------------------------------------------------------------------------
    use snf_molecule,       only: type_atom ,hauptax_trans,calc_delta_i,testlin
    use snf_symmat, only : vec_product
    use snf_tools! , only : inverse_matrix
    implicit none
    type(type_atom),intent(inout) :: molecule(:)
    type(type_peak),pointer :: spectrum(:),new_spectrum(:)
    integer,optional :: unit
    ! flow control:  -1 : only test run, no modes will be omitted 
    !                >0 : new spectrum is exactly this number smaller than the old one
    integer :: number_modes_to_omit,out,nvar

    ! dimensions
    integer :: natoms

    ! thresholds 
    real(double) :: delta_rot,delta_trans     ! for translational,rotational movement

    ! actual values
    real(double) :: actual_momentum(3)                       ! momentum         
    real(double) :: actual_omega_vec(3),actual_omega2        ! angular frequency
    real(double) :: actual_ang_mom(3),actual_ang_mom2        ! angular momentum 

    ! desired values
    real(double) :: desired_momentum                         ! momentum         
    real(double) :: desired_omega_vec(3),desired_omega2      ! angular frequency
    real(double) :: desired_ang_mom_vec(3),desired_ang_mom2  ! angular momentum 

    real(double) :: trans_fraction,rot_fraction   ! fractions of translational,rotational movement

    ! mass depended values
    real(double) :: mkoord(3),molecule_mass       ! center of mass, total mass of molecule
    real(double) :: i_matrix(3,3),inv_i_matrix(3,3) ! inertia matrix and its inverse
    ! indices
    integer :: mode,atom,new_mode,row,col,imode

    ! variables for nicer output
    character(len=13) :: mode_type
    integer :: output_index,transrot_index,info
    logical,allocatable :: copy_mode(:)
    logical :: testrun
    real(double) :: ematrix(3,3)                  ! used for kronecker-delta
    real(double) :: i_eigvec(3,3),i_eigval(3),sqrtmass,work(8)
    data ematrix /1.d0,0.d0,0.d0, &
         &        0.d0,1.d0,0.d0, &
         &        0.d0,0.d0,1.d0 /
    if(present(unit)) then
       out=unit
    else
       out=6
    endif
    delta_trans= 0.5d0
    delta_rot  = 0.5d0
    natoms=ubound(molecule,1)
    testrun = (number_modes_to_omit .eq. -1)
    !-----------------------------------------------------------------------------
    ! allocate memory for shorter list of peaks
    ! the value number_modes_to_omit=1 is only a test run, 
    ! so that no modes will ne omitted.
    !-----------------------------------------------------------------------------
    nvar=ubound(spectrum,1)
    allocate(copy_mode(nvar))
    !-----------------------------------------------------------------------------
    ! determine center of mass and mass of the molecule
    !-----------------------------------------------------------------------------

    mkoord = 0
    molecule_mass = 0
    do atom=1,natoms
       mkoord = mkoord+molecule(atom)%cartcoord*molecule(atom)%mass
       molecule_mass = molecule_mass+molecule(atom)%mass
    enddo
    if(molecule_mass .le. 1.d-10) then
       call serror('get_translation_and_rotation',2,'Zero molecular mass detected !')
    endif
    mkoord = mkoord/molecule_mass

    !-----------------------------------------------------------------------------
    ! move molecule to center of mass
    !-----------------------------------------------------------------------------

    do atom=1,natoms
       molecule(atom)%cartcoord = molecule(atom)%cartcoord-mkoord
    enddo

    !-----------------------------------------------------------------------------
    ! calculate inertia matrix and its inverse
    !-----------------------------------------------------------------------------
    if(.not.testlin(molecule)) then
      do atom=1,natoms
         do col=1,3
            do row=1,col
               i_matrix(row,col) = sum(molecule(:)%mass*(ematrix(col,row)* &
                    (molecule(:)%cartcoord(1)**2+molecule(:)%cartcoord(2)**2+molecule(:)%cartcoord(3)**2) &
                    -molecule(:)%cartcoord(col)*molecule(:)%cartcoord(row))) 
            enddo
         enddo
      enddo
      if(.not. testrun) then
         write(out,*)
         write(out,'(''molecular weight:'',f10.2,'' g/mol'')') molecule_mass
         write(out,*)
         write(out,'(''moments of inertia matrix'')')
         write(out,'(''-------------------------'')')
         write(out,*)
         do row=1,3
            write(out,'(3f14.5)')(i_matrix(row,col),col=1,row)
         enddo
         i_eigvec=i_matrix
         call dsyev('V','L',3,i_eigvec(1,1),3,i_eigval(1),work,8,info)
         if(info.ne.0) then
           call serror('get_translation_and_rotation',2,'Could not calculate eigenvectors of the##&
            &moments of inertia matrix!')
         end if
         write(out,*)
         write(out,'(''principal moments of inertia matrix and eigenvectors'')')
         write(out,'(''----------------------------------------------------'')')
         write(out,*)
         write(out,'(3f14.5)')(i_eigval(col),col=1,3)
         write(out,*)
         do row=1,3
            write(out,'(3f14.5)')(i_eigvec(row,col),col=1,3)
         enddo
      endif
      call inverse_matrix(i_matrix,inv_i_matrix)
    else
      !-----------------------------------------------------------------
      ! This part might be done by the above procedure, but it is not
      ! possible to invert the i_matrix for a linear molecule (see
      ! special treatment below)
      !-----------------------------------------------------------------
      i_eigval(1)=0.0d0
      do atom=1,natoms
        i_eigval(1)=i_eigval(1)+molecule(atom)%mass*(molecule(atom)%cartcoord(1)**2+&
          & molecule(atom)%cartcoord(2)**2+molecule(atom)%cartcoord(3)**2)
      end do
      i_eigval(2)=i_eigval(1)
      i_eigval(3)=0.0d0
      i_eigvec=0.0d0
      i_eigvec(1,1)=1.0d0
      i_eigvec(2,2)=1.0d0
      !-----------------------------------------------------------------
      ! only two entries in the i-matrix for linear molecules
      !-----------------------------------------------------------------
      i_matrix=0.0d0
      i_matrix(1,1)=i_eigval(1)
      i_matrix(2,2)=i_eigval(1)
    end if
    !-----------------------------------------------------------------------------
    ! calculate desired translational movement
    !-----------------------------------------------------------------------------
    desired_momentum=sqrt(molecule_mass)
    new_mode=0
    transrot_index=0
    !    write(out,*)
    !    write(out,'(''identifying translational and rotation normal modes'')')
    write(out,*)
    if(testrun) then
       !       write(out,'(''==================================================='')')
       write(out,'(''   Type identification for irrep: '',A5)') spectrum(1)%irrep
       write(out,*)
       write(out,'(''   type of mode | No. | wavenumber | trans. frac. | rot. frac. '')')
       write(out,'(''   -------------|-----|------------|--------------|------------'')')
    endif
    do mode=1,nvar
       !----------------------------------------------------------------
       ! calculate actual change of center of gravity and
       ! actual angular momentum
       !----------------------------------------------------------------
       actual_momentum=0.d0
       actual_ang_mom=0.d0
       do atom=1,natoms
          sqrtmass=dsqrt(molecule(atom)%mass)
          actual_momentum(:)= actual_momentum(:)+ &
               &              sqrtmass * spectrum(mode)%norm_mode(atom*3-2:atom*3)
          actual_ang_mom=actual_ang_mom + vec_product(sqrtmass*molecule(atom)%cartcoord(:), &
               &              spectrum(mode)%norm_mode(atom*3-2:atom*3))
       enddo
       trans_fraction=dsqrt(dot_product(actual_momentum,actual_momentum))/desired_momentum

       actual_ang_mom2=dot_product(actual_ang_mom,actual_ang_mom)
       !-----------------------------------------------------------------------------
       ! calculate actual angular frequency
       !-----------------------------------------------------------------------------
       if(.not.testlin(molecule)) then
         actual_omega_vec=matmul(actual_ang_mom,inv_i_matrix)
       else
         !-------------------------------------------------
         ! moment of inertia can be treated as a scalar
         !-------------------------------------------------
         actual_omega_vec(:)=actual_ang_mom(:)/i_eigval(1)
       end if
       actual_omega2=dot_product(actual_omega_vec,actual_omega_vec)
       if(actual_omega2 .gt. 1.0d-5) then
          !---------------------------------------------------------------------------
          ! calculate desired angular frequency so that sum_atoms (r x omega)^2 = 1  
          !---------------------------------------------------------------------------
          desired_omega2=0.d0
          do atom=1,natoms
             desired_omega2 = desired_omega2 + (dot_product(molecule(atom)%cartcoord(:),molecule(atom)%cartcoord(:)) &
                  & -  dot_product(molecule(atom)%cartcoord(:),actual_omega_vec)**2 / actual_omega2) &
                  & * molecule(atom)%mass
          enddo
          if(desired_omega2 .gt. 1.0d-5) then
             desired_omega2 = 1.d0/desired_omega2                      
          else
             desired_omega2=0.d0
          endif
          desired_omega_vec=sqrt(desired_omega2/actual_omega2)*actual_omega_vec(:)
          !---------------------------------------------------------------------------
          ! calculate desired angular momentum
          !---------------------------------------------------------------------------
          desired_ang_mom_vec=matmul(i_matrix,desired_omega_vec)     
          desired_ang_mom2=dot_product(desired_ang_mom_vec,desired_ang_mom_vec)
          if(desired_ang_mom2 .gt. 1.0d-5) then
             rot_fraction=sqrt(actual_ang_mom2/desired_ang_mom2)
          else
             rot_fraction=0.d0
             call serror('get_translation_and_rotation',0,'angular momentum for pure&
              & rotational mode## < threshold. Set rotational fraction to 0.0d0.',unit)
          endif
       else
          rot_fraction=0.d0
       endif

       !----------------------------------------------------------------
       ! check if translation or rotation is present
       !----------------------------------------------------------------
       if(rot_fraction  .lt. delta_rot .and. trans_fraction.lt. delta_trans) then
          !----------------------------------------------------------------
          ! mode accepted
          !----------------------------------------------------------------
          copy_mode(mode) = .true.
          new_mode=new_mode+1
          output_index=new_mode
          mode_type='vibrational'
       else
          !----------------------------------------------------------------
          ! mode not accepted
          !----------------------------------------------------------------
          copy_mode(mode) = .false.
          transrot_index=transrot_index+1
          output_index=transrot_index
          if((trans_fraction.ge. delta_trans) .and. &
               & (rot_fraction  .ge. delta_rot)) then
             mode_type='rot+trans. '
          else if (trans_fraction.ge. delta_trans) then
             mode_type='translation'
          else
             mode_type='rotational '
          endif
       endif
       !----------------------------------------------------------------
       ! advanced output about character of modes
       !----------------------------------------------------------------
       if(testrun) then
          write(out,'(4X,a11,a2,i4,a2,f11.2,a2,f13.2,a2,f11.2)') &
               & mode_type,' |', &
               & output_index,' |',&
               & spectrum(mode)%freq,' |',&
               & trans_fraction*100,' |',&
               & rot_fraction*100
          !               & spectrum(mode)%number_in_irrep,spectrum(mode)%irrep,' |',&
       endif
    enddo
    !----------------------------------------------------------------
    ! extract only vibrational modes
    !----------------------------------------------------------------
    if( .not. testrun) then
       imode=0
       allocate(new_spectrum(new_mode))
       do mode=1,nvar
          if(copy_mode(mode)) then
             imode=imode+1
             new_spectrum(imode) = spectrum(mode)          
          else
             if(associated(spectrum(mode)%norm_mode)) &
                  & deallocate(spectrum(mode)%norm_mode)
          endif
       enddo
       deallocate(spectrum)
       spectrum => new_spectrum
       !----------------------------------------------------------------
       ! test on number of vibrational and translational/rotational modes
       !----------------------------------------------------------------
       if(transrot_index .eq.number_modes_to_omit) then
          write(out,'(''omitted'',i2,'' translational and/or rotational modes'')')&
               & number_modes_to_omit

       else if(transrot_index .lt. number_modes_to_omit) then
          write(out,*)'**********************************'
          write(out,'('' ERROR: Too many vibrational modes!'')')
          write(out,'('' There are          '',i5)') new_mode+1
          write(out,'('' There should be    '',i5)') nvar - number_modes_to_omit
          write(out,*)'**********************************'
       else
          write(out,*)
          write(out,*)'*********************************************************'
          write(out,*)'ERROR: Too many translational and rotational modes found! '
          write(out,'('' There are          '',i5)') transrot_index
          write(out,'('' There should be    '',i5)') number_modes_to_omit
          write(out,*)'*********************************************************'
       endif
    endif
    deallocate(copy_mode)
  end subroutine get_translation_and_rotation

  !************************************************************************

  subroutine assign_irrep_number(spectrum,char_table)
    !----------------------------------------------------------------
    ! this routine numerates the modes per irreducible representations.
    ! this must be performed in this primitive way, because you cannot
    ! easily predict the irreps of translation and rotation modes before 
    ! determining the normal modes.
    !----------------------------------------------------------------
    use snf_char_table,only : type_char_table
    implicit none
    type(type_peak),pointer :: spectrum(:)
    type(type_char_table) :: char_table
    integer,pointer :: irrep_counter(:)
    integer :: irr_index,mode

    allocate(irrep_counter(char_table%nirrep))
    irrep_counter = 0
    !----------------------------------------------------------------
    ! loop over all modes and look for every mode the matching entry in the 
    ! character table
    !----------------------------------------------------------------

    do mode=1,ubound(spectrum,1)
       irr_index=1
       if(spectrum(mode)%irrep.ne.'ERR') then
          do while(char_table%mulliken(irr_index) .ne. spectrum(mode)%irrep)
             irr_index=irr_index+1
          enddo
          irrep_counter(irr_index) = irrep_counter(irr_index) +1
          spectrum(mode)%number_in_irrep = irrep_counter(irr_index)
       else
          spectrum(mode)%number_in_irrep = 0
       end if
    enddo
    deallocate(irrep_counter)
  end subroutine assign_irrep_number

  !************************************************************************

  subroutine zeropoint_energy(spectrum,energy)
    use snf_param
    implicit none
    !----------------------------------------------------------------
    ! zeropointlevel energy in kj/mol
    !          
    ! energy = 0.5 * hc * sum(1/lambda)
    !          
    ! the faktor 0.1 is the conversion factor from cm-->m and j-->kj
    ! only sum up positive frequencies
    !
    ! written by Carsten Kind, Uni Erlangen, 2000
    !----------------------------------------------------------------
    type(type_peak),pointer :: spectrum(:)
    real(double) :: energy
    integer :: i,n
    energy=0
    n=ubound(spectrum,1)
    do i=1,n
       if(spectrum(i)%freq.gt.0.d0) energy=energy+spectrum(i)%freq
    enddo
    energy=0.5*speedoflight *planck_h*avogadro*1.d-01*energy
  end subroutine zeropoint_energy

  !************************************************************************  

end module snf_spectrum
