
!********************************************************************************
!*   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
!*   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
!*   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
!*
!*
!*   This file is part of the package SNF.
!*
!*   SNF is free software; you can redistribute it and/or modify
!*   it under the terms of the GNU General Public License as published by
!*   the Free Software Foundation; either version 2 of the License, or
!*   (at your option) any later version.
!*
!*   This program is distributed in the hope that it will be useful,
!*   but WITHOUT ANY WARRANTY; without even the implied warranty of
!*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*   GNU General Public License for more details.
!*
!*   You should have received a copy of the GNU General Public License
!*   along with this program; if not, write to the Free Software
!*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!*
!********************************************************************************
module snf_symmat
  use f90_kind
  use snf_error, only: serror
  use snf_tools, only: string2integer
  !
  ! generation of of symmetry transformation matrices 
  !    trans_mat (function)

  real(double) :: sigma_h_p(3,3)
  data sigma_h_p &
       &/ 1.d0, 0.d0, 0.d0,  &
       &  0.d0, 1.d0, 0.d0,  &
       &  0.d0, 0.d0,-1.d0/
  real(double) :: sigma_h_d(5,5)
  data sigma_h_d &
       &/ 1.d0, 0.d0, 0.d0, 0.d0, 0.d0,  &
       &  0.d0, 1.d0, 0.d0, 0.d0, 0.d0,  &
       &  0.d0, 0.d0, 1.d0, 0.d0, 0.d0,  &
       &  0.d0, 0.d0, 0.d0, 1.d0, 0.d0,  &
       &  0.d0, 0.d0, 0.d0, 0.d0, 1.d0/

  !*****************************************************************************

contains

  !*****************************************************************************

  function trans_mat(z_vec,x_vec)
    !----------------------------------------------------------------
    ! this function yields the matrix for the transformation of symmetry elements
    ! and coordinates into a new coordinate system, which is determinend by
    ! x_vec and z_vec.
    ! 
    ! written by C. Kind
    ! extended by J. Neugebauer, Universitaet Erlangen-Nuernberg, 05/2001
    !----------------------------------------------------------------
    implicit none
    real(double) :: trans_mat(3,3)
    real(double) :: x1_mat(3,3),hilf_vec(3)
    real(double),intent(in) :: z_vec(3),x_vec(3)
    real(double) :: x2_mat(3,3)
    data x2_mat &
         &/1.d0, 0.d0, 0.d0,  &
         & 0.d0, 1.d0, 0.d0,  &
         & 0.d0, 0.d0, 1.d0/ 
    x1_mat(3,:) = 1/dsqrt(dot_product(z_vec,z_vec))* z_vec
    hilf_vec = x_vec-dot_product(x_vec,x1_mat(3,:))*x1_mat(3,:)
    x1_mat(1,:) = 1/dsqrt(dot_product(hilf_vec,hilf_vec))* hilf_vec
    x1_mat(2,:) = vec_product(x1_mat(3,:),x1_mat(1,:))
    trans_mat = matmul(x2_mat,transpose(x1_mat))

  end function trans_mat

  !*****************************************************************************

  function symel_mat(symel)
    !----------------------------------------------------------------
    ! this function sets up matrices for all symmetry elements and stores 
    ! them in char_tbl%sym_mat. They are generated as follows:            
    ! c-axes are created in terms of z-rotation matrices                  
    ! s-axes are created in the same way, the reflection is performed by  
    ! changing the sign of the (3,3) matrix element.                      
    ! sd- and sv-planes are generated by transformation of an xz-mirror-  
    ! plane with a rotation matrix (rotation around z-axis)               
    ! e and i are represented as c1 and s2, respectively                  
    !                                                                     
    ! written by C. Kind                                                  
    ! modified and extended by J. Neugebauer, Uni Erlangen, 05/2001       
    !----------------------------------------------------------------
    !                                                                     
    ! Note: Special definitions are applied for symmetry operations of    
    !       point groups i and ih. (J. Neugebauer, 05/2001)               
    !                                                                     
    !----------------------------------------------------------------
    use snf_tools,only:getint
    implicit none
    character(len=132) :: errbuff
    character(len=7) :: symel
    real(double) :: symel_mat(3,3)
    integer ax_order,ndreh,ax_number

    !------------------------------------------------------
    ! determine order "ax_order" and exponent "ndreh" of
    ! axis C_(ax_order)^(ndreh) / S_(ax_order)^(ndreh)
    !
    ! in case of icosahedral or cubic symmetry, also the 
    ! number "ax_number" is necessary to determine the axis
    !------------------------------------------------------

    if(index('ijop',symel(1:1)).ne.0) then
       ax_order = index('1234567890',symel(3:3))
       ndreh = index('123456789',symel(2:2))
       ax_number = 10*index('123456789',symel(4:4))+ &
            index('123456789',symel(5:5))
       if(ax_order*ndreh*ax_number.eq.0) then
         write(errbuff,*) 'Could not find symmetry element: ',symel 
         call serror('symel_mat',2,errbuff)
       end if
       symel_mat(:,:)=icsym(ax_order,ndreh,ax_number,symel(1:1))
    else 
       if (.not. (symel(1:2) .eq. 'sh')) then
          ax_order = getint(symel,3)
          ndreh    = getint(symel,index(symel,'_')+1)
       endif
       !----------------------------------------------------------------
       ! determination of symmetry element and appropriate 
       ! transformation matrix
       !----------------------------------------------------------------
       select case (symel(1:2))
       case ('cz')
          symel_mat(:,:)= cnz_p(ax_order,ndreh) 
       case ('sz')
          symel_mat(:,:)= snz_p(ax_order,ndreh)
       case ('cx')
          symel_mat(:,:)= c2x_p(ax_order,ndreh)
       case ('cy')
          symel_mat(:,:)= c2y_p(ax_order,ndreh)
       case ('sv')
          symel_mat(:,:)= sigma_v_p(ax_order,ndreh)
       case ('sd')
          symel_mat(:,:)= sigma_d_p(ax_order,ndreh)
       case ('sh')
          symel_mat(:,:)= sigma_h_p
       case default
         write(errbuff,*) 'Could not find symmetry element: ',symel 
         call serror('symel_mat',2,errbuff)
       end select
    end if
  end function symel_mat

  !*****************************************************************************

  subroutine sym_mat_schreiben(mat)
    !----------------------------------------------------------------
    ! prints individual symmtry matrices.
    ! written by C. Kind
    !----------------------------------------------------------------
    use snf_tools,only:printmat
    implicit none
    real(double) :: mat(:,:)

    call printmat(mat,text='transformationsmatrix',fmt='F8.3')
  end subroutine sym_mat_schreiben

  !*****************************************************************************

  function cnz_p(ax_order,ndreh)
    !----------------------------------------------------------------
    ! sets up matrix for symmetry operation C_n^p
    ! written by C. Kind
    !----------------------------------------------------------------
    use snf_param, only : pi

    implicit none
    integer :: ax_order, ndreh,special_angle6,special_angle4
    real(double):: cnz_p(3,3)

    special_angle6=ndreh*6/ax_order
    special_angle4=ndreh*4/ax_order
    if(mod(ndreh*600/ax_order,100) .eq. 0) then
       select case(special_angle6)
       case(0)                          ! 0 o 
          cnz_p(1,1)= 1.0d0
          cnz_p(1,2)= 0.0d0
       case(1)                          ! 60 o
          cnz_p(1,1)= 0.5d0
          cnz_p(1,2)= dsqrt(3.0d0)/2.0d0
       case(2)                          ! 120 o
          cnz_p(1,1)=-0.5d0
          cnz_p(1,2)= dsqrt(3.0d0)/2.0d0
       case(3)                          ! 180 o
          cnz_p(1,1)=-1.0d0
          cnz_p(1,2)= 0.0d0
       case(4)                          ! 240 o
          cnz_p(1,1)=-0.5d0
          cnz_p(1,2)=-dsqrt(3.0d0)/2.0d0
       case(5)                          ! 300 o
          cnz_p(1,1)= 0.5d0
          cnz_p(1,2)=-dsqrt(3.0d0)/2.0d0
       case(6)                          ! 360 o 
          cnz_p(1,1)= 1.0d0
          cnz_p(1,2)= 0.0d0
       end select
    else if(mod(ndreh*400/ax_order,100) .eq. 0) then
       select case(special_angle4)
       case(1)                          ! 90  o
          cnz_p(1,1)= 0.0d0
          cnz_p(1,2)= 1.0d0
       case(3)                          ! 270 o
          cnz_p(1,1)=-0.0d0
          cnz_p(1,2)=-1.0d0
       end select
    else
       cnz_p(1,1)= cos(ndreh*2*pi/ax_order)
       cnz_p(1,2)=-sin(ndreh*2*pi/ax_order)
    endif
    cnz_p(1,3)= 0.d0
    cnz_p(2,1)=-cnz_p(1,2)
    cnz_p(2,2)= cnz_p(1,1)
    cnz_p(2,3)= 0.d0
    cnz_p(3,1)= 0.d0
    cnz_p(3,2)= 0.d0
    cnz_p(3,3)= 1.d0 
  end function cnz_p

  !*****************************************************************************

  function c2x_p(ax_order,ndreh)
    !----------------------------------------------------------------
    ! sets up matrix for symmetry operation C_2x^p
    ! written by C. Kind
    !----------------------------------------------------------------
    implicit none
    integer :: ax_order, ndreh
    real(double) :: c2x_p(3,3), hilf_mat(3,3)
    real(double) :: cx2_mat(3,3)
    data cx2_mat &
         &/ 1.d0, 0.d0, 0.d0,  &
         &  0.d0,-1.d0, 0.d0,  &
         &  0.d0, 0.d0,-1.d0/

    hilf_mat=cnz_p(2*ax_order,ndreh)
    c2x_p = matmul(matmul(transpose(hilf_mat),cx2_mat),hilf_mat)
  end function c2x_p

  !*****************************************************************************

  function c2y_p(ax_order,ndreh)
    !----------------------------------------------------------------
    ! sets up matrix for symmetry operation C_2y^p
    ! written by C. Kind
    !----------------------------------------------------------------
    implicit none
    integer :: ax_order, ndreh
    real(double):: c2y_p(3,3), hilf_mat(3,3)
    real(double) :: cx2_mat(3,3)
    data cx2_mat &
         &/ 1.d0, 0.d0, 0.d0,  &
         &  0.d0,-1.d0, 0.d0,  &
         &  0.d0, 0.d0,-1.d0/

    hilf_mat=cnz_p(ax_order*2,ndreh*2+1)
    c2y_p = matmul(matmul(transpose(hilf_mat),cx2_mat),hilf_mat)
  end function c2y_p

  !*****************************************************************************

  function sigma_v_p(ax_order,ndreh)
    !----------------------------------------------------------------
    ! sets up matrix for symmetry operation sigma_v
    ! written by C. Kind
    !----------------------------------------------------------------
    implicit none
    integer :: ax_order, ndreh
    real(double) :: sigma_v_p(3,3), hilf_mat(3,3)
    real(double) :: sigmaxz_mat(3,3)
    data sigmaxz_mat &
         &/ 1.d0, 0.d0, 0.d0,  &
         &  0.d0,-1.d0, 0.d0,  &
         &  0.d0, 0.d0, 1.d0/

    hilf_mat=cnz_p(ax_order,ndreh)
    sigma_v_p = matmul(matmul(transpose(hilf_mat),sigmaxz_mat),hilf_mat)
  end function sigma_v_p

  !*****************************************************************************

  function sigma_d_p(ax_order,ndreh)
    !----------------------------------------------------------------
    ! sets up matrix for symmetry operation sigma_d
    ! written by C. Kind
    !----------------------------------------------------------------
    implicit none
    integer :: ax_order, ndreh
    real(double):: sigma_d_p(3,3),hilf_mat(3,3)
    real(double) :: sigmaxz_mat(3,3)
    data sigmaxz_mat &
         &/ 1.d0, 0.d0, 0.d0,  &
         &  0.d0,-1.d0, 0.d0,  &
         &  0.d0, 0.d0, 1.d0/

    hilf_mat=cnz_p(ax_order*2,ndreh*2+1)
    sigma_d_p = matmul(matmul(transpose(hilf_mat),sigmaxz_mat),hilf_mat)
  end function sigma_d_p

  !*****************************************************************************

  function snz_p(ax_order,ndreh)
    !----------------------------------------------------------------
    ! sets up matrix for symmetry operation  S_n^p
    ! written by C. Kind
    !----------------------------------------------------------------
    implicit none
    integer :: ax_order, ndreh
    real(double):: snz_p(3,3)

    snz_p = cnz_p(ax_order,ndreh)
    snz_p(3,3) = -1.d0
  end function snz_p

  !*****************************************************************************

  function cn_p(ax_order,ndreh,ax_vec,second_vec)
    !----------------------------------------------------------------
    ! sets up matrix for symmetry operation  C_n^p
    ! written by C. Kind
    !----------------------------------------------------------------
    implicit none
    integer :: ax_order, ndreh
    real(double):: hilf_mat(3,3),second_vec(3),ax_vec(3),cn_p(3,3)

    hilf_mat = trans_mat(ax_vec,second_vec)
    cn_p = matmul(matmul(transpose(hilf_mat),cnz_p(ax_order,ndreh)),hilf_mat)
  end function cn_p

  !*****************************************************************************

  function sn_p(ax_order, ndreh, ax_vec,second_vec)
    !----------------------------------------------------------------
    ! sets up matrix for symmetry operation  S_n^p
    ! written by C. Kind
    !----------------------------------------------------------------
    implicit none
    integer :: ax_order, ndreh
    real(double) :: ax_vec(3), sn_p(3,3),hilf_mat(3,3),second_vec(3)

    hilf_mat = trans_mat(ax_vec,second_vec)
    sn_p = matmul(matmul(transpose(hilf_mat),snz_p(ax_order,ndreh)),hilf_mat)
  end function sn_p

  !*****************************************************************************

  function sigma_p(norm_vec,second_vec)
    !----------------------------------------------------------------
    ! sets up matrix for symmetry operation sigma
    ! written by C. Kind
    !----------------------------------------------------------------
    implicit none
    real(double) :: norm_vec(3), sigma_p(3,3),second_vec(3),hilf_mat(3,3)

    hilf_mat = trans_mat(norm_vec,second_vec)
    sigma_p = matmul(matmul(transpose(hilf_mat),sigma_h_p),hilf_mat)
  end function sigma_p

  !*****************************************************************************

  function icsym(ax_order,ndreh,ax_number,ijc)
    !----------------------------------------------------------------------------
    ! this function provides the transformation matrices for the symmtry elements
    ! of icosahedral and cubic point groups. The coordinates specify points
    ! which define - together with O(0,0,0) - rotation axes.
    !
    ! written by J. Neugebauer, Universitaet Erlangen-Nuernberg, 05/2001
    !----------------------------------------------------------------------------
    implicit none
    real(double) :: c3i(3,10),c2i(3,15),c5i(3,6),icsym(3,3)
    real(double) :: phi,pi,c4o(3,3),c2o(3,6),c3o(3,4)
    parameter (pi = 3.1415926535897932384626433832795d0)
    integer :: i,j
    character(len=1) :: ijc
    integer :: ax_order, ax_number, ndreh 

    data ((c2i(i,j),i=1,3),j=1,15) &
         / 0.42532540417602d0,   -0.30901699437495d0,    0.85065080835204d0, &
         0.42532540417602d0,    0.30901699437495d0,    0.85065080835204d0, &
         -0.16245984811645d0,    0.50000000000000d0,    0.85065080835204d0, &
         -0.52573111211913d0,    0.00000000000000d0,    0.85065080835204d0, &
         -0.16245984811645d0,   -0.50000000000000d0,    0.85065080835204d0, &
         0.85065080835204d0,    0.00000000000000d0,    0.52573111211913d0, &
         0.26286555605957d0,    0.80901699437495d0,    0.52573111211914d0, &
         -0.68819096023559d0,    0.50000000000000d0,    0.52573111211913d0, &
         -0.68819096023559d0,   -0.50000000000000d0,    0.52573111211913d0, &
         0.26286555605957d0,   -0.80901699437495d0,    0.52573111211914d0, &
         0.58778525229247d0,   -0.80901699437495d0,    0.00000000000000d0, &
         0.95105651629515d0,   -0.30901699437494d0,    0.00000000000000d0, &
         0.95105651629515d0,    0.30901699437494d0,    0.00000000000000d0, &
         0.58778525229247d0,    0.80901699437495d0,    0.00000000000000d0, &
         0.00000000000000d0,    1.00000000000000d0,    0.00000000000000d0 /


    data ((c3i(i,j),i=1,3),j=1,10) &
         / 0.60706199820669d0,    0.00000000000000d0,    0.79465447229177d0, &
         0.18759247408508d0,    0.57735026918962d0,    0.79465447229177d0, &
         -0.49112347318842d0,    0.35682208977309d0,    0.79465447229177d0, &
         -0.49112347318842d0,   -0.35682208977309d0,    0.79465447229177d0, &
         0.18759247408508d0,   -0.57735026918962d0,    0.79465447229177d0, &
         0.79465447229177d0,   -0.57735026918962d0,   -0.18759247408508d0, &
         0.98224694637685d0,    0.00000000000000d0,    0.18759247408508d0, &
         0.79465447229177d0,    0.57735026918962d0,   -0.18759247408508d0, &
         0.30353099910334d0,    0.93417235896272d0,    0.18759247408508d0, &
         -0.30353099910334d0,    0.93417235896272d0,   -0.18759247408508d0 /

    data ((c5i(i,j),i=1,3),j=1,6) &
         / 0.00000000000000d0,   0.00000000000000d0,   1.00000000000000d0, &
         0.72360679774998d0,  -0.52573111211913d0,   0.44721359549996d0, &
         0.72360679774998d0,   0.52573111211913d0,   0.44721359549996d0, &
         -0.27639320225002d0,   0.85065080835204d0,   0.44721359549996d0, &
         -0.89442719099992d0,   0.00000000000000d0,   0.44721359549996d0, &
         -0.27639320225002d0,  -0.85065080835204d0,   0.44721359549996d0 /

    data ((c4o(i,j),i=1,3),j=1,3) &
         / 1.00000000000000d0,   0.00000000000000d0,   0.00000000000000d0, &
         0.00000000000000d0,   1.00000000000000d0,   0.00000000000000d0, &
         0.00000000000000d0,   0.00000000000000d0,   1.00000000000000d0 /

    data ((c2o(i,j),i=1,3),j=1,6) &
         / 1.00000000000000d0,   1.00000000000000d0,   0.00000000000000d0, &
         -1.00000000000000d0,   1.00000000000000d0,   0.00000000000000d0, &
         1.00000000000000d0,   0.00000000000000d0,   1.00000000000000d0, &
         0.00000000000000d0,  -1.00000000000000d0,  -1.00000000000000d0, &
         1.00000000000000d0,   0.00000000000000d0,  -1.00000000000000d0, &
         0.00000000000000d0,  -1.00000000000000d0,   1.00000000000000d0 /

    data ((c3o(i,j),i=1,3),j=1,4) &
         / 1.00000000000000d0,   1.00000000000000d0,   1.00000000000000d0, &
         1.00000000000000d0,   1.00000000000000d0,  -1.00000000000000d0, &
         1.00000000000000d0,  -1.00000000000000d0,  -1.00000000000000d0, &
         -1.00000000000000d0,   1.00000000000000d0,  -1.00000000000000d0 /

    phi=2.d0*pi*dble(ndreh)/dble(ax_order)
    !   write(*,*) ijc,ax_number,ax_order,ndreh ! debug-output
    if(ijc.eq.'i') then
       select case(ax_order)
       case(2)
          call initrtm(c2i(:,ax_number),phi,icsym)
       case(3)
          call initrtm(c3i(:,ax_number),phi,icsym)
       case(5)
          call initrtm(c5i(:,ax_number),phi,icsym)
       case default
          call serror('icsym',2,'No such symmetry element!')
       end select
    else if(ijc.eq.'j') then
       select case(ax_order)
       case(1)
          call rotref(c2i(:,ax_number),phi,icsym)
       case(6)
          call rotref(c3i(:,ax_number),phi,icsym)
       case(10)
          call rotref(c5i(:,ax_number),phi,icsym)
       case default
          call serror('icsym',2,'No such symmetry element!')
       end select
    else if(ijc.eq.'o') then
       select case(ax_order)
       case(2)
          call initrtm(c2o(:,ax_number),phi,icsym)
       case(3)
          call initrtm(c3o(:,ax_number),phi,icsym)
       case(4)
          call initrtm(c4o(:,ax_number),phi,icsym)
       case default
          call serror('icsym',2,'No such symmetry element!')
       end select
    else if(ijc.eq.'p') then
       select case(ax_order)
       case(1) !oh,td
          call rotref(c2o(:,ax_number),phi,icsym)
       case(6) !oh,th
          call rotref(c3o(:,ax_number),phi,icsym)
       case(4) !oh,td
          call rotref(c4o(:,ax_number),phi,icsym)
       case default
          call serror('icsym',2,'No such symmetry element!')
       end select
    else
       call serror('icsym',2,'No such symmetry element!')
    end if
  end function icsym

  !*****************************************************************************

  subroutine initrtm(ps2,phi,rotm)

    !--------------------------------------------------------------------
    ! sets up the rotation matrix for a rotation by an angle phi [rad]
    ! around an axis defined by the points O(0,0,0) and ps2(x,y,z)
    !
    ! source: Simon L. Altmann, Peter Herzig, Point-Group Theory Tables,
    !         Oxford Science Publications, Clarendon Press, Oxford 1994,
    !         p. 57.
    !
    ! written by J. Neugebauer, Universitaet Erlangen-Nuernberg, 05/2001
    !-------------------------------------------------------------------

    implicit none
    double precision   ps(3),s1,s2,ps2(3)
    double precision   phi,length
    double precision   rotm(3,3)

    s1=sin(phi/2.0d0)
    s1=2.d0*s1**2

    s2=sin(phi)

    !-------------------------------------------------------------------
    ! Normalisation of vector ps
    !-------------------------------------------------------------------

    length=sqrt(ps2(1)**2+ps2(2)**2+ps2(3)**2)
    if(length.le.1.d-14) then
       call serror('initrtm',2,'axis of rotation setup failed since &
         &P1 = 0 = (0,0,0)')
    end if
    ps=ps2/length

    !-------------------------------------------------------------------
    ! set up rotation matrix
    !-------------------------------------------------------------------

    rotm(1,1)=1.d0-(ps(2)**2+ps(3)**2)*s1
    rotm(1,2)=-ps(3)*s2+ps(1)*ps(2)*s1
    rotm(1,3)= ps(2)*s2+ps(3)*ps(1)*s1
    rotm(2,1)= ps(3)*s2+ps(1)*ps(2)*s1
    rotm(2,2)=1.d0-(ps(3)**2+ps(1)**2)*s1
    rotm(2,3)=-ps(1)*s2+ps(2)*ps(3)*s1
    rotm(3,1)=-ps(2)*s2+ps(3)*ps(1)*s1
    rotm(3,2)= ps(1)*s2+ps(2)*ps(3)*s1
    rotm(3,3)=1.d0-(ps(1)**2+ps(2)**2)*s1

  end subroutine initrtm

  !*****************************************************************************

  subroutine rotref(ps,phi,rotm)

    !--------------------------------------------------------------------
    ! sets up the matrix for an operation Sn^k (==> angle phi = 2*pi*k/n) 
    ! around an axis defined by the points O(0,0,0) and ps(x,y,z) 
    !
    ! written by J. Neugebauer, Universitaet Erlangen-Nuernberg,  05/2001
    !--------------------------------------------------------------------

    implicit none
    double precision   ps(3),py(3),pz(3)
    double precision   mirz(3,3),rotsy(3,3),rotsz(3,3),auxm(3,3)
    double precision   rota(3,3),phim,thetam,mirphi(3,3),rotm(3,3)
    double precision   ps2(3),length,scr1,phi
    intrinsic sin,sqrt,asin,acos
    integer i,j
    !-------------------------------------------------------------------
    ! 1. choose xy-plane as mirror plane.
    ! 2. rotate mirror plane by an angle phim around y-axis, then
    ! 3. by an angle thetam around z-axis to get the mirror plane
    !    perpendicular to the rotation axis.
    !-------------------------------------------------------------------

    !-------------------------------------------------------------------
    ! 1. 
    !-------------------------------------------------------------------

    data ((mirz(i,j),i=1,3),j=1,3) &
         / 1.d0, 0.d0, 0.d0, &
         0.d0, 1.d0, 0.d0, &
         0.d0, 0.d0,-1.d0 /
    !-------------------------------------------------------------------
    ! 2. set up matrix for rotation of mirror plane (xy as defined by mirz)
    ! around y-axis; rotation angle phim = angle between choosen
    ! rotation axis and z-axis
    !-------------------------------------------------------------------
    data py / 0.d0, 1.d0, 0.d0 /
    data pz / 0.d0, 0.d0, 1.d0 /

    length=sqrt(ps(1)**2+ps(2)**2+ps(3)**2)
    if(length.le.1.d-14) then
       call serror('rotref',2,'axis of rotation setup failed since &
         &P1 = 0 = (0,0,0)')
    end if

    scr1=sqrt(ps(1)**2+ps(2)**2)
    !-------------------------------------------------------------------
    ! factor -1 results from the definition of the angle phim, which
    ! is measured clockwise from z-axis.
    !-------------------------------------------------------------------
    phim=-sign(1.d0,ps(3))*asin(scr1/length)
    call initrtm(py,phim,rotsy)

    auxm=matmul(matmul(transpose(rotsy),mirz),rotsy)
    !-------------------------------------------------------------------
    ! 3. set up matrix for rotation of mirror plane (xy as defined by mirz)
    ! around z-axis; rotation angle phim = arctan(ps(1)/ps(3))
    !-------------------------------------------------------------------
    if(scr1.le.1.d-14) then
       thetam=0.d0
    else
       thetam=-sign(1.d0,ps(2))*acos(ps(1)/scr1)
    end if
    call initrtm(pz,thetam,rotsz)

    mirphi=matmul(matmul(transpose(rotsz),auxm),rotsz)
    !-------------------------------------------------------------------
    ! set up rotation matrix
    !-------------------------------------------------------------------
    ps2=ps/length
    call initrtm(ps2,phi,rota)
    rotm = matmul(mirphi,rota)

  end subroutine rotref

  !*****************************************************************************

  function vec_product(vec1,vec2)
    !----------------------------------------------------------------*****
    ! this function yields the vector product of vectors vec1 and vec2
    !
    ! written by C. Kind
    !----------------------------------------------------------------*****
    implicit none
    real(double) :: vec_product(3)
    real(double),intent(in)  :: vec1(3),vec2(3)

    vec_product(1)=vec1(2)*vec2(3)-vec1(3)*vec2(2)
    vec_product(2)=vec1(3)*vec2(1)-vec1(1)*vec2(3)
    vec_product(3)=vec1(1)*vec2(2)-vec1(2)*vec2(1)

  end function vec_product

  !*****************************************************************************

end module snf_symmat
