
!********************************************************************************
!*   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
!*   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
!*   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
!*
!*
!*   This file is part of the package SNF.
!*
!*   SNF is free software; you can redistribute it and/or modify
!*   it under the terms of the GNU General Public License as published by
!*   the Free Software Foundation; either version 2 of the License, or
!*   (at your option) any later version.
!*
!*   This program is distributed in the hope that it will be useful,
!*   but WITHOUT ANY WARRANTY; without even the implied warranty of
!*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*   GNU General Public License for more details.
!*
!*   You should have received a copy of the GNU General Public License
!*   along with this program; if not, write to the Free Software
!*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!*
!********************************************************************************
module snf_thermo
  use f90_kind
  use snf_error

  private
  public :: get_tdvalues

  !************************************************************************

  type type_elecont
     real(double) :: energy,partfunc,entropy,cv
  end type type_elecont

  type type_vibcont
     real(double) :: energy,partfunc,entropy,partfuncv0,cv
  end type type_vibcont

  type type_rotcont
     real(double) :: energy,partfunc,entropy,cv
  end type type_rotcont

  type type_tracont
     real(double) :: energy,partfunc,entropy,cv
  end type type_tracont

  type type_internal
     real(double) :: zpe
  end type type_internal

  type type_tempcorr
     type(type_elecont) :: elecont
     type(type_vibcont) :: vibcont
     type(type_rotcont) :: rotcont
     type(type_tracont) :: tracont
     type(type_internal) :: internal
     real(double) :: partfunc,rovibtra,etotal,entropy,temper,press,gibbs
     real(double) :: partfuncv0,cv
  end type type_tempcorr


  !************************************************************************

contains

  !************************************************************************

  subroutine get_tdvalues(zpe,totene,freqs,temp,press,out,mass,inertia,&
   & symnumb,lintest,spin)
    use snf_param
    implicit none
    !----------------------------------------------------------------
    ! this sub collects thermodynamical data 
    ! for a canonical ensemble of an ideal gas
    !
    ! td_data%elecont%energy is in [hartree]
    ! td_data%etotal is in [hartree]
    ! td_data%rovibtra is in [kj/mol]
    !
    ! written by Markus Reiher, uni erlangen, march 2001
    ! extension for linear molecules by Johannes Neugebauer, june 2001
    ! slightly modified by Markus Reiher, aug. and nov. 2001
    !----------------------------------------------------------------
    integer :: irrepno,out,symnumb
    real(double) :: zpe,totene,freqs(:),temp,press,mass,inertia(3),spin
    type(type_tempcorr) :: td_data
    logical, intent(in) :: lintest

    irrepno=ubound(freqs,1) 

!CMR    write(out,*)
!CMR    write(out,*) ' >>>>>>>>>>>>>>>>>>>  WARNING  <<<<<<<<<<<<<<<<<<'
!CMR    write(out,*) ' Electronic degeneracies are not yet considered !'
!CMR    write(out,*) ' >>>>>>>>>>>>>>>>>>>  WARNING  <<<<<<<<<<<<<<<<<<'
!CMR    write(out,*)

    td_data%internal%zpe=zpe
    td_data%elecont%energy=totene
    td_data%elecont%entropy=gasconstant*log(2.0d0*spin+1.0d0)
    td_data%temper=temp
    td_data%press=press
    call calc_partfunc(td_data,freqs,irrepno,lintest,out) 
    call calc_cv(td_data,freqs,irrepno,lintest,out) 
    call calc_entropy(td_data,freqs,irrepno,mass,inertia,lintest,symnumb,out)
    call calc_energy(td_data,freqs,irrepno,lintest)
    ! sum up all contributions
    td_data%partfunc=td_data%vibcont%partfunc* &
         &                         td_data%rotcont%partfunc* &
         &                         td_data%tracont%partfunc* &
         &                         td_data%elecont%partfunc
    td_data%partfuncv0=td_data%vibcont%partfuncv0* &
         &                         td_data%rotcont%partfunc* &
         &                         td_data%tracont%partfunc* &
         &                         td_data%elecont%partfunc
    td_data%entropy=td_data%vibcont%entropy+ &
         &                         td_data%rotcont%entropy+ &
         &                         td_data%tracont%entropy+ &
         &                         td_data%elecont%entropy 
    td_data%rovibtra=td_data%vibcont%energy + &
         &                          td_data%rotcont%energy+  &
         &                          td_data%tracont%energy
    td_data%etotal=kjmol2hartree*td_data%rovibtra+td_data%elecont%energy &
         &         -kjmol2hartree*(td_data%temper*td_data%entropy/1000.0d0)
    td_data%gibbs=td_data%rovibtra-td_data%entropy*td_data%temper/1000.0d0
    call print_tdvalues(td_data,symnumb,spin,out)
  end subroutine get_tdvalues


  !************************************************************************

  subroutine calc_entropy(td_data,freq,irrepno,mass,inertia,lintest,symnumb,out)
    use snf_param
    implicit none
    !----------------------------------------------------------------------------
    ! calculation of entropy
    ! following formulae given in the manual
    !
    ! written by Markus Reiher, Uni Erlangen, 2001
    ! extension for linear molecules by Johannes Neugebauer, Uni Erlangen, 2001
    ! modified by J. Neugebauer, Uni Erlangen, 2001
    !---------------------------------------------------------------------------- 
    integer :: irrepno,i,out,symnumb
    real(double) :: freq(:),cont1,cont2,cont3,hnukt,mass,inertia(3),dummy,rott(3)
    type(type_tempcorr) :: td_data 
    logical, intent(in) :: lintest

    !---------------------------------------------------------------- 
    ! vibrational contributions  
    !---------------------------------------------------------------- 
    td_data%vibcont%entropy=0.0d0
    do i=1,irrepno
       if (freq(i).lt.0.0d0) then
          call serror('calc_entropy',0,'Imaginary frequency found !##&
           &Imaginary frequency is skipped for entropy calculation;##&
           &check whether this is a numerical artifact or not !##&
	   &=> S, Cv, and partition function may be affected by this.',out)
       else
          hnukt=planck_h*speedoflight*100.d0*freq(i)/(boltz*td_data%temper)
          cont1=1.0d0-1.0d0/exp(hnukt)
          cont2=exp(hnukt)-1.0d0
          cont3=hnukt/cont2-log(cont1) 
          td_data%vibcont%entropy=td_data%vibcont%entropy+cont3
       end if
    enddo
    td_data%vibcont%entropy=td_data%vibcont%entropy*boltz*avogadro          
    !---------------------------------------------------------------- 
    ! rotational contributions  
    !---------------------------------------------------------------- 
    dummy=planck_h*planck_h/(8.0d0*boltz*pi*pi)
    if(.not.lintest) then
      rott(1)=dummy/inertia(1)*1000.d0*avogadro/bohr**2
      rott(2)=dummy/inertia(2)*1000.d0*avogadro/bohr**2
      rott(3)=dummy/inertia(3)*1000.d0*avogadro/bohr**2
      cont1=sqrt((td_data%temper/rott(1))*(td_data%temper/rott(2))*&
       &(td_data%temper/rott(3)))
      cont3=sqrt(pi)/symnumb*cont1
      td_data%rotcont%partfunc=cont3
      td_data%rotcont%entropy=log(cont3)+1.5d0
    else
      !---------------------------------------------------------------- 
      ! there is one zero entry in "inertia" for linear molecules,
      ! but you don't know which one
      !---------------------------------------------------------------- 
      rott(1)=0.0d0
      do i=1,2
        if(inertia(i).ge.1.0d0) then
          rott(1)=dummy/inertia(i)*1000.d0*avogadro/bohr**2
          exit
        end if
      end do
      if (abs(rott(1)).le.1.0d-10) &
           &  call serror('calc_entropy',2,'Found two zero entries for&
           & moments of inertia')
      !---------------------------------------------------------------- 
      ! for molecules with c_{infty,v} symmetry one must divide cont1 
      ! by 2 !!!
      !---------------------------------------------------------------- 
      cont1=td_data%temper/rott(1)/dble(symnumb)
      td_data%rotcont%partfunc=cont1
      td_data%rotcont%entropy=log(cont1)+1.0d0
    end if
    td_data%rotcont%entropy=td_data%rotcont%entropy*boltz*avogadro         
    !---------------------------------------------------------------- 
    ! translational contributions  
    !---------------------------------------------------------------- 
    cont1=(boltz*td_data%temper)**2.5d0/(td_data%press*planck_h**3.0d0)*(2.0d0*pi*mass)**1.5d0
    td_data%tracont%partfunc=cont1
    td_data%tracont%entropy=boltz*avogadro*(log(cont1)+2.5d0)
  end subroutine calc_entropy

  !************************************************************************

  subroutine calc_partfunc(td_data,freq,irrepno,lintest,out)
    use snf_param
    implicit none
    !----------------------------------------------------------------
    ! calculation of partition function for vibration
    ! the other partition functions are calculated in calc_entropy,
    ! they are set to one here
    ! following
    ! 
    ! written by J. Neugebauer and M. Reiher, Uni Erlangen, 2001
    !----------------------------------------------------------------
    integer :: irrepno,i,out
    real(double) :: freq(:),cont1,cont2,hnukt,cont3,cont4
    type(type_tempcorr) :: td_data 
    logical :: lintest
 
    ! electronic contributions  
    !caution: electronic states without degeneracy assumed
    td_data%elecont%partfunc=1.0d0
    ! vibrational contributions  
    cont3=1.0d0
    cont4=1.0d0
    do i=1,irrepno
       if (freq(i).lt.0.0d0) then
!CMR          call serror('calc_partfunc',0,'Imaginary frequency found !##&
!CMR           &Imaginary frequency is skipped for partition function calculation;##&
!CMR           &check whether this is a numerical artifact or not !',out)
       else
          hnukt=planck_h*speedoflight*100.d0*freq(i)/(boltz*td_data%temper)
          cont1=1.0d0-exp(-1.0d0*hnukt)
          cont2=exp(-0.5d0*hnukt)/(1.0d0-exp(-1.0d0*hnukt))
          cont3=cont3/cont1
          cont4=cont4*cont2
       end if
    enddo
    td_data%vibcont%partfuncv0=cont3
    td_data%vibcont%partfunc=cont4
    ! rotational contributions  (these are set in calc_entropy)
    td_data%rotcont%partfunc=1.0d0
    ! translational contributions (these are set in calc_entropy)
    td_data%tracont%partfunc=1.0d0
  end subroutine calc_partfunc

  !************************************************************************

  subroutine calc_cv(td_data,freq,irrepno,lintest,out)
    use snf_param
    implicit none
    !----------------------------------------------------------------
    ! calculation of heat capacities c_v
    ! 
    ! written by J. Neugebauer, Uni Erlangen, 2001
    !----------------------------------------------------------------
    integer :: irrepno,i,out
    real(double) :: freq(:),cont1,cont2,hnukt,cont3
    type(type_tempcorr) :: td_data 
    logical :: lintest

    ! electronic contributions  
    !caution: electronic states without degeneracy assumed
    td_data%elecont%cv=0.0d0
    ! vibrational contributions  
    cont3=0.0d0
    do i=1,irrepno
       if (freq(i).lt.0.0d0) then
!CMR          call serror('calc_cv',0,'Imaginary frequency found !##&
!CMR           &Imaginary frequency is skipped for heat capacity calculation;##&
!CMR           &check whether this is a numerical artifact or not !',out)
       else
          hnukt=planck_h*speedoflight*100.d0*freq(i)/(boltz*td_data%temper)
          cont1=exp(hnukt)-1.0d0
          cont2=hnukt*hnukt*exp(hnukt)/cont1/cont1
          cont3=cont3+cont2
       end if
    enddo
    td_data%vibcont%cv=cont3*boltz*avogadro

    ! rotational contributions  
    if(lintest) then
      td_data%rotcont%cv=1.0d0*boltz*avogadro
    else
      td_data%rotcont%cv=1.5d0*boltz*avogadro
    end if
    ! translational contributions  
    td_data%tracont%cv=1.5d0*boltz*avogadro
    td_data%cv=td_data%tracont%cv+td_data%rotcont%cv+td_data%vibcont%cv
  end subroutine calc_cv

  !************************************************************************

  subroutine calc_energy(td_data,freqs,irrepno,lintest)
    use snf_param
    implicit none
    !----------------------------------------------------------------
    ! calculation of rot, vib, trans energy
    ! following
    ! 
    ! written by Markus Reiher, Uni Erlangen, 2001
    ! extension for linear molecules by Johannes Neugebauer,
    ! Uni Erlangen, 2001
    !----------------------------------------------------------------
    integer :: i,irrepno
    real(double) :: freqs(:),hnukt
    type(type_tempcorr) :: td_data 
    logical, intent(in) :: lintest
    ! vibrational contributions  
    td_data%vibcont%energy=0.0d0
    do i=1,irrepno
       hnukt=planck_h*speedoflight*100.d0*freqs(i)/(boltz*td_data%temper)
       td_data%vibcont%energy=td_data%vibcont%energy+hnukt*(0.5d0+1.0d0/(exp(hnukt)-1.0d0))
    enddo
    td_data%vibcont%energy=td_data%vibcont%energy*boltz*avogadro*td_data%temper/1000.0d0

    ! rotational contributions  
    if(.not.lintest) then
      td_data%rotcont%energy=1.5d0*boltz*avogadro*td_data%temper/1000.0d0 
    else
      td_data%rotcont%energy=boltz*avogadro*td_data%temper/1000.0d0 
    end if
    ! translational contributions  
    td_data%tracont%energy=1.5d0*boltz*avogadro*td_data%temper/1000.0d0
  end subroutine calc_energy

  !************************************************************************

  subroutine print_tdvalues(td_data,symnumb,spin,out)
    use snf_param
    implicit none
    !----------------------------------------------------------------
    ! print data obtained in this module
    !
    ! written by Markus Reiher, uni erlangen, march 2001
    ! sightly modified by Markus Reiher, aug. and nov. 2001
    ! modified by J. Neugebauer, Uni Erlangen, 2001
    !----------------------------------------------------------------  
    integer :: out,symnumb
    real(double) :: spin
    type(type_tempcorr),intent(in) :: td_data
    write(out,*)
    write(out,'('' Thermochemical data'')') 
    write(out,'('' -------------------'')')
    write(out,'(''   Symmetry number              :'',i15)')symnumb
    write(out,'(''   Temperature  [K]             :'',F15.2)')td_data%temper
    write(out,'(''   Pressure  [Pa]               :'',F15.2)')td_data%press
    write(out,'(''   Total spin quantum number    :'',F15.2)')spin
    write(out,'(''   Partition function (bottom)  :'',E15.6)')td_data%partfunc
    write(out,'(''   Partition function (v=0)     :'',E15.6)')td_data%partfuncv0
    write(out,'(''   ln(Q(bottom))                :'',E15.6)')log(td_data%partfunc)
    write(out,'(''   ln(Q(v=0))                   :'',E15.6)')log(td_data%partfuncv0)
    write(out,'(''   Entropy  S [J/(Mol K)]       :'',F15.2)')td_data%entropy
    write(out,'(''   Enthalpy H [kJ/mol]          :'',F15.2)')td_data%rovibtra
    write(out,'(''   Cv [kJ/(mol*K)]              :'',F15.2)')td_data%cv
    write(out,'(''   Gibbs Enthalpy G [kJ/mol]    :'',F15.2)')td_data%gibbs
    write(out,'(''   Electronic Energy  [Hartree] :'',F20.10)')td_data%elecont%energy
    write(out,'(''   Total Energy  [Hartree]      :'',F20.10)')td_data%etotal
    write(out,*)
    write(out,*) "   NOTE: * H and G do NOT contain <E_elec> !"
    write(out,*) "             (only the Total Energy contains everything) "
    write(out,*) "         * H contains the ZPE!"
    write(out,*) "         * the electronic contribution to S is the spin-only value!"
    write(out,*)
    write(out,'(''  Contributions                 :    translation&
      &       rotation      vibration     electronic'')')
    write(out,'(''   Entropy  [J/(Mol K)]         :'',4F15.2)') &
      & td_data%tracont%entropy,td_data%rotcont%entropy,&
      & td_data%vibcont%entropy,td_data%elecont%entropy
    write(out,'(''   Enthalpy  [kJ/mol]           :'',3F15.2)')  &
      & td_data%tracont%energy,td_data%rotcont%energy,&
      & td_data%vibcont%energy
    write(out,'(''   Cv [kJ/(mol*K)]              :'',3F15.2)') &
      & td_data%tracont%cv,td_data%rotcont%cv,&
      & td_data%vibcont%cv
    write(out,'(''   Partition function (bottom)  :'',3E15.6)')  &
      & td_data%tracont%partfunc,td_data%rotcont%partfunc,&
      & td_data%vibcont%partfunc
    write(out,'(''   Partition function (v=0)     :'',3E15.6)')  &
      & td_data%tracont%partfunc,td_data%rotcont%partfunc,&
      & td_data%vibcont%partfuncv0
    write(out,'(''   ln(Q(bottom))                :'',3F15.6)')  &
      & log(td_data%tracont%partfunc),log(td_data%rotcont%partfunc),&
      & log(td_data%vibcont%partfunc)
    write(out,'(''   ln(Q(v=0))                   :'',3F15.6)')  &
      & log(td_data%tracont%partfunc),log(td_data%rotcont%partfunc),&
      & log(td_data%vibcont%partfuncv0)
    write(out,*)
  end subroutine print_tdvalues


  !************************************************************************

end module snf_thermo
