
!********************************************************************************
!*   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
!*   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
!*   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
!*
!*
!*   This file is part of the package SNF.
!*
!*   SNF is free software; you can redistribute it and/or modify
!*   it under the terms of the GNU General Public License as published by
!*   the Free Software Foundation; either version 2 of the License, or
!*   (at your option) any later version.
!*
!*   This program is distributed in the hope that it will be useful,
!*   but WITHOUT ANY WARRANTY; without even the implied warranty of
!*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*   GNU General Public License for more details.
!*
!*   You should have received a copy of the GNU General Public License
!*   along with this program; if not, write to the Free Software
!*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!*
!********************************************************************************
module snf_turbomole
  use f90_kind
  use snf_error
  use snf_param

  private 
  public :: type_tm_ctrl
  public :: init_tm_ctrl,read_tm_ctrl,modify_tm_ctrl,print_tm_ctrl
  public :: read_tm_coord, write_tm_coord

  type type_tm_ctrl
     integer :: mode,    &           ! program mode
          &     inputpol,&           ! flag for polarizabilities
          &     scfconv, &           ! scfconv parameter
          &     numderiv
     real(double) :: cstep       ! derivation of cartesian coordinates
     character(len=3) :: symmetry    ! schoenfliess symbol of point group
     character(len=80) :: coord      ! filename for coord file
     character(len=80) :: control    ! filename for control file
     real(double),pointer :: dynfreq(:) ! angular frequencies for dyn. pol.
     logical :: tmpdirclean, logdirclean ! flags for cleaning up tmpdir and logdir
  end type type_tm_ctrl

  !************************************************************************

contains

  !************************************************************************

  subroutine init_tm_ctrl(ctrl)
    implicit none
    !----------------------------------------------------------------
    ! initialize default ctrl values 
    !
    ! inputpol values:
    ! 0 - No Raman calculation
    ! 1 - Raman intensities using static polarizabilities
    ! 2 - Raman intensities using dynamic polarizabilities
    ! 
    ! written by C. Kind and M. Reiher, Uni Erlangen, 1999
    ! modified by J. Neugebauer, Uni Erlangen, 2001
    !----------------------------------------------------------------
    type(type_tm_ctrl),intent(inout) :: ctrl
    nullify(ctrl%dynfreq)
    ctrl%mode= 1
    ctrl%inputpol= 0
    ctrl%scfconv = -1
    ctrl%symmetry='c1 '
    ctrl%coord='coord'
    ctrl%control='control'
    ctrl%cstep=-1.d0
    ctrl%numderiv = -1
    ctrl%tmpdirclean=.false. 
    ctrl%logdirclean=.false. 
  end subroutine init_tm_ctrl

  !************************************************************************

  subroutine read_tm_ctrl(ctrl)
    use snf_tools, only: string2integer,string2real
    implicit none
    !----------------------------------------------------------------
    ! read all avaible data from control
    !
    ! written by C. Kind and M. Reiher, Uni Erlangen, 1999
    ! modified by J. Neugebauer, 2001
    !----------------------------------------------------------------
    type(type_tm_ctrl),intent(inout) :: ctrl
    character(len=132)          :: buf,errbuff
    integer :: ucf
    logical ex,ridft
    ! --------- read whole control file  -------------------------
    ridft=.false.
    ucf = 42
    buf='  '
    inquire(file=ctrl%control,exist=ex)
    if(.not. ex) then
      write(errbuff,'(2a)') 'Cannot find file: ',ctrl%control
      call serror('read_tm_ctrl',2,errbuff)
    end if
    open(unit=ucf,file=ctrl%control,status='unknown')
    loop: do 
       read(ucf,'(a)') buf
       if(index(buf,'$end').ne.0) then
          exit loop
       else if( index(buf,'$cstep ').ne.0) then
          ctrl%cstep=string2real(buf)
       else if( index(buf,'$symmetry ').ne.0) then
          ctrl%symmetry=trim(buf(11:))
          call lower(ctrl%symmetry)
    !-----------------------------------------------------------------
    ! The next "if" is for compatibility with older versions of SNF,
    ! which did not use the snf_control file
    !-----------------------------------------------------------------
       else if( index(buf,'$nf_symmetry ').ne.0) then
          ctrl%symmetry=trim(buf(14:))
          call lower(ctrl%symmetry)
       else if( index(buf,'$scfconv ').ne.0) then
          ctrl%scfconv=string2integer(buf)
       else if( index(buf,'$scfinstab').ne.0.and.&
        &index(buf,'polly').ne.0) then
          ctrl%inputpol = 1
       else if( index(buf,'$scfinstab').ne.0.and.&
        &index(buf,'dynpol').ne.0) then
          ctrl%inputpol = 2
!          call getdfreq(buf(index(buf,'dynpol')+6:),ctrl)
       else if( index(buf,'$workescf.intel').ne.0) then
          ctrl%mode=5
       else if( index(buf,'$workescf.athlon').ne.0) then
          ctrl%mode=6
       else if( index(buf,'$tmpdirclean').ne.0) then
          ctrl%tmpdirclean=.true.
       else if( index(buf,'$logdirclean').ne.0) then
          ctrl%logdirclean=.true.
       else if( index(buf,'$ridft ').ne.0) then
          ridft=.true.
       else if( index(buf,'$traloop').ne.0) then ! this means MP2
          ctrl%mode=4
       else if( index(buf,'$cbas ').ne.0) then     ! this means rimp2
          ctrl%mode=3
       else if( index(buf,'$coord ').ne.0) then
          ctrl%coord=trim(buf(index(buf,'=')+1:))
       else if( index(buf,'$numderiv ').ne.0) then
          ctrl%numderiv=string2integer(buf)
       endif
    enddo loop
    if(ridft) then
      if(ctrl%inputpol.eq.0) then
        ctrl%mode=2
      else
        if(ctrl%mode.eq.6) then 
          ctrl%mode=8
        else
          ctrl%mode=7
        end if
      end if
    end if
    close(ucf)    
  end subroutine read_tm_ctrl


  subroutine modify_tm_ctrl(ctrl)
    implicit none
    !----------------------------------------------------------------
    ! this routine prepares the control file for a define run and 
    ! a following nf-calculation:
    ! it introduces $properties, $moment and $cstep if not present.
    ! it changes $scfconv. $symmetry is set to c1. 
    ! furthermore, $numderiv is introduced.
    !
    ! written by C. Kind and M. Reiher, Uni Erlangen, 1999
    ! modified by J. Neugebauer, Uni Erlangen, 2001
    !----------------------------------------------------------------
    type(type_tm_ctrl),intent(in) :: ctrl
    logical :: cstep_found,moments_found,&
         &     properties_found,symmetry_found,&
         &     inputpol_found,rimp2_found
    integer :: row,nrow,i
    character(len=132),allocatable :: buf(:)    ! buffer for control file   
    character(len=80) :: form
    integer :: ucf,nfreq
    !--------- read whole control file ------------------------
    ucf = 42
    open(unit=ucf,file=ctrl%control,status='unknown')
    ! count rows
    nrow=0
    allocate(buf(1))
    do 
       nrow=nrow+1
       read(ucf,'(a)') buf(1)
       if(index(buf(1),'$end').ne. 0) exit
    enddo
    rewind(ucf)
    deallocate(buf)
    allocate(buf(nrow))
    do row=1,nrow
       read(ucf,'(a)') buf(row)
    enddo
    close(ucf)     
    ! --------- set right control-keywords ---------------------
    if(ctrl%inputpol.eq.2) then
      nfreq=1
      if(associated(ctrl%dynfreq)) nfreq=ubound(ctrl%dynfreq,1)
      write(form,'(a,i2,a)') '(a,',nfreq,'f16.8)'
      if(ctrl%mode.ge.5.and.ctrl%mode.le.8) &
      write(form,'(a,i2,a)') '(a,',nfreq,'(/1f16.8))'
    end if

    cstep_found = .false.
    moments_found = .false.
    properties_found = .false.
    symmetry_found = .false.
    inputpol_found = .false.
    rimp2_found = .false.
    row=0
    open(unit=ucf,file=ctrl%control,status='unknown')
    do 
100    row=row+1
110    if( index(buf(row),'$scfconv ').ne.0) then
          write(ucf,'(a9,i2)') '$scfconv ',ctrl%scfconv
       else if( index(buf(row),'$properties').ne.0) then
          properties_found=.true.
          write(ucf,'(a)')'$properties '
          write(ucf,'(a)')'    trace                              off '  
          write(ucf,'(a)')'    moments                            active'  
          write(ucf,'(a)')'    potential                          off '  
          write(ucf,'(a)')'    cowan-griffin                      off '  
          write(ucf,'(a)')'    localization                       off '  
          write(ucf,'(a)')'    population analyses                off '  
          write(ucf,'(a)')'    plot                               off '  
          write(ucf,'(a)')'    firstorder                         off '  
          write(ucf,'(a)')'    fit                                off '  
          row=row+1
          do while(index(buf(row),'$') .eq.0)
             row=row+1
          end do
          row=row-1
       else if( index(buf(row),'$moments ').ne.0) then
          moments_found = .true.
          write(ucf,'(a)')'$moments'
          write(ucf,'(a)')' 0th 1st'
          write(ucf,'(a)')' point       0.00000000000000      0.00000000000000      0.00000000000000'
          row=row+1
          do while(index(buf(row),'$') .eq.0)
             row=row+1
          end do
          row=row-1
       else if( index(buf(row),'$cstep ').ne.0) then
          cstep_found = .true.
          write(ucf,'(''$cstep  '',F12.3)')Ctrl%cstep
       else if( index(buf(row),'$scfinstab ').ne.0) then
          if(ctrl%inputpol.eq.1.and.index(buf(row),'polly').ne.0) then
             inputpol_found = .true.
             write(ucf,'(a)') trim(buf(row))
          else if(ctrl%inputpol.eq.2.and.index(buf(row),'dynpol').ne.0) then
             inputpol_found = .true.
             if(associated(ctrl%dynfreq)) then
               write(ucf,form) '$scfinstab dynpol ',(ctrl%dynfreq(i),i=1,&
                &ubound(ctrl%dynfreq,1))
             else
               write(ucf,form) '$scfinstab dynpol ',nm2freq/514.5d0
             end if
             !---------------------------------------------
             ! skip old frequencies in case of new escf
             !---------------------------------------------
             if(ctrl%mode.ge.5.and.ctrl%mode.le.8) then
200            row=row+1
               if( index(buf(row),'$').eq.0) then
                 goto 200
               else
                 goto 110
               end if
             end if
          endif
       else if( index(buf(row),'$symmetry ').ne.0) then
          symmetry_found = .true.
          write(ucf,'(''$symmetry '',a3)') ctrl%symmetry
       else if( index(buf(row),'$rimp2').ne.0) then
          rimp2_found = .true.
          write(ucf,'(''$rimp2'')') 
       else if( index(buf(row),'$workescf.athlon').ne.0) then
          continue
       else if( index(buf(row),'$workescf.intel').ne.0) then
          continue
       else if( index(buf(row),'$numderiv ').ne.0) then
          continue
       else if( index(buf(row),'$tmpdirclean').ne.0) then
          continue
       else if( index(buf(row),'$logdirclean').ne.0) then
          continue
       else if( index(buf(row),'$end').ne.0) then
          if(.not.properties_found) then
             write(ucf,'(a)')'$properties'
             write(ucf,'(a)')'    trace                              off   '  
             write(ucf,'(a)')'    moments                            active'  
             write(ucf,'(a)')'    potential                          off   '  
             write(ucf,'(a)')'    cowan-griffin                      off   '  
             write(ucf,'(a)')'    localization                       off   '  
             write(ucf,'(a)')'    population analyses                off   '  
             write(ucf,'(a)')'    plot                               off   '  
             write(ucf,'(a)')'    firstorder                         off   '  
             write(ucf,'(a)')'    fit                                off   '  
          endif
          if(.not.moments_found) then
             write(ucf,'(a)')'$moments'
             write(ucf,'(a)')' 0th 1st'
             write(ucf,'(a)')' point       0.00000000000000      0.00000000000000      0.00000000000000 '
          endif
          if(ctrl%tmpdirclean) write(ucf,'(a)')'$tmpdirclean'
          if(ctrl%logdirclean) write(ucf,'(a)')'$logdirclean'
          if (ctrl%numderiv.eq.1) then
             write(ucf,'(a)')'$numderiv 1'
          else if (ctrl%numderiv.eq.2) then
             write(ucf,'(a)')'$numderiv 2'
          else
             call serror('modify_tm_ctrl',2,'ctrl%numderiv contains&
              & wrong entry !')
          end if
          if(.not.cstep_found) then
             write(ucf,'(''$cstep  '',F12.3)')ctrl%cstep
          endif
          if(.not.inputpol_found.and. ctrl%inputpol.eq.1) then
             write(ucf,'(a)')'$scfinstab polly'
          else if(.not.inputpol_found.and. ctrl%inputpol.eq.2) then
             if(associated(ctrl%dynfreq)) then
               write(ucf,form) '$scfinstab dynpol ',(ctrl%dynfreq(i),i=1,&
                &ubound(ctrl%dynfreq,1))
             else
               write(ucf,form) '$scfinstab dynpol ',nm2freq/514.5
             end if
          endif
          if(.not. symmetry_found) then
             call serror('modify_tm_ctrl',2,'$symmetry not found in control file')
          endif
          if(.not.rimp2_found.and.ctrl%mode.eq.3) then
             write(ucf,'(''$rimp2  '')')
          endif
          if(ctrl%mode.eq.6.or.ctrl%mode.eq.8) then 
            write(ucf,'(''$workescf.athlon'')') 
          else if(ctrl%mode.eq.5.or.ctrl%mode.eq.7) then
            write(ucf,'(''$workescf.intel'')')
          end if
          if(ctrl%mode.ge.20.and.ctrl%mode.le.25) &
     &      write(ucf,'(a,i3)') '$daltonmode',ctrl%mode
          write(ucf,'(a)')'$end' 
          exit
       else if((.not.(((index(buf(row),'ridft') .ne.0) .or. &
            &   (index(buf(row),'ricore') .ne.0) .or. &
            &   (index(buf(row),'jbas') .ne.0)) .and. &
            &  (ctrl%inputpol .ge. 1.and.ctrl%mode.eq.2))).and.&
            &  (index(buf(row),'daltonmode').eq.0)) then
          write(ucf,'(a)') trim(buf(row))
       endif
    enddo
    close(ucf)
    deallocate(buf)
  end subroutine modify_tm_ctrl

  !************************************************************************

  subroutine print_tm_ctrl(ctrl,unit)
    implicit none
    !----------------------------------------------------------------
    ! output routine
    !
    ! written by C. Kind and M. Reiher, Uni Erlangen, 1999
    !
    ! modified by J. Neugebauer, Universitaet Erlangen-Nuernberg,
    ! 05/2001
    !----------------------------------------------------------------
    type(type_tm_ctrl),intent(in) :: ctrl
    integer,optional :: unit
    integer :: out
    if(present(unit)) then
       out=unit
    else
       out=6
    endif
    write(out,*)
    write(out,*)' s-numfreq essentials from turbomole control file:'
    write(out,'(a16,i3)')' mode         = ',ctrl%mode
    write(out,'(a16,i3)')' inputpol     = ',ctrl%inputpol
    write(out,'(a16,i3)')' numderiv     = ',ctrl%numderiv 
    write(out,'(a16,i3)')' scfconv      = ',ctrl%scfconv 
    write(out,'(a16,f10.4)')' cstep        = ',ctrl%cstep
    write(out,'(a16,a3)')' symmetry     = ',ctrl%symmetry
    write(out,'(a16,80a)')' coord file   = ',trim(ctrl%coord)
  end subroutine print_tm_ctrl

  subroutine read_tm_coord(tempname,molecule)
    !-----------------------------------------------------------------
    ! read turbomole coord files
    !
    ! written by C. Kind and M. Reiher, Uni Erlangen, 1999
    ! modified by J. Neugebauer, Universitaet Erlangen-Nuernberg, 2001
    !-----------------------------------------------------------------
    use snf_molecule,only : type_atom,find_nucl_charge,find_mass,fiso_mass
    implicit none
    !
    ! read coord file an allocate molecule
    !
    type(type_atom),pointer :: molecule(:)
    character*(*) tempname
    character(len=255) :: b1,errbuff
    character(len=2) :: element
    integer :: lu,atom,natoms
    lu=1
    open (unit=lu,file=tempname,status='unknown',err=1001)
    do 
       read (lu,'(a)',end=999) b1
       if (b1(1:6).eq.'$coord') exit
    enddo
    natoms=0
    do 
       read (lu,'(a)',end=999) b1
       if(b1(1:1).eq.'$') exit
       natoms=natoms+1
    enddo
    rewind(lu)
    allocate(molecule(natoms))
    molecule(:)%changem=.false.
    do 
       read (lu,'(a)',end=999) b1
       if (b1(1:6).eq.'$coord') exit
    enddo

    do atom=1,natoms
       read(lu,*) molecule(atom)%cartcoord(1), &
            &     molecule(atom)%cartcoord(2), &
            &     molecule(atom)%cartcoord(3), &
            &     element
       call upper(element)
       molecule(atom)%element=element
       molecule(atom)%nucl_charge = find_nucl_charge(element)
       if(molecule(atom)%nucl_charge .eq. 0) &
     &    write(*,'(a,i2,a)') ' Found dummy atom (atom No.',atom,')'
       if(molecule(atom)%nucl_charge .eq. -1) then
          write(errbuff,'(2a)')'Not a valid element symbol:',element
          call serror('read_tm_coord',2,errbuff)
       endif
       ! if deuterium
       if(element .eq. 'D') then
          molecule(atom)%mass  = fiso_mass(1,2)
       else
          molecule(atom)%mass  = find_mass(molecule(atom)%nucl_charge)
       end if
    enddo
999 close (unit=lu)
    return
1001 call serror('read_tm_coord',2,'Error while reading file ''coord''.')
  end subroutine read_tm_coord

  !************************************************************************

  subroutine write_tm_coord(fname,molecule)
    !
    ! written by C. Kind and M. Reiher, Uni Erlangen, 1999
    use snf_molecule,only: type_atom
    implicit none
    !
    ! write coord file
    !
    type(type_atom),intent(in) :: molecule(:)
    character*(*) fname
    integer :: atom
    open(unit=44,file=fname,status='unknown')
    write(44,'(''$coord'')')
    do atom=1,ubound(molecule,1)
       write(44,'(3f18.10,5x,a2)') molecule(atom)%cartcoord(1), &
            &        molecule(atom)%cartcoord(2), &
            &        molecule(atom)%cartcoord(3), &
            &        molecule(atom)%element
    enddo
    write(44,'(''$end'')')
    close(44)
  end subroutine write_tm_coord

  !************************************************************************

end module snf_turbomole
